@@ -91,6 +91,7 @@ typedef enum cmd_code {
 	CMD_RUN,
 	CMD_HOLD,
 	CMD_ALTER,
+	CMD_STEAL,
 	CMD_SLEEP,
 
     /* Store commands */	
@@ -103,31 +104,18 @@ typedef enum cmd_code {
 	CMD_SUICIDE,
 
 	CMD_HELP,
-	CMD_REPEAT
+	CMD_REPEAT,
+	CMD_COMMAND_MONSTER
 } cmd_code;
 
 typedef enum cmd_context {
-	CMD_INIT,
-	CMD_BIRTH,
-	CMD_GAME,
-	CMD_STORE,
-	CMD_DEATH
+	CTX_INIT,
+	CTX_BIRTH,
+	CTX_GAME,
+	CTX_STORE,
+	CTX_DEATH
 } cmd_context;
 
-enum {
-	DIR_UNKNOWN = 0,
-	DIR_NW = 7,
-	DIR_N = 8,
-	DIR_NE = 9,
-	DIR_W = 4,
-	DIR_TARGET = 5,
-	DIR_NONE = 5,
-	DIR_E = 6,
-	DIR_SW = 1,
-	DIR_S = 2,
-	DIR_SE = 3,
-};
-
 /**
  * ------------------------------------------------------------------------
  * Argument structures
@@ -260,6 +248,17 @@ errr cmdq_push(cmd_code c);
  */
 void cmdq_execute(cmd_context ctx);
 
+/**
+ * Remove all commands from the queue.
+ */
+void cmdq_flush(void);
+
+/**
+ * Return true if the previous command used an item from the floor.
+ * Otherwise, return false.
+ */
+bool cmdq_does_previous_use_floor_item(void);
+
 /**
  * ------------------------------------------------------------------------
  * Command repeat manipulation
@@ -301,7 +300,7 @@ void cmd_set_arg_choice(struct command *cmd, const char *arg, int choice);
 void cmd_set_arg_string(struct command *cmd, const char *arg, const char *str);
 void cmd_set_arg_direction(struct command *cmd, const char *arg, int dir);
 void cmd_set_arg_target(struct command *cmd, const char *arg, int target);
-void cmd_set_arg_point(struct command *cmd, const char *arg, int x, int y);
+void cmd_set_arg_point(struct command *cmd, const char *arg, struct loc grid);
 void cmd_set_arg_item(struct command *cmd, const char *arg, struct object *obj);
 void cmd_set_arg_number(struct command *cmd, const char *arg, int amt);
 
@@ -313,7 +312,7 @@ int cmd_get_arg_choice(struct command *cmd, const char *arg, int *choice);
 int cmd_get_arg_string(struct command *cmd, const char *arg, const char **str);
 int cmd_get_arg_direction(struct command *cmd, const char *arg, int *dir);
 int cmd_get_arg_target(struct command *cmd, const char *arg, int *target);
-int cmd_get_arg_point(struct command *cmd, const char *arg, int *x, int *y);
+int cmd_get_arg_point(struct command *cmd, const char *arg, struct loc *grid);
 int cmd_get_arg_item(struct command *cmd, const char *arg, struct object **obj);
 int cmd_get_arg_number(struct command *cmd, const char *arg, int *amt);
 
