@@ -115,101 +115,22 @@ static int beam_chance(int tval)
 }
 
 
-typedef enum {
-	ART_TAG_NONE,
-	ART_TAG_NAME,
-	ART_TAG_KIND,
-	ART_TAG_VERB,
-	ART_TAG_VERB_IS
-} art_tag_t;
-
-static art_tag_t art_tag_lookup(const char *tag)
-{
-	if (strncmp(tag, "name", 4) == 0)
-		return ART_TAG_NAME;
-	else if (strncmp(tag, "kind", 4) == 0)
-		return ART_TAG_KIND;
-	else if (strncmp(tag, "s", 1) == 0)
-		return ART_TAG_VERB;
-	else if (strncmp(tag, "is", 2) == 0)
-		return ART_TAG_VERB_IS;
-	else
-		return ART_TAG_NONE;
-}
-
 /**
  * Print an artifact activation message.
- *
- * In order to support randarts, with scrambled names, we re-write
- * the message to replace instances of {name} with the artifact name
- * and instances of {kind} with the type of object.
- *
- * This code deals with plural and singular forms of verbs correctly
- * when encountering {s}, though in fact both names and kinds are
- * always singular in the current code (gloves are "Set of" and boots
- * are "Pair of")
  */
 static void activation_message(struct object *obj)
 {
-	char buf[1024] = "\0";
-	const char *next;
-	const char *s;
-	const char *tag;
-	const char *in_cursor;
-	size_t end = 0;
+	const char *message;
 
-	/* See if we have a message */
+	/* See if we have a message, then print it */
 	if (!obj->activation) return;
 	if (!obj->activation->message) return;
-	if (obj->artifact && obj->artifact->alt_msg)
-		in_cursor = obj->artifact->alt_msg;
-	else
-		in_cursor = obj->activation->message;
-
-	next = strchr(in_cursor, '{');
-	while (next) {
-		/* Copy the text leading up to this { */
-		strnfcat(buf, 1024, &end, "%.*s", next - in_cursor, in_cursor); 
-
-		s = next + 1;
-		while (*s && isalpha((unsigned char) *s)) s++;
-
-		/* Valid tag */
-		if (*s == '}') {
-			/* Start the tag after the { */
-			tag = next + 1;
-			in_cursor = s + 1;
-
-			switch(art_tag_lookup(tag)) {
-			case ART_TAG_NAME:
-				end += object_desc(buf, 1024, obj, ODESC_PREFIX | ODESC_BASE); 
-				break;
-			case ART_TAG_KIND:
-				object_kind_name(&buf[end], 1024-end, obj->kind, true);
-				end += strlen(&buf[end]);
-				break;
-			case ART_TAG_VERB:
-				if (obj->number == 1) {
-					strnfcat(buf, 1024, &end, "s");
-				}
-				break;
-			case ART_TAG_VERB_IS:
-				if (obj->number > 1)
-					strnfcat(buf, 1024, &end, "are");
-				else
-					strnfcat(buf, 1024, &end, "is");
-			default:
-				break;
-			}
-		} else
-			/* An invalid tag, skip it */
-			in_cursor = next + 1;
-
-		next = strchr(in_cursor, '{');
+	if (obj->artifact && obj->artifact->alt_msg) {
+		message = obj->artifact->alt_msg;
+	} else {
+		message = obj->activation->message;
 	}
-	strnfcat(buf, 1024, &end, in_cursor);
-
-	msg("%s", buf);
+	print_custom_message(obj, message, MSG_GENERIC);
 }
 
 
@@ -227,6 +148,15 @@ void do_cmd_uninscribe(struct command *cmd)
 {
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get arguments */
 	if (cmd_get_item(cmd, "item", &obj,
 			/* Prompt */ "Uninscribe which item?",
@@ -253,6 +183,15 @@ void do_cmd_inscribe(struct command *cmd)
 	char prompt[1024];
 	char o_name[80];
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get arguments */
 	if (cmd_get_item(cmd, "item", &obj,
 			/* Prompt */ "Inscribe which item?",
@@ -271,6 +210,7 @@ void do_cmd_inscribe(struct command *cmd)
 		return;
 
 	obj->note = quark_add(str);
+	string_free((char *)str);
 
 	player->upkeep->notice |= (PN_COMBINE | PN_IGNORE);
 	player->upkeep->redraw |= (PR_INVEN | PR_EQUIP);
@@ -282,6 +222,8 @@ void do_cmd_inscribe(struct command *cmd)
  */
 void do_cmd_autoinscribe(struct command *cmd)
 {
+	if (player_is_shapechanged(player)) return;
+
 	autoinscribe_ground();
 	autoinscribe_pack();
 
@@ -302,6 +244,15 @@ void do_cmd_takeoff(struct command *cmd)
 {
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get arguments */
 	if (cmd_get_item(cmd, "item", &obj,
 			/* Prompt */ "Take off or unwield which item?",
@@ -331,6 +282,15 @@ void do_cmd_wield(struct command *cmd)
 	int slot;
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get arguments */
 	if (cmd_get_item(cmd, "item", &obj,
 			/* Prompt */ "Wear or wield which item?",
@@ -399,10 +359,10 @@ void do_cmd_wield(struct command *cmd)
 	else
 		act = "You were wearing";
 
+	inven_wield(obj, slot);
+
 	/* Message */
 	msgt(MSG_WIELD, "%s %s (%c).", act, o_name, gear_to_label(equip_obj));
-
-	inven_wield(obj, slot);
 }
 
 /**
@@ -413,6 +373,15 @@ void do_cmd_drop(struct command *cmd)
 	int amt;
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get arguments */
 	if (cmd_get_item(cmd, "item", &obj,
 			/* Prompt */ "Drop which item?",
@@ -453,12 +422,12 @@ static void use_aux(struct command *cmd, struct object *obj, enum use use,
 					int snd)
 {
 	struct effect *effect = object_effect(obj);
-	bool ident = false, used = false, can_use = true;
-	bool was_aware;
+	bool can_use = true;
+	bool was_aware, from_floor;
 	bool known_aim = false;
 	bool none_left = false;
 	int dir = 5;
-	int px = player->px, py = player->py;
+	char label = gear_to_label(obj);
 	struct trap_kind *rune = lookup_trap("glyph of warding");
 
 	/* Get arguments */
@@ -499,7 +468,9 @@ static void use_aux(struct command *cmd, struct object *obj, enum use use,
 	/* Execute the effect */
 	if (can_use) {
 		int beam = beam_chance(obj->tval);
-		int boost, level;
+		int boost, level, charges = 0;
+		bool ident = false, used;
+		struct object *work_obj;
 
 		/* Get the level */
 		if (obj->artifact)
@@ -519,9 +490,44 @@ static void use_aux(struct command *cmd, struct object *obj, enum use use,
 		}
 
 		/* Boost damage effects if skill > difficulty */
-		boost = MAX(player->state.skills[SKILL_DEVICE] - level, 0);
+		boost = MAX((player->state.skills[SKILL_DEVICE] - level) / 2, 0);
 
-		/* Do effect */
+		/*
+		 * Tentatively deduct the amount used - the effect could leave
+		 * the object inaccessible making it difficult to do after a
+		 * successful use.  For the same reason, get a copy of the
+		 * object to use for propagating knowledge.
+		 */
+		if (use == USE_SINGLE) {
+			if (object_is_carried(player, obj)) {
+				work_obj = gear_object_for_use(obj, 1, false, &none_left);
+				from_floor = false;
+			} else {
+				work_obj = floor_object_for_use(obj, 1, false, &none_left);
+				from_floor = true;
+			}
+		} else  {
+			if (use == USE_CHARGE) {
+				charges = obj->pval;
+				/* Use a single charge */
+				obj->pval--;
+			} else if (use == USE_TIMEOUT) {
+				charges = obj->timeout;
+				obj->timeout += randcalc(obj->time, 0, RANDOMISE);
+			}
+			work_obj = object_new();
+			object_copy(work_obj, obj);
+			work_obj->oidx = 0;
+			if (obj->known) {
+				work_obj->known = object_new();
+				object_copy(work_obj->known, obj->known);
+				work_obj->known->oidx = 0;
+			}
+			from_floor = !object_is_carried(player, obj);
+		}
+
+		/* Do effect; use original not copy (proj. effect handling) */
+		target_fix();
 		used = effect_do(effect,
 							source_player(),
 							obj,
@@ -529,63 +535,98 @@ static void use_aux(struct command *cmd, struct object *obj, enum use use,
 							was_aware,
 							dir,
 							beam,
-							boost);
+							boost,
+							cmd);
+		target_release();
 
-		/* Quit if the item wasn't used and no knowledge was gained */
-		if (!used && (was_aware || !ident)) return;
+		if (!used) {
+			/* Restore the tentative deduction. */
+			if (use == USE_SINGLE) {
+				/* Drop copy to simplify subsequent logic */
+				struct object *dropped = object_new();
+
+				object_copy(dropped, work_obj);
+				if (work_obj->known) {
+					dropped->known = object_new();
+					object_copy(dropped->known, work_obj->known);
+				}
+				if (from_floor) {
+					drop_near(cave, &dropped, 0, player->grid, true, true);
+				} else {
+					inven_carry(player, dropped, true, true);
+				}
+			} else if (use == USE_CHARGE) {
+				obj->pval = charges;
+			} else if (use == USE_TIMEOUT) {
+				obj->timeout = charges;
+			}
+
+			/*
+			 * Quit if the item wasn't used and no knowledge was
+			 * gained
+			 */
+			if (was_aware || !ident) {
+				if (work_obj->known) {
+					object_delete(&work_obj->known);
+				}
+				object_delete(&work_obj);
+				return;
+			}
+		}
+
+		/* Increase knowledge */
+		if (use == USE_SINGLE) {
+			char name[80];
+			/* Single use items are automatically learned */
+			if (!was_aware) {
+				object_learn_on_use(player, obj);
+			}
+			/* Get a description */
+			if (used && none_left) {
+				obj->number--;
+			}
+			object_desc(name, sizeof(name), obj, ODESC_PREFIX | ODESC_FULL);
+			if (from_floor) {
+				/* Print a message */
+				msg("You see %s.", name);
+			} else {
+				msg("You have %s (%c).", name, label);
+			}
+			if (used && none_left) {
+				obj->number++;
+			}
+		} else {
+			/* Wearables may need update, other things become known or tried */
+			if (tval_is_wearable(work_obj)) {
+				update_player_object_knowledge(player);
+			} else if (!was_aware && ident) {
+				object_learn_on_use(player, work_obj);
+			} else {
+				object_flavor_tried(work_obj);
+			}
+		}
+
+		if (used && use == USE_CHARGE) {
+			/* Describe charges */
+			if (from_floor)
+				floor_item_charges(work_obj);
+			else
+				inven_item_charges(work_obj);
+		}
+
+		/* Clean up created copy. */
+		if (work_obj->known)
+			object_delete(&work_obj->known);
+		object_delete(&work_obj);
+	} else {
+		from_floor = !object_is_carried(player, obj);
 	}
 
-	/* If the item is a null pointer or has been wiped, be done now */
-	if (!obj) return;
-
 	/* Use the turn */
 	player->upkeep->energy_use = z_info->move_energy;
 
-	/* Increase knowledge */
-	if (use == USE_SINGLE) {
-		/* Single use items are automatically learned */
-		if (!was_aware) {
-			object_learn_on_use(player, obj);
-		}
-	} else {
-		/* Wearables may need update, other things become known or tried */
-		if (tval_is_wearable(obj)) {
-			update_player_object_knowledge(player);
-		} else if (!was_aware && ident) {
-			object_learn_on_use(player, obj);
-		} else if (can_use) {
-			object_flavor_tried(obj);
-		}
-	}
-
-	/* Chargeables act differently to single-used items when not used up */
-	if (used && use == USE_CHARGE) {
-		/* Use a single charge */
-		obj->pval--;
-
-		/* Describe charges */
-		if (object_is_carried(player, obj))
-			inven_item_charges(obj);
-		else
-			floor_item_charges(obj);
-	} else if (used && use == USE_TIMEOUT) {
-		obj->timeout += randcalc(obj->time, 0, RANDOMISE);
-	} else if (used && use == USE_SINGLE) {
-		struct object *used_obj;
-
-		/* Destroy an item in the pack */
-		if (object_is_carried(player, obj))
-			used_obj = gear_object_for_use(obj, 1, true, &none_left);
-		else
-			/* Destroy an item on the floor */
-			used_obj = floor_object_for_use(obj, 1, true, &none_left);
-		if (used_obj->known)
-			object_delete(&used_obj->known);
-		object_delete(&used_obj);
-	}
-
-	/* Autoinscribe if we still have any */
-	if (!none_left)
+	/* Autoinscribe if we are guaranteed to still have any */
+	if (!none_left && !from_floor)
 		apply_autoinscription(obj);
 
 	/* Mark as tried and redisplay */
@@ -593,10 +634,10 @@ static void use_aux(struct command *cmd, struct object *obj, enum use use,
 	player->upkeep->redraw |= (PR_INVEN | PR_EQUIP | PR_OBJECT);
 
 	/* Hack to make Glyph of Warding work properly */
-	if (square_trap_specific(cave, py, px, rune->tidx)) {
+	if (square_trap_specific(cave, player->grid, rune->tidx)) {
 		/* Push objects off the grid */
-		if (square_object(cave, py, px))
-			push_object(py, px);
+		if (square_object(cave, player->grid))
+			push_object(player->grid);
 	}
 }
 
@@ -608,6 +649,15 @@ void do_cmd_read_scroll(struct command *cmd)
 {
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Check player can use scroll */
 	if (!player_can_read(player, true))
 		return;
@@ -629,6 +679,15 @@ void do_cmd_use_staff(struct command *cmd)
 {
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get an item */
 	if (cmd_get_item(cmd, "item", &obj,
 			"Use which staff? ",
@@ -651,6 +710,15 @@ void do_cmd_aim_wand(struct command *cmd)
 {
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get an item */
 	if (cmd_get_item(cmd, "item", &obj,
 			"Aim which wand? ",
@@ -673,6 +741,15 @@ void do_cmd_zap_rod(struct command *cmd)
 {
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get an item */
 	if (cmd_get_item(cmd, "item", &obj,
 			"Zap which rod? ",
@@ -695,6 +772,15 @@ void do_cmd_activate(struct command *cmd)
 {
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get an item */
 	if (cmd_get_item(cmd, "item", &obj,
 			"Activate which item? ",
@@ -734,6 +820,15 @@ void do_cmd_quaff_potion(struct command *cmd)
 {
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get an item */
 	if (cmd_get_item(cmd, "item", &obj,
 			"Quaff which potion? ",
@@ -751,6 +846,15 @@ void do_cmd_use(struct command *cmd)
 {
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get an item */
 	if (cmd_get_item(cmd, "item", &obj,
 			"Use which item? ",
@@ -767,8 +871,13 @@ void do_cmd_use(struct command *cmd)
 	else if (tval_is_staff(obj))		do_cmd_use_staff(cmd);
 	else if (tval_is_scroll(obj))		do_cmd_read_scroll(cmd);
 	else if (obj_can_refill(obj))		do_cmd_refill(cmd);
-	else if (obj_is_activatable(obj))	do_cmd_activate(cmd);
-	else
+	else if (obj_is_activatable(obj)) {
+		if (object_is_equipped(player->body, obj)) {
+			do_cmd_activate(cmd);
+		} else {
+			msg("Equip the item to use it.");
+		}
+	} else
 		msg("The item cannot be used at the moment");
 }
 
@@ -807,7 +916,7 @@ static void refill_lamp(struct object *lamp, struct object *obj)
 			if (object_is_carried(player, obj))
 				inven_carry(player, used, true, true);
 			else
-				drop_near(cave, &used, 0, player->py, player->px, false);
+				drop_near(cave, &used, 0, player->grid, false, true);
 		} else
 			/* Empty a single lantern */
 			obj->timeout = 0;
@@ -844,6 +953,15 @@ void do_cmd_refill(struct command *cmd)
 	struct object *light = equipped_item_by_slot_name(player, "light");
 	struct object *obj;
 
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	/* Get an item */
 	if (cmd_get_item(cmd, "item", &obj,
 			"Refuel with with fuel source? ",
@@ -852,7 +970,7 @@ void do_cmd_refill(struct command *cmd)
 			USE_INVEN | USE_FLOOR) != CMD_OK) return;
 
 	/* Check what we're wielding. */
-	if (!tval_is_light(light)) {
+	if (!light || !tval_is_light(light)) {
 		msg("You are not wielding a light.");
 		return;
 	} else if (of_has(light->flags, OF_NO_FUEL)) {
@@ -882,11 +1000,15 @@ void do_cmd_refill(struct command *cmd)
 void do_cmd_cast(struct command *cmd)
 {
 	int spell_index, dir = 0;
-
-	const char *verb = player->class->magic.spell_realm->verb;
-	const char *noun = player->class->magic.spell_realm->spell_noun;
 	const struct class_spell *spell;
 
+	if (player_is_shapechanged(player)) {
+		if (get_check("Change back to your original form? " )) {
+			player_resume_normal_shape(player);
+		}
+		return;
+	}
+
 	/* Check the player can cast spells at all */
 	if (!player_can_cast(player, true))
 		return;
@@ -911,6 +1033,9 @@ void do_cmd_cast(struct command *cmd)
 
 	/* Verify "dangerous" spells */
 	if (spell->smana > player->csp) {
+		const char *verb = spell->realm->verb;
+		const char *noun = spell->realm->spell_noun;
+
 		/* Warning */
 		msg("You do not have enough mana to %s this %s.", verb, noun);
 
@@ -922,8 +1047,15 @@ void do_cmd_cast(struct command *cmd)
 	}
 
 	/* Cast a spell */
-	if (spell_cast(spell_index, dir))
-		player->upkeep->energy_use = z_info->move_energy;
+	target_fix();
+	if (spell_cast(spell_index, dir, cmd)) {
+		if (player->timed[TMD_FASTCAST]) {
+			player->upkeep->energy_use = (z_info->move_energy * 3) / 4;
+		} else {
+			player->upkeep->energy_use = z_info->move_energy;
+		}
+	}
+	target_release();
 }
 
 
@@ -960,8 +1092,6 @@ void do_cmd_study_book(struct command *cmd)
 	struct class_spell *spell;
 	int i, k = 0;
 
-	const char *p = player->class->magic.spell_realm->spell_noun;
-
 	if (cmd_get_item(cmd, "item", &book_obj,
 			/* Prompt */ "Study which book? ",
 			/* Error  */ "You cannot learn any new spells from the books you have.",
@@ -969,7 +1099,7 @@ void do_cmd_study_book(struct command *cmd)
 			/* Choice */ USE_INVEN | USE_FLOOR) != CMD_OK)
 		return;
 
-	book = object_to_book(book_obj);
+	book = player_object_to_book(player, book_obj);
 	track_object(player->upkeep, book_obj);
 	handle_stuff(player);
 
@@ -986,9 +1116,9 @@ void do_cmd_study_book(struct command *cmd)
 		spell_index = spell->sidx;
 	}
 
-	if (spell_index < 0)
-		msg("You cannot learn any %ss in that book.", p);
-	else {
+	if (spell_index < 0) {
+		msg("You cannot learn any %ss in that book.", book->realm->spell_noun);
+	} else {
 		spell_learn(spell_index);
 		player->upkeep->energy_use = z_info->move_energy;
 	}
@@ -1001,6 +1131,15 @@ void do_cmd_study_book(struct command *cmd)
  */
 void do_cmd_study(struct command *cmd)
 {
+	if (player_is_shapechanged(player)) {
+		msg("You cannot do this while in %s form.",	player->shape->name);
+		if (get_check("Do you want to change back? " )) {
+			player_resume_normal_shape(player);
+		} else {
+			return;
+		}
+	}
+
 	if (player_has(player, PF_CHOOSE_SPELLS))
 		do_cmd_study_spell(cmd);
 	else
