@@ -26,15 +26,12 @@
 /* cmd-cave.c */
 void do_cmd_go_up(struct command *cmd);
 void do_cmd_go_down(struct command *cmd);
-bool search(bool verbose);
-void do_cmd_search(struct command *cmd);
-void do_cmd_toggle_search(struct command *cmd);
 void do_cmd_open(struct command *cmd);
 void do_cmd_close(struct command *cmd);
 void do_cmd_tunnel(struct command *cmd);
 void do_cmd_disarm(struct command *cmd);
-void do_cmd_alter_aux(int dir);
 void do_cmd_alter(struct command *cmd);
+void do_cmd_steal(struct command *cmd);
 void move_player(int dir, bool disarm);
 void do_cmd_walk(struct command *cmd);
 void do_cmd_jump(struct command *cmd);
@@ -45,11 +42,11 @@ void do_cmd_rest(struct command *cmd);
 void do_cmd_sleep(struct command *cmd);
 void display_feeling(bool obj_only);
 void do_cmd_feeling(void);
+void do_cmd_mon_command(struct command *cmd);
 
 /* cmd-misc.c */
 void do_cmd_wizard(void);
 void do_cmd_suicide(struct command *cmd);
-void do_cmd_version(void);
 void do_cmd_note(void);
 
 /* cmd-obj.c */
@@ -74,7 +71,7 @@ void do_cmd_study_book(struct command *cmd);
 void do_cmd_study(struct command *cmd);
 
 /* cmd-pickup.c */
-int do_autopickup(void);
+int do_autopickup(struct player *p);
 void do_cmd_pickup(struct command *cmd);
 void do_cmd_autopickup(struct command *cmd);
 
