@@ -21,6 +21,9 @@
 
 #include "source.h"
 #include "object.h"
+#include "player-attack.h"
+#include "cmds.h"
+
 
 /* Types of effect */
 typedef enum
@@ -35,12 +38,12 @@ typedef enum
 /*** Functions ***/
 
 void free_effect(struct effect *source);
-bool effect_valid(struct effect *effect);
-bool effect_aim(struct effect *effect);
-const char *effect_info(struct effect *effect);
-const char *effect_desc(struct effect *effect);
+bool effect_valid(const struct effect *effect);
+bool effect_aim(const struct effect *effect);
+const char *effect_info(const struct effect *effect);
+const char *effect_desc(const struct effect *effect);
 effect_index effect_lookup(const char *name);
-int effect_param(int index, const char *type);
+int effect_subtype(int index, const char *type);
 bool effect_do(struct effect *effect,
 	struct source origin,
 	struct object *obj,
@@ -48,13 +51,17 @@ bool effect_do(struct effect *effect,
 	bool aware,
 	int dir,
 	int beam,
-	int boost);
+	int boost,
+	struct command *cmd);
 void effect_simple(int index,
 	struct source origin,
 	const char *dice_string,
-	int p1,
-	int p2,
-	int p3,
+	int subtype,
+	int radius,
+	int other,
+	int y,
+	int x,
 	bool *ident);
+int recharge_failure_chance(const struct object *obj, int strength);
 
 #endif /* INCLUDED_EFFECTS_H */
