@@ -28,6 +28,7 @@
 #include "game-world.h"
 #include "generate.h"
 #include "init.h"
+#include "mon-group.h"
 #include "mon-make.h"
 #include "obj-util.h"
 #include "trap.h"
@@ -37,7 +38,7 @@ struct chunk **chunk_list;     /**< list of pointers to saved chunks */
 u16b chunk_list_max = 0;      /**< current max actual chunk index */
 
 /**
- * Write a the terrain info of a chunk to memory and return a pointer to it
+ * Write the terrain info of a chunk to memory and return a pointer to it
  *
  * \param c chunk being written
  * \return the memory location of the chunk
@@ -52,8 +53,8 @@ struct chunk *chunk_write(struct chunk *c)
 	for (y = 0; y < new->height; y++) {
 		for (x = 0; x < new->width; x++) {
 			/* Terrain */
-			new->squares[y][x].feat = c->squares[y][x].feat;
-			sqinfo_copy(new->squares[y][x].info, c->squares[y][x].info);
+			new->squares[y][x].feat = square(c, loc(x, y)).feat;
+			sqinfo_copy(square(new, loc(x, y)).info, square(c, loc(x, y)).info);
 		}
 	}
 
@@ -70,9 +71,7 @@ void chunk_list_add(struct chunk *c)
 	int newsize = (chunk_list_max + CHUNK_LIST_INCR) *	sizeof(struct chunk *);
 
 	/* Lengthen the list if necessary */
-	if (chunk_list_max == 0)
-		chunk_list = mem_zalloc(newsize);
-	else if ((chunk_list_max % CHUNK_LIST_INCR) == 0)
+	if ((chunk_list_max % CHUNK_LIST_INCR) == 0)
 		chunk_list = (struct chunk **) mem_realloc(chunk_list, newsize);
 
 	/* Add the new one */
@@ -86,27 +85,20 @@ void chunk_list_add(struct chunk *c)
  */
 bool chunk_list_remove(char *name)
 {
-	int i, j;
-	int newsize = 0;
+	int i;
 
+	/* Find the match */
 	for (i = 0; i < chunk_list_max; i++) {
-		/* Find the match */
 		if (!strcmp(name, chunk_list[i]->name)) {
-			/* Copy all the succeeding ones back one */
-			for (j = i + 1; j < chunk_list_max; j++)
+			/* Copy all the succeeding chunks back one */
+			int j;
+			for (j = i + 1; j < chunk_list_max; j++) {
 				chunk_list[j - 1] = chunk_list[j];
+			}
 
-			/* Destroy the last one, and shorten the list */
-			/* Don't do this for now - may be unnecessary, as chunk list never
-			 * really gets shorter */
-			//if ((chunk_list_max % CHUNK_LIST_INCR) == 0)
-			//	newsize = (chunk_list_max - CHUNK_LIST_INCR) *	
-			//		sizeof(struct chunk *);
+			/* Shorten the list and return */
 			chunk_list_max--;
 			chunk_list[chunk_list_max] = NULL;
-			if (newsize)
-				chunk_list = (struct chunk **) mem_realloc(chunk_list, newsize);
-
 			return true;
 		}
 	}
@@ -163,39 +155,43 @@ struct chunk *chunk_find_adjacent(struct player *p, bool above)
 /**
  * Transform y, x coordinates by rotation, reflection and translation
  * Stolen from PosChengband
- * \param y the coordinates being transformed
- * \param x the coordinates being transformed
- * \param y0 how much the coordinates are being translated
- * \param x0 how much the coordinates are being translated
+ * \param grid the grid being transformed
+ * \param y0 how much the grid is being translated vertically
+ * \param x0 how much the grid is being translated horizontally
  * \param height height of the chunk
  * \param width width of the chunk
  * \param rotate how much to rotate, in multiples of 90 degrees clockwise
  * \param reflect whether to reflect horizontally
  */
-void symmetry_transform(int *y, int *x, int y0, int x0, int height, int width,
+void symmetry_transform(struct loc *grid, int y0, int x0, int height, int width,
 						int rotate, bool reflect)
 {
 	int i;
 
 	/* Rotate (in multiples of 90 degrees clockwise) */
     for (i = 0; i < rotate % 4; i++) {
-        int temp = *x;
-        *x = height - 1 - (*y);
-        *y = temp;
+        int temp = grid->x;
+        grid->x = height - 1 - (grid->y);
+        grid->y = temp;
     }
 
 	/* Reflect (horizontally) */
 	if (reflect)
-		*x = width - 1 - *x;
+		grid->x = width - 1 - grid->x;
 
 	/* Translate */
-	*y += y0;
-	*x += x0;
+	grid->y += y0;
+	grid->x += x0;
 }
 
 /**
- * Write a chunk, transformed, to a given offset in another chunk.  Note that
- * objects are copied from the old chunk and not retained there
+ * Write a chunk, transformed, to a given offset in another chunk.
+ *
+ * This function assumes that it is being called at level generation, when
+ * there has been no interaction between the player and the level, monsters
+ * have not been activated, all monsters are in only one group, and objects
+ * are in their original positions.
+ *
  * \param dest the chunk where the copy is going
  * \param source the chunk being copied
  * \param y0 transformation parameters  - see symmetry_transform()
@@ -207,9 +203,10 @@ void symmetry_transform(int *y, int *x, int y0, int x0, int height, int width,
 bool chunk_copy(struct chunk *dest, struct chunk *source, int y0, int x0,
 				int rotate, bool reflect)
 {
-	int i;
-	int y, x;
+	int i, max_group_id = 0;
+	struct loc grid;
 	int h = source->height, w = source->width;
+	int mon_skip = dest->mon_max - 1;
 
 	/* Check bounds */
 	if (rotate % 1) {
@@ -220,89 +217,116 @@ bool chunk_copy(struct chunk *dest, struct chunk *source, int y0, int x0,
 			return false;
 	}
 
-	/* Write the location stuff */
-	for (y = 0; y < h; y++) {
-		for (x = 0; x < w; x++) {
+	/* Write the location stuff (terrain, objects, traps) */
+	for (grid.y = 0; grid.y < h; grid.y++) {
+		for (grid.x = 0; grid.x < w; grid.x++) {
 			/* Work out where we're going */
-			int dest_y = y;
-			int dest_x = x;
-			symmetry_transform(&dest_y, &dest_x, y0, x0, h, w, rotate, reflect);
+			struct loc dest_grid = grid;
+			symmetry_transform(&dest_grid, y0, x0, h, w, rotate, reflect);
 
 			/* Terrain */
-			dest->squares[dest_y][dest_x].feat = source->squares[y][x].feat;
-			sqinfo_copy(dest->squares[dest_y][dest_x].info,
-						source->squares[y][x].info);
+			dest->squares[dest_grid.y][dest_grid.x].feat =
+				square(source, grid).feat;
+			sqinfo_copy(square(dest, dest_grid).info,
+						square(source, grid).info);
 
 			/* Dungeon objects */
-			if (square_object(source, y, x)) {
+			if (square_object(source, grid)) {
 				struct object *obj;
-				dest->squares[dest_y][dest_x].obj = square_object(source, y, x);
+				dest->squares[dest_grid.y][dest_grid.x].obj =
+					square_object(source, grid);
 
-				for (obj = square_object(source, y, x); obj; obj = obj->next) {
+				for (obj = square_object(source, grid); obj; obj = obj->next) {
 					/* Adjust position */
-					obj->iy = dest_y;
-					obj->ix = dest_x;
+					obj->grid = dest_grid;
 				}
-				source->squares[y][x].obj = NULL;
-			}
-
-			/* Monsters */
-			if (source->squares[y][x].mon > 0) {
-				struct monster *source_mon = square_monster(source, y, x);
-				struct monster *dest_mon = NULL;
-				int idx;
-
-				/* Valid monster */
-				if (!source_mon->race)
-					continue;
-
-				/* Make a monster */
-				idx = mon_pop(dest);
-
-				/* Hope this never happens */
-				if (!idx)
-					break;
-
-				/* Copy over */
-				dest_mon = cave_monster(dest, idx);
-				dest->squares[dest_y][dest_x].mon = idx;
-				memcpy(dest_mon, source_mon, sizeof(*source_mon));
-
-				/* Adjust stuff */
-				dest_mon->midx = idx;
-				dest_mon->fy = dest_y;
-				dest_mon->fx = dest_x;
-
-				/* Held objects */
-				if (source_mon->held_obj)
-					dest_mon->held_obj = source_mon->held_obj;
+				source->squares[grid.y][grid.x].obj = NULL;
 			}
 
 			/* Traps */
-			if (source->squares[y][x].trap) {
-				struct trap *trap = source->squares[y][x].trap;
-				dest->squares[dest_y][dest_x].trap = trap;
+			if (square(source, grid).trap) {
+				struct trap *trap = square(source, grid).trap;
+				dest->squares[dest_grid.y][dest_grid.x].trap = trap;
 
 				/* Traverse the trap list */
 				while (trap) {
 					/* Adjust location */
-					trap->fy = dest_y;
-					trap->fx = dest_x;
+					trap->grid = dest_grid;
 					trap = trap->next;
 				}
-				source->squares[y][x].trap = NULL;
+				source->squares[grid.y][grid.x].trap = NULL;
 			}
 
 			/* Player */
-			if (source->squares[y][x].mon == -1) 
-				dest->squares[dest_y][dest_x].mon = -1;
+			if (square(source, grid).mon == -1) 
+				dest->squares[dest_grid.y][dest_grid.x].mon = -1;
 		}
 	}
 
-	/* Miscellany */
-	for (i = 0; i < z_info->f_max + 1; i++)
-		dest->feat_count[i] += source->feat_count[i];
+	/* Monsters */
+	dest->mon_max += source->mon_max;
+	dest->mon_cnt += source->mon_cnt;
+	dest->num_repro += source->num_repro;
+	for (i = 1; i < source->mon_max; i++) {
+		struct monster *source_mon = &source->monsters[i];
+		struct monster *dest_mon = &dest->monsters[mon_skip + i];
 
+		/* Valid monster */
+		if (!source_mon->race) continue;
+
+		/* Copy */
+		memcpy(dest_mon, source_mon, sizeof(struct monster));
+
+		/* Adjust monster index */
+		dest_mon->midx += mon_skip;
+
+		/* Move grid */
+		symmetry_transform(&dest_mon->grid, y0, x0, h, w, rotate, reflect);
+		dest->squares[dest_mon->grid.y][dest_mon->grid.x].mon = dest_mon->midx;
+
+		/* Held or mimicked objects */
+		if (source_mon->held_obj) {
+			struct object *obj;
+			dest_mon->held_obj = source_mon->held_obj;
+			for (obj = source_mon->held_obj; obj; obj = obj->next) {
+				obj->held_m_idx = dest_mon->midx;
+			}
+		}
+		if (source_mon->mimicked_obj) {
+			dest_mon->mimicked_obj = source_mon->mimicked_obj;
+		}
+	}
+
+	/* Find max monster group id */
+	for (i = 1; i < z_info->level_monster_max; i++) {
+		if (dest->monster_groups[i]) max_group_id = i;
+	}
+
+	/* Copy monster groups */
+	for (i = 1; i < z_info->level_monster_max - max_group_id; i++) {
+		struct monster_group *group = source->monster_groups[i];
+		struct mon_group_list_entry *entry;
+
+		/* Copy monster group list */
+		dest->monster_groups[i + max_group_id] = source->monster_groups[i];
+
+		/* Adjust monster group indices */
+		if (!group) continue;
+		entry = group->member_list;
+		group->index += max_group_id;
+		group->leader += mon_skip;
+		while (entry) {
+			int idx = entry->midx;
+			struct monster *mon = &dest->monsters[mon_skip + idx];
+			entry->midx = mon->midx;
+			assert(entry->midx == mon_skip + idx);
+			mon->group_info[0].index += max_group_id;
+			entry = entry->next;
+		}
+	}
+	monster_groups_verify(dest);
+
+	/* Copy object list */
 	dest->objects = mem_realloc(dest->objects,
 								(dest->obj_max + source->obj_max + 2)
 								* sizeof(struct object*));
@@ -310,8 +334,15 @@ bool chunk_copy(struct chunk *dest, struct chunk *source, int y0, int x0,
 		dest->objects[dest->obj_max + i] = source->objects[i];
 		if (dest->objects[dest->obj_max + i] != NULL)
 			dest->objects[dest->obj_max + i]->oidx = dest->obj_max + i;
+		source->objects[i] = NULL;
 	}
 	dest->obj_max += source->obj_max + 1;
+	source->obj_max = 1;
+	object_lists_check_integrity(dest, NULL);
+
+	/* Miscellany */
+	for (i = 0; i < z_info->f_max + 1; i++)
+		dest->feat_count[i] += source->feat_count[i];
 
 	dest->obj_rating += source->obj_rating;
 	dest->mon_rating += source->mon_rating;
@@ -334,10 +365,11 @@ void chunk_validate_objects(struct chunk *c) {
 
 	for (y = 0; y < c->height; y++) {
 		for (x = 0; x < c->width; x++) {
-			for (obj = square_object(c, y, x); obj; obj = obj->next)
+			struct loc grid = loc(x, y);
+			for (obj = square_object(c, grid); obj; obj = obj->next)
 				assert(obj->tval != 0);
-			if (c->squares[y][x].mon > 0) {
-				struct monster *mon = square_monster(c, y, x);
+			if (square(c, grid).mon > 0) {
+				struct monster *mon = square_monster(c, grid);
 				if (mon->held_obj)
 					for (obj = mon->held_obj; obj; obj = obj->next)
 						assert(obj->tval != 0);
