@@ -48,6 +48,11 @@ MON_MSG(FLEE_IN_TERROR,		MSG_FLEE,		false,	"flee[s] in terror!")
 MON_MSG(NOT_AFRAID,			MSG_GENERIC,	false,	"[is|are] no longer afraid.")
 MON_MSG(HELD,				MSG_GENERIC,    false,  "[is|are] frozen to the spot!")
 MON_MSG(NOT_HELD,			MSG_GENERIC,    false,  "can move again.")
+MON_MSG(DISEN,				MSG_GENERIC,    false,  "seem[s] less magical!")
+MON_MSG(NOT_DISEN,			MSG_GENERIC,    false,  "seem[s] magical again.")
+MON_MSG(COMMAND,			MSG_GENERIC,    false,  "falls under your spell!")
+MON_MSG(NOT_COMMAND,		MSG_GENERIC,    false,  "is no longer under your control.")
+MON_MSG(SHAPE_FAIL,			MSG_GENERIC,    false,  "shimmers for a moment.")
 MON_MSG(MORIA_DEATH,		MSG_KILL,		true,	"You hear [a|several] scream[|s] of agony!")
 MON_MSG(DISINTEGRATES,		MSG_KILL,		false,	"disintegrate[s]!")
 MON_MSG(FREEZE_SHATTER,		MSG_KILL,		false,	"freeze[s] and shatter[s]!")
