@@ -40,10 +40,12 @@ OF(AGGRAVATE)
 OF(DRAIN_EXP)
 OF(STICKY)
 OF(FRAGILE)
-OF(LIGHT_1)
 OF(LIGHT_2)
+OF(LIGHT_3)
 OF(DIG_1)
 OF(DIG_2)
 OF(DIG_3)
 OF(EXPLODE)
+OF(TRAP_IMMUNE)
+OF(THROWING)
 OF(MAX)
