@@ -22,9 +22,11 @@ PARSE_ERROR(INVALID_LIGHTING,		"invalid lighting")
 PARSE_ERROR(INVALID_MESSAGE,		"invalid message")
 PARSE_ERROR(INVALID_MONSTER_BASE,	"invalid monster base")
 PARSE_ERROR(INVALID_MONSTER,		"invalid monster race")
+PARSE_ERROR(INVALID_MONSTER_ROLE,	"invalid monster group role")
 PARSE_ERROR(INVALID_OBJ_PROP_CODE,	"invalid object property code")
 PARSE_ERROR(INVALID_OPTION,			"invalid option")
 PARSE_ERROR(INVALID_OPERATION,		"invalid power calculation operation")
+PARSE_ERROR(INVALID_PLAY_PROP_CODE,	"invalid player property code")
 PARSE_ERROR(INVALID_PROPERTY,		"invalid object property type")
 PARSE_ERROR(INVALID_SLAY,			"invalid slay")
 PARSE_ERROR(INVALID_SPELL_FREQ,		"invalid spell frequency")
@@ -35,6 +37,7 @@ PARSE_ERROR(INVALID_VALUE,			"invaild value")
 PARSE_ERROR(MISSING_BLOW_EFF_TYPE,	"missing blow effect type")
 PARSE_ERROR(MISSING_FIELD,			"missing field")
 PARSE_ERROR(MISSING_OBJ_PROP_TYPE,	"missing object property type")
+PARSE_ERROR(MISSING_PLAY_PROP_TYPE,	"missing player property type")
 PARSE_ERROR(MISSING_RECORD_HEADER,	"missing record header")
 PARSE_ERROR(NO_ARTIFACT_NAME,		"artifact name not found")
 PARSE_ERROR(NO_BUILDER_FOUND,		"no builder found")
