@@ -41,6 +41,9 @@
 #  define NCURSES_ENABLE_STDbool_H 0
 # endif
 
+/* Mac needs _XOPEN_SOURCE_EXTENDED to expose mvwaddnstr(). */
+# define _XOPEN_SOURCE_EXTENDED 1
+
 # include <ncurses.h>
 #else
 # include <curses.h>
@@ -104,11 +107,33 @@ static struct termios  game_termios;
 static char *termtype;
 static bool loaded_terminfo;
 
+/**
+ * Simple rectangle type
+ */
+struct rect_s
+{
+    int  x,  y;
+    int cx, cy;
+};
+typedef struct rect_s rect_t, *rect_ptr;
+
+/* Trivial rectangle utility to make code a bit more readable */
+static rect_t rect(int x, int y, int cx, int cy)
+{
+    rect_t r;
+    r.x = x;
+    r.y = y;
+    r.cx = cx;
+    r.cy = cy;
+    return r;
+}
+
 /**
  * Information about a term
  */
 typedef struct term_data {
 	term t;                 /* All term info */
+	rect_t r;
 	WINDOW *win;            /* Pointer to the curses window */
 } term_data;
 
@@ -149,10 +174,16 @@ static int can_use_color = false;
  */
 static int colortable[BASIC_COLORS];
 
+/**
+ * Same colors as in the colortable, except fg and bg are both set to the fg value.
+ * These pairs are used for drawing solid walls.
+ */
+static int same_colortable[BASIC_COLORS];
+
 /* Screen info: use one big Term 0, or other subwindows? */
 static bool bold_extended = false;
-static bool ascii_walls = false;
-static int term_count = 4;
+static bool use_default_background = false;
+static int term_count = 1;
 
 /**
  * Background color we should draw with; either BLACK or DEFAULT
@@ -168,6 +199,14 @@ static int bg_color = COLOR_BLACK;
 #define PAIR_MAGENTA 5
 #define PAIR_CYAN 6
 #define PAIR_BLACK 7
+#define PAIR_WHITE_WHITE 8
+#define PAIR_RED_RED 9
+#define PAIR_GREEN_GREEN 10
+#define PAIR_YELLOW_YELLOW 11
+#define PAIR_BLUE_BLUE 12
+#define PAIR_MAGENTA_MAGENTA 13
+#define PAIR_CYAN_CYAN 14
+#define PAIR_BLACK_BLACK 15
 
 #endif
 
@@ -289,16 +328,15 @@ static errr Term_xtra_gcu_alive(int v) {
 	return 0;
 }
 
-const char help_gcu[] = "Text mode, subopts\n              -a     Use ASCII walls\n              -b     Big screen (equivalent to -n1)\n              -B     Use brighter bold characters\n              -nN    Use N terminals (up to 6)";
+const char help_gcu[] = "Text mode, subopts\n              -B     Use brighter bold characters\n              -D     Use terminal default background color\n              -nN    Use N terminals (up to 6)";
 
 /**
  * Usage:
  *
- * angband -mgcu -- [-a] [-b] [-B] [-nN]
+ * angband -mgcu -- [-B] [-D] [-nN]
  *
- *   -a      Use ASCII walls
- *   -b      Big screen (equivalent to -n1)
  *   -B      Use brighter bold characters
+ *   -D      Use terminal default background color
  *   -nN     Use N terminals (up to 6)
  */
 
@@ -317,6 +355,11 @@ static void Term_init_gcu(term *t) {
 	/* Count init's, handle first */
 	if (active++ != 0) return;
 
+	#if defined(USE_NCURSES) && defined(KEY_MOUSE)
+	/* Turn on the mouse. */
+	mousemask(ALL_MOUSE_EVENTS, NULL);
+	#endif
+
 	/* Erase the window */
 	wclear(td->win);
 
@@ -548,6 +591,25 @@ static errr Term_xtra_gcu_event(int v) {
 	}
 	#endif
 
+	#if defined(USE_NCURSES) && defined(KEY_MOUSE)
+	if (i == KEY_MOUSE) {
+		MEVENT m;
+		if (getmouse(&m) != OK) return (0);
+
+		int b = 0;
+		if (m.bstate & BUTTON1_CLICKED) b = 1;
+		else if (m.bstate & BUTTON2_CLICKED) b = 2;
+		else if (m.bstate & BUTTON3_CLICKED) b = 3;
+		else if (m.bstate & BUTTON4_CLICKED) b = 4;
+		if (m.bstate & BUTTON_SHIFT) b |= (KC_MOD_SHIFT << 4);
+		if (m.bstate & BUTTON_CTRL) b |= (KC_MOD_CONTROL << 4);
+		if (m.bstate & BUTTON_ALT) b |= (KC_MOD_ALT << 4);
+
+		if (b != 0) Term_mousepress(m.x, m.y, b);
+		return (0);
+	}
+	#endif
+
 	/* uncomment to debug keycode issues */
 	#if 0
 	printw("key %d", i);
@@ -669,32 +731,6 @@ static int create_color(int i, int scale) {
  * React to changes
  */
 static errr Term_xtra_gcu_react(void) {
-	if (ascii_walls) {
-		int i;
-		ascii_walls = false;
-		for (i = 0; i < 4; i++) {
-			// magma as %:D
-			feat_x_char[i][FEAT_MAGMA] = 0x23;
-			feat_x_attr[i][FEAT_MAGMA] = 0x01;
-
-			// quartz as %:D
-			feat_x_char[i][FEAT_QUARTZ] = 0x23;
-			feat_x_attr[i][FEAT_QUARTZ] = 0x01;
-
-			// quartz/magma w treasure as *:o
-			feat_x_char[i][FEAT_MAGMA_K] = feat_x_char[i][FEAT_QUARTZ_K] = 0x2A;
-			feat_x_attr[i][FEAT_MAGMA_K] = feat_x_attr[i][FEAT_QUARTZ_K] = 0x03;
-
-			// granite walls as #:D
-			feat_x_char[i][FEAT_GRANITE] = 0x23;
-			feat_x_attr[i][FEAT_GRANITE] = 0x01;
-
-			// permanent walls as #:r
-			feat_x_char[i][FEAT_PERM] = 0x23;
-			feat_x_attr[i][FEAT_PERM] = 0x04;
-		}
-	}
-
 #ifdef A_COLOR
 	if (COLORS == 256 || COLORS == 88) {
 		/* If we have more than 16 colors, find the best matches. These numbers
@@ -715,6 +751,8 @@ static errr Term_xtra_gcu_react(void) {
 			int isbold = bold_extended ? A_BRIGHT : A_NORMAL;
 			init_pair(i + 1, fg, bg_color);
 			colortable[i] = COLOR_PAIR(i + 1) | isbold;
+			init_pair(BASIC_COLORS + i, fg, fg);
+			same_colortable[i] = COLOR_PAIR(BASIC_COLORS + i) | isbold;
 		}
 	}
 #endif
@@ -804,13 +842,22 @@ static errr Term_text_gcu(int x, int y, int n, int a, const wchar_t *s) {
 
 #ifdef A_COLOR
 	if (can_use_color) {
+
 		/* the lower 7 bits of the attribute indicate the fg/bg */
 		int attr = a & 127;
-		int color = colortable[attr];
 
 		/* the high bit of the attribute indicates a reversed fg/bg */
 		bool reversed = a > 127;
 
+		int color;
+
+		/* Set bg and fg to the same color when drawing solid walls */
+		if (a / MAX_COLORS == BG_SAME) {
+			color = same_colortable[attr];
+		} else {
+			color = colortable[attr];
+		}
+
 		/* the following check for A_BRIGHT is to avoid #1813 */
 		int mode;
 		if (reversed && (color & A_BRIGHT))
@@ -831,7 +878,6 @@ static errr Term_text_gcu(int x, int y, int n, int a, const wchar_t *s) {
 	return 0;
 }
 
-
 /**
  * Create a window for the given "term_data" argument.
  *
@@ -881,6 +927,49 @@ static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x)
 	return (0);
 }
 
+/**
+ * Simple helper
+ */ 
+static errr term_data_init(term_data *td)
+{
+	return term_data_init_gcu(td, td->r.cy, td->r.cx, td->r.y, td->r.x);
+}
+
+
+/* Parse 27,15,*x30 up to the 'x'. * gets converted to a big number
+   Parse 32,* until the end. Return count of numbers parsed */
+static int _parse_size_list(const char *arg, int sizes[], int max)
+{
+    int i = 0;
+    const char *start = arg;
+    const char *stop = arg;
+
+    for (;;)
+    {
+        if (!*stop || !isdigit(*stop))
+        {
+            if (i >= max) break;
+            if (*start == '*')
+                sizes[i] = 255;
+            else
+            {
+                /* rely on atoi("23,34,*") -> 23
+                   otherwise, copy [start, stop) into a new buffer first.*/
+                sizes[i] = atoi(start);
+            }
+            i++;
+            if (!*stop || *stop != ',') break;
+
+            stop++;
+            start = stop;
+        }
+        else
+            stop++;
+    }
+    return i;
+}
+
+
 static void hook_quit(const char *str) {
 	int i;
 
@@ -902,8 +991,6 @@ static void hook_quit(const char *str) {
  */
 errr init_gcu(int argc, char **argv) {
 	int i;
-	int rows, cols, y, x;
-	int next_win = 0;
 
 	/* Initialize info about terminal capabilities */
 	termtype = getenv("TERM");
@@ -911,18 +998,14 @@ errr init_gcu(int argc, char **argv) {
 
 	/* Parse args */
 	for (i = 1; i < argc; i++) {
-		if (prefix(argv[i], "-b")) {
-			term_count = 1;
-		} else if (prefix(argv[i], "-B")) {
+		if (prefix(argv[i], "-B")) {
 			bold_extended = true;
-		} else if (prefix(argv[i], "-a")) {
-			ascii_walls = true;
 		} else if (prefix(argv[i], "-n")) {
 			term_count = atoi(&argv[i][2]);
 			if (term_count > MAX_TERM_DATA) term_count = MAX_TERM_DATA;
 			else if (term_count < 1) term_count = 1;
-		} else {
-			plog_fmt("Ignoring option: %s", argv[i]);
+		} else if (prefix(argv[i], "-D")) {
+			use_default_background = true;
 		}
 	}
 
@@ -951,7 +1034,7 @@ errr init_gcu(int argc, char **argv) {
 
 #ifdef HAVE_USE_DEFAULT_COLORS
 	/* Should we use curses' "default color" */
-	if (use_default_colors() == OK) bg_color = -1;
+	if (use_default_background && use_default_colors() == OK) bg_color = -1;
 #endif
 
 	/* Attempt to use colors */
@@ -966,6 +1049,16 @@ errr init_gcu(int argc, char **argv) {
 		init_pair(PAIR_CYAN, COLOR_CYAN, bg_color);
 		init_pair(PAIR_BLACK, COLOR_BLACK, bg_color);
 
+		/* These pairs are used for drawing solid walls */
+		init_pair(PAIR_WHITE_WHITE, COLOR_WHITE, COLOR_WHITE);
+		init_pair(PAIR_RED_RED, COLOR_RED, COLOR_RED);
+		init_pair(PAIR_GREEN_GREEN, COLOR_GREEN, COLOR_GREEN);
+		init_pair(PAIR_YELLOW_YELLOW, COLOR_YELLOW, COLOR_YELLOW);
+		init_pair(PAIR_BLUE_BLUE, COLOR_BLUE, COLOR_BLUE);
+		init_pair(PAIR_MAGENTA_MAGENTA, COLOR_MAGENTA, COLOR_MAGENTA);
+		init_pair(PAIR_CYAN_CYAN, COLOR_CYAN, COLOR_CYAN);
+		init_pair(PAIR_BLACK_BLACK, COLOR_BLACK, COLOR_BLACK);
+
 		/* Prepare the colors */
 		colortable[COLOUR_DARK]     = (COLOR_PAIR(PAIR_BLACK));
 		colortable[COLOUR_WHITE]    = (COLOR_PAIR(PAIR_WHITE) | A_BRIGHT);
@@ -996,6 +1089,36 @@ errr init_gcu(int argc, char **argv) {
 		colortable[COLOUR_MUSTARD]     = (COLOR_PAIR(PAIR_YELLOW));
 		colortable[COLOUR_BLUE_SLATE]  = (COLOR_PAIR(PAIR_BLUE));
 		colortable[COLOUR_DEEP_L_BLUE] = (COLOR_PAIR(PAIR_BLUE));
+
+		same_colortable[COLOUR_DARK]     = (COLOR_PAIR(PAIR_BLACK_BLACK));
+		same_colortable[COLOUR_WHITE]    = (COLOR_PAIR(PAIR_WHITE_WHITE) | A_BRIGHT);
+		same_colortable[COLOUR_SLATE]    = (COLOR_PAIR(PAIR_WHITE_WHITE));
+		same_colortable[COLOUR_ORANGE]   = (COLOR_PAIR(PAIR_YELLOW_YELLOW) | A_BRIGHT);
+		same_colortable[COLOUR_RED]      = (COLOR_PAIR(PAIR_RED_RED));
+		same_colortable[COLOUR_GREEN]    = (COLOR_PAIR(PAIR_GREEN_GREEN));
+		same_colortable[COLOUR_BLUE]     = (COLOR_PAIR(PAIR_BLUE_BLUE));
+		same_colortable[COLOUR_UMBER]    = (COLOR_PAIR(PAIR_YELLOW_YELLOW));
+		same_colortable[COLOUR_L_DARK]   = (COLOR_PAIR(PAIR_BLACK_BLACK) | A_BRIGHT);
+		same_colortable[COLOUR_L_WHITE]  = (COLOR_PAIR(PAIR_WHITE_WHITE));
+		same_colortable[COLOUR_L_PURPLE] = (COLOR_PAIR(PAIR_MAGENTA_MAGENTA));
+		same_colortable[COLOUR_YELLOW]   = (COLOR_PAIR(PAIR_YELLOW_YELLOW) | A_BRIGHT);
+		same_colortable[COLOUR_L_RED]    = (COLOR_PAIR(PAIR_MAGENTA_MAGENTA) | A_BRIGHT);
+		same_colortable[COLOUR_L_GREEN]  = (COLOR_PAIR(PAIR_GREEN_GREEN) | A_BRIGHT);
+		same_colortable[COLOUR_L_BLUE]   = (COLOR_PAIR(PAIR_BLUE_BLUE) | A_BRIGHT);
+		same_colortable[COLOUR_L_UMBER]  = (COLOR_PAIR(PAIR_YELLOW_YELLOW));
+
+		same_colortable[COLOUR_PURPLE]      = (COLOR_PAIR(PAIR_MAGENTA_MAGENTA));
+		same_colortable[COLOUR_VIOLET]      = (COLOR_PAIR(PAIR_MAGENTA_MAGENTA));
+		same_colortable[COLOUR_TEAL]        = (COLOR_PAIR(PAIR_CYAN_CYAN));
+		same_colortable[COLOUR_MUD]         = (COLOR_PAIR(PAIR_YELLOW_YELLOW));
+		same_colortable[COLOUR_L_YELLOW]    = (COLOR_PAIR(PAIR_YELLOW_YELLOW) | A_BRIGHT);
+		same_colortable[COLOUR_MAGENTA]     = (COLOR_PAIR(PAIR_MAGENTA_MAGENTA) | A_BRIGHT);
+		same_colortable[COLOUR_L_TEAL]      = (COLOR_PAIR(PAIR_CYAN_CYAN) | A_BRIGHT);
+		same_colortable[COLOUR_L_VIOLET]    = (COLOR_PAIR(PAIR_MAGENTA_MAGENTA) | A_BRIGHT);
+		same_colortable[COLOUR_L_PINK]      = (COLOR_PAIR(PAIR_MAGENTA_MAGENTA) | A_BRIGHT);
+		same_colortable[COLOUR_MUSTARD]     = (COLOR_PAIR(PAIR_YELLOW));
+		same_colortable[COLOUR_BLUE_SLATE]  = (COLOR_PAIR(PAIR_BLUE));
+		same_colortable[COLOUR_DEEP_L_BLUE] = (COLOR_PAIR(PAIR_BLUE));
 	}
 #endif
 
@@ -1015,24 +1138,191 @@ errr init_gcu(int argc, char **argv) {
 	keymap_game_prepare();
 
 	/* Now prepare the term(s) */
-	for (i = 0; i < term_count; i++) {
-		/* Get the terminal dimensions; if the user asked for a big screen
-		 * then we'll put the whole screen in term 0; otherwise we'll divide
-		 * it amongst the available terms */
-		get_gcu_term_size(i, &rows, &cols, &y, &x);
-		
-		/* Skip non-existant windows */
-		if (rows <= 0 || cols <= 0) continue;
-		
-		/* Create a term */
-		term_data_init_gcu(&data[next_win], rows, cols, y, x);
-		
-		/* Remember the term */
-		angband_term[next_win] = &data[next_win].t;
-		
-		/* One more window */
-		next_win++;
+	if (term_count > 1) 
+	{	
+		int rows, cols, y, x;
+		int next_win = 0;
+		for (i = 0; i < term_count; i++) {
+			/* Get the terminal dimensions; if the user asked for a big screen
+			 * then we'll put the whole screen in term 0; otherwise we'll divide
+			 * it amongst the available terms */
+			get_gcu_term_size(i, &rows, &cols, &y, &x);
+			
+			/* Skip non-existant windows */
+			if (rows <= 0 || cols <= 0) continue;
+			
+			/* Create a term */
+			term_data_init_gcu(&data[next_win], rows, cols, y, x);
+			
+			/* Remember the term */
+			angband_term[next_win] = &data[next_win].t;
+			
+			/* One more window */
+			next_win++;
+		}
 	}
+	else
+/* Parse Args and Prepare the Terminals. Rectangles are specified
+      as Width x Height, right? The game will allow you to have two
+      strips of extra terminals, one on the right and one on the bottom.
+      The map terminal will than fit in as big as possible in the remaining
+      space.
+
+      Examples:
+        angband -mgcu -- -right 30x27,* -bottom *x7 will layout as
+
+        Term-0: Map (COLS-30)x(LINES-7) | Term-1: 30x27
+        --------------------------------|----------------------
+        <----Term-3: (COLS-30)x7------->| Term-2: 30x(LINES-27)
+
+        composband -mgcu -- -bottom *x7 -right 30x27,* will layout as
+
+        Term-0: Map (COLS-30)x(LINES-7) | Term-2: 30x27
+                                        |------------------------------
+                                        | Term-3: 30x(LINES-27)
+        ---------------------------------------------------------------
+        <----------Term-1: (COLS)x7----------------------------------->
+
+        Notice the effect on the bottom terminal by specifying its argument
+        second or first. Notice the sequence numbers for the various terminals
+        as you will have to blindly configure them in the window setup screen.
+
+        EDIT: Added support for -left and -top.
+    */
+    {
+        rect_t remaining = rect(0, 0, COLS, LINES);
+        int    spacer_cx = 1;
+        int    spacer_cy = 1;
+        int    next_term = 1;
+        int    term_ct = 1;
+
+        for (i = 1; i < argc; i++)
+        {
+            if (strcmp(argv[i], "-spacer") == 0)
+            {
+                i++;
+                if (i >= argc)
+                    quit("Missing size specifier for -spacer");
+                sscanf(argv[i], "%dx%d", &spacer_cx, &spacer_cy);
+            }
+            else if (strcmp(argv[i], "-right") == 0 || strcmp(argv[i], "-left") == 0)
+            {
+                const char *arg, *tmp;
+                bool left = strcmp(argv[i], "-left") == 0;
+                int  cx, cys[MAX_TERM_DATA] = {0}, ct, j, x, y;
+
+                i++;
+                if (i >= argc)
+                    quit(format("Missing size specifier for -%s", left ? "left" : "right"));
+
+                arg = argv[i];
+                tmp = strchr(arg, 'x');
+                if (!tmp)
+                    quit(format("Expected something like -%s 60x27,* for two %s hand terminals of 60 columns, the first 27 lines and the second whatever is left.", left ? "left" : "right", left ? "left" : "right"));
+                cx = atoi(arg);
+                remaining.cx -= cx;
+                if (left)
+                {
+                    x = remaining.x;
+                    y = remaining.y;
+                    remaining.x += cx;
+                }
+                else
+                {
+                    x = remaining.x + remaining.cx;
+                    y = remaining.y;
+                }
+                remaining.cx -= spacer_cx;
+                if (left)
+                    remaining.x += spacer_cx;
+                
+                tmp++;
+                ct = _parse_size_list(tmp, cys, MAX_TERM_DATA);
+                for (j = 0; j < ct; j++)
+                {
+                    int cy = cys[j];
+                    if (y + cy > remaining.y + remaining.cy)
+                        cy = remaining.y + remaining.cy - y;
+                    if (next_term >= MAX_TERM_DATA)
+                        quit(format("Too many terminals. Only %d are allowed.", MAX_TERM_DATA));
+                    if (cy <= 0)
+                    {
+                        quit(format("Out of bounds in -%s: %d is too large (%d rows max for this strip)", 
+                            left ? "left" : "right", cys[j], remaining.cy));
+                    }
+                    data[next_term++].r = rect(x, y, cx, cy);
+                    y += cy + spacer_cy;
+                    term_ct++;
+                }
+            }
+            else if (strcmp(argv[i], "-top") == 0 || strcmp(argv[i], "-bottom") == 0)
+            {
+                const char *arg, *tmp;
+                bool top = strcmp(argv[i], "-top") == 0;
+                int  cy, cxs[MAX_TERM_DATA] = {0}, ct, j, x, y;
+
+                i++;
+                if (i >= argc)
+                    quit(format("Missing size specifier for -%s", top ? "top" : "bottom"));
+
+                arg = argv[i];
+                tmp = strchr(arg, 'x');
+                if (!tmp)
+                    quit(format("Expected something like -%s *x7 for a single %s terminal of 7 lines using as many columns as are available.", top ? "top" : "bottom", top ? "top" : "bottom"));
+                tmp++;
+                cy = atoi(tmp);
+                ct = _parse_size_list(arg, cxs, MAX_TERM_DATA);
+
+                remaining.cy -= cy;
+                if (top)
+                {
+                    x = remaining.x;
+                    y = remaining.y;
+                    remaining.y += cy;
+                }
+                else
+                {
+                    x = remaining.x;
+                    y = remaining.y + remaining.cy;
+                }
+                remaining.cy -= spacer_cy;
+                if (top)
+                    remaining.y += spacer_cy;
+                
+                tmp++;
+                for (j = 0; j < ct; j++)
+                {
+                    int cx = cxs[j];
+                    if (x + cx > remaining.x + remaining.cx)
+                        cx = remaining.x + remaining.cx - x;
+                    if (next_term >= MAX_TERM_DATA)
+                        quit(format("Too many terminals. Only %d are allowed.", MAX_TERM_DATA));
+                    if (cx <= 0)
+                    {
+                        quit(format("Out of bounds in -%s: %d is too large (%d cols max for this strip)", 
+                            top ? "top" : "bottom", cxs[j], remaining.cx));
+                    }
+                    data[next_term++].r = rect(x, y, cx, cy);
+                    x += cx + spacer_cx;
+                    term_ct++;
+                }
+            }
+        }
+
+        /* Map Terminal */
+        if (remaining.cx < MIN_TERM0_COLS || remaining.cy < MIN_TERM0_LINES)
+            quit(format("Failed: angband needs an %dx%d map screen, not %dx%d", MIN_TERM0_COLS, MIN_TERM0_LINES, remaining.cx, remaining.cy));
+        data[0].r = remaining;
+        term_data_init(&data[0]);
+        angband_term[0] = Term;
+
+        /* Child Terminals */
+        for (next_term = 1; next_term < term_ct; next_term++)
+        {
+            term_data_init(&data[next_term]);
+            angband_term[next_term] = Term;
+        }
+    }
 
 	/* Activate the "Angband" window screen */
 	Term_activate(&data[0].t);
