@@ -35,14 +35,17 @@
  * locale junk
  */
 #include "locale.h"
+
+#if !defined(WINDOWS)
 #include "langinfo.h"
+#endif
 
 /**
  * Some machines have a "main()" function in their "main-xxx.c" file,
  * all the others use this file for their "main()" function.
  */
 
-#if defined(WIN32_CONSOLE_MODE) || !defined(WINDOWS) || defined(USE_SDL)
+#if defined(WIN32_CONSOLE_MODE) || !defined(WINDOWS) || defined(USE_SDL) || defined(USE_SDL2)
 
 #include "main.h"
 
@@ -59,6 +62,10 @@ static const struct module modules[] =
 	{ "sdl", help_sdl, init_sdl },
 #endif /* USE_SDL */
 
+#ifdef USE_SDL2
+	{ "sdl2", help_sdl2, init_sdl2 },
+#endif /* USE_SDL2 */
+
 #ifdef USE_GCU
 	{ "gcu", help_gcu, init_gcu },
 #endif /* USE_GCU */
@@ -198,6 +205,7 @@ static void change_path(const char *info)
 			/* the directory may not exist and may need to be created. */
 			path_build(dirpath, sizeof(dirpath), dir, "");
 			if (!dir_create(dirpath)) quit_fmt("Cannot create '%s'", dirpath);
+			string_free(info_copy);
 			return;
 		}
 	}
@@ -464,12 +472,13 @@ int main(int argc, char *argv[])
 	/* If we were told which mode to use, then use it */
 	if (mstr)
 		ANGBAND_SYS = mstr;
-
+#if !defined(WINDOWS)
 	if (setlocale(LC_CTYPE, "")) {
 		/* Require UTF-8 */
 		if (strcmp(nl_langinfo(CODESET), "UTF-8") != 0)
 			quit("Angband requires UTF-8 support");
 	}
+#endif
 
 	/* Try the modules in the order specified by modules[] */
 	for (i = 0; i < (int)N_ELEMENTS(modules); i++) {
