@@ -37,6 +37,7 @@ extern errr init_ami(int argc, char **argv);
 extern errr init_vme(int argc, char **argv);
 extern errr init_vcs(int argc, char **argv);
 extern errr init_sdl(int argc, char **argv);
+extern errr init_sdl2(int argc, char **argv);
 extern errr init_test(int argc, char **argv);
 extern errr init_stats(int argc, char **argv);
 
@@ -56,6 +57,7 @@ extern const char help_emx[];
 extern const char help_ibm[];
 extern const char help_dos[];
 extern const char help_sdl[];
+extern const char help_sdl2[];
 extern const char help_test[];
 extern const char help_stats[];
 
