@@ -38,6 +38,9 @@
 /* "someone", "something", or "the kobold" at the start of a message */
 #define MDESC_STANDARD  (MDESC_CAPITAL | MDESC_IND_HID | MDESC_PRO_HID)
 
+/* "someone", "something", or "the kobold" as the target of an attack */
+#define MDESC_TARG  (MDESC_OBJE | MDESC_IND_HID | MDESC_PRO_HID)
+
  /* Reveal the full, indefinite name of a monster */
 #define MDESC_DIED_FROM (MDESC_SHOW | MDESC_IND_VIS)
 
