@@ -55,54 +55,61 @@ typedef enum monster_sex monster_sex_t;
  * dangerous the attack is to the player given current state. Spells may be
  * colored green (least dangerous), yellow, orange, or red (most dangerous).
  */
-int spell_color(struct player *p, int spell_index)
+static int spell_color(struct player *p, const struct monster_race *race,
+					   int spell_index)
 {
 	const struct monster_spell *spell = monster_spell_by_index(spell_index);
+	struct monster_spell_level *level = spell->level;
 	struct effect *eff = spell ? spell->effect : NULL;
 
 	/* No spell */
 	if (!spell) return COLOUR_DARK;
 
+	/* Get the right level */
+	while (level->next && race->spell_power >= level->next->power) {
+		level = level->next;
+	}
+
 	/* Unresistable spells just use the default color */
-	if (!spell->lore_attr_resist && !spell->lore_attr_immune) {
-		return spell->lore_attr;
+	if (!level->lore_attr_resist && !level->lore_attr_immune) {
+		return level->lore_attr;
 	}
 
 	/* Spells with a save */
-	if (spell->save_message) {
+	if (level->save_message) {
 		/* Mixed results if the save may fail, perfect result if it can't */
 		if (p->known_state.skills[SKILL_SAVE] < 100) {
 			if (eff->index == EF_TELEPORT_LEVEL) {
 				/* Special case - teleport level */
 				if (p->known_state.el_info[ELEM_NEXUS].res_level > 0) {
-					return spell->lore_attr_resist;
+					return level->lore_attr_resist;
 				} else {
-					return spell->lore_attr;
+					return level->lore_attr;
 				}
 			} else if (eff->index == EF_TIMED_INC) {
 				/* Simple timed effects */
-				if (player_inc_check(p, eff->params[0], true)) {
-					return spell->lore_attr;
+				if (player_inc_check(p, eff->subtype, true)) {
+					return level->lore_attr;
 				} else {
-					return spell->lore_attr_resist;
+					return level->lore_attr_resist;
 				}
-			} else if (spell->lore_attr_immune) {
+			} else if (level->lore_attr_immune) {
 				/* Multiple timed effects plus damage */
 				for (; eff; eff = eff->next) {
 					if (eff->index != EF_TIMED_INC) continue;
-					if (player_inc_check(p, eff->params[0], true)) {
-						return spell->lore_attr;
+					if (player_inc_check(p, eff->subtype, true)) {
+						return level->lore_attr;
 					}
 				}
-				return spell->lore_attr_resist;
+				return level->lore_attr_resist;
 			} else {
 				/* Straight damage */
-				return spell->lore_attr;
+				return level->lore_attr;
 			}
-		} else if (spell->lore_attr_immune) {
-			return spell->lore_attr_immune;
+		} else if (level->lore_attr_immune) {
+			return level->lore_attr_immune;
 		} else {
-			return spell->lore_attr_resist;
+			return level->lore_attr_resist;
 		}
 	}
 
@@ -110,25 +117,25 @@ int spell_color(struct player *p, int spell_index)
 	if ((eff->index == EF_BOLT) || (eff->index == EF_BALL) ||
 		(eff->index == EF_BREATH)) {
 		/* Treat by element */
-		switch (eff->params[0]) {
+		switch (eff->subtype) {
 			/* Special case - sound */
 			case ELEM_SOUND:
 				if (p->known_state.el_info[ELEM_SOUND].res_level > 0) {
-					return spell->lore_attr_immune;
+					return level->lore_attr_immune;
 				} else if (of_has(p->known_state.flags, OF_PROT_STUN)) {
-					return spell->lore_attr_resist;
+					return level->lore_attr_resist;
 				} else {
-					return spell->lore_attr;
+					return level->lore_attr;
 				}
 				break;
 			/* Special case - nexus */
 			case ELEM_NEXUS:
 				if (p->known_state.el_info[ELEM_NEXUS].res_level > 0) {
-					return spell->lore_attr_immune;
+					return level->lore_attr_immune;
 				} else if (p->known_state.skills[SKILL_SAVE] >= 100) {
-					return spell->lore_attr_resist;
+					return level->lore_attr_resist;
 				} else {
-					return spell->lore_attr;
+					return level->lore_attr;
 				}
 				break;
 			/* Elements that stun or confuse */
@@ -137,27 +144,27 @@ int spell_color(struct player *p, int spell_index)
 			case ELEM_PLASMA:
 			case ELEM_WATER:
 				if (!of_has(p->known_state.flags, OF_PROT_STUN)) {
-					return spell->lore_attr;
+					return level->lore_attr;
 				} else if (!of_has(p->known_state.flags, OF_PROT_CONF) &&
-						   (eff->params[0] == ELEM_WATER)){
-					return spell->lore_attr;
+						   (eff->subtype == ELEM_WATER)){
+					return level->lore_attr;
 				} else {
-					return spell->lore_attr_resist;
+					return level->lore_attr_resist;
 				}
 				break;
 			/* All other elements */
 			default:
-				if (p->known_state.el_info[eff->params[0]].res_level == 3) {
-					return spell->lore_attr_immune;
-				} else if (p->known_state.el_info[eff->params[0]].res_level > 0) {
-					return spell->lore_attr_resist;
+				if (p->known_state.el_info[eff->subtype].res_level == 3) {
+					return level->lore_attr_immune;
+				} else if (p->known_state.el_info[eff->subtype].res_level > 0) {
+					return level->lore_attr_resist;
 				} else {
-					return spell->lore_attr;
+					return level->lore_attr;
 				}
 		}
 	}
 
-	return spell->lore_attr;
+	return level->lore_attr;
 }
 
 /**
@@ -331,8 +338,12 @@ void lore_update(const struct monster_race *race, struct monster_lore *lore)
 		lore->sleep_known = true;
 
 	/* Spellcasting frequency */
-	if ((lore->cast_innate + lore->cast_spell > 100) || lore->all_known)
+	if (lore->cast_innate > 50 || lore->all_known) {
+		lore->innate_freq_known = true;
+	}
+	if (lore->cast_spell > 50 || lore->all_known) {
 		lore->spell_freq_known = true;
+	}
 
 	/* Flags for probing and cheating */
 	if (lore->all_known) {
@@ -545,7 +556,8 @@ static const char *lore_describe_speed(byte speed)
 	} lore_speed_description[] = {
 		{130,	"incredibly quickly"},
 		{120,	"very quickly"},
-		{110,	"quickly"},
+		{115,	"quickly"},
+		{110,	"fairly quickly"},
 		{109,	"normal speed"}, /* 110 is normal speed */
 		{99,	"slowly"},
 		{89,	"very slowly"},
@@ -592,36 +604,39 @@ void lore_multiplier_speed(textblock *tb, const struct monster_race *race)
 	// moves at 2.3x normal speed (0.9x your current speed)
 	textblock_append(tb, "at ");
 
-	char buf[5] = "";
+	char buf[8] = "";
 	int multiplier = 10 * extract_energy[race->speed] / extract_energy[110];
 	byte int_mul = multiplier / 10;
 	byte dec_mul = multiplier % 10;
+	byte attr = COLOUR_ORANGE;
 
 	strnfmt(buf, sizeof(buf), "%d.%dx", int_mul, dec_mul);
 	textblock_append_c(tb, COLOUR_L_BLUE, buf);
 
 	textblock_append(tb, " normal speed, which is ");
+	multiplier = 100 * extract_energy[race->speed]
+		/ extract_energy[player->state.speed];
+	int_mul = multiplier / 100;
+	dec_mul = multiplier % 100;
+	if (!dec_mul) {
+		strnfmt(buf, sizeof(buf), "%dx", int_mul);
+	} else if (!(dec_mul % 10)) {
+		strnfmt(buf, sizeof(buf), "%d.%dx", int_mul, dec_mul / 10);
+	} else {
+		strnfmt(buf, sizeof(buf), "%d.%02dx", int_mul, dec_mul);
+	}
 
 	if (player->state.speed > race->speed) {
-		char buf[13] = "";
-		multiplier = 10 * extract_energy[player->state.speed] / extract_energy[race->speed];
-		int_mul = multiplier / 10;
-		dec_mul = multiplier % 10;
-		strnfmt(buf, sizeof(buf), "%d.%dx slower ", int_mul, dec_mul);
-		textblock_append_c(tb, COLOUR_L_GREEN, buf);
+		attr = COLOUR_L_GREEN;
+	} else if (player->state.speed < race->speed) {
+		attr = COLOUR_RED;
 	}
-	else if (player->state.speed < race->speed) {
-		char buf[13] = "";
-		multiplier = 100 * extract_energy[race->speed] / extract_energy[player->state.speed];
-		int_mul = multiplier / 100;
-		dec_mul = ((multiplier + 9) / 10) % 10;
-		strnfmt(buf, sizeof(buf), "%d.%dx faster ", int_mul, dec_mul);
-		textblock_append_c(tb, COLOUR_RED, buf);
-	}
-	if (player->state.speed == race->speed)
+	if (player->state.speed == race->speed) {
 		textblock_append(tb, "the same as you");
-	else
-		textblock_append(tb, "than you");
+	} else {
+		textblock_append_c(tb, attr, buf);
+		textblock_append(tb, " your speed");
+	}
 }
 
 /**
@@ -763,7 +778,7 @@ static void lore_append_spell_clause(textblock *tb, bitflag *f, bool know_hp,
 		int spell;
 		for (spell = rsf_next(f, FLAG_START); spell;
 			 spell = rsf_next(f, spell + 1)) {
-			int color = spell_color(player, spell);
+			int color = spell_color(player, race, spell);
 			int damage = mon_spell_lore_damage(spell, race, know_hp);
 
 			/* First entry starts immediately */
@@ -1028,7 +1043,7 @@ void lore_append_toughness(textblock *tb, const struct monster_race *race,
 		textblock_append(tb, ".  ");
 
 		/* Player's base chance to hit */
-		chance = py_attack_hit_chance(player, weapon);
+		chance = chance_of_melee_hit(player, weapon);
 
 		/* The following calculations are based on test_hit();
 		 * make sure to keep it in sync */
@@ -1068,7 +1083,9 @@ void lore_append_exp(textblock *tb, const struct monster_race *race,
 	long exp_integer, exp_fraction;
 	s16b level;
 
+	/* Check legality and that this is a placeable monster */
 	assert(tb && race && lore);
+	if (!race->rarity) return;
 
 	/* Introduction */
 	if (rf_has(known_flags, RF_UNIQUE))
@@ -1231,9 +1248,19 @@ void lore_append_abilities(textblock *tb, const struct monster_race *race,
 						   initial_pronoun);
 	if (rf_has(known_flags, RF_REGENERATE))
 		textblock_append(tb, "%s regenerates quickly.  ", initial_pronoun);
-	if (rf_has(known_flags, RF_HAS_LIGHT))
+
+	/* Describe light */
+	if (race->light > 1) {
 		textblock_append(tb, "%s illuminates %s surroundings.  ",
 						 initial_pronoun, lore_pronoun_possessive(msex, false));
+	} else if (race->light == 1) {
+		textblock_append(tb, "%s is illuminated.  ", initial_pronoun);
+	} else if (race->light == -1) {
+		textblock_append(tb, "%s is darkened.  ", initial_pronoun);
+	} else if (race->light < -1) {
+		textblock_append(tb, "%s shrouds %s surroundings in darkness.  ",
+						 initial_pronoun, lore_pronoun_possessive(msex, false));
+	}
 
 	/* Collect susceptibilities */
 	create_mon_flag_mask(current_flags, RFT_VULN, RFT_VULN_I, RFT_MAX);
@@ -1292,6 +1319,12 @@ void lore_append_abilities(textblock *tb, const struct monster_race *race,
 	else
 		my_strcpy(start, format("%s does not resist ", initial_pronoun),
 				  sizeof(start));
+
+	/* Special case for undead */
+	if (rf_has(known_flags, RF_UNDEAD)) {
+		rf_off(current_flags, RF_IM_NETHER);
+	}
+
 	lore_append_clause(tb, current_flags, COLOUR_L_UMBER, start, "or", "");
 	if (!rf_is_empty(current_flags)) {
 		prev = true;
@@ -1393,10 +1426,9 @@ void lore_append_spells(textblock *tb, const struct monster_race *race,
 						const struct monster_lore *lore,
 						bitflag known_flags[RF_SIZE])
 {
-	int average_frequency;
 	monster_sex_t msex = MON_SEX_NEUTER;
+	bool innate = false;
 	bool breath = false;
-	bool magic = false;
 	const char *initial_pronoun;
 	bool know_hp;
 	bitflag current_flags[RSF_SIZE], test_flags[RSF_SIZE];
@@ -1417,32 +1449,53 @@ void lore_append_spells(textblock *tb, const struct monster_race *race,
 	rsf_diff(current_flags, test_flags);
 	if (!rsf_is_empty(current_flags)) {
 		textblock_append(tb, "%s may ", initial_pronoun);
-		lore_append_spell_clause(tb, current_flags, know_hp, race, "or", ".  ");
+		lore_append_spell_clause(tb, current_flags, know_hp, race, "or", "");
+		innate = true;
 	}
 
 	/* Collect breaths */
 	create_mon_spell_mask(current_flags, RST_BREATH, RST_NONE);
 	rsf_inter(current_flags, lore->spell_flags);
 	if (!rsf_is_empty(current_flags)) {
-		textblock_append(tb, "%s may ", initial_pronoun);
+		if (innate) {
+			textblock_append(tb, ", and may ");
+		} else {
+			textblock_append(tb, "%s may ", initial_pronoun);
+		}
 		textblock_append_c(tb, COLOUR_L_RED, "breathe ");
 		lore_append_spell_clause(tb, current_flags, know_hp, race, "or", "");
 		breath = true;
 	}
 
+	/* End the sentence about innate spells and breaths */
+	if ((innate || breath) && race->freq_innate) {
+		if (lore->innate_freq_known) {
+			/* Describe the spell frequency */
+			textblock_append(tb, "; ");
+			textblock_append_c(tb, COLOUR_L_GREEN, "1");
+			textblock_append(tb, " time in ");
+			textblock_append_c(tb, COLOUR_L_GREEN, "%d",
+							   100 / race->freq_innate);
+		} else if (lore->cast_innate) {
+			/* Guess at the frequency */
+			int approx_frequency = MAX(((race->freq_innate + 9) / 10) * 10, 1);
+			textblock_append(tb, "; about ");
+			textblock_append_c(tb, COLOUR_L_GREEN, "1");
+			textblock_append(tb, " time in ");
+			textblock_append_c(tb, COLOUR_L_GREEN, "%d",
+							   100 / approx_frequency);
+		}
+
+		textblock_append(tb, ".  ");
+	}
 
 	/* Collect spell information */
 	rsf_copy(current_flags, lore->spell_flags);
 	create_mon_spell_mask(test_flags, RST_BREATH, RST_INNATE, RST_NONE);
 	rsf_diff(current_flags, test_flags);
 	if (!rsf_is_empty(current_flags)) {
-		magic = true;
-
 		/* Intro */
-		if (breath)
-			textblock_append(tb, ", and may ");
-		else
-			textblock_append(tb, "%s may ", initial_pronoun);
+		textblock_append(tb, "%s may ", initial_pronoun);
 
 		/* Verb Phrase */
 		textblock_append_c(tb, COLOUR_L_RED, "cast spells");
@@ -1454,26 +1507,26 @@ void lore_append_spells(textblock *tb, const struct monster_race *race,
 		/* List */
 		textblock_append(tb, " which ");
 		lore_append_spell_clause(tb, current_flags, know_hp, race, "or", "");
-	}
 
-	/* End the sentence about innate/other spells */
-	if (breath || magic) {
-		/* Calculate total casting and average frequency */
-		average_frequency = (race->freq_innate + race->freq_spell) / 2;
-
-		if (lore->spell_freq_known) {
-			/* Describe the spell frequency */
-			textblock_append(tb, "; ");
-			textblock_append_c(tb, COLOUR_L_GREEN, "1");
-			textblock_append(tb, " time in ");
-			textblock_append_c(tb, COLOUR_L_GREEN, "%d", 100 / average_frequency);
-		} else if (lore->cast_innate || lore->cast_spell) {
-			/* Guess at the frequency */
-			average_frequency = ((average_frequency + 9) / 10) * 10;
-			textblock_append(tb, "; about ");
-			textblock_append_c(tb, COLOUR_L_GREEN, "1");
-			textblock_append(tb, " time in ");
-			textblock_append_c(tb, COLOUR_L_GREEN, "%d", 100 / average_frequency);
+		/* End the sentence about innate/other spells */
+		if (race->freq_spell) {
+			if (lore->spell_freq_known) {
+				/* Describe the spell frequency */
+				textblock_append(tb, "; ");
+				textblock_append_c(tb, COLOUR_L_GREEN, "1");
+				textblock_append(tb, " time in ");
+				textblock_append_c(tb, COLOUR_L_GREEN, "%d",
+								   100 / race->freq_spell);
+			} else if (lore->cast_spell) {
+				/* Guess at the frequency */
+				int approx_frequency = MAX(((race->freq_spell + 9) / 10) * 10,
+										   1);
+				textblock_append(tb, "; about ");
+				textblock_append_c(tb, COLOUR_L_GREEN, "1");
+				textblock_append(tb, " time in ");
+				textblock_append_c(tb, COLOUR_L_GREEN, "%d",
+								   100 / approx_frequency);
+			}
 		}
 
 		textblock_append(tb, ".  ");
@@ -1607,7 +1660,8 @@ void lore_append_attack(textblock *tb, const struct monster_race *race,
 		textblock_append_c(tb, COLOUR_ORANGE, " at least");
 	}
 	textblock_append_c(tb, COLOUR_L_GREEN, " %d", total_centidamage/100);
-	textblock_append(tb, " damage.  ");
+	textblock_append(tb, " damage on each of %s turns.  ",
+					 lore_pronoun_possessive(msex, false));
 }
 
 /**
@@ -1691,11 +1745,18 @@ void write_lore_entries(ang_file *fff)
 			char name[120] = "";
 
 			while (drop) {
-				object_short_name(name, sizeof name, kind->name);
-				file_putf(fff, "drop:%s:%s:%d:%d:%d\n",
-						  tval_find_name(kind->tval), name,
-						  drop->percent_chance, drop->min, drop->max);
-				drop = drop->next;
+				if (kind) {
+					object_short_name(name, sizeof name, kind->name);
+					file_putf(fff, "drop:%s:%s:%d:%d:%d\n",
+							  tval_find_name(kind->tval), name,
+							  drop->percent_chance, drop->min, drop->max);
+					drop = drop->next;
+				} else {
+					file_putf(fff, "drop-base:%s:%d:%d:%d\n",
+							  tval_find_name(drop->tval), drop->percent_chance,
+							  drop->min, drop->max);
+					drop = drop->next;
+				}
 			}
 		}
 
@@ -1704,8 +1765,21 @@ void write_lore_entries(ang_file *fff)
 			struct monster_friends *f = lore->friends;
 
 			while (f) {
-				file_putf(fff, "friends:%d:%dd%d:%s\n", f->percent_chance,
-						  f->number_dice, f->number_side, f->race->name);
+				if (f->role == MON_GROUP_MEMBER) {
+					file_putf(fff, "friends:%d:%dd%d:%s\n", f->percent_chance,
+							  f->number_dice, f->number_side, f->race->name);
+				} else {
+					char *role_name = NULL;
+					if (f->role == MON_GROUP_SERVANT) {
+						role_name = string_make("servant");
+					} else if (f->role == MON_GROUP_BODYGUARD) {
+						role_name = string_make("bodyguard");
+					}
+					file_putf(fff, "friends:%d:%dd%d:%s:%s\n",
+							  f->percent_chance, f->number_dice,
+							  f->number_side, f->race->name, role_name);
+					string_free(role_name);
+				}
 				f = f->next;
 			}
 		}
@@ -1715,8 +1789,22 @@ void write_lore_entries(ang_file *fff)
 			struct monster_friends_base *b = lore->friends_base;
 
 			while (b) {
-				file_putf(fff, "friends-base:%d:%dd%d:%s\n", b->percent_chance,
-						  b->number_dice, b->number_side, b->base->name);
+				if (b->role == MON_GROUP_MEMBER) {
+					file_putf(fff, "friends-base:%d:%dd%d:%s\n",
+							  b->percent_chance, b->number_dice,
+							  b->number_side, b->base->name);
+				} else {
+					char *role_name = NULL;
+					if (b->role == MON_GROUP_SERVANT) {
+						role_name = string_make("servant");
+					} else if (b->role == MON_GROUP_BODYGUARD) {
+						role_name = string_make("bodyguard");
+					}
+					file_putf(fff, "friends-base:%d:%dd%d:%s:%s\n",
+							  b->percent_chance, b->number_dice,
+							  b->number_side, b->base->name, role_name);
+					string_free(role_name);
+				}
 				b = b->next;
 			}
 		}
