@@ -20,6 +20,7 @@
 #include "alloc.h"
 #include "game-world.h"
 #include "init.h"
+#include "mon-group.h"
 #include "mon-lore.h"
 #include "mon-make.h"
 #include "mon-predicate.h"
@@ -31,19 +32,36 @@
 #include "obj-tval.h"
 #include "obj-util.h"
 #include "player-calcs.h"
+#include "player-timed.h"
 #include "target.h"
 
-s16b num_repro;
-
+/**
+ * ------------------------------------------------------------------------
+ * Monster race allocation
+ *
+ * Monster race allocation is done using an allocation table (see alloc.h).
+ * This table is sorted by depth.  Each line of the table contains the
+ * monster race index, the monster race level, and three probabilities:
+ * - prob1 is the base probability of the race, calculated from monster.txt.
+ * - prob2 is calculated by get_mon_num_prep(), which decides whether a
+ *         monster is appropriate based on a secondary function; prob2 is
+ *         always either prob1 or 0.
+ * - prob3 is calculated by get_mon_num(), which checks whether universal
+ *         restrictions apply (for example, unique monsters can only appear
+ *         once on a given level); prob3 is always either prob2 or 0.
+ * ------------------------------------------------------------------------ */
 static s16b alloc_race_size;
 static struct alloc_entry *alloc_race_table;
 
+/**
+ * Initialize monster allocation info
+ */
 static void init_race_allocs(void) {
 	int i;
 	struct monster_race *race;
 	alloc_entry *table;
 	s16b *num = mem_zalloc(z_info->max_depth * sizeof(s16b));
-	s16b *aux = mem_zalloc(z_info->max_depth * sizeof(s16b));
+	s16b *already_counted = mem_zalloc(z_info->max_depth * sizeof(s16b));
 
 	/* Size of "alloc_race_table" */
 	alloc_race_size = 0;
@@ -63,7 +81,7 @@ static void init_race_allocs(void) {
 		}
 	}
 
-	/* Collect the level indexes */
+	/* Calculate the cumultive level totals */
 	for (i = 1; i < z_info->max_depth; i++) {
 		/* Group by level */
 		num[i] += num[i - 1];
@@ -72,9 +90,6 @@ static void init_race_allocs(void) {
 	/* Paranoia */
 	if (!num[0]) quit("No town monsters!");
 
-
-	/*** Initialize monster allocation info ***/
-
 	/* Allocate the alloc_race_table */
 	alloc_race_table = mem_zalloc(alloc_race_size * sizeof(alloc_entry));
 
@@ -86,34 +101,37 @@ static void init_race_allocs(void) {
 		/* Get the i'th race */
 		race = &r_info[i];
 
-		/* Count valid pairs */
+		/* Count valid races */
 		if (race->rarity) {
-			int p, x, y, z;
+			int p, lev, prev_lev_count, race_index;
 
-			/* Extract the base level */
-			x = race->level;
+			/* Extract this race's level */
+			lev = race->level;
 
 			/* Extract the base probability */
 			p = (100 / race->rarity);
 
-			/* Skip entries preceding our locale */
-			y = (x > 0) ? num[x-1] : 0;
+			/* Multiply by depth factor (experimental) */
+			p *= (1 + lev / 10);
 
-			/* Skip previous entries at this locale */
-			z = y + aux[x];
+			/* Skip entries preceding this monster's level */
+			prev_lev_count = (lev > 0) ? num[lev - 1] : 0;
+
+			/* Skip entries already counted for this level */
+			race_index = prev_lev_count + already_counted[lev];
 
 			/* Load the entry */
-			table[z].index = i;
-			table[z].level = x;
-			table[z].prob1 = p;
-			table[z].prob2 = p;
-			table[z].prob3 = p;
+			table[race_index].index = i;
+			table[race_index].level = lev;
+			table[race_index].prob1 = p;
+			table[race_index].prob2 = p;
+			table[race_index].prob3 = p;
 
 			/* Another entry complete for this locale */
-			aux[x]++;
+			already_counted[lev]++;
 		}
 	}
-	mem_free(aux);
+	mem_free(already_counted);
 	mem_free(num);
 }
 
@@ -121,6 +139,167 @@ static void cleanup_race_allocs(void) {
 	mem_free(alloc_race_table);
 }
 
+
+/**
+ * Apply a monster restriction function to the monster allocation table.
+ * This way, we can use get_mon_num() to get a level-appropriate monster that
+ * satisfies certain conditions (such as belonging to a particular monster
+ * family).
+ */
+void get_mon_num_prep(bool (*get_mon_num_hook)(struct monster_race *race))
+{
+	int i;
+
+	/* Scan the allocation table */
+	for (i = 0; i < alloc_race_size; i++) {
+		alloc_entry *entry = &alloc_race_table[i];
+
+		/* Check the restriction, if any */
+		if (!get_mon_num_hook || (*get_mon_num_hook)(&r_info[entry->index])) {
+			/* Accept this monster */
+			entry->prob2 = entry->prob1;
+
+		} else {
+			/* Do not use this monster */
+			entry->prob2 = 0;
+		}
+	}
+}
+
+/**
+ * Helper function for get_mon_num(). Scans the prepared monster allocation
+ * table and picks a random monster. Returns the index of a monster in
+ * `table`.
+ */
+static struct monster_race *get_mon_race_aux(long total,
+											 const alloc_entry *table)
+{
+	int i;
+
+	/* Pick a monster */
+	long value = randint0(total);
+
+	/* Find the monster */
+	for (i = 0; i < alloc_race_size; i++) {
+		/* Found the entry */
+		if (value < table[i].prob3) break;
+
+		/* Decrement */
+		value -= table[i].prob3;
+	}
+
+	return &r_info[table[i].index];
+}
+
+/**
+ * Chooses a monster race that seems appropriate to the given level
+ *
+ * This function uses the "prob2" field of the monster allocation table,
+ * and various local information, to calculate the "prob3" field of the
+ * same table, which is then used to choose an appropriate monster, in
+ * a relatively efficient manner.
+ *
+ * Note that town monsters will *only* be created in the town, and
+ * "normal" monsters will *never* be created in the town, unless the
+ * level is modified, for example, by polymorph or summoning.
+ *
+ * There is a small chance (1/25) of boosting the given depth by
+ * a small amount (up to four levels), except in the town.
+ *
+ * It is (slightly) more likely to acquire a monster of the given level
+ * than one of a lower level.  This is done by choosing several monsters
+ * appropriate to the given level and keeping the deepest one.
+ *
+ * Note that if no monsters are appropriate, then this function will
+ * fail, and return zero, but this should *almost* never happen.
+ */
+struct monster_race *get_mon_num(int level)
+{
+	int i, p;
+	long total;
+	struct monster_race *race;
+	alloc_entry *table = alloc_race_table;
+	time_t cur_time = time(NULL);
+	struct tm *date = localtime(&cur_time);
+
+	/* Occasionally produce a nastier monster in the dungeon */
+	if (level > 0 && one_in_(z_info->ood_monster_chance))
+		level += MIN(level / 4 + 2, z_info->ood_monster_amount);
+
+	total = 0L;
+
+	/* Process probabilities */
+	for (i = 0; i < alloc_race_size; i++) {
+		/* Monsters are sorted by depth */
+		if (table[i].level > level) break;
+
+		/* Default */
+		table[i].prob3 = 0;
+
+		/* No town monsters in dungeon */
+		if ((level > 0) && (table[i].level <= 0)) continue;
+
+		/* Get the chosen monster */
+		race = &r_info[table[i].index];
+
+		/* No seasonal monsters outside of Christmas */
+		if (rf_has(race->flags, RF_SEASONAL) &&
+			!(date->tm_mon == 11 && date->tm_mday >= 24 && date->tm_mday <= 26))
+			continue;
+
+		/* Only one copy of a unique must be around at the same time */
+		if (rf_has(race->flags, RF_UNIQUE) && (race->cur_num >= race->max_num))
+			continue;
+
+		/* Some monsters never appear out of depth */
+		if (rf_has(race->flags, RF_FORCE_DEPTH) && race->level > player->depth)
+			continue;
+
+		/* Accept */
+		table[i].prob3 = table[i].prob2;
+
+		/* Total */
+		total += table[i].prob3;
+	}
+
+	/* No legal monsters */
+	if (total <= 0) return NULL;
+
+	/* Pick a monster */
+	race = get_mon_race_aux(total, table);
+
+	/* Try for a "harder" monster once (50%) or twice (10%) */
+	p = randint0(100);
+
+	if (p < 60) {
+		struct monster_race *old = race;
+
+		/* Pick a new monster */
+		race = get_mon_race_aux(total, table);
+
+		/* Keep the deepest one */
+		if (race->level < old->level) race = old;
+	}
+
+	/* Try for a "harder" monster twice (10%) */
+	if (p < 10) {
+		struct monster_race *old = race;
+
+		/* Pick a monster */
+		race = get_mon_race_aux(total, table);
+
+		/* Keep the deepest one */
+		if (race->level < old->level) race = old;
+	}
+
+	/* Result */
+	return race;
+}
+
+/**
+ * ------------------------------------------------------------------------
+ * Deleting of monsters and monster list handling
+ * ------------------------------------------------------------------------ */
 /**
  * Deletes a monster by index.
  *
@@ -128,20 +307,20 @@ static void cleanup_race_allocs(void) {
  */
 void delete_monster_idx(int m_idx)
 {
-	assert(m_idx > 0);
-
 	struct monster *mon = cave_monster(cave, m_idx);
+	struct loc grid;
 
-	int y = mon->fy;
-	int x = mon->fx;
-	assert(square_in_bounds(cave, y, x));
+	assert(m_idx > 0);
+	assert(square_in_bounds(cave, mon->grid));
+	grid = mon->grid;
 
 	/* Hack -- Reduce the racial counter */
 	mon->race->cur_num--;
 
-	/* Hack -- count the number of "reproducers" */
-	if (rf_has(mon->race->flags, RF_MULTIPLY))
-		num_repro--;
+	/* Count the number of "reproducers" */
+	if (rf_has(mon->race->flags, RF_MULTIPLY)) {
+		cave->num_repro--;
+	}
 
 	/* Hack -- remove target monster */
 	if (target_get_monster() == mon)
@@ -151,8 +330,14 @@ void delete_monster_idx(int m_idx)
 	if (player->upkeep->health_who == mon)
 		health_track(player->upkeep, NULL);
 
-	/* Monster is gone */
-	cave->squares[y][x].mon = 0;
+	/* Hack -- remove any command status */
+	if (mon->m_timed[MON_TMD_COMMAND]) {
+		(void) player_clear_timed(player, TMD_COMMAND, true);
+	}
+
+	/* Monster is gone from square and group */
+	square_set_mon(cave, grid, 0);
+	monster_remove_from_groups(cave, mon);
 
 	/* Delete objects */
 	struct object *obj = mon->held_obj;
@@ -165,7 +350,19 @@ void delete_monster_idx(int m_idx)
 		if (obj->artifact && !(obj->known && obj->known->artifact))
 			obj->artifact->created = false;
 
-		/* Delete the object */
+		/* Delete the object.  Since it's in the cave's list do
+		 * some additional bookkeeping. */
+		if (obj->known) {
+			/* It's not in a floor pile so remove it completely.
+			 * Once compatibility with old savefiles isn't needed
+			 * can skip the test and simply delist and delete
+			 * since any obj->known from a monster's inventory
+			 * will not be in a floor pile. */
+			if (loc_is_zero(obj->known->grid)) {
+				delist_object(player->cave, obj->known);
+				object_delete(&obj->known);
+			}
+		}
 		delist_object(cave, obj);
 		object_delete(&obj);
 		obj = next;
@@ -173,9 +370,7 @@ void delete_monster_idx(int m_idx)
 
 	/* Delete mimicked objects */
 	if (mon->mimicked_obj) {
-		square_excise_object(cave, y, x, mon->mimicked_obj);
-		delist_object(cave, mon->mimicked_obj);
-		object_delete(&mon->mimicked_obj);
+		square_delete_object(cave, mon->grid, mon->mimicked_obj, true, false);
 	}
 
 	/* Wipe the Monster */
@@ -185,29 +380,30 @@ void delete_monster_idx(int m_idx)
 	cave->mon_cnt--;
 
 	/* Visual update */
-	square_light_spot(cave, y, x);
+	square_light_spot(cave, grid);
 }
 
 
 /**
  * Deletes the monster, if any, at the given location.
  */
-void delete_monster(int y, int x)
+void delete_monster(struct loc grid)
 {
-	assert(square_in_bounds(cave, y, x));
+	assert(square_in_bounds(cave, grid));
 
 	/* Delete the monster (if any) */
-	if (cave->squares[y][x].mon > 0)
-		delete_monster_idx(cave->squares[y][x].mon);
+	if (square(cave, grid).mon > 0)
+		delete_monster_idx(square(cave, grid).mon);
 }
 
 
 /**
  * Move a monster from index i1 to index i2 in the monster list.
+ *
+ * This should only be called when there is an actual monster at i1
  */
-static void compact_monsters_aux(int i1, int i2)
+void monster_index_move(int i1, int i2)
 {
-	int y, x;
 	struct monster *mon;
 	struct object *obj;
 
@@ -216,15 +412,20 @@ static void compact_monsters_aux(int i1, int i2)
 
 	/* Old monster */
 	mon = cave_monster(cave, i1);
-	y = mon->fy;
-	x = mon->fx;
+	if (!mon) return;
 
 	/* Update the cave */
-	cave->squares[y][x].mon = i2;
+	square_set_mon(cave, mon->grid, i2);
 
 	/* Update midx */
 	mon->midx = i2;
 
+	/* Update group */
+	if (!monster_group_change_index(cave, i2, i1)) {
+		quit("Bad monster group info!") ;
+		monster_groups_verify(cave);
+	}
+
 	/* Repair objects being carried by monster */
 	for (obj = mon->held_obj; obj; obj = obj->next)
 		obj->held_m_idx = i2;
@@ -233,20 +434,20 @@ static void compact_monsters_aux(int i1, int i2)
 	if (mon->mimicked_obj)
 		mon->mimicked_obj->mimicking_m_idx = i2;
 
-	/* Hack -- Update the target */
+	/* Update the target */
 	if (target_get_monster() == mon)
 		target_set_monster(cave_monster(cave, i2));
 
-	/* Hack -- Update the health bar */
+	/* Update the health bar */
 	if (player->upkeep->health_who == mon)
 		player->upkeep->health_who = cave_monster(cave, i2);
 
-	/* Hack -- move monster */
+	/* Move monster */
 	memcpy(cave_monster(cave, i2),
 			cave_monster(cave, i1),
 			sizeof(struct monster));
 
-	/* Hack -- wipe hole */
+	/* Wipe hole */
 	memset(cave_monster(cave, i1), 0, sizeof(struct monster));
 }
 
@@ -312,7 +513,7 @@ void compact_monsters(int num_to_compact)
 			if (randint0(100) < chance) continue;
 
 			/* Delete the monster */
-			delete_monster(mon->fy, mon->fx);
+			delete_monster(mon->grid);
 
 			/* Count the monster */
 			num_compacted++;
@@ -328,7 +529,7 @@ void compact_monsters(int num_to_compact)
 		if (mon->race) continue;
 
 		/* Move last monster into open hole */
-		compact_monsters_aux(cave_monster_max(cave) - 1, m_idx);
+		monster_index_move(cave_monster_max(cave) - 1, m_idx);
 
 		/* Compress "cave->mon_max" */
 		cave->mon_max--;
@@ -347,7 +548,7 @@ void compact_monsters(int num_to_compact)
  */
 void wipe_mon_list(struct chunk *c, struct player *p)
 {
-	int m_idx;
+	int m_idx, i;
 
 	/* Delete all the monsters */
 	for (m_idx = cave_monster_max(c) - 1; m_idx >= 1; m_idx--) {
@@ -364,6 +565,16 @@ void wipe_mon_list(struct chunk *c, struct player *p)
 			while (obj) {
 				if (obj->artifact && !(obj->known && obj->known->artifact))
 					obj->artifact->created = false;
+				/*
+				 * Also, remove from the cave's object list.
+				 * That way, the scan for orphaned objects
+				 * in cave_free() doesn't attempt to
+				 * access freed memory or free memory
+				 * twice.
+				 */
+				if (obj->oidx) {
+					c->objects[obj->oidx] = NULL;
+				}
 				obj = obj->next;
 			}
 			object_pile_free(held_obj);
@@ -372,21 +583,28 @@ void wipe_mon_list(struct chunk *c, struct player *p)
 		/* Reduce the racial counter */
 		mon->race->cur_num--;
 
-		/* Monster is gone */
-		c->squares[mon->fy][mon->fx].mon = 0;
+		/* Monster is gone from square */
+		square_set_mon(c, mon->grid, 0);
 
 		/* Wipe the Monster */
 		memset(mon, 0, sizeof(struct monster));
 	}
 
+	/* Delete all the monster groups */
+	for (i = 1; i < z_info->level_monster_max; i++) {
+		if (c->monster_groups[i]) {
+			monster_group_free(c, c->monster_groups[i]);
+		}
+	}
+
 	/* Reset "cave->mon_max" */
 	c->mon_max = 1;
 
 	/* Reset "mon_cnt" */
 	c->mon_cnt = 0;
 
-	/* Hack -- reset "reproducer" count */
-	num_repro = 0;
+	/* Reset "reproducer" count */
+	c->num_repro = 0;
 
 	/* Hack -- no more target */
 	target_set_monster(0);
@@ -395,6 +613,13 @@ void wipe_mon_list(struct chunk *c, struct player *p)
 	health_track(p->upkeep, 0);
 }
 
+/**
+ * ------------------------------------------------------------------------
+ * Monster creation utilities:
+ *  Getting a new monster index
+ *  Creating objects for monsters to carry or mimic
+ *  Calculating hitpoints
+ * ------------------------------------------------------------------------ */
 /**
  * Returns the index of a "free" monster, or 0 if no slot is available.
  *
@@ -442,166 +667,6 @@ s16b mon_pop(struct chunk *c)
 }
 
 
-/**
- * Apply a "monster restriction function" to the "monster allocation table".
- * This way, we can use get_mon_num() to get a level-appropriate monster that
- * satisfies certain conditions (such as belonging to a particular monster
- * family).
- */
-void get_mon_num_prep(bool (*get_mon_num_hook)(struct monster_race *race))
-{
-	int i;
-
-	/* Scan the allocation table */
-	for (i = 0; i < alloc_race_size; i++) {
-		alloc_entry *entry = &alloc_race_table[i];
-
-		/* Accept monsters which pass the restriction, if any */
-		if (!get_mon_num_hook || (*get_mon_num_hook)(&r_info[entry->index]))
-			entry->prob2 = entry->prob1;
-
-		/* Do not use this monster */
-		else
-			entry->prob2 = 0;
-	}
-}
-
-/**
- * Helper function for get_mon_num(). Scans the prepared monster allocation
- * table and picks a random monster. Returns the index of a monster in
- * `table`.
- */
-static struct monster_race *get_mon_race_aux(long total,
-											 const alloc_entry *table)
-{
-	int i;
-
-	/* Pick a monster */
-	long value = randint0(total);
-
-	/* Find the monster */
-	for (i = 0; i < alloc_race_size; i++) {
-		/* Found the entry */
-		if (value < table[i].prob3) break;
-
-		/* Decrement */
-		value -= table[i].prob3;
-	}
-
-	return &r_info[table[i].index];
-}
-
-/**
- * Chooses a monster race that seems "appropriate" to the given level
- *
- * This function uses the "prob2" field of the "monster allocation table",
- * and various local information, to calculate the "prob3" field of the
- * same table, which is then used to choose an "appropriate" monster, in
- * a relatively efficient manner.
- *
- * Note that "town" monsters will *only* be created in the town, and
- * "normal" monsters will *never* be created in the town, unless the
- * "level" is "modified", for example, by polymorph or summoning.
- *
- * There is a small chance (1/50) of "boosting" the given depth by
- * a small amount (up to four levels), except in the town.
- *
- * It is (slightly) more likely to acquire a monster of the given level
- * than one of a lower level.  This is done by choosing several monsters
- * appropriate to the given level and keeping the "hardest" one.
- *
- * Note that if no monsters are "appropriate", then this function will
- * fail, and return zero, but this should *almost* never happen.
- */
-struct monster_race *get_mon_num(int level)
-{
-	int i, p;
-
-	long total;
-
-	struct monster_race *race;
-
-	alloc_entry *table = alloc_race_table;
-
-	/* Occasionally produce a nastier monster in the dungeon */
-	if (level > 0 && one_in_(z_info->ood_monster_chance))
-		level += MIN(level / 4 + 2, z_info->ood_monster_amount);
-
-	total = 0L;
-
-	/* Process probabilities */
-	for (i = 0; i < alloc_race_size; i++) {
-		time_t cur_time = time(NULL);
-		struct tm *date = localtime(&cur_time);
-
-		/* Monsters are sorted by depth */
-		if (table[i].level > level) break;
-
-		/* Default */
-		table[i].prob3 = 0;
-
-		/* No town monsters in dungeon */
-		if ((level > 0) && (table[i].level <= 0)) continue;
-
-		/* Get the chosen monster */
-		race = &r_info[table[i].index];
-
-		/* No seasonal monsters outside of Christmas */
-		if (rf_has(race->flags, RF_SEASONAL) &&
-			!(date->tm_mon == 11 && date->tm_mday >= 24 && date->tm_mday <= 26))
-			continue;
-
-		/* Only one copy of a a unique must be around at the same time */
-		if (rf_has(race->flags, RF_UNIQUE) &&
-				race->cur_num >= race->max_num)
-			continue;
-
-		/* Some monsters never appear out of depth */
-		if (rf_has(race->flags, RF_FORCE_DEPTH) && race->level > player->depth)
-			continue;
-
-		/* Accept */
-		table[i].prob3 = table[i].prob2;
-
-		/* Total */
-		total += table[i].prob3;
-	}
-
-	/* No legal monsters */
-	if (total <= 0) return NULL;
-
-	/* Pick a monster */
-	race = get_mon_race_aux(total, table);
-
-	/* Try for a "harder" monster once (50%) or twice (10%) */
-	p = randint0(100);
-
-	if (p < 60) {
-		struct monster_race *old = race;
-
-		/* Pick a new monster */
-		race = get_mon_race_aux(total, table);
-
-		/* Keep the deepest one */
-		if (race->level < old->level) race = old;
-	}
-
-	/* Try for a "harder" monster twice (10%) */
-	if (p < 10) {
-		struct monster_race *old = race;
-
-		/* Pick a monster */
-		race = get_mon_race_aux(total, table);
-
-		/* Keep the deepest one */
-		if (race->level < old->level) race = old;
-	}
-
-	/* Result */
-	return race;
-}
-
-
 /**
  * Return the number of things dropped by a monster.
  *
@@ -646,6 +711,7 @@ int mon_create_drop_count(const struct monster_race *race, bool maximize)
 static bool mon_create_drop(struct chunk *c, struct monster *mon, byte origin)
 {
 	struct monster_drop *drop;
+	struct monster_lore *lore = get_lore(mon->race);
 
 	bool great, good, gold_ok, item_ok;
     bool extra_roll = false;
@@ -665,7 +731,12 @@ static bool mon_create_drop(struct chunk *c, struct monster *mon, byte origin)
 	/* Determine how much we can drop */
 	number = mon_create_drop_count(mon->race, false);
 
-    /* Give added bonus for unique monters */
+	/* Uniques that have been stolen from get their quantity reduced */
+    if (rf_has(mon->race->flags, RF_UNIQUE)) {
+		number = MAX(0, number - lore->thefts);
+	}
+
+    /* Give added bonus for unique monsters */
     monlevel = mon->race->level;
     if (rf_has(mon->race->flags, RF_UNIQUE)) {
         monlevel = MIN(monlevel + 15, monlevel * 2);
@@ -716,10 +787,21 @@ static bool mon_create_drop(struct chunk *c, struct monster *mon, byte origin)
 		if ((unsigned int)randint0(100) >= drop->percent_chance)
 			continue;
 
-		/* Allocate by hand, prep, apply magic */
-		obj = mem_zalloc(sizeof(*obj));
-		object_prep(obj, drop->kind, level, RANDOMISE);
-		apply_magic(obj, level, true, good, great, extra_roll);
+		/* Specified by tval or by kind */
+		if (drop->kind) {
+			/* Allocate by hand, prep, apply magic */
+			obj = mem_zalloc(sizeof(*obj));
+			object_prep(obj, drop->kind, level, RANDOMISE);
+			apply_magic(obj, level, true, good, great, extra_roll);
+		} else {
+			/* Choose by set tval */
+			assert(drop->tval);
+			obj = make_object(c, level, good, great, extra_roll, NULL,
+							  drop->tval);
+		}
+
+		/* Abort if no good object is found */
+		if (!obj) continue;
 
 		/* Set origin details */
 		obj->origin = origin;
@@ -799,7 +881,7 @@ void mon_create_mimicked_object(struct chunk *c, struct monster *mon, int index)
 	mon->mimicked_obj = obj;
 
 	/* Put the object on the floor if it goes, otherwise no mimicry */
-	if (floor_carry(c, mon->fy, mon->fx, obj, &dummy)) {
+	if (floor_carry(c, mon->grid, obj, &dummy)) {
 		list_object(c, obj);
 	} else {
 		/* Clear the mimicry */
@@ -816,68 +898,6 @@ void mon_create_mimicked_object(struct chunk *c, struct monster *mon, int index)
 	}
 }
 
-/**
- * Attempts to place a copy of the given monster at the given position in
- * the dungeon.
- *
- * All of the monster placement routines eventually call this function. This
- * is what actually puts the monster in the dungeon (i.e., it notifies the cave
- * and sets the monsters position). The dungeon loading code also calls this
- * function directly.
- *
- * `origin` is the item origin to use for any monster drops (e.g. ORIGIN_DROP,
- * ORIGIN_DROP_PIT, etc.) The dungeon loading code calls this with origin = 0,
- * which prevents the monster's drops from being generated again.
- *
- * Returns the m_idx of the newly copied monster, or 0 if the placement fails.
- */
-s16b place_monster(struct chunk *c, int y, int x, struct monster *mon,
-				   byte origin)
-{
-	s16b m_idx;
-	struct monster *new_mon;
-
-	assert(square_in_bounds(c, y, x));
-	assert(!square_monster(c, y, x));
-
-	/* Get a new record */
-	m_idx = mon_pop(c);
-	if (!m_idx) return 0;
-
-	/* Copy the monster */
-	new_mon = cave_monster(c, m_idx);
-	memcpy(new_mon, mon, sizeof(struct monster));
-
-	/* Set the ID */
-	new_mon->midx = m_idx;
-
-	/* Set the location */
-	c->squares[y][x].mon = new_mon->midx;
-	new_mon->fy = y;
-	new_mon->fx = x;
-	assert(square_monster(c, y, x) == new_mon);
-
-	update_mon(new_mon, c, true);
-
-	/* Hack -- Count the number of "reproducers" */
-	if (rf_has(new_mon->race->flags, RF_MULTIPLY)) num_repro++;
-
-	/* Count racial occurrences */
-	new_mon->race->cur_num++;
-
-	/* Create the monster's drop, if any */
-	if (origin)
-		(void)mon_create_drop(c, new_mon, origin);
-
-	/* Make mimics start mimicking */
-	if (origin && new_mon->race->mimic_kinds) {
-		mon_create_mimicked_object(c, new_mon, m_idx);
-	}
-
-	/* Result */
-	return m_idx;
-}
-
 /**
  * Calculates hp for a monster. This function assumes that the Rand_normal
  * function has limits of +/- 4x std_dev. If that changes, this function
@@ -909,6 +929,83 @@ int mon_hp(const struct monster_race *race, aspect hp_aspect)
 }
 
 
+/**
+ * ------------------------------------------------------------------------
+ * Placement of a single monster
+ * These are the functions that actually put the monster into the world
+ * ------------------------------------------------------------------------ */
+/**
+ * Attempts to place a copy of the given monster at the given position in
+ * the dungeon.
+ *
+ * All of the monster placement routines eventually call this function. This
+ * is what actually puts the monster in the dungeon (i.e., it notifies the cave
+ * and sets the monster's position). The dungeon loading code also calls this
+ * function directly.
+ *
+ * `origin` is the item origin to use for any monster drops (e.g. ORIGIN_DROP,
+ * ORIGIN_DROP_PIT, etc.) The dungeon loading code calls this with origin = 0,
+ * which prevents the monster's drops from being generated again.
+ *
+ * Returns the m_idx of the newly copied monster, or 0 if the placement fails.
+ */
+s16b place_monster(struct chunk *c, struct loc grid, struct monster *mon,
+				   byte origin)
+{
+	s16b m_idx;
+	struct monster *new_mon;
+	struct monster_group_info *info = mon->group_info;
+	bool loading = mon->midx > 0;
+
+	assert(square_in_bounds(c, grid));
+	assert(!square_monster(c, grid));
+
+	/* Get a new record, or recycle the old one */
+	if (loading) {
+		m_idx = mon->midx;
+		c->mon_max++;
+		c->mon_cnt++;
+	} else {
+		m_idx = mon_pop(c);
+		if (!m_idx) return 0;
+	}
+
+	/* Copy the monster */
+	new_mon = cave_monster(c, m_idx);
+	memcpy(new_mon, mon, sizeof(struct monster));
+
+	/* Set the ID */
+	new_mon->midx = m_idx;
+
+	/* Set the location */
+	square_set_mon(c, grid, new_mon->midx);
+	new_mon->grid = grid;
+	assert(square_monster(c, grid) == new_mon);
+
+	/* Assign monster to its monster group */
+	monster_group_assign(c, new_mon, info, loading);
+
+	update_mon(new_mon, c, true);
+
+	/* Count the number of "reproducers" */
+	if (rf_has(new_mon->race->flags, RF_MULTIPLY)) c->num_repro++;
+
+	/* Count racial occurrences */
+	new_mon->race->cur_num++;
+
+	/* Create the monster's drop, if any */
+	if (origin)
+		(void)mon_create_drop(c, new_mon, origin);
+
+	/* Make mimics start mimicking */
+	if (origin && new_mon->race->mimic_kinds) {
+		mon_create_mimicked_object(c, new_mon, m_idx);
+	}
+
+	/* Result */
+	return m_idx;
+}
+
 /**
  * Attempts to place a monster of the given race at the given location.
  *
@@ -931,40 +1028,38 @@ int mon_hp(const struct monster_race *race, aspect hp_aspect)
  * except for the savefile loading code, which calls place_monster()
  * directly.
  */
-static bool place_new_monster_one(struct chunk *c, int y, int x,
+static bool place_new_monster_one(struct chunk *c, struct loc grid,
 								  struct monster_race *race, bool sleep,
+								  struct monster_group_info group_info,
 								  byte origin)
 {
 	int i;
-
 	struct monster *mon;
 	struct monster monster_body;
 
-	assert(square_in_bounds(c, y, x));
+	assert(square_in_bounds(c, grid));
 	assert(race && race->name);
 
 	/* Not where monsters already are */
-	if (square_monster(c, y, x))
-		return false;
+	if (square_monster(c, grid)) return false;
 
 	/* Not where the player already is */
-	if ((player->py == y) && (player->px == x))
-		return false;
+	if (loc_eq(player->grid, grid)) return false;
 
-	/* Prevent monsters from being placed where they cannot walk, but allow other feature types */
-	if (!square_is_monster_walkable(c, y, x))
-		return false;
+	/* Prevent monsters from being placed where they cannot walk, but allow
+	 * other feature types */
+	if (!square_is_monster_walkable(c, grid)) return false;
 
-	/* No creation on glyph of warding */
-	if (square_iswarded(c, y, x)) return false;
+	/* No creation on glyphs */
+	if (square_iswarded(c, grid) || square_isdecoyed(c, grid)) return false;
 
 	/* "unique" monsters must be "unique" */
-	if (rf_has(race->flags, RF_UNIQUE) && race->cur_num >= race->max_num)
-		return (false);
+	if (rf_has(race->flags, RF_UNIQUE) && (race->cur_num >= race->max_num))
+		return false;
 
 	/* Depth monsters may NOT be created out of depth */
 	if (rf_has(race->flags, RF_FORCE_DEPTH) && player->depth < race->level)
-		return (false);
+		return false;
 
 	/* Add to level feeling, note uniques for cheaters */
 	c->mon_rating += race->level * race->level;
@@ -980,8 +1075,9 @@ static bool place_new_monster_one(struct chunk *c, int y, int x,
 		}
 		/* Boost rating by power per 10 levels OOD */
 		c->mon_rating += (race->level - c->depth) * race->level * race->level;
-	} else if (rf_has(race->flags, RF_UNIQUE) && OPT(player, cheat_hear))
+	} else if (rf_has(race->flags, RF_UNIQUE) && OPT(player, cheat_hear)) {
 		msg("Unique (%s).", race->name);
+	}
 
 	/* Get local monster */
 	mon = &monster_body;
@@ -1026,8 +1122,8 @@ static bool place_new_monster_one(struct chunk *c, int y, int x,
 	if (rf_has(race->flags, RF_FORCE_SLEEP))
 		mflag_on(mon->mflag, MFLAG_NICE);
 
-	/* Radiate light? */
-	if (rf_has(race->flags, RF_HAS_LIGHT))
+	/* Affect light? */
+	if (mon->race->light != 0)
 		player->upkeep->update |= PU_UPDATE_VIEW;
 
 	/* Is this obviously a monster? (Mimics etc. aren't) */
@@ -1040,8 +1136,12 @@ static bool place_new_monster_one(struct chunk *c, int y, int x,
 	if (rf_has(race->flags, RF_ATTR_RAND))
 		mon->attr = randint1(BASIC_COLORS - 1);
 
+	/* Set the group info */
+	mon->group_info[PRIMARY_GROUP].index = group_info.index;
+	mon->group_info[PRIMARY_GROUP].role = group_info.role;
+
 	/* Place the monster in the dungeon */
-	if (!place_monster(c, y, x, mon, origin))
+	if (!place_monster(c, grid, mon, origin))
 		return (false);
 
 	/* Success */
@@ -1049,12 +1149,10 @@ static bool place_new_monster_one(struct chunk *c, int y, int x,
 }
 
 
-/*
- * Maximum size of a group of monsters
- */
-#define GROUP_MAX	25
-
-
+/**
+ * ------------------------------------------------------------------------
+ * More complex monster placement routines
+ * ------------------------------------------------------------------------ */
 /**
  * Attempts to place a group of monsters of race `r_idx` around
  * the given location. The number of monsters to place is `total`.
@@ -1065,60 +1163,54 @@ static bool place_new_monster_one(struct chunk *c, int y, int x,
  * `origin` is the item origin to use for any monster drops (e.g. ORIGIN_DROP,
  * ORIGIN_DROP_PIT, etc.)
  */
-static bool place_new_monster_group(struct chunk *c, int y, int x,
+static bool place_new_monster_group(struct chunk *c, struct loc grid,
 									struct monster_race *race, bool sleep,
+									struct monster_group_info group_info,
 									int total, byte origin)
 {
 	int n, i;
 
-	int hack_n;
+	int loc_num;
 
-	/* x and y coordinates of the placed monsters */
-	byte hack_y[GROUP_MAX];
-	byte hack_x[GROUP_MAX];
+	/* Locations of the placed monsters */
+	struct loc *loc_list = mem_zalloc(sizeof(struct loc) *
+									  z_info->monster_group_max);
 
+	/* Sanity and bounds check */
 	assert(race);
+	total = MIN(total, z_info->monster_group_max);
 
 	/* Start on the monster */
-	hack_n = 1;
-	hack_x[0] = x;
-	hack_y[0] = y;
+	loc_num = 1;
+	loc_list[0] = grid;
 
 	/* Puddle monsters, breadth first, up to total */
-	for (n = 0; (n < hack_n) && (hack_n < total); n++) {
-		/* Grab the location */
-		int hx = hack_x[n];
-		int hy = hack_y[n];
-
+	for (n = 0; (n < loc_num) && (loc_num < total); n++) {
 		/* Check each direction, up to total */
-		for (i = 0; (i < 8) && (hack_n < total); i++) {
-			int mx = hx + ddx_ddd[i];
-			int my = hy + ddy_ddd[i];
+		for (i = 0; (i < 8) && (loc_num < total); i++) {
+			struct loc try = loc_sum(loc_list[n], ddgrid_ddd[i]);
 
 			/* Walls and Monsters block flow */
-			if (!square_isempty(c, my, mx)) continue;
+			if (!square_isempty(c, try)) continue;
 
 			/* Attempt to place another monster */
-			if (place_new_monster_one(c, my, mx, race, sleep, origin)) {
+			if (place_new_monster_one(c, try, race, sleep, group_info, origin)){
 				/* Add it to the "hack" set */
-				hack_y[hack_n] = my;
-				hack_x[hack_n] = mx;
-				hack_n++;
+				loc_list[loc_num] = try;
+				loc_num++;
 			}
 		}
 	}
 
 	/* Success */
+	mem_free(loc_list);
 	return (true);
 }
 
-/* Maximum distance from center for a group of monsters */
-#define GROUP_DISTANCE 5
-
 static struct monster_base *place_monster_base = NULL;
 
 /**
- * Predicate function for get_mon_num_prep)
+ * Predicate function for get_mon_num_prep()
  * Check to see if the monster race has the same base as
  * place_monter_base.
  */
@@ -1139,10 +1231,10 @@ static bool place_monster_base_okay(struct monster_race *race)
 /**
  * Helper function to place monsters that appear as friends or escorts
  */
- bool place_friends(struct chunk *c, int y, int x, struct monster_race *race,
+bool place_friends(struct chunk *c, struct loc grid, struct monster_race *race,
 					struct monster_race *friends_race, int total, bool sleep,
-					byte origin)
- {
+					struct monster_group_info group_info, byte origin)
+{
 	int extra_chance;
 
 	/* Find the difference between current dungeon depth and monster level */
@@ -1152,8 +1244,8 @@ static bool place_monster_base_okay(struct monster_race *race)
 	bool is_unique = rf_has(friends_race->flags, RF_UNIQUE);
 
 	/* Make sure the unique hasn't been killed already */
-	if (is_unique) {
-		total = friends_race->cur_num < friends_race->max_num ? 1 : 0;
+	if (is_unique && (friends_race->cur_num >= friends_race->max_num)) {
+		return false;
 	}
 
 	/* More than 4 levels OoD, no groups allowed */
@@ -1177,24 +1269,26 @@ static bool place_monster_base_okay(struct monster_race *race)
 	if (total > 0) {
 		/* Handle friends same as original monster */
 		if (race->ridx == friends_race->ridx) {
-			return place_new_monster_group(c, y, x, race, sleep, total, origin);
+			return place_new_monster_group(c, grid, race, sleep, group_info,
+										   total, origin);
 		} else {
 			int j;
-			int nx = 0;
-			int ny = 0;
+			struct loc new;
 
 			/* Find a nearby place to put the other groups */
 			for (j = 0; j < 50; j++) {
-				scatter(c, &ny, &nx, y, x, GROUP_DISTANCE, false);
-				if (square_isopen(c, ny, nx)) {
+				scatter(c, &new, grid, z_info->monster_group_dist, false);
+				if (square_isopen(c, new)) {
 					break;
 				}
 			}
 
 			/* Place the monsters */
-			bool success = place_new_monster_one(c, ny, nx, friends_race, sleep, origin);
+			bool success = place_new_monster_one(c, new, friends_race, sleep,
+												 group_info, origin);
 			if (total > 1)
-				success = place_new_monster_group(c, ny, nx, friends_race, sleep, total, origin);
+				success = place_new_monster_group(c, new, friends_race, sleep,
+												  group_info, total, origin);
 
 			return success;
 		}
@@ -1216,8 +1310,9 @@ static bool place_monster_base_okay(struct monster_race *race)
  * `origin` is the item origin to use for any monster drops (e.g. ORIGIN_DROP,
  * ORIGIN_DROP_PIT, etc.)
  */
-bool place_new_monster(struct chunk *c, int y, int x, struct monster_race *race,
-					   bool sleep, bool group_okay, byte origin)
+bool place_new_monster(struct chunk *c, struct loc grid,
+					   struct monster_race *race, bool sleep, bool group_ok,
+					   struct monster_group_info group_info, byte origin)
 {
 	struct monster_friends *friends;
 	struct monster_friends_base *friends_base;
@@ -1226,11 +1321,19 @@ bool place_new_monster(struct chunk *c, int y, int x, struct monster_race *race,
 	assert(c);
 	assert(race);
 
+	/* If we don't have a group index already, make one; our first monster
+	 * will be the leader */
+	if (!group_info.index) {
+		group_info.index = monster_group_index_new(c);
+	}
+
 	/* Place one monster, or fail */
-	if (!place_new_monster_one(c, y, x, race, sleep, origin)) return (false);
+	if (!place_new_monster_one(c, grid, race, sleep, group_info, origin)) {
+		return (false);
+	}
 
 	/* We're done unless the group flag is set */
-	if (!group_okay) return (true);
+	if (!group_ok) return (true);
 
 	/* Go through friends flags */
 	for (friends = race->friends; friends; friends = friends->next) {
@@ -1240,7 +1343,12 @@ bool place_new_monster(struct chunk *c, int y, int x, struct monster_race *race,
 		/* Calculate the base number of monsters to place */
 		total = damroll(friends->number_dice, friends->number_side);
 
-		place_friends(c, y, x, race, friends->race, total, sleep, origin);
+		/* Set group role */
+		group_info.role = friends->role;
+
+		/* Place them */
+		place_friends(c, grid, race, friends->race, total, sleep, group_info,
+					  origin);
 
 	}
 
@@ -1270,7 +1378,12 @@ bool place_new_monster(struct chunk *c, int y, int x, struct monster_race *race,
 		/* Handle failure */
 		if (!friends_race) break;
 
-		place_friends(c, y, x, race, friends_race, total, sleep, origin);
+		/* Set group role */
+		group_info.role = friends_base->role;
+
+		/* Place them */
+		place_friends(c, grid, race, friends_race, total, sleep, group_info,
+					  origin);
 	}
 
 	/* Success */
@@ -1294,13 +1407,16 @@ bool place_new_monster(struct chunk *c, int y, int x, struct monster_race *race,
  *
  * Returns true if we successfully place a monster.
  */
-bool pick_and_place_monster(struct chunk *c, int y, int x, int depth,
+bool pick_and_place_monster(struct chunk *c, struct loc grid, int depth,
 							bool sleep, bool group_okay, byte origin)
 {
-	/* Pick a monster race */
+	/* Pick a monster race, no specified group */
 	struct monster_race *race = get_mon_num(depth);
+	struct monster_group_info info = { 0, 0 };
+
 	if (race) {
-		return place_new_monster(c, y, x, race, sleep, group_okay, origin);
+		return place_new_monster(c, grid, race, sleep, group_okay, info,
+								 origin);
 	} else {
 		return false;
 	}
@@ -1320,7 +1436,7 @@ bool pick_and_place_monster(struct chunk *c, int y, int x, int depth,
 bool pick_and_place_distant_monster(struct chunk *c, struct player *p, int dis,
 		bool sleep, int depth)
 {
-	int y = 0, x = 0;
+	struct loc grid;
 	int	attempts_left = 10000;
 
 	assert(c);
@@ -1328,18 +1444,17 @@ bool pick_and_place_distant_monster(struct chunk *c, struct player *p, int dis,
 	/* Find a legal, distant, unoccupied, space */
 	while (--attempts_left) {
 		/* Pick a location */
-		y = randint0(c->height);
-		x = randint0(c->width);
+		grid = loc(randint0(c->width), randint0(c->height));
 
 		/* Require "naked" floor grid */
-		if (!square_isempty(c, y, x)) continue;
+		if (!square_isempty(c, grid)) continue;
 
 		/* Do not put random monsters in marked rooms. */
-		if ((!character_dungeon) && square_ismon_restrict(c, y, x))
+		if ((!character_dungeon) && square_ismon_restrict(c, grid))
 			continue;
 
 		/* Accept far away grids */
-		if (distance(y, x, p->py, p->px) > dis) break;
+		if (distance(grid, p->grid) > dis) break;
 	}
 
 	if (!attempts_left) {
@@ -1350,7 +1465,7 @@ bool pick_and_place_distant_monster(struct chunk *c, struct player *p, int dis,
 	}
 
 	/* Attempt to place the monster, allow groups */
-	if (pick_and_place_monster(c, y, x, depth, sleep, true, ORIGIN_DROP))
+	if (pick_and_place_monster(c, grid, depth, sleep, true, ORIGIN_DROP))
 		return (true);
 
 	/* Nope */
