@@ -134,8 +134,7 @@ static bool does_resist(const struct monster *mon, int effect_type, int timer, i
 static bool mon_set_timed(struct monster *mon,
 		int effect_type,
 		int timer,
-		int flag,
-		bool id)
+		int flag)
 {
 	assert(mon != NULL);
 	assert(mon->race != NULL);
@@ -147,6 +146,7 @@ static bool mon_set_timed(struct monster *mon,
 
 	bool check_resist;
 	bool resisted = false;
+	bool update = false;
 
 	int m_note = 0;
 	int old_timer = mon->m_timed[effect_type];
@@ -184,12 +184,21 @@ static bool mon_set_timed(struct monster *mon,
 		m_note = MON_MSG_UNAFFECTED;
 	} else {
 		mon->m_timed[effect_type] = timer;
+		update = true;
+	}
 
-		if (player->upkeep->health_who == mon)
-			player->upkeep->redraw |= (PR_HEALTH);
-
-		/* Update the visuals, as appropriate. */
-		player->upkeep->redraw |= (PR_MONLIST);
+	/* Special case - deal with monster shapechanges */
+	if (effect_type == MON_TMD_CHANGED) {
+		if (timer > old_timer) {
+			if (!monster_change_shape(mon)) {
+				m_note = MON_MSG_SHAPE_FAIL;
+				mon->m_timed[effect_type] = old_timer;
+			}
+		} else if (timer == 0) {
+			if (!monster_revert_shape(mon)) {
+				quit ("Monster shapechange reversion failed!");
+			}
+		}
 	}
 
 	/* Print a message if there is one, if the effect allows for it, and if
@@ -198,11 +207,17 @@ static bool mon_set_timed(struct monster *mon,
 		!(flag & MON_TMD_FLG_NOMESSAGE) &&
 		(flag & MON_TMD_FLG_NOTIFY)
 		&& monster_is_obvious(mon)) {
-			char m_name[80];
-			monster_desc(m_name, sizeof(m_name), mon, MDESC_IND_HID);
 			add_monster_message(mon, m_note, true);
 	}
 
+	/* Update the visuals, as appropriate. */
+	if (update) {
+		if (player->upkeep->health_who == mon)
+			player->upkeep->redraw |= (PR_HEALTH);
+
+		player->upkeep->redraw |= (PR_MONLIST);
+	}
+
 	return !resisted;
 }
 
@@ -219,8 +234,7 @@ static bool mon_set_timed(struct monster *mon,
  *
  * Returns true if the monster's timer changed.
  */
-bool mon_inc_timed(struct monster *mon, int effect_type, int timer, int flag,
-				   bool id)
+bool mon_inc_timed(struct monster *mon, int effect_type, int timer, int flag)
 {
 	assert(effect_type >= 0);
 	assert(effect_type < MON_TMD_MAX);
@@ -255,7 +269,7 @@ bool mon_inc_timed(struct monster *mon, int effect_type, int timer, int flag,
 		}
 	}
 
-	return mon_set_timed(mon, effect_type, new_value, flag, id);
+	return mon_set_timed(mon, effect_type, new_value, flag);
 }
 
 /**
@@ -267,8 +281,7 @@ bool mon_inc_timed(struct monster *mon, int effect_type, int timer, int flag,
  *
  * Returns true if the monster's timer changed.
  */
-bool mon_dec_timed(struct monster *mon, int effect_type, int timer, int flag,
-				   bool id)
+bool mon_dec_timed(struct monster *mon, int effect_type, int timer, int flag)
 {
 	assert(effect_type >= 0);
 	assert(effect_type < MON_TMD_MAX);
@@ -279,7 +292,7 @@ bool mon_dec_timed(struct monster *mon, int effect_type, int timer, int flag,
 		new_level = 0;
 	}
 
-	return mon_set_timed(mon, effect_type, new_level, flag, id);
+	return mon_set_timed(mon, effect_type, new_level, flag);
 }
 
 /**
@@ -287,7 +300,7 @@ bool mon_dec_timed(struct monster *mon, int effect_type, int timer, int flag,
  *
  * Returns true if the monster's timer was changed.
  */
-bool mon_clear_timed(struct monster *mon, int effect_type, int flag, bool id)
+bool mon_clear_timed(struct monster *mon, int effect_type, int flag)
 {
 	assert(effect_type >= 0);
 	assert(effect_type < MON_TMD_MAX);
@@ -295,6 +308,17 @@ bool mon_clear_timed(struct monster *mon, int effect_type, int flag, bool id)
 	if (mon->m_timed[effect_type] == 0) {
 		return false;
 	} else {
-		return mon_set_timed(mon, effect_type, 0, flag, id);
+		return mon_set_timed(mon, effect_type, 0, flag);
 	}
 }
+
+/**
+ * The level at which an effect is affecting a monster.
+ * Levels range from 0 (unaffected) to 5 (maximum effect).
+ */
+int monster_effect_level(struct monster *mon, int effect_type)
+{
+	struct mon_timed_effect *effect = &effects[effect_type];
+	int divisor = MAX(effect->max_timer / 5, 1);
+	return MIN((mon->m_timed[effect_type] + divisor - 1) / divisor, 5);
+}
