@@ -20,17 +20,6 @@
 #ifndef OBJECT_CHEST_H
 #define OBJECT_CHEST_H
 
-/**
- * Chest trap flags (see "obj-chest.c")
- */
-#define CHEST_LOSE_STR	0x01
-#define CHEST_LOSE_CON	0x02
-#define CHEST_POISON	0x04
-#define CHEST_PARALYZE	0x08
-#define CHEST_EXPLODE	0x10
-#define CHEST_SUMMON	0x20
-
-
 /**
  * Chest check types
  */
@@ -40,13 +29,16 @@ enum chest_query {
 	CHEST_TRAPPED
 };
 
-byte chest_trap_type(const struct object *obj);
+extern struct file_parser chest_trap_parser;
+
+char *chest_trap_name(const struct object *obj);
 bool is_trapped_chest(const struct object *obj);
 bool is_locked_chest(const struct object *obj);
+int pick_chest_traps(struct object *obj);
 void unlock_chest(struct object *obj);
-struct object *chest_check(int y, int x, enum chest_query check_type);
-int count_chests(int *y, int *x, enum chest_query check_type);
-bool do_cmd_open_chest(int y, int x, struct object *obj);
-bool do_cmd_disarm_chest(int y, int x, struct object *obj);
+struct object *chest_check(struct loc grid, enum chest_query check_type);
+int count_chests(struct loc *grid, enum chest_query check_type);
+bool do_cmd_open_chest(struct loc grid, struct object *obj);
+bool do_cmd_disarm_chest(struct object *obj);
 
 #endif /* OBJECT_CHEST_H */
