@@ -98,7 +98,7 @@ bool slot_type_is(int slot, int type)
 struct object *slot_object(struct player *p, int slot)
 {
 	/* Ensure a valid body */
-	if (p->body.slots) {
+	if (p->body.slots && p->body.slots[slot].obj) {
 		return p->body.slots[slot].obj;
 	}
 
@@ -163,18 +163,24 @@ static bool object_is_in_quiver(struct player *p, const struct object *obj)
 int pack_slots_used(struct player *p)
 {
 	struct object *obj;
-	int quiver_slots = 0;
-	int pack_slots = 0;
+	int i, pack_slots = 0;
 	int quiver_ammo = 0;
 
 	for (obj = p->gear; obj; obj = obj->next) {
+		bool found = false;
 		/* Equipment doesn't count */
 		if (!object_is_equipped(p->body, obj)) {
-			/* Check if it could be in the quiver */
-			if (tval_is_ammo(obj) && quiver_slots < z_info->quiver_size) {
-				quiver_slots++;
-				quiver_ammo += obj->number;
-			} else {
+			/* Check if it is in the quiver */
+			if (tval_is_ammo(obj)) {
+				for (i = 0; i < z_info->quiver_size; i++) {
+					if (p->upkeep->quiver[i] == obj) {
+						quiver_ammo += obj->number;
+						found = true;
+						break;
+					}
+				}
+			}
+			if (!found) {
 				/* Count regular slots */
 				pack_slots++;
 			}
@@ -612,7 +618,7 @@ void inven_carry(struct player *p, struct object *obj, bool absorb,
 		struct object *combine_item = NULL;
 
 		struct object *gear_obj = p->gear;
-		while (combine_item == false && gear_obj) {
+		while ((combine_item == NULL) && (gear_obj != NULL)) {
 			if (!object_is_equipped(p->body, gear_obj) &&
 					object_similar(gear_obj, obj, OSTACK_PACK)) {
 				combine_item = gear_obj;
@@ -630,6 +636,9 @@ void inven_carry(struct player *p, struct object *obj, bool absorb,
 			obj->known = NULL;
 			object_absorb(combine_item, obj);
 
+			/* Ensure numbers are aligned (should not be necessary, but safe) */
+			combine_item->known->number = combine_item->number;
+
 			obj = combine_item;
 			combining = true;
 		}
@@ -645,8 +654,8 @@ void inven_carry(struct player *p, struct object *obj, bool absorb,
 
 		/* Remove cave object details */
 		obj->held_m_idx = 0;
-		obj->iy = obj->ix = 0;
-		obj->known->iy = obj->known->ix = 0;
+		obj->grid = loc(0, 0);
+		obj->known->grid = loc(0, 0);
 
 		/* Update the inventory */
 		p->upkeep->total_weight += (obj->number * obj->weight);
@@ -801,12 +810,13 @@ void inven_takeoff(struct object *obj)
 	player->body.slots[slot].obj = NULL;
 	player->upkeep->equip_cnt--;
 
-	/* Message */
-	msgt(MSG_WIELD, "%s %s (%c).", act, o_name, I2A(slot));
-
 	player->upkeep->update |= (PU_BONUS | PU_INVEN | PU_UPDATE_VIEW);
 	player->upkeep->notice |= (PN_IGNORE);
 	update_stuff(player);
+
+	/* Message */
+	msgt(MSG_WIELD, "%s %s (%c).", act, o_name, gear_to_label(obj));
+
 	return;
 }
 
@@ -820,8 +830,6 @@ void inven_takeoff(struct object *obj)
  */
 void inven_drop(struct object *obj, int amt)
 {
-	int py = player->py;
-	int px = player->px;
 	struct object *dropped;
 	bool none_left = false;
 	bool quiver = false;
@@ -880,7 +888,7 @@ void inven_drop(struct object *obj, int amt)
 	}
 
 	/* Drop it near the player */
-	drop_near(cave, &dropped, 0, py, px, false);
+	drop_near(cave, &dropped, 0, player->grid, false, true);
 
 	/* Sound for quiver objects */
 	if (quiver)
@@ -935,12 +943,21 @@ void combine_pack(void)
 				object_absorb(obj2->known, obj1->known);
 				obj1->known = NULL;
 				object_absorb(obj2, obj1);
+
+				/* Ensure numbers align (should not be necessary, but safer) */
+				obj2->known->number = obj2->number;
+
 				break;
 			} else if (inven_can_stack_partial(obj2, obj1, OSTACK_PACK)) {
 				/* Setting this to true spams the combine message. */
 				display_message = false;
 				object_absorb_partial(obj2->known, obj1->known);
 				object_absorb_partial(obj2, obj1);
+
+				/* Ensure numbers align (should not be necessary, but safer) */
+				obj2->known->number = obj2->number;
+				obj1->known->number = obj1->number;
+
 				break;
 			}
 		}
@@ -991,7 +1008,7 @@ void pack_overflow(struct object *obj)
 	if (!pack_is_overfull()) return;
 
 	/* Disturbing */
-	disturb(player, 0);
+	disturb(player);
 
 	/* Warning */
 	msg("Your pack overflows!");
@@ -1020,7 +1037,7 @@ void pack_overflow(struct object *obj)
 
 	/* Excise the object and drop it (carefully) near the player */
 	gear_excise_object(obj);
-	drop_near(cave, &obj, 0, player->py, player->px, false);
+	drop_near(cave, &obj, 0, player->grid, false, true);
 
 	/* Describe */
 	if (artifact)
