@@ -62,7 +62,7 @@ static quality_ignore_struct quality_mapping[] =
 	{ ITYPE_BASIC_DRAGON_ARMOR,		TV_DRAG_ARMOR,	"Red" },
 	{ ITYPE_BASIC_DRAGON_ARMOR,		TV_DRAG_ARMOR,	"Green" },
 	{ ITYPE_MULTI_DRAGON_ARMOR,		TV_DRAG_ARMOR,	"Multi" },
-	{ ITYPE_HIGH_DRAGON_ARMOR,		TV_DRAG_ARMOR,	"Pseudo" },
+	{ ITYPE_HIGH_DRAGON_ARMOR,		TV_DRAG_ARMOR,	"Shining" },
 	{ ITYPE_HIGH_DRAGON_ARMOR,		TV_DRAG_ARMOR,	"Law" },
 	{ ITYPE_HIGH_DRAGON_ARMOR,		TV_DRAG_ARMOR,	"Gold" },
 	{ ITYPE_HIGH_DRAGON_ARMOR,		TV_DRAG_ARMOR,	"Chaos" },
@@ -193,8 +193,6 @@ static void rune_add_autoinscription(struct object *obj, int i)
 void rune_autoinscribe(int i)
 {
 	struct object *obj;
-	int py = player->py;
-	int px = player->px;
 
 	/* Check the player knows the rune */
 	if (!player_knows_rune(player, i)) {
@@ -203,7 +201,7 @@ void rune_autoinscribe(int i)
 
 	/* Autoinscribe each object on the ground */
 	if (cave)
-		for (obj = square_object(cave, py, px); obj; obj = obj->next)
+		for (obj = square_object(cave, player->grid); obj; obj = obj->next)
 			if (object_has_rune(obj, i))
 				rune_add_autoinscription(obj, i);
 
@@ -336,12 +334,10 @@ int add_autoinscription(s16b kind, const char *inscription, bool aware)
  */
 void autoinscribe_ground(void)
 {
-	int py = player->py;
-	int px = player->px;
 	struct object *obj;
 
 	/* Autoinscribe each object in the pile */
-	for (obj = square_object(cave, py, px); obj; obj = obj->next)
+	for (obj = square_object(cave, player->grid); obj; obj = obj->next)
 		apply_autoinscription(obj);
 }
 
@@ -680,7 +676,7 @@ void ignore_drop(void)
 			}
 
 			/* We're allowed to drop it. */
-			if (!square_isshop(cave, player->py, player->px)) {
+			if (!square_isshop(cave, player->grid)) {
 				player->upkeep->dropping = true;
 				cmdq_push(CMD_DROP);
 				cmd_set_arg_item(cmdq_peek(), "item", obj);
