@@ -79,7 +79,7 @@ static struct {
 
 /**
  * ------------------------------------------------------------------------
- * List-writiing utility code
+ * List-writing utility code
  * ------------------------------------------------------------------------ */
 
 /**
@@ -112,7 +112,7 @@ static size_t element_info_collect(const bool list[], const char *recepticle[])
 		if (list[i])
 			recepticle[count++] = projections[i].name;
 	}
-	
+
 	return count;
 }
 
@@ -144,6 +144,10 @@ static bool describe_curses(textblock *tb, const struct object *obj,
 			textblock_append(tb, ".\n");
 		}
 	}
+	/* Say if curse removal has been tried */
+	if (of_has(obj->flags, OF_FRAGILE)) {
+		textblock_append(tb, "Attempting to uncurse it may destroy it.\n");
+	}
 
 	return true;
 }
@@ -458,7 +462,8 @@ static bool describe_brands(textblock *tb, const struct object *obj)
 static void calculate_melee_crits(struct player_state *state, int weight,
 		int plus, int *mult, int *add, int *div)
 {
-	int k, to_crit = weight + 5 * (state->to_h + plus) + 3 * player->lev;
+	int k, to_crit = weight + 5 * (state->to_h + plus) +
+		3 * state->skills[SKILL_TO_HIT_MELEE] - 60;
 	to_crit = MIN(5000, MAX(0, to_crit));
 
 	*mult = *add = 0;
@@ -468,18 +473,33 @@ static void calculate_melee_crits(struct player_state *state, int weight,
 		if (k <  700) { *mult += 4; *add += 20; continue; }
 		if (k <  900) { *mult += 6; *add += 30; continue; }
 		if (k < 1300) { *mult += 6; *add += 40; continue; }
-		                *mult += 7; *add += 50;
+		                *mult += 8; *add += 40;
 	}
 
-	/*
-	 * Scale the output down to a more reasonable size, to prevent
-	 * integer overflow downstream.
-	 */
-	*mult = 100 + to_crit*(*mult - 1300)/(50*1300);
-	*add  = *add * to_crit / (500*50);
+	/* Scale the output to a reasonable size to prevent integer overflow. */
+	*mult = 100 + to_crit * (*mult - 1300) / (50 * 1300);
+	*add  = *add * to_crit / (500 * 50);
 	*div  = 100;
 }
 
+/**
+ * Account for criticals in the calculation of melee prowess for O-combat;
+ * crit chance * average number of dice added
+ *
+ * Return value is 100x number of dice
+ */
+static int o_calculate_melee_crits(struct player_state state,
+								   const struct object *obj)
+{
+	int dice = 0;
+	int chance = BTH_PLUS_ADJ * (state.to_h + obj->known->to_h) +
+		state.skills[SKILL_TO_HIT_MELEE];
+	chance = (100 * chance) / (chance + 240);
+	dice = (537 * chance) / 240;
+
+	return dice;
+}
+
 /**
  * Missile crits follow the same approach as melee crits.
  */
@@ -502,6 +522,29 @@ static void calculate_missile_crits(struct player_state *state, int weight,
 	*div  = 100;
 }
 
+/**
+ * Missile crits follow the same approach as melee crits.
+ */
+static int o_calculate_missile_crits(struct player_state state,
+									 const struct object *obj,
+									 const struct object *launcher)
+{
+	int dice = 0;
+	int bonus = state.to_h + obj->known->to_h
+		+ (launcher ? launcher->known->to_h : 0);
+	int chance = BTH_PLUS_ADJ * bonus;
+	if (launcher) {
+		chance += state.skills[SKILL_TO_HIT_BOW];
+	} else {
+		chance += state.skills[SKILL_TO_HIT_THROW];
+		chance *= 3 / 2;
+	}
+	chance = (100 * chance) / (chance + 360);
+	dice = (569 * chance) / 500;
+
+	return dice;
+}
+
 /**
  * Get the object flags the player should know about for the given object/
  * viewing mode combination.
@@ -722,7 +765,7 @@ static bool describe_blows(textblock *tb, const struct object *obj)
  */
 static bool obj_known_damage(const struct object *obj, int *normal_damage,
 							 int *brand_damage, int *slay_damage,
-							 bool *nonweap_slay)
+							 bool *nonweap_slay, bool throw)
 {
 	int i;
 	int dice, sides, dam, total_dam, plus = 0;
@@ -734,8 +777,8 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 	bool has_brands_or_slays = false;
 
 	struct object *bow = equipped_item_by_slot_name(player, "shooting");
-	bool weapon = tval_is_melee_weapon(obj);
-	bool ammo   = (player->state.ammo_tval == obj->tval) && (bow);
+	bool weapon = tval_is_melee_weapon(obj) && !throw;
+	bool ammo   = (player->state.ammo_tval == obj->tval) && (bow) && !throw;
 	int melee_adj_mult = ammo ? 0 : 1;
 	int multiplier = 1;
 
@@ -773,7 +816,7 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 							  &crit_div);
 
 		old_blows = state.num_blows;
-	} else { /* Ammo */
+	} else if (ammo) {
 		plus += obj->known->to_h;
 
 		calculate_missile_crits(&player->state, obj->weight, plus, &crit_mult,
@@ -781,6 +824,14 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 
 		dam += (obj->known->to_d * 10);
 		dam += (bow->known->to_d * 10);
+	} else {
+		plus += obj->known->to_h;
+
+		calculate_missile_crits(&player->state, obj->weight, plus, &crit_mult,
+								&crit_add, &crit_div);
+
+		dam += (obj->known->to_d * 10);
+		dam *= 1 + player->lev / 12;
 	}
 
 	if (ammo) multiplier = player->state.ammo_mult;
@@ -818,11 +869,13 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 
 	/* Get damage for each brand on the objects */
 	for (i = 1; i < z_info->brand_max; i++) {
-		/* Must have the brand */
-		if (total_brands[i])
-			has_brands_or_slays = true;
-		else
+		/* Must have the brand, possibly from a spell */
+		if (player_has_temporary_brand(i)) {
+			*nonweap_slay = true;
+		} else if (!total_brands[i]) {
 			continue;
+		}
+		has_brands_or_slays = true;
 
 		/* Include bonus damage and brand in stated average */
 		total_dam = dam * (multiplier + brands[i].multiplier - melee_adj_mult)
@@ -830,21 +883,25 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 		total_dam = (total_dam * crit_mult + crit_add) / crit_div;
 		total_dam += xtra_postcrit;
 
-		if (weapon)
+		if (weapon) {
 			total_dam = (total_dam * old_blows) / 100;
-		else
+		} else if (ammo) {
 			total_dam *= player->state.num_shots;
+			total_dam /= 10;
+		}
 
 		brand_damage[i] = total_dam;
 	}
 
 	/* Get damage for each slay on the objects */
 	for (i = 1; i < z_info->slay_max; i++) {
-		/* Must have the slay */
-		if (total_slays[i])
-			has_brands_or_slays = true;
-		else
+		/* Must have the slay, possibly from a spell */
+		if (player_has_temporary_slay(i)) {
+			*nonweap_slay = true;
+		} else if (!total_slays[i]) {
 			continue;
+		}
+		has_brands_or_slays = true;
 
 		/* Include bonus damage and slay in stated average */
 		total_dam = dam * (multiplier + slays[i].multiplier - melee_adj_mult)
@@ -852,10 +909,12 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 		total_dam = (total_dam * crit_mult + crit_add) / crit_div;
 		total_dam += xtra_postcrit;
 
-		if (weapon)
+		if (weapon) {
 			total_dam = (total_dam * old_blows) / 100;
-		else
+		} else if (ammo) {
 			total_dam *= player->state.num_shots;
+			total_dam /= 10;
+		}
 
 		slay_damage[i] = total_dam;
 	}
@@ -866,10 +925,12 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 	total_dam += xtra_postcrit;
 
 	/* Normal damage, not considering brands or slays */
-	if (weapon)
+	if (weapon) {
 		total_dam = (total_dam * old_blows) / 100;
-	else
+	} else if (ammo) {
 		total_dam *= player->state.num_shots;
+		total_dam /= 10;
+	}
 
 	*normal_damage = total_dam;
 
@@ -879,10 +940,203 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 }
 
 
+/**
+ * Gets information about the average damage/turn that can be inflicted if
+ * the player wields the given weapon.
+ *
+ * Fills in the damage against normal adversaries in `normal_damage`, as well
+ * as the slays on the weapon in slay_list[] and corresponding damages in 
+ * slay_damage[].  These must both be at least SL_MAX long to be safe.
+ * `nonweap_slay` is set to whether other items being worn could add to the
+ * damage done by branding attacks.
+ *
+ * Returns the number of slays populated in slay_list[] and slay_damage[].
+ *
+ * Note that the results are meaningless if called on a fake ego object as
+ * the actual ego may have different properties.
+ */
+static bool o_obj_known_damage(const struct object *obj, int *normal_damage,
+								 int *brand_damage, int *slay_damage,
+							   bool *nonweap_slay, bool throw)
+{
+	int i;
+	int dice, sides, die_average, total_dam;
+	int deadliness = obj->known->to_d;
+	int old_blows = 0;
+	bool *total_brands;
+	bool *total_slays;
+	bool has_brands_or_slays = false;
+
+	struct object *bow = equipped_item_by_slot_name(player, "shooting");
+	bool weapon = tval_is_melee_weapon(obj) && !throw;
+	bool ammo   = (player->state.ammo_tval == obj->tval) && (bow) && !throw;
+	int multiplier = 1;
+
+	struct player_state state;
+	int weapon_slot = slot_by_name(player, "weapon");
+	struct object *current_weapon = slot_object(player, weapon_slot);
+
+	/* Pretend we're wielding the object if it's a weapon */
+	if (weapon)
+		player->body.slots[weapon_slot].obj = (struct object *) obj;
+
+	/* Calculate the player's hypothetical state */
+	memcpy(&state, &player->state, sizeof(state));
+	state.stat_ind[STAT_STR] = 0; //Hack - NRM
+	state.stat_ind[STAT_DEX] = 0; //Hack - NRM
+	calc_bonuses(player, &state, true, false);
+
+	/* Stop pretending */
+	player->body.slots[weapon_slot].obj = current_weapon;
+
+	/* Finish if dice not known */
+	dice = obj->known->dd * 100;
+	sides = obj->known->ds;
+	if (!dice || !sides) return false;
+
+	/* Get the number of additional dice from criticals (x100) */
+	if (weapon)	{
+		dice += o_calculate_melee_crits(state, obj);
+		old_blows = state.num_blows;
+	} else if (ammo) {
+		dice += o_calculate_missile_crits(player->state, obj, bow);
+	} else {
+		dice += o_calculate_missile_crits(player->state, obj, NULL);
+		dice *= 2 + player->lev / 12;
+	}
+
+	if (ammo) multiplier = player->state.ammo_mult;
+
+	/* Get the average value of a single damage die. (x10) */
+	die_average = 5 * (sides + 1);
+
+	/* Apply the launcher multiplier. */
+	die_average *= multiplier;
+
+	/* Apply deadliness to average. (100x inflation) */
+	if (ammo) {
+		deadliness = obj->known->to_d + bow->known->to_d + state.to_d;
+	} else {
+		deadliness = obj->known->to_d + state.to_d;
+	}
+	apply_deadliness(&die_average, MIN(deadliness, 150));
+
+	/* Get the brands */
+	total_brands = mem_zalloc(z_info->brand_max * sizeof(bool));
+	copy_brands(&total_brands, obj->known->brands);
+	if (ammo && bow->known)
+		copy_brands(&total_brands, bow->known->brands);
+
+	/* Get the slays */
+	total_slays = mem_zalloc(z_info->slay_max * sizeof(bool));
+	copy_slays(&total_slays, obj->known->slays);
+	if (ammo && bow->known)
+		copy_slays(&total_slays, bow->known->slays);
+
+	/* Melee weapons may get slays and brands from other items */
+	*nonweap_slay = false;
+	if (weapon)	{
+		for (i = 2; i < player->body.count; i++) {
+			struct object *slot_obj = slot_object(player, i);
+			if (!slot_obj)
+				continue;
+
+			if (slot_obj->known->brands || slot_obj->known->slays)
+				*nonweap_slay = true;
+			else
+				continue;
+
+			/* Replace the old lists with new ones */
+			copy_brands(&total_brands, slot_obj->known->brands);
+			copy_slays(&total_slays, slot_obj->known->slays);
+		}
+	}
+
+	/* Increase die average for each brand on the objects */
+	for (i = 1; i < z_info->brand_max; i++) {
+		int brand_average, add = brands[i].o_multiplier - 10;
+
+		/* Must have the brand */
+		if (total_brands[i])
+			has_brands_or_slays = true;
+		else
+			continue;
+
+		/* Include brand in stated average (x10), deflate (/1000) */
+		brand_average = die_average * brands[i].o_multiplier;
+		brand_average /= 1000;
+
+		/* Damage per hit is now dice * die average, (still x1000) */
+		total_dam = (dice * brand_average);
+
+		/* Now adjust for blows and shots and deflate again */
+		if (weapon) {
+			total_dam *= old_blows;
+			total_dam /= 10000;
+		} else if (ammo) {
+			total_dam *= player->state.num_shots;
+			total_dam /= 1000;
+		} else {
+			total_dam /= 100;
+		}
+
+		brand_damage[i] = total_dam + add;
+	}
+
+	/* Get damage for each slay on the objects */
+	for (i = 1; i < z_info->slay_max; i++) {
+		int slay_average, add = slays[i].o_multiplier - 10;
+
+		/* Must have the slay */
+		if (total_slays[i])
+			has_brands_or_slays = true;
+		else
+			continue;
+
+		/* Include slay in stated average (x10), deflate (/1000) */
+		slay_average = die_average * slays[i].o_multiplier;
+		slay_average /= 1000;
+
+		/* Damage per hit is now dice * die average, (still x1000) */
+		total_dam = (dice * slay_average);
+
+		/* Now adjust for blows and shots and deflate again */
+		if (weapon) {
+			total_dam *= old_blows;
+			total_dam /= 10000;
+		} else if (ammo) {
+			total_dam *= player->state.num_shots;
+			total_dam /= 1000;
+		} else {
+			total_dam /= 100;
+		}
+
+		slay_damage[i] = total_dam + add;
+	}
+
+	/* Normal damage, not considering brands or slays */
+	total_dam = (dice * die_average) / 1000;
+	if (weapon) {
+		total_dam *= old_blows;
+		total_dam /= 1000;
+	} else if (ammo) {
+		total_dam *= player->state.num_shots;
+		total_dam /= 100;
+	} else {
+		total_dam /= 10;
+	}
+	*normal_damage = total_dam;
+
+	mem_free(total_brands);
+	mem_free(total_slays);
+	return has_brands_or_slays;
+}
+
+
 /**
  * Describe damage.
  */
-static bool describe_damage(textblock *tb, const struct object *obj)
+static bool describe_damage(textblock *tb, const struct object *obj, bool throw)
 {
 	int i;
 	bool nonweap_slay = false;
@@ -891,47 +1145,151 @@ static bool describe_damage(textblock *tb, const struct object *obj)
 	int *slay_damage = mem_zalloc(z_info->slay_max * sizeof(int));
 
 	/* Collect brands and slays */
-	bool has_brands_or_slays = obj_known_damage(obj, &normal_damage,
-												brand_damage, slay_damage,
-												&nonweap_slay);
+	bool has_brands_or_slays = OPT(player, birth_percent_damage) ?
+		o_obj_known_damage(obj, &normal_damage, brand_damage, slay_damage,
+						   &nonweap_slay, throw) :
+		obj_known_damage(obj, &normal_damage, brand_damage, slay_damage,
+						 &nonweap_slay, throw);
 
 	/* Mention slays and brands from other items */
 	if (nonweap_slay)
 		textblock_append(tb, "This weapon may benefit from one or more off-weapon brands or slays.\n");
 
-	textblock_append(tb, "Average damage/round: ");
+	if (throw) {
+		textblock_append(tb, "Average thrown damage: ");
+	} else {
+		textblock_append(tb, "Average damage/round: ");
+	}
 
 	if (has_brands_or_slays) {
-		/* Output damage for creatures effected by the brands */
-		for (i = 0; i < z_info->brand_max; i++) {
-			if (brand_damage[i] <= 0) {
-				continue;
-			} else if (brand_damage[i] % 10) {
-				textblock_append_c(tb, COLOUR_L_GREEN, "%d.%d",
-								   brand_damage[i] / 10, brand_damage[i] % 10);
-			} else {
-				textblock_append_c(tb, COLOUR_L_GREEN, "%d",
-								   brand_damage[i] / 10);
-			}
-			textblock_append(tb, " vs. creatures not resistant to %s, ",
-							 brands[i].name);
-		}
+		/*
+		 * Sort by decreasing damage so entries with the same damage
+		 * can be printed together.
+		 */
+		int *sortind = mem_alloc(
+			(z_info->brand_max + z_info->slay_max) *
+			sizeof(*sortind));
+		int nsort = 0;
+		const char *lastnm;
+		int lastdam, groupn;
+		bool last_is_brand;
 
-		/* Output damage for creatures effected by the slays */
+		/*
+		 * Assemble the indices.  Do the slays first so, if tied
+		 * for damage, they'll appear first.  That's easier to read.
+		 */
 		for (i = 0; i < z_info->slay_max; i++) {
-			if (slay_damage[i] <= 0) {
-				continue;
-			} else if (slay_damage[i] % 10) {
-				textblock_append_c(tb, COLOUR_L_GREEN, "%d.%d",
-								   slay_damage[i] / 10, slay_damage[i] % 10);
-			} else {
-				textblock_append_c(tb, COLOUR_L_GREEN, "%d",
-								   slay_damage[i] / 10);
+			if (slay_damage[i] > 0) {
+				sortind[nsort] = i + z_info->brand_max;
+				++nsort;
+			}
+		}
+		for (i = 0; i < z_info->brand_max; i++) {
+			if (brand_damage[i] > 0) {
+				sortind[nsort] = i;
+				++nsort;
+			}
+		}
+		/* Sort.  Since the number is small, insertion sort is fine. */
+		for (i = 0; i < nsort - 1; i++) {
+			int maxdam = (sortind[i] < z_info->brand_max) ?
+				brand_damage[sortind[i]] :
+				slay_damage[sortind[i] - z_info->brand_max];
+			int maxind = i;
+			int j;
+
+			for (j = i + 1; j < nsort; j++) {
+				int dam = (sortind[j] < z_info->brand_max) ?
+					brand_damage[sortind[j]] :
+					slay_damage[sortind[j] -
+						z_info->brand_max];
+
+				if (maxdam < dam) {
+					maxdam = dam;
+					maxind = j;
+				}
+			}
+			if (maxind != i) {
+				int tmp = sortind[maxind];
+
+				sortind[maxind] = sortind[i];
+				sortind[i] = tmp;
 			}
-			textblock_append(tb, " vs. %s, ", slays[i].name);
 		}
 
-		textblock_append(tb, "and ");
+		/* Output. */
+		lastdam = 0;
+		groupn = 0;
+		lastnm = NULL;
+		last_is_brand = false;
+		for (i = 0; i < nsort; i++) {
+			const char *tgt;
+			int dam;
+			bool is_brand;
+
+			if (sortind[i] < z_info->brand_max) {
+				is_brand = true;
+				tgt = brands[sortind[i]].name;
+				dam = brand_damage[sortind[i]];
+			} else {
+				is_brand = false;
+				tgt = slays[sortind[i] -
+					z_info->brand_max].name;
+				dam = slay_damage[sortind[i] -
+					z_info->brand_max];
+			}
+
+			if (groupn > 0) {
+				if (dam != lastdam) {
+					if (groupn > 2) {
+						textblock_append(tb, ", and");
+					} else if (groupn == 2) {
+						textblock_append(tb, " and");
+					}
+				} else if (groupn > 1) {
+					textblock_append(tb, ",");
+				}
+				if (last_is_brand) {
+					textblock_append(tb,
+						" creatures not resistant to");
+				}
+				textblock_append(tb, " %s", lastnm);
+			}
+			if (dam != lastdam) {
+				if (i != 0) {
+					textblock_append(tb, ", ");
+				}
+				if (dam % 10) {
+					textblock_append_c(tb, COLOUR_L_GREEN,
+						"%d.%d vs", dam / 10, dam % 10);
+				} else {
+					textblock_append_c(tb, COLOUR_L_GREEN,
+						"%d vs", dam / 10);
+				}
+				groupn = 1;
+				lastdam = dam;
+			} else {
+				assert(groupn > 0);
+				++groupn;
+			}
+			lastnm = tgt;
+			last_is_brand = is_brand;
+		}
+		if (groupn > 0) {
+			if (groupn > 2) {
+				textblock_append(tb, ", and");
+			} else if (groupn == 2) {
+				textblock_append(tb, " and");
+			}
+			if (last_is_brand) {
+				textblock_append(tb,
+					" creatures not resistant to");
+			}
+			textblock_append(tb, " %s", lastnm);
+		}
+
+		textblock_append(tb, (nsort == 1) ? " and " : ", and ");
+		mem_free(sortind);
 	}
 
 	if (normal_damage <= 0)
@@ -1012,13 +1370,15 @@ static bool describe_combat(textblock *tb, const struct object *obj)
 	struct object *bow = equipped_item_by_slot_name(player, "shooting");
 	bool weapon = tval_is_melee_weapon(obj);
 	bool ammo   = (player->state.ammo_tval == obj->tval) && (bow);
+	bool throwing_weapon = weapon && of_has(obj->flags, OF_THROWING);
+	bool rock = tval_is_ammo(obj) && of_has(obj->flags, OF_THROWING);
 
 	int range, break_chance;
 	bool thrown_effect, heavy;
 
 	obj_known_misc_combat(obj, &thrown_effect, &range, &break_chance, &heavy);
 
-	if (!weapon && !ammo) {
+	if (!weapon && !ammo && !rock) {
 		if (thrown_effect) {
 			textblock_append(tb, "It can be thrown at creatures with damaging effect.\n");
 			return true;
@@ -1033,13 +1393,18 @@ static bool describe_combat(textblock *tb, const struct object *obj)
 
 	describe_blows(tb, obj);
 
-	if (!weapon) { /* Ammo */
-		textblock_append(tb, "Hits targets up to ");
+	if (ammo) {
+		textblock_append(tb, "When fired, hits targets up to ");
 		textblock_append_c(tb, COLOUR_L_GREEN, format("%d", range));
 		textblock_append(tb, " feet away.\n");
 	}
 
-	describe_damage(tb, obj);
+	if (weapon || ammo) {
+		describe_damage(tb, obj, false);
+	}
+	if (throwing_weapon || rock) {
+		describe_damage(tb, obj, true);
+	}
 
 	if (ammo) {
 		textblock_append_c(tb, COLOUR_L_GREEN, "%d%%", break_chance);
@@ -1154,14 +1519,14 @@ static bool describe_digger(textblock *tb, const struct object *obj)
 /**
  * Gives the known light-sourcey characteristics of the given object.
  *
- * Fills in the radius of the light in `rad`, whether it uses fuel and
+ * Fills in the intensity of the light in `intensity`, whether it uses fuel and
  * how many turns light it can refuel in similar items.
  *
  * Return false if the object is not known to be a light source (which 
  * includes it not actually being a light source).
  */
 static bool obj_known_light(const struct object *obj, oinfo_detail_t mode,
-							int *rad, bool *uses_fuel, int *refuel_turns)
+							int *intensity, bool *uses_fuel, int *refuel_turns)
 {
 	bool no_fuel;
 	bool is_light = tval_is_light(obj);
@@ -1169,16 +1534,16 @@ static bool obj_known_light(const struct object *obj, oinfo_detail_t mode,
 	if (!is_light && (obj->modifiers[OBJ_MOD_LIGHT] <= 0))
 		return false;
 
-	/* Work out radius */
-	if (of_has(obj->flags, OF_LIGHT_1))
-		*rad = 1;
-	else if (of_has(obj->flags, OF_LIGHT_2))
-		*rad = 2;
-	*rad += obj->known->modifiers[OBJ_MOD_LIGHT];
+	/* Work out intensity */
+	if (of_has(obj->flags, OF_LIGHT_2))
+		*intensity = 2;
+	else if (of_has(obj->flags, OF_LIGHT_3))
+		*intensity = 3;
+	*intensity += obj->known->modifiers[OBJ_MOD_LIGHT];
 
 	/* Prevent unidentified objects (especially artifact lights) from showing
-	 * bad radius and refueling info. */
-	if (*rad == 0)
+	 * bad intensity and refueling info. */
+	if (*intensity == 0)
 		return false;
 
 	no_fuel = of_has(obj->known->flags, OF_NO_FUEL) ? true : false;
@@ -1204,17 +1569,17 @@ static bool obj_known_light(const struct object *obj, oinfo_detail_t mode,
 static bool describe_light(textblock *tb, const struct object *obj,
 						   oinfo_detail_t mode)
 {
-	int rad = 0;
+	int intensity = 0;
 	bool uses_fuel = false;
 	int refuel_turns = 0;
 	bool terse = mode & OINFO_TERSE ? true : false;
 
-	if (!obj_known_light(obj, mode, &rad, &uses_fuel, &refuel_turns))
+	if (!obj_known_light(obj, mode, &intensity, &uses_fuel, &refuel_turns))
 		return false;
 
 	if (tval_is_light(obj)) {
-		textblock_append(tb, "Radius ");
-		textblock_append_c(tb, COLOUR_L_GREEN, format("%d", rad));
+		textblock_append(tb, "Intensity ");
+		textblock_append_c(tb, COLOUR_L_GREEN, format("%d", intensity));
 		textblock_append(tb, " light.");
 
 		if (!obj->artifact && !uses_fuel)
@@ -1233,6 +1598,20 @@ static bool describe_light(textblock *tb, const struct object *obj,
 }
 
 
+/**
+ * Describe readable books.
+ */
+static bool describe_book(textblock *tb, const struct object *obj,
+						   oinfo_detail_t mode)
+{
+	if (!obj_can_browse(obj)) return false;
+
+	textblock_append(tb, "\nYou can read this book.\n");
+
+	return true;
+}
+
+
 /**
  * Gives the known effects of using the given item.
  *
@@ -1385,6 +1764,10 @@ static bool describe_effect(textblock *tb, const struct object *obj,
 			/* Check all the possible types of description format */
 			switch (base_descs[effect->index].efinfo_flag) {
 				/* Healing sometimes has a minimum percentage */
+			case EFINFO_HURT: {
+				strnfmt(desc, sizeof(desc), effect_desc(effect), dice_string);
+				break;
+			}
 			case EFINFO_HEAL: {
 				char min_string[50];
 				if (value.m_bonus)
@@ -1397,76 +1780,90 @@ static bool describe_effect(textblock *tb, const struct object *obj,
 				break;
 			}
 
-			/* Nourishment is just a flat amount */
 			case EFINFO_CONST: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect), value.base/2);
 				break;
 			}
+			case EFINFO_FOOD: {
+				char *fed = effect->subtype ? "leaves you nourished" :
+					"feeds you";
+				strnfmt(desc, sizeof(desc), effect_desc(effect), fed,
+						value.base * z_info->food_value, value.base);
+				break;
+			}
 			case EFINFO_CURE: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						timed_effects[effect->params[0]].desc);
+						timed_effects[effect->subtype].desc);
 				break;
 			}
 			case EFINFO_TIMED: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						timed_effects[effect->params[0]].desc, dice_string);
+						timed_effects[effect->subtype].desc, dice_string);
 				break;
 			}
 			case EFINFO_STAT: {
-				int stat = effect->params[0];
+				int stat = effect->subtype;
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
 						lookup_obj_property(OBJ_PROPERTY_STAT, stat)->name);
 				break;
 			}
 			case EFINFO_SEEN: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->params[0]].desc);
+						projections[effect->subtype].desc);
 				break;
 			}
 			case EFINFO_SUMM: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						summon_desc(effect->params[0]));
+						summon_desc(effect->subtype));
 				break;
 			}
 
 			/* Only currently used for the player, but can handle monsters */
 			case EFINFO_TELE: {
-				if (effect->params[0])
+				char *dist = value.m_bonus ?
+					" a level dependent distance" :
+					format(" %d grids", value.base);
+
+				if (effect->subtype) {
 					strnfmt(desc, sizeof(desc), effect_desc(effect),
-							"a monster", value.base);
-				else
+							"a monster", dist);
+				} else {
 					strnfmt(desc, sizeof(desc), effect_desc(effect), "you",
-							value.base);
+							dist);
+				}
 				break;
 			}
 			case EFINFO_QUAKE: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						effect->params[1]);
-				break;
-			}
-			case EFINFO_LIGHT: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect), dice_string,
-						effect->params[1]);
+						effect->radius);
 				break;
 			}
 
 			/* Object generated balls are elemental */
 			case EFINFO_BALL: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->params[0]].player_desc,
-						effect->params[1], dice_string);
+						projections[effect->subtype].player_desc,
+						effect->radius, dice_string);
 				if (boost)
 					my_strcat(desc, format(", which your device skill increases by %d per cent", boost),
 							  sizeof(desc));
 				break;
 			}
 
+			case EFINFO_SPOT: {
+				int i_radius = effect->other ? effect->other : effect->radius;
+				strnfmt(desc, sizeof(desc), effect_desc(effect),
+						projections[effect->subtype].player_desc,
+						effect->radius, i_radius, dice_string);
+				break;
+			}
+
 			/* Object generated breaths are elemental */
 			case EFINFO_BREATH: {
 				/* Special treatment for several random breaths */
 				if (random_breath) {
 					my_strcat(breaths,
-							  projections[effect->params[0]].player_desc,
+							  projections[effect->subtype].player_desc,
 							  sizeof(breaths));
 					if (random_choices > 3) {
 						my_strcat(breaths, ", ", sizeof(breaths));
@@ -1479,25 +1876,42 @@ static bool describe_effect(textblock *tb, const struct object *obj,
 						random_breath = false;
 					}
 					strnfmt(desc, sizeof(desc), effect_desc(effect), breaths,
-							effect->params[1], dice_string);
+							effect->other, dice_string);
 				} else {
 					strnfmt(desc, sizeof(desc), effect_desc(effect),
-							projections[effect->params[0]].player_desc,
-							effect->params[1], dice_string);
+							projections[effect->subtype].player_desc,
+							effect->other, dice_string);
 				}
 				break;
 			}
 
+			case EFINFO_SHORT: {
+				strnfmt(desc, sizeof(desc), effect_desc(effect), 
+						projections[effect->subtype].player_desc,
+						effect->radius +
+						effect->other ? effect->other / player->lev : 0,
+						dice_string);
+				break;
+			}
+
+			/* Currently no object generated lashes */
+			case EFINFO_LASH: {
+				strnfmt(desc, sizeof(desc), effect_desc(effect),
+						projections[effect->subtype].lash_desc,
+						effect->subtype);
+				break;
+			}
+
 			/* Bolts that inflict status */
 			case EFINFO_BOLT: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->params[0]].desc);
+						projections[effect->subtype].desc);
 				break;
 			}
 			/* Bolts and beams that damage */
 			case EFINFO_BOLTD: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->params[0]].desc, dice_string);
+						projections[effect->subtype].desc, dice_string);
 				if (boost)
 					my_strcat(desc, format(", which your device skill increases by %d per cent", boost),
 							  sizeof(desc));
@@ -1505,7 +1919,12 @@ static bool describe_effect(textblock *tb, const struct object *obj,
 			}
 			case EFINFO_TOUCH: {
 				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->params[0]].desc);
+						projections[effect->subtype].desc);
+				break;
+			}
+			case EFINFO_TAP: {
+				strnfmt(desc, sizeof(desc), effect_desc(effect),
+						dice_string);
 				break;
 			}
 			case EFINFO_NONE: {
@@ -1753,6 +2172,7 @@ static textblock *object_info_out(const struct object *obj, int mode)
 	if (describe_sustains(tb, flags)) something = true;
 	if (describe_misc_magic(tb, flags)) something = true;
 	if (describe_light(tb, obj, mode)) something = true;
+	if (describe_book(tb, obj, mode)) something = true;
 	if (ego && describe_ego(tb, obj->ego)) something = true;
 	if (something) textblock_append(tb, "\n");
 
@@ -1763,12 +2183,12 @@ static textblock *object_info_out(const struct object *obj, int mode)
 			something = true;
 			textblock_append(tb, "\n");
 		}
-		
+
 		if (subjective && describe_combat(tb, obj)) {
 			something = true;
 			textblock_append(tb, "\n");
 		}
-		
+
 		if (!terse && subjective && describe_digger(tb, obj)) something = true;
 	}
 
@@ -1800,6 +2220,7 @@ textblock *object_info_ego(struct ego_item *ego)
 	struct object_kind *kind = NULL;
 	struct object obj = OBJECT_NULL, known_obj = OBJECT_NULL;
 	size_t i;
+	textblock *result;
 
 	for (i = 0; i < z_info->k_max; i++) {
 		kind = &k_info[i];
@@ -1818,7 +2239,10 @@ textblock *object_info_ego(struct ego_item *ego)
 	object_copy(&known_obj, &obj);
 	obj.known = &known_obj;
 
-	return object_info_out(&obj, OINFO_NONE | OINFO_EGO);
+	result = object_info_out(&obj, OINFO_NONE | OINFO_EGO);
+	object_wipe(&known_obj);
+	object_wipe(&obj);
+	return result;
 }
 
 
