@@ -40,8 +40,10 @@ typedef enum {
  */
 enum {
 	EFINFO_NONE,
+	EFINFO_HURT,
 	EFINFO_HEAL,
 	EFINFO_CONST,
+	EFINFO_FOOD,
 	EFINFO_CURE,
 	EFINFO_TIMED,
 	EFINFO_STAT,
@@ -49,12 +51,15 @@ enum {
 	EFINFO_SUMM,
 	EFINFO_TELE,
 	EFINFO_QUAKE,
-	EFINFO_LIGHT,
 	EFINFO_BALL,
+	EFINFO_SPOT,
 	EFINFO_BREATH,
+	EFINFO_SHORT,
+	EFINFO_LASH,
 	EFINFO_BOLT,
 	EFINFO_BOLTD,
-	EFINFO_TOUCH
+	EFINFO_TOUCH,
+	EFINFO_TAP
 };
 
 textblock *object_info(const struct object *obj, oinfo_detail_t mode);
