@@ -43,6 +43,7 @@
 #include "player-spell.h"
 #include "player-util.h"
 #include "randname.h"
+#include "trap.h"
 #include "z-queue.h"
 
 /* #define LIST_DEBUG */
@@ -326,8 +327,7 @@ void object_delete(struct object **obj_address)
 	if (cave && player && player->cave && obj->oidx &&
 		(obj == cave->objects[obj->oidx]) &&
 		player->cave->objects[obj->oidx]) {
-		obj->iy = 0;
-		obj->ix = 0;
+		obj->grid = loc(0, 0);
 		obj->held_m_idx = 0;
 		obj->mimicking_m_idx = 0;
 
@@ -528,13 +528,16 @@ void object_origin_combine(struct object *obj1, const struct object *obj2)
 			obj1->origin = obj2->origin;
 			obj1->origin_depth = obj2->origin_depth;
 			obj1->origin_race = obj2->origin_race;
+			break;
 		}
 
 		/* Set as "mixed" */
 		case 2:
 		{
 			obj1->origin = ORIGIN_MIXED;
+			break;
 		}
+		default: break;
 	}
 }
 
@@ -596,12 +599,20 @@ void object_absorb_partial(struct object *obj1, struct object *obj2)
  */
 void object_absorb(struct object *obj1, struct object *obj2)
 {
+	struct object *known = obj2->known;
 	int total = obj1->number + obj2->number;
 
 	/* Add together the item counts */
 	obj1->number = MIN(total, obj1->kind->base->max_stack);
 
 	object_absorb_merge(obj1, obj2);
+	if (known) {
+		if (!loc_is_zero(known->grid)) {
+			square_excise_object(player->cave, known->grid, known);
+		}
+		delist_object(player->cave, known);
+		object_delete(&known);
+	}
 	object_delete(&obj2);
 }
 
@@ -696,8 +707,12 @@ struct object *object_split(struct object *src, int amt)
 	/* Get a copy of the object */
 	object_copy(dest, src);
 
-	/* Prepare a new known object if necessary */
+	/* Do we need a new known object? */
 	if (src->known) {
+		/* Ensure numbers are aligned (should not be necessary, but safer) */
+		src->known->number = src->number;
+
+		/* Make the new object */
 		dest_known = object_new();
 		object_copy(dest_known, src->known);
 		dest->known = dest_known;
@@ -717,8 +732,8 @@ struct object *object_split(struct object *src, int amt)
 	if (src->note)
 		dest->note = src->note;
 	if (src->known) {
-		dest->known->number = amt;
-		src->known->number -= amt;
+		dest->known->number = dest->number;
+		src->known->number = src->number;
 		dest->known->note = src->known->note;
 	}
 
@@ -750,10 +765,9 @@ struct object *floor_object_for_use(struct object *obj, int num, bool message,
 		usable = object_split(obj, num);
 	} else {
 		usable = obj;
-		square_excise_object(player->cave, usable->iy, usable->ix,
-							 usable->known);
+		square_excise_object(player->cave, usable->grid, usable->known);
 		delist_object(player->cave, usable->known);
-		square_excise_object(cave, usable->iy, usable->ix, usable);
+		square_excise_object(cave, usable->grid, usable);
 		delist_object(cave, usable);
 		*none_left = true;
 
@@ -766,10 +780,8 @@ struct object *floor_object_for_use(struct object *obj, int num, bool message,
 	}
 
 	/* Object no longer has a location */
-	usable->known->iy = 0;
-	usable->known->ix = 0;
-	usable->iy = 0;
-	usable->ix = 0;
+	usable->known->grid = loc(0, 0);
+	usable->grid = loc(0, 0);
 
 	/* Housekeeping */
 	player->upkeep->update |= (PU_BONUS | PU_INVEN);
@@ -798,11 +810,11 @@ struct object *floor_object_for_use(struct object *obj, int num, bool message,
 /**
  * Find and return the oldest object on the given grid marked as "ignore".
  */
-static struct object *floor_get_oldest_ignored(struct chunk *c, int y, int x)
+static struct object *floor_get_oldest_ignored(struct chunk *c, struct loc grid)
 {
 	struct object *obj, *ignore = NULL;
 
-	for (obj = square_object(c, y, x); obj; obj = obj->next)
+	for (obj = square_object(c, grid); obj; obj = obj->next)
 		if (ignore_item_ok(obj))
 			ignore = obj;
 
@@ -816,25 +828,26 @@ static struct object *floor_get_oldest_ignored(struct chunk *c, int y, int x)
  *
  * Optionally put the object at the top or bottom of the pile
  */
-bool floor_carry(struct chunk *c, int y, int x, struct object *drop, bool *note)
+bool floor_carry(struct chunk *c, struct loc grid, struct object *drop,
+				 bool *note)
 {
 	int n = 0;
-	struct object *obj, *ignore = floor_get_oldest_ignored(c, y, x);
+	struct object *obj, *ignore = floor_get_oldest_ignored(c, grid);
 
 	/* Fail if the square can't hold objects */
-	if (!square_isobjectholding(c, y, x))
+	if (!square_isobjectholding(c, grid))
 		return false;
 
 	/* Scan objects in that grid for combination */
-	for (obj = square_object(c, y, x); obj; obj = obj->next) {
+	for (obj = square_object(c, grid); obj; obj = obj->next) {
 		/* Check for combination */
 		if (object_similar(obj, drop, OSTACK_FLOOR)) {
 			/* Combine the items */
 			object_absorb(obj, drop);
 
 			/* Note the pile */
-			if (square_isview(c, y, x)) {
-				square_note_spot(c, y, x);
+			if (square_isview(c, grid)) {
+				square_note_spot(c, grid);
 			}
 
 			/* Don't mention if ignored */
@@ -854,7 +867,7 @@ bool floor_carry(struct chunk *c, int y, int x, struct object *drop, bool *note)
 	if (n >= z_info->floor_size || (!OPT(player, birth_stacking) && n)) {
 		/* Delete the oldest ignored object */
 		if (ignore) {
-			square_excise_object(c, y, x, ignore);
+			square_excise_object(c, grid, ignore);
 			delist_object(c, ignore);
 			object_delete(&ignore);
 		} else {
@@ -863,21 +876,30 @@ bool floor_carry(struct chunk *c, int y, int x, struct object *drop, bool *note)
 	}
 
 	/* Location */
-	drop->iy = y;
-	drop->ix = x;
+	drop->grid = grid;
 
 	/* Forget monster */
 	drop->held_m_idx = 0;
 
 	/* Link to the first object in the pile */
-	pile_insert(&c->squares[y][x].obj, drop);
+	pile_insert(&c->squares[grid.y][grid.x].obj, drop);
 
 	/* Record in the level list */
 	list_object(c, drop);
 
+	/* If there's a known version, put it in the player's view of the
+	 * cave but at an unknown location.  square_note_spot() will move
+	 * it to the correct place if seen. */
+	if (drop->known) {
+		drop->known->oidx = drop->oidx;
+		drop->known->held_m_idx = 0;
+		drop->known->grid = loc(0, 0);
+		player->cave->objects[drop->oidx] = drop->known;
+	}
+
 	/* Redraw */
-	square_note_spot(c, y, x);
-	square_light_spot(c, y, x);
+	square_note_spot(c, grid);
+	square_light_spot(c, grid);
 
 	/* Don't mention if ignored */
 	if (ignore_item_ok(drop)) {
@@ -903,8 +925,8 @@ static void floor_carry_fail(struct object *drop, bool broke)
 			: VERB_AGREEMENT(drop->number, "disappears", "disappear");
 		object_desc(o_name, sizeof(o_name), drop, ODESC_BASE);
 		msg("The %s %s.", o_name, verb);
-		if (known->iy && known->ix)
-			square_excise_object(player->cave, known->iy, known->ix, known);
+		if (!loc_is_zero(known->grid))
+			square_excise_object(player->cave, known->grid, known);
 		delist_object(player->cave, known);
 		object_delete(&known);
 	}
@@ -919,13 +941,16 @@ static void floor_carry_fail(struct object *drop, bool broke)
  * the object can combine, stack, or be placed.  Artifacts will try very
  * hard to be placed, including "teleporting" to a useful grid if needed.
  *
+ * If prefer_pile is true, does not apply a penalty for putting different types
+ * items in the same grid.
+ *
  * If no appropriate grid is found, the given grid is unchanged
  */
-static void drop_find_grid(struct object *drop, int *y, int *x)
+static void drop_find_grid(struct object *drop, bool prefer_pile, struct loc *grid)
 {
 	int best_score = -1;
-	int best_y = *y;
-	int best_x = *x;
+	struct loc start = *grid;
+	struct loc best = start;
 	int i, dy, dx;
 	struct object *obj;
 
@@ -934,23 +959,21 @@ static void drop_find_grid(struct object *drop, int *y, int *x)
 		for (dx = -3; dx <= 3; dx++) {
 			bool combine = false;
 			int dist = (dy * dy) + (dx * dx);
-			int ty = *y + dy;
-			int tx = *x + dx;
+			struct loc try = loc_sum(start, loc(dx, dy));
 			int num_shown = 0;
 			int num_ignored = 0;
 			int score;
 
 			/* Lots of reasons to say no */
 			if ((dist > 10) ||
-				!square_in_bounds_fully(cave, ty, tx) ||
-				!los(cave, *y, *x, ty, tx) ||
-				!square_isfloor(cave, ty, tx) ||
-				square_isplayertrap(cave, ty, tx) ||
-				square_iswarded(cave, ty, tx))
+				!square_in_bounds_fully(cave, try) ||
+				!los(cave, start, try) ||
+				!square_isfloor(cave, try) ||
+				square_istrap(cave, try))
 				continue;
 
 			/* Analyse the grid for carrying the new object */
-			for (obj = square_object(cave, ty, tx); obj; obj = obj->next) {
+			for (obj = square_object(cave, try); obj; obj = obj->next){
 				/* Check for possible combination */
 				if (object_similar(obj, drop, OSTACK_FLOOR))
 					combine = true;
@@ -967,25 +990,24 @@ static void drop_find_grid(struct object *drop, int *y, int *x)
 			/* Disallow if the stack size is too big */
 			if ((!OPT(player, birth_stacking) && (num_shown > 1)) ||
 				((num_shown + num_ignored) > z_info->floor_size &&
-				 !floor_get_oldest_ignored(cave, ty, tx)))
+				 !floor_get_oldest_ignored(cave, try)))
 				continue;
 
 			/* Score the location based on how close and how full the grid is */
-			score = 1000 - (dist + num_shown * 5);
+			score = 1000 -
+				(dist + (prefer_pile ? 0 : num_shown * 5));
 
 			if ((score < best_score) || ((score == best_score) && one_in_(2)))
 				continue;
 
 			best_score = score;
-			best_y = ty;
-			best_x = tx;
+			best = try;
 		}
 	}
 
 	/* Return if we have a score, otherwise fail or try harder for artifacts */
 	if (best_score >= 0) {
-		*y = best_y;
-		*x = best_x;
+		*grid = best;
 		return;
 	} else if (!drop->artifact) {
 		return;
@@ -993,16 +1015,13 @@ static void drop_find_grid(struct object *drop, int *y, int *x)
 	for (i = 0; i < 2000; i++) {
 		/* Start bouncing from grid to grid, stopping if we find an empty one */
 		if (i < 1000) {
-			best_y = rand_spread(best_y, 1);
-			best_x = rand_spread(best_x, 1);
+			best = rand_loc(best, 1, 1);
 		} else {
 			/* Now go to purely random locations */
-			best_y = randint0(cave->height);
-			best_x = randint0(cave->width);
+			best = loc(randint0(cave->width), randint0(cave->height));
 		}
-		if (square_canputitem(cave, best_y, best_x)) {
-			*y = best_y;
-			*x = best_x;
+		if (square_canputitem(cave, best)) {
+			*grid = best;
 			return;
 		}
 	}
@@ -1020,15 +1039,17 @@ static void drop_find_grid(struct object *drop, int *y, int *x)
  * This function will produce a description of a drop event under the player
  * when "verbose" is true.
  *
+ * If "prefer_pile" is true, the penalty for putting different types of items
+ * in the same square is not applied.
+ *
  * The calling function needs to deal with the consequences of the dropped
  * object being destroyed or absorbed into an existing pile.
  */
-void drop_near(struct chunk *c, struct object **dropped, int chance, int y,
-			   int x, bool verbose)
+void drop_near(struct chunk *c, struct object **dropped, int chance,
+			   struct loc grid, bool verbose, bool prefer_pile)
 {
 	char o_name[80];
-	int best_y = y;
-	int best_x = x;
+	struct loc best = grid;
 	bool dont_ignore = verbose && !ignore_item_ok(*dropped);
 
 	/* Only called in the current level */
@@ -1044,10 +1065,10 @@ void drop_near(struct chunk *c, struct object **dropped, int chance, int y,
 	}
 
 	/* Find the best grid and drop the item, destroying if there's no space */
-	drop_find_grid(*dropped, &best_y, &best_x);
-	if (floor_carry(c, best_y, best_x, *dropped, &dont_ignore)) {
+	drop_find_grid(*dropped, prefer_pile, &best);
+	if (floor_carry(c, best, *dropped, &dont_ignore)) {
 		sound(MSG_DROP);
-		if (dont_ignore && (c->squares[best_y][best_x].mon < 0)) {
+		if (dont_ignore && (square(c, best).mon < 0)) {
 			msg("You feel something roll beneath your feet.");
 		}
 	} else {
@@ -1062,38 +1083,46 @@ void drop_near(struct chunk *c, struct object **dropped, int chance, int y,
  * the previous square with a type that does not allow for objects. Drop the
  * objects. Last, put the square back to its original type.
  */
-void push_object(int y, int x)
+void push_object(struct loc grid)
 {
 	/* Save the original terrain feature */
-	struct feature *feat_old = square_feat(cave, y, x);
-
-	struct object *obj = square_object(cave, y, x);
-
+	struct feature *feat_old = square_feat(cave, grid);
+	struct object *obj = square_object(cave, grid);
 	struct queue *queue = q_new(z_info->floor_size);
-
-	bool glyph = square_iswarded(cave, y, x);
+	struct trap *trap = square_trap(cave, grid);
 
 	/* Push all objects on the square, stripped of pile info, into the queue */
 	while (obj) {
 		struct object *next = obj->next;
-		q_push_ptr(queue, obj);
+		/* In case the object is known, make a copy to work with
+		 * and try to delete the original which will orphan it to
+		 * serve as a placeholder for the known version. */
+		struct object *newobj = object_new();
 
-		/* Orphan the object */
-		obj->next = NULL;
-		obj->prev = NULL;
-		obj->iy = 0;
-		obj->ix = 0;
+		object_copy(newobj, obj);
+		newobj->oidx = 0;
+		newobj->grid = loc(0, 0);
+		if (newobj->known) {
+			newobj->known = object_new();
+			object_copy(newobj->known, obj->known);
+			newobj->known->oidx = 0;
+			newobj->known->grid = loc(0, 0);
+		}
+		q_push_ptr(queue, newobj);
+
+		delist_object(cave, obj);
+		object_delete(&obj);
 
 		/* Next object */
 		obj = next;
 	}
 
 	/* Disassociate the objects from the square */
-	cave->squares[y][x].obj = NULL;
+	square_set_obj(cave, grid, NULL);
 
 	/* Set feature to an open door */
-	square_force_floor(cave, y, x);
-	square_add_door(cave, y, x, false);
+	square_force_floor(cave, grid);
+	square_add_door(cave, grid, false);
 
 	/* Drop objects back onto the floor */
 	while (q_len(queue) > 0) {
@@ -1101,13 +1130,14 @@ void push_object(int y, int x)
 		obj = q_pop_ptr(queue);
 
 		/* Drop the object */
-		drop_near(cave, &obj, 0, y, x, false);
+		drop_near(cave, &obj, 0, grid, false, false);
 	}
 
-	/* Reset cave feature and rune if needed */
-	square_set_feat(cave, y, x, feat_old->fidx);
-	if (glyph)
-		square_add_ward(cave, y, x);
+	/* Reset cave feature, remove trap if needed */
+	square_set_feat(cave, grid, feat_old->fidx);
+	if (trap && !square_istrappable(cave, grid)) {
+		square_destroy_trap(cave, grid);
+	}
 
 	q_free(queue);
 }
@@ -1139,15 +1169,13 @@ int scan_floor(struct object **items, int max_size, object_floor_t mode,
 			   item_tester tester)
 {
 	struct object *obj;
-	int py = player->py;
-	int px = player->px;
 	int num = 0;
 
 	/* Sanity */
-	if (!square_in_bounds(cave, py, px)) return 0;
+	if (!square_in_bounds(cave, player->grid)) return 0;
 
 	/* Scan all objects in the grid */
-	for (obj = square_object(cave, py, px); obj; obj = obj->next) {
+	for (obj = square_object(cave, player->grid); obj; obj = obj->next) {
 		/* Enforce limit */
 		if (num >= max_size) break;
 
@@ -1176,16 +1204,16 @@ int scan_floor(struct object **items, int max_size, object_floor_t mode,
  *
  * Return the number of objects acquired.
  */
-int scan_distant_floor(struct object **items, int max_size, int y, int x)
+int scan_distant_floor(struct object **items, int max_size, struct loc grid)
 {
 	struct object *obj;
 	int num = 0;
 
 	/* Sanity */
-	if (!square_in_bounds(player->cave, y, x)) return 0;
+	if (!square_in_bounds(player->cave, grid)) return 0;
 
 	/* Scan all objects in the grid */
-	for (obj = square_object(player->cave, y, x); obj; obj = obj->next) {
+	for (obj = square_object(player->cave, grid); obj; obj = obj->next) {
 		/* Enforce limit */
 		if (num >= max_size) break;
 
@@ -1271,7 +1299,7 @@ int scan_items(struct object **item_list, size_t item_max, int mode,
 bool item_is_available(struct object *obj)
 {
 	if (object_is_carried(player, obj)) return true;
-	if (cave && square_holds_object(cave, player->py, player->px, obj))
+	if (cave && square_holds_object(cave, player->grid, obj))
 		return true;
 	return false;
 }
