@@ -88,6 +88,7 @@ enum object_flag_type {
 	OFT_MELEE,		/* applicable only to melee weapons */
 	OFT_BAD,		/* an undesirable flag */
 	OFT_DIG,		/* applicable only to diggers */
+	OFT_THROW,		/* applicable only to throwables */
 
 	OFT_MAX
 };
@@ -188,7 +189,7 @@ extern struct obj_property *obj_properties;
  * Functions
  * ------------------------------------------------------------------------ */
 struct obj_property *lookup_obj_property(int type, int index);
-void create_obj_flag_mask(bitflag *f, bool id, ...);
+void create_obj_flag_mask(bitflag *f, int id, ...);
 void flag_message(int flag, char *name);
 int sustain_flag(int stat);
 
