@@ -34,6 +34,7 @@ bool tval_can_have_timeout(const struct object *obj);
 int tval_find_idx(const char *name);
 const char *tval_find_name(int tval);
 bool tval_is_ammo(const struct object *obj);
+bool tval_is_sharp_missile(const struct object *obj);
 bool tval_is_armor(const struct object *obj);
 bool tval_is_body_armor(const struct object *obj);
 bool tval_is_book_k(const struct object_kind *kind);
@@ -51,7 +52,6 @@ bool tval_is_light_k(const struct object_kind *kind);
 bool tval_is_melee_weapon(const struct object *obj);
 bool tval_is_money(const struct object *obj);
 bool tval_is_money_k(const struct object_kind *kind);
-bool tval_is_pointy(const struct object *obj);
 bool tval_is_digger(const struct object *obj);
 bool tval_is_potion(const struct object *obj);
 bool tval_is_ring(const struct object *obj);
