@@ -324,7 +324,7 @@ bool is_unknown(const struct object *obj)
 		.is_player = false,
 		.hallucinate = false,
 	};
-	map_info(obj->iy, obj->ix, &gd);
+	map_info(obj->grid, &gd);
 	return gd.unseen_object;
 }	
 
@@ -356,6 +356,44 @@ unsigned check_for_inscrip(const struct object *obj, const char *inscrip)
 	return i;
 }
 
+/**
+ * Looks if "inscrip" immediately followed by a decimal integer without a
+ * leading sign character is present on the given object.  Returns the number
+ * of times such an inscription occurs and, if that value is at least one,
+ * sets *ival to the value of the integer that followed the first such
+ * inscription.
+ */
+unsigned check_for_inscrip_with_int(const struct object *obj, const char *inscrip, int* ival)
+{
+	unsigned i = 0;
+	size_t inlen = strlen(inscrip);
+	const char *s;
+
+	if (!obj->note) return 0;
+
+	s = quark_str(obj->note);
+
+	/* Needing this implies there are bad instances of obj->note around,
+	 * but I haven't been able to track down their origins - NRM */
+	if (!s) return 0;
+
+	do {
+		s = strstr(s, inscrip);
+		if (!s) break;
+		if (isdigit(s[inlen])) {
+			if (i == 0) {
+				long inarg = strtol(s + inlen, 0, 10);
+
+				*ival = (inarg < INT_MAX) ? (int) inarg : INT_MAX;
+			}
+			i++;
+		}
+		s++;
+	} while (s);
+
+	return i;
+}
+
 /*** Object kind lookup functions ***/
 
 /**
@@ -479,7 +517,7 @@ void object_short_name(char *buf, size_t max, const char *name)
 	size_t j, k;
 	/* Copy across the name, stripping modifiers & and ~) */
 	size_t len = strlen(name);
-	for (j = 0, k = 0; j < len && k < max; j++) {
+	for (j = 0, k = 0; j < len && k < max - 1; j++) {
 		if (j == 0 && name[0] == '&' && name[1] == ' ')
 			j += 2;
 		if (name[j] == '~')
@@ -581,8 +619,7 @@ bool obj_is_activatable(const struct object *obj)
  */
 bool obj_can_activate(const struct object *obj)
 {
-	if (obj_is_activatable(obj))
-	{
+	if (obj_is_activatable(obj)) {
 		/* Check the recharge */
 		if (!obj->timeout) return true;
 	}
@@ -612,19 +649,24 @@ bool obj_can_refill(const struct object *obj)
 	return false;
 }
 
-bool obj_can_browse(const struct object *obj)
+bool obj_kind_can_browse(const struct object_kind *kind)
 {
 	int i;
 
 	for (i = 0; i < player->class->magic.num_books; i++) {
 		struct class_book book = player->class->magic.books[i];
-		if (obj->kind == lookup_kind(book.tval, book.sval))
+		if (kind == lookup_kind(book.tval, book.sval))
 			return true;
 	}
 
 	return false;
 }
 
+bool obj_can_browse(const struct object *obj)
+{
+	return obj_kind_can_browse(obj->kind);
+}
+
 bool obj_can_cast_from(const struct object *obj)
 {
 	return obj_can_browse(obj) &&
@@ -634,7 +676,7 @@ bool obj_can_cast_from(const struct object *obj)
 bool obj_can_study(const struct object *obj)
 {
 	return obj_can_browse(obj) &&
-			spell_book_count_spells(obj, spell_okay_to_study) > 0;
+		spell_book_count_spells(obj, spell_okay_to_study) > 0;
 }
 
 
@@ -656,6 +698,14 @@ bool obj_can_fire(const struct object *obj)
 	return obj->tval == player->state.ammo_tval;
 }
 
+/**
+ * Determine if an object is designed for throwing
+ */
+bool obj_is_throwing(const struct object *obj)
+{
+	return of_has(obj->flags, OF_THROWING);
+}
+
 /* Can has inscrip pls */
 bool obj_has_inscrip(const struct object *obj)
 {
@@ -894,3 +944,95 @@ bool verify_object(const char *prompt, struct object *obj)
 }
 
 
+typedef enum {
+	MSG_TAG_NONE,
+	MSG_TAG_NAME,
+	MSG_TAG_KIND,
+	MSG_TAG_VERB,
+	MSG_TAG_VERB_IS
+} msg_tag_t;
+
+static msg_tag_t msg_tag_lookup(const char *tag)
+{
+	if (strncmp(tag, "name", 4) == 0) {
+		return MSG_TAG_NAME;
+	} else if (strncmp(tag, "kind", 4) == 0) {
+		return MSG_TAG_KIND;
+	} else if (strncmp(tag, "s", 1) == 0) {
+		return MSG_TAG_VERB;
+	} else if (strncmp(tag, "is", 2) == 0) {
+		return MSG_TAG_VERB_IS;
+	} else {
+		return MSG_TAG_NONE;
+	}
+}
+
+/**
+ * Print a message from a string, customised to include details about an object
+ */
+void print_custom_message(struct object *obj, const char *string, int msg_type)
+{
+	char buf[1024] = "\0";
+	const char *next;
+	const char *s;
+	const char *tag;
+	size_t end = 0;
+
+	/* Not always a string */
+	if (!string) return;
+
+	next = strchr(string, '{');
+	while (next) {
+		/* Copy the text leading up to this { */
+		strnfcat(buf, 1024, &end, "%.*s", next - string, string); 
+
+		s = next + 1;
+		while (*s && isalpha((unsigned char) *s)) s++;
+
+		/* Valid tag */
+		if (*s == '}') {
+			/* Start the tag after the { */
+			tag = next + 1;
+			string = s + 1;
+
+			switch(msg_tag_lookup(tag)) {
+			case MSG_TAG_NAME:
+				if (obj) {
+					end += object_desc(buf, 1024, obj,
+									   ODESC_PREFIX | ODESC_BASE);
+				} else {
+					strnfcat(buf, 1024, &end, "hands");
+				}
+				break;
+			case MSG_TAG_KIND:
+				if (obj) {
+					object_kind_name(&buf[end], 1024 - end, obj->kind, true);
+					end += strlen(&buf[end]);
+				} else {
+					strnfcat(buf, 1024, &end, "hands");
+				}
+				break;
+			case MSG_TAG_VERB:
+				if (obj && obj->number == 1) {
+					strnfcat(buf, 1024, &end, "s");
+				}
+				break;
+			case MSG_TAG_VERB_IS:
+				if ((!obj) || (obj->number > 1)) {
+					strnfcat(buf, 1024, &end, "are");
+				} else {
+					strnfcat(buf, 1024, &end, "is");
+				}
+			default:
+				break;
+			}
+		} else
+			/* An invalid tag, skip it */
+			string = next + 1;
+
+		next = strchr(string, '{');
+	}
+	strnfcat(buf, 1024, &end, string);
+
+	msgt(msg_type, "%s", buf);
+}
