@@ -33,6 +33,7 @@ bool object_test(item_tester tester, const struct object *o);
 bool item_test(item_tester tester, int item);
 bool is_unknown(const struct object *obj);
 unsigned check_for_inscrip(const struct object *obj, const char *inscrip);
+unsigned check_for_inscrip_with_int(const struct object *obj, const char *insrip, int *ival);
 struct object_kind *lookup_kind(int tval, int sval);
 struct object_kind *objkind_byid(int kidx);
 struct artifact *lookup_artifact_name(const char *name);
@@ -45,12 +46,14 @@ bool obj_can_zap(const struct object *obj);
 bool obj_is_activatable(const struct object *obj);
 bool obj_can_activate(const struct object *obj);
 bool obj_can_refill(const struct object *obj);
+bool obj_kind_can_browse(const struct object_kind *kind);
 bool obj_can_browse(const struct object *obj);
 bool obj_can_cast_from(const struct object *obj);
 bool obj_can_study(const struct object *obj);
 bool obj_can_takeoff(const struct object *obj);
 bool obj_can_wear(const struct object *obj);
 bool obj_can_fire(const struct object *obj);
+bool obj_is_throwing(const struct object *obj);
 bool obj_has_inscrip(const struct object *obj);
 bool obj_has_flag(const struct object *obj, int flag);
 bool obj_is_useable(const struct object *obj);
@@ -63,6 +66,7 @@ void distribute_charges(struct object *source, struct object *dest, int amt);
 int number_charging(const struct object *obj);
 bool recharge_timeout(struct object *obj);
 bool verify_object(const char *prompt, struct object *obj);
+void print_custom_message(struct object *obj, const char *string, int msg_type);
 
 
 #endif /* OBJECT_UTIL_H */
