@@ -5,7 +5,7 @@
 #ifndef INCLUDED_OBJECT_H
 #define INCLUDED_OBJECT_H
 
-#include "z-rand.h"
+#include "z-type.h"
 #include "z-quark.h"
 #include "z-bitflag.h"
 #include "z-dice.h"
@@ -53,7 +53,28 @@ struct effect {
 	struct effect *next;
 	u16b index;		/**< The effect index */
 	dice_t *dice;	/**< Dice expression used in the effect */
-	int params[3];	/**< Extra parameters to be passed to the handler */
+	int y;			/**< Y coordinate or distance */
+	int x;			/**< X coordinate or distance */
+	int subtype;	/**< Projection type, timed effect type, etc. */
+	int radius;		/**< Radius of the effect (if it has one) */
+	int other;		/**< Extra parameter to be passed to the handler */
+	char *msg;		/**< Message for deth or whatever */
+};
+
+/**
+ * Chests
+ */
+struct chest_trap {
+	struct chest_trap *next;
+	char *name;
+	char *code;
+	int level;
+	struct effect *effect;
+	int pval;
+	bool destroy;
+	bool magic;
+	char *msg;
+	char *msg_death;
 };
 
 /**
@@ -64,7 +85,9 @@ struct brand {
 	char *name;
 	char *verb;
 	int resist_flag;
+	int vuln_flag;
 	int multiplier;
+	int o_multiplier;
 	int power;
 	struct brand *next;
 };
@@ -80,6 +103,7 @@ struct slay {
 	char *range_verb;
 	int race_flag;
 	int multiplier;
+	int o_multiplier;
 	int power;
 	struct slay *next;
 };
@@ -384,7 +408,7 @@ struct curse_data {
  * monster's inventory.
  *
  * The "held_m_idx" field is used to indicate which monster, if any,
- * is holding the object.  Objects being held have "ix = 0" and "iy = 0".
+ * is holding the object.  Objects being held have (0, 0) as a grid.
  *
  * Note that object records are not now copied, but allocated on object
  * creation and freed on object destruction.  These records are handed
@@ -402,8 +426,7 @@ struct object {
 
 	u16b oidx;				/**< Item list index, if any */
 
-	byte iy;				/**< Y-position on map, or zero */
-	byte ix;				/**< X-position on map, or zero */
+	struct loc grid;		/**< position on map, or (0, 0) */
 
 	byte tval;				/**< Item type (from kind) */
 	byte sval;				/**< Item sub-type (from kind) */
@@ -456,8 +479,7 @@ static struct object const OBJECT_NULL = {
 	.next = NULL,
 	.known = NULL,
 	.oidx = 0,
-	.iy = 0,
-	.ix = 0,
+	.grid = { 0, 0 },
 	.tval = 0,
 	.sval = 0,
 	.pval = 0,
