@@ -44,135 +44,196 @@
  */
 #define MAX_PF_LENGTH 500
 
+/**
+ * Bounds of the search rectangle
+ */
+static struct loc top_left, bottom_right;
 
-static int terrain[MAX_PF_RADIUS][MAX_PF_RADIUS];
-static char pf_result[MAX_PF_LENGTH];
-static int pf_result_index;
+/**
+ * Array of distances from the player
+ */
+static int path_distance[MAX_PF_RADIUS][MAX_PF_RADIUS];
 
-static int ox, oy, ex, ey;
-static int dir_search[8] = {2,4,6,8,1,3,7,9};
+/**
+ * Pathfinding results
+ */
+static int path_step_dir[MAX_PF_LENGTH];
+static int path_step_idx;
 
-
-static bool is_valid_pf(int y, int x)
+/**
+ * Determine whether a grid is OK for the pathfinder to check
+ */
+static bool is_valid_pf(struct loc grid)
 {
 	/* Unvisited means allowed */
-	if (!square_isknown(cave, y, x)) return (true);
+	if (!square_isknown(cave, grid)) return true;
 
 	/* No damaging terrain */
-	if (!square_isdamaging(cave, y, x)) return (true);
+	if (square_isdamaging(cave, grid)) return false;
+
+	/* No trapped squares (unless trapsafe) */
+	if (square_isvisibletrap(cave, grid) &&	!player_is_trapsafe(player)) {
+		return false;
+	}
 
 	/* Require open space */
-	return (square_ispassable(cave, y, x));
+	return (square_ispassable(cave, grid));
 }
 
-static void fill_terrain_info(void)
+/**
+ * Get pathfinding region
+ */
+static void get_pathfind_region(void)
 {
-	int i, j;
+	top_left.x = MAX(player->grid.x - MAX_PF_RADIUS / 2, 0);
+	top_left.y = MAX(player->grid.y - MAX_PF_RADIUS / 2, 0);
 
-	ox = MAX(player->px - MAX_PF_RADIUS / 2, 0);
-	oy = MAX(player->py - MAX_PF_RADIUS / 2, 0);
-
-	ex = MIN(player->px + MAX_PF_RADIUS / 2 - 1, cave->width);
-	ey = MIN(player->py + MAX_PF_RADIUS / 2 - 1, cave->height);
-
-	for (i = 0; i < MAX_PF_RADIUS; i++)
-		for (j = 0; j < MAX_PF_RADIUS; j++)
-			terrain[i][j] = -1;
-
-	for (j = oy; j < ey; j++)
-		for (i = ox; i < ex; i++)
-			if (is_valid_pf(j, i))
-				terrain[j - oy][i - ox] = MAX_PF_LENGTH;
-
-	terrain[player->py - oy][player->px - ox] = 1;
+	bottom_right.x = MIN(player->grid.x + MAX_PF_RADIUS / 2 - 1, cave->width);
+	bottom_right.y = MIN(player->grid.y + MAX_PF_RADIUS / 2 - 1, cave->height);
 }
 
-#define MARK_DISTANCE(c,d) if ((c <= MAX_PF_LENGTH) && (c > d)) \
-							{ c = d; try_again = (true); }
-
-bool findpath(int y, int x)
+/**
+ * Get the path distance info for a grid
+ */
+static int path_dist(struct loc grid)
 {
-	int i, j, k;
-	int dir = 10;
-	bool try_again;
-	int cur_distance;
+	return path_distance[grid.y - top_left.y][grid.x - top_left.x];
+}
 
-	fill_terrain_info();
+/**
+ * Set the path distance info for a grid
+ */
+static void set_path_dist(struct loc grid, int dist)
+{
+	assert(grid.y >= top_left.y);
+	assert(grid.x >= top_left.x);
+	assert(grid.y < bottom_right.y);
+	assert(grid.x < bottom_right.x);
+	path_distance[grid.y - top_left.y][grid.x - top_left.x] = dist;
+}
 
-	terrain[player->py - oy][player->px - ox] = 1;
+/**
+ * Initialize terrain information
+ */
+static void path_dist_info_init(void)
+{
+	struct loc grid;
 
-	if ((x >= ox) && (x < ex) && (y >= oy) && (y < ey)) {
-		if ((cave->squares[y][x].mon > 0) &&
-			monster_is_visible(square_monster(cave, y, x))) {
-			terrain[y - oy][x - ox] = MAX_PF_LENGTH;
+	/* Set distance of all grids in the region to -1 */
+	for (grid.y = 0; grid.y < MAX_PF_RADIUS; grid.y++)
+		for (grid.x = 0; grid.x < MAX_PF_RADIUS; grid.x++)
+			path_distance[grid.y][grid.x] = -1;
+
+	/* Set distance of valid grids to MAX_PF_LENGTH */
+	for (grid.y = top_left.y; grid.y < bottom_right.y; grid.y++)
+		for (grid.x = top_left.x; grid.x < bottom_right.x; grid.x++)
+			if (is_valid_pf(grid))
+				set_path_dist(grid, MAX_PF_LENGTH);
+
+	/* Set distance of the player's grid to 0 */
+	set_path_dist(player->grid, 0);
+}
+
+/**
+ * Try to find a path from the player's grid
+ * \param grid the target grid
+ */
+static bool set_up_path_distances(struct loc grid)
+{
+	int i;
+	struct point_set *reached = point_set_new(MAX_PF_RADIUS * MAX_PF_RADIUS);
+
+	/* Initialize the pathfinding region */
+	get_pathfind_region();
+	path_dist_info_init();
+
+	/* Check bounds */
+	if ((grid.x >= top_left.x) && (grid.x < bottom_right.x) &&
+		(grid.y >= top_left.y) && (grid.y < bottom_right.y)) {
+		if ((square(cave, grid).mon > 0) &&
+			monster_is_visible(square_monster(cave, grid))) {
+			set_path_dist(grid, MAX_PF_LENGTH);
 		}
 	} else {
 		bell("Target out of range.");
-		return (false);
+		return false;
 	}
 
-	/* 
-	 * And now starts the very naive and very 
-	 * inefficient pathfinding algorithm
-	 */
-	do {
-		try_again = false;
+	/* Add the player's grid to the set of marked grids */
+	add_to_point_set(reached, player->grid);
 
-		for (j = oy + 1; j < ey - 1; j++) {
-			for (i = ox + 1; i < ex - 1; i++) {
-				cur_distance = terrain[j - oy][i - ox] + 1;
-
-				if ((cur_distance > 0) && (cur_distance < MAX_PF_LENGTH)) {
-					for (dir = 1; dir < 10; dir++) {
-						int next_y, next_x;
-						if (dir == 5)
-							dir++;
-						next_y = j - oy + ddy[dir];
-						next_x = i - ox + ddx[dir];
-
-						MARK_DISTANCE(terrain[next_y][next_x], cur_distance);
-					}
-				}
-			}
-		}
-
-		if (terrain[y - oy][x - ox] < MAX_PF_LENGTH)
-			try_again = false;
-
-	}
-	while (try_again) ;
-
-	/* Failure */
-	if (terrain[y - oy][x - ox] == MAX_PF_LENGTH) {
-		bell("Target space unreachable.");
-		return (false);
-	}
-
-	/* Success */
-	i = x;
-	j = y;
-
-	pf_result_index = 0;
-
-	while ((i != player->px) || (j != player->py)) {
-		cur_distance = terrain[j - oy][i - ox] - 1;
+	/* Add the neighbours of any marked grid in the area */
+	for (i = 0; i < reached->n; i++) {
+		int k, cur_distance = path_dist(reached->pts[i]) + 1;
 		for (k = 0; k < 8; k++) {
-			dir = dir_search[k];
-			if (terrain[j - oy + ddy[dir]][i - ox + ddx[dir]] == cur_distance)
+			struct loc next = loc_sum(reached->pts[i], ddgrid_ddd[k]);
+
+			/* Enforce length and area bounds */
+			if ((path_dist(next) <= cur_distance) ||
+				(path_dist(next) > MAX_PF_LENGTH) ||
+				(next.y < top_left.y) ||
+				(next.y >= bottom_right.y) ||
+				(next.x < top_left.x) ||
+				(next.x >= bottom_right.x)) {
+				continue;
+			}
+
+			/* Add the grid */
+			set_path_dist(next, cur_distance);
+			add_to_point_set(reached, next);
+		}
+	}
+
+	/* Grid distances have been recorded, so we can get rid of the point set */
+	point_set_dispose(reached);
+
+	/* Failure to find a path */
+	if (path_dist(grid) == -1 || path_dist(grid) == MAX_PF_LENGTH) {
+		bell("Target space unreachable.");
+		return false;
+	}
+
+	/* Found a path */
+	return true;
+}
+
+/**
+ * Fill the array of path step directions
+ * \param grid the target grid
+ */
+bool find_path(struct loc grid)
+{
+	struct loc new = grid;
+
+	/* Attempt to find a path if necessary */
+	if (loc_eq(new, player->grid)) return false;
+	if (!set_up_path_distances(grid)) return false;
+
+	/* Now travel along the path, backwards */
+	path_step_idx = 0;
+	while (!loc_eq(new, player->grid)) {
+		int k, next_distance = path_dist(new) - 1;
+
+		/* Find the next step */
+		for (k = 0; k < 8; k++) {
+			struct loc next = loc_sum(new, ddgrid_ddd[k]);
+			if (path_dist(next) == next_distance)
 				break;
 		}
 
-		/* Should never happen */
-		assert ((dir != 10) && (dir != 5));
+		/* Check we haven't exceeded path length */
+		if (path_step_idx >= MAX_PF_LENGTH) return false;
 
-		pf_result[pf_result_index++] = '0' + (char)(10 - dir);
-		i += ddx[dir];
-		j += ddy[dir];
+		/* Record the opposite of the backward direction */
+		path_step_dir[path_step_idx++] = 10 - ddd[k];
+		new = loc_sum(new, ddgrid_ddd[k]);
 	}
 
-	pf_result_index--;
+	/* Reduce to the actual number of steps */
+	path_step_idx--;
+	assert(path_step_idx >= 0);
 
-	return (true);
+	return true;
 }
 
 /**
@@ -191,33 +252,21 @@ int pathfind_direction_to(struct loc from, struct loc to)
 		return DIR_NONE;
 
 	if (dx >= 0 && dy >= 0) {
-		if (adx < ady * 2 && ady < adx * 2)
-			return DIR_SE;
-		else if (adx > ady)
-			return DIR_E;
-		else
-			return DIR_S;
+		if (adx < ady * 2 && ady < adx * 2)	return DIR_SE;
+		else if (adx > ady) return DIR_E;
+		else return DIR_S;
 	} else if (dx > 0 && dy < 0) {
-		if (adx < ady * 2 && ady < adx * 2)
-			return DIR_NE;
-		else if (adx > ady)
-			return DIR_E;
-		else
-			return DIR_N;
+		if (adx < ady * 2 && ady < adx * 2)	return DIR_NE;
+		else if (adx > ady)	return DIR_E;
+		else return DIR_N;
 	} else if (dx < 0 && dy > 0) {
-		if (adx < ady * 2 && ady < adx * 2)
-			return DIR_SW;
-		else if (adx > ady)
-			return DIR_W;
-		else
-			return DIR_S;
+		if (adx < ady * 2 && ady < adx * 2)	return DIR_SW;
+		else if (adx > ady)	return DIR_W;
+		else return DIR_S;
 	} else if (dx <= 0 && dy <= 0) {
-		if (adx < ady * 2 && ady < adx * 2)
-			return DIR_NW;
-		else if (adx > ady)
-			return DIR_W;
-		else
-			return DIR_N;
+		if (adx < ady * 2 && ady < adx * 2)	return DIR_NW;
+		else if (adx > ady)	return DIR_W;
+		else return DIR_N;
 	}
 
 	assert(0);
@@ -397,24 +446,25 @@ static const byte chome[] =
 /**
  * Hack -- Check for a "known wall" (see below)
  */
-static int see_wall(int dir, int y, int x)
+static bool see_wall(int dir, struct loc grid)
 {
 	/* Get the new location */
-	y += ddy[dir];
-	x += ddx[dir];
+	grid = loc_sum(grid, ddgrid[dir]);
 
 	/* Illegal grids are not known walls XXX XXX XXX */
-	if (!square_in_bounds(cave, y, x)) return (false);
+	if (!square_in_bounds(cave, grid)) return false;
+
+	/* Webs are enough like walls */
+	if (square_iswebbed(cave, grid)) return true;
 
 	/* Non-wall grids are not known walls */
-	if (!square_seemslikewall(cave, y, x))
-		return false;
+	if (!square_seemslikewall(cave, grid)) return false;
 
 	/* Unknown walls are not known walls */
-	if (!square_isknown(cave, y, x)) return (false);
+	if (!square_isknown(cave, grid)) return false;
 
 	/* Default */
-	return (true);
+	return true;
 }
 
 
@@ -434,10 +484,8 @@ static int see_wall(int dir, int y, int x)
  */
 static void run_init(int dir)
 {
-	int py = player->py;
-	int px = player->px;
-
-	int i, row, col;
+	int i;
+	struct loc grid;
 
 	bool deepleft, deepright;
 	bool shortleft, shortright;
@@ -463,26 +511,29 @@ static void run_init(int dir)
 	shortright = shortleft = false;
 
 	/* Find the destination grid */
-	row = py + ddy[dir];
-	col = px + ddx[dir];
+	grid = loc_sum(player->grid, ddgrid[dir]);
 
 	/* Extract cycle index */
 	i = chome[dir];
 
 	/* Check for nearby or distant wall */
-	if (see_wall(cycle[i+1], py, px)) {
+	if (see_wall(cycle[i + 1], player->grid)) {
+		/* Wall diagonally left of player's current grid */
 		run_break_left = true;
 		shortleft = true;
-	} else if (see_wall(cycle[i+1], row, col)) {
+	} else if (see_wall(cycle[i + 1], grid)) {
+		/* Wall diagonally left of the grid the player is stepping to */
 		run_break_left = true;
 		deepleft = true;
 	}
 
 	/* Check for nearby or distant wall */
-	if (see_wall(cycle[i-1], py, px)) {
+	if (see_wall(cycle[i - 1], player->grid)) {
+		/* Wall diagonally right of player's current grid */
 		run_break_right = true;
 		shortright = true;
-	} else if (see_wall(cycle[i-1], row, col)) {
+	} else if (see_wall(cycle[i - 1], grid)) {
+		/* Wall diagonally right of the grid the player is stepping to */
 		run_break_right = true;
 		deepright = true;
 	}
@@ -492,13 +543,13 @@ static void run_init(int dir)
 		/* Not looking for open area */
 		run_open_area = false;
 
-		/* Angled or blunt corridor entry */
+		/* Check angled or blunt corridor entry for diagonal directions */
 		if (dir & 0x01) {
 			if (deepleft && !deepright)
 				run_old_dir = cycle[i - 1];
 			else if (deepright && !deepleft)
 				run_old_dir = cycle[i + 1];
-		} else if (see_wall(cycle[i], row, col)) {
+		} else if (see_wall(cycle[i], player->grid)) {
 			if (shortleft && !shortright)
 				run_old_dir = cycle[i - 2];
 			else if (shortright && !shortleft)
@@ -515,17 +566,13 @@ static void run_init(int dir)
  */
 static bool run_test(void)
 {
-	int py = player->py;
-	int px = player->px;
-
 	int prev_dir;
 	int new_dir;
 
-	int row, col;
+	struct loc grid;
 	int i, max, inv;
 	int option, option2;
 
-
 	/* No options yet */
 	option = 0;
 	option2 = 0;
@@ -533,11 +580,9 @@ static bool run_test(void)
 	/* Where we came from */
 	prev_dir = run_old_dir;
 
-
-	/* Range of newly adjacent grids */
+	/* Range of newly adjacent grids - 5 for diagonals, 3 for cardinals */
 	max = (prev_dir & 0x01) + 1;
 
-
 	/* Look at every newly adjacent square. */
 	for (i = -max; i <= max; i++) {
 		struct object *obj;
@@ -546,39 +591,43 @@ static bool run_test(void)
 		new_dir = cycle[chome[prev_dir] + i];
 
 		/* New location */
-		row = py + ddy[new_dir];
-		col = px + ddx[new_dir];
-
+		grid = loc_sum(player->grid, ddgrid[new_dir]);
 
 		/* Visible monsters abort running */
-		if (cave->squares[row][col].mon > 0) {
-			struct monster *mon = square_monster(cave, row, col);
+		if (square(cave, grid).mon > 0) {
+			struct monster *mon = square_monster(cave, grid);
 			if (monster_is_visible(mon)) {
-				return (true);
+				return true;
 			}
 		}
 
+		/* Visible traps abort running (unless trapsafe) */
+		if (square_isvisibletrap(cave, grid) &&
+			!player_is_trapsafe(player)) {
+			return true;
+		}
+
 		/* Visible objects abort running */
-		for (obj = square_object(cave, row, col); obj; obj = obj->next)
+		for (obj = square_object(cave, grid); obj; obj = obj->next)
 			/* Visible object */
-			if (obj->known && !ignore_item_ok(obj)) return (true);
+			if (obj->known && !ignore_item_ok(obj)) return true;
 
 		/* Assume unknown */
 		inv = true;
 
 		/* Check memorized grids */
-		if (square_isknown(cave, row, col)) {
-			bool notice = square_isinteresting(cave, row, col);
+		if (square_isknown(cave, grid)) {
+			bool notice = square_isinteresting(cave, grid);
 
 			/* Interesting feature */
-			if (notice) return (true);
+			if (notice) return true;
 
 			/* The grid is "visible" */
 			inv = false;
 		}
 
 		/* Analyze unknown grids and floors */
-		if (inv || square_ispassable(cave, row, col)) {
+		if (inv || square_ispassable(cave, grid)) {
 			/* Looking for open area */
 			if (run_open_area) {
 				/* Nothing */
@@ -587,10 +636,10 @@ static bool run_test(void)
 				option = new_dir;
 			} else if (option2) {
 				/* Three new directions. Stop running. */
-				return (true);
+				return true;
 			} else if (option != cycle[chome[prev_dir] + i - 1]) {
 				/* Two non-adjacent new directions.  Stop running. */
-				return (true);
+				return true;
 			} else if (new_dir & 0x01) {
 				/* Two new (adjacent) directions (case 1) */
 				option2 = new_dir;
@@ -619,19 +668,18 @@ static bool run_test(void)
 		new_dir = cycle[chome[prev_dir] + i];
 		
 		/* New location */
-		row = py + ddy[prev_dir] + ddy[new_dir];
-		col = px + ddx[prev_dir] + ddx[new_dir];
+		grid = loc_sum(player->grid,
+					   loc_sum(ddgrid[prev_dir], ddgrid[new_dir]));
 		
 		/* HACK: Ugh. Sometimes we come up with illegal bounds. This will
 		 * treat the symptom but not the disease. */
-		if (row >= cave->height || col >= cave->width) continue;
-		if (row < 0 || col < 0) continue;
+		if (!square_in_bounds(cave, grid)) continue;
 
 		/* Obvious monsters abort running */
-		if (cave->squares[row][col].mon > 0) {
-			struct monster *mon = square_monster(cave, row, col);
+		if (square(cave, grid).mon > 0) {
+			struct monster *mon = square_monster(cave, grid);
 			if (monster_is_obvious(mon))
-				return (true);
+				return true;
 		}
 	}
 
@@ -640,22 +688,18 @@ static bool run_test(void)
 		/* Hack -- look again */
 		for (i = -max; i < 0; i++) {
 			new_dir = cycle[chome[prev_dir] + i];
-
-			row = py + ddy[new_dir];
-			col = px + ddx[new_dir];
+			grid = loc_sum(player->grid, ddgrid[new_dir]);
 
 			/* Unknown grid or non-wall */
-			/* Was: square_ispassable(cave, row, col) */
-			if (!square_isknown(cave, row, col) ||
-			    (square_ispassable(cave, row, col))) {
+			if (!square_isknown(cave, grid) || square_ispassable(cave, grid)) {
 				/* Looking to break right */
 				if (run_break_right) {
-					return (true);
+					return true;
 				}
 			} else { /* Obstacle */
 				/* Looking to break left */
 				if (run_break_left) {
-					return (true);
+					return true;
 				}
 			}
 		}
@@ -663,29 +707,25 @@ static bool run_test(void)
 		/* Hack -- look again */
 		for (i = max; i > 0; i--) {
 			new_dir = cycle[chome[prev_dir] + i];
-
-			row = py + ddy[new_dir];
-			col = px + ddx[new_dir];
+			grid = loc_sum(player->grid, ddgrid[new_dir]);
 
 			/* Unknown grid or non-wall */
-			/* Was: square_ispassable(cave, row, col) */
-			if (!square_isknown(cave, row, col) ||
-			    (square_ispassable(cave, row, col))) {
+			if (!square_isknown(cave, grid) || square_ispassable(cave, grid)) {
 				/* Looking to break left */
 				if (run_break_left) {
-					return (true);
+					return true;
 				}
 			} else { /* Obstacle */
 				/* Looking to break right */
 				if (run_break_right) {
-					return (true);
+					return true;
 				}
 			}
 		}
 	} else { /* Not looking for open area */
 		/* No options */
 		if (!option) {
-			return (true);
+			return true;
 		} else if (!option2) { /* One option */
 			/* Primary option */
 			run_cur_dir = option;
@@ -701,14 +741,12 @@ static bool run_test(void)
 		}
 	}
 
-
 	/* About to hit a known wall, stop */
-	if (see_wall(run_cur_dir, py, px))
-		return (true);
-
+		if (see_wall(run_cur_dir, player->grid))
+		return true;
 
 	/* Failure */
-	return (false);
+	return false;
 }
 
 
@@ -722,7 +760,7 @@ static bool run_test(void)
 void run_step(int dir)
 {
 	/* Trapsafe player will treat the trap as if it isn't there */
-	bool disarm = player->timed[TMD_TRAPSAFE] ? false : true;
+	bool disarm = player_is_trapsafe(player) ? false : true;
 
 	/* Start or continue run */
 	if (dir) {
@@ -741,27 +779,27 @@ void run_step(int dir)
 			/* Update regular running */
 			if (run_test()) {
 				/* Disturb */
-				disturb(player, 0);
+				disturb(player);
 				return;
 			}
-		} else if (pf_result_index < 0) {
+		} else if (path_step_idx < 0) {
 			/* Pathfinding, and the path is finished */
-			disturb(player, 0);
+			disturb(player);
 			player->upkeep->running_withpathfind = false;
 			return;
 		} else {
-			int y = player->py + ddy[pf_result[pf_result_index] - '0'];
-			int x = player->px + ddx[pf_result[pf_result_index] - '0'];
+			struct loc grid = loc_sum(player->grid,
+									  ddgrid[path_step_dir[path_step_idx]]);
 
-			if (pf_result_index == 0) {
+			if (path_step_idx == 0) {
 				/* Known wall */
-				if (square_isknown(cave, y, x) &&
-					!square_ispassable(cave, y, x)) {
-					disturb(player, 0);
+				if (square_isknown(cave, grid) &&
+					!square_ispassable(cave, grid)) {
+					disturb(player);
 					player->upkeep->running_withpathfind = false;
 					return;
 				}
-			} else if (pf_result_index > 0) {
+			} else if (path_step_idx > 0) {
 				struct object *obj;
 
 				/* If the player has computed a path that is going to end up
@@ -771,57 +809,56 @@ void run_step(int dir)
 				 * We have to look ahead two, otherwise we don't know which is
 				 * the last direction moved and don't initialise the run
 				 * properly. */
-				y = player->py + ddy[pf_result[pf_result_index] - '0'];
-				x = player->px + ddx[pf_result[pf_result_index] - '0'];
+				grid = loc_sum(player->grid,
+							   ddgrid[path_step_dir[path_step_idx]]);
 
 				/* Known wall */
-				if (square_isknown(cave, y, x) &&
-					!square_ispassable(cave, y, x)) {
-					disturb(player, 0);
+				if (square_isknown(cave, grid) &&
+					!square_ispassable(cave, grid)) {
+					disturb(player);
 					player->upkeep->running_withpathfind = false;
 					return;
 				}
 
 				/* Visible monsters abort running */
-				if (cave->squares[y][x].mon > 0) {
-					struct monster *mon = square_monster(cave, y, x);
+				if (square(cave, grid).mon > 0) {
+					struct monster *mon = square_monster(cave, grid);
 
 					/* Visible monster */
 					if (monster_is_visible(mon)) {
-						disturb(player, 0);
+						disturb(player);
 						player->upkeep->running_withpathfind = false;
 						return;
 					}
 				}
 
 				/* Visible objects abort running */
-				for (obj = square_object(cave, y, x); obj; obj = obj->next)
+				for (obj = square_object(cave, grid); obj; obj = obj->next)
 					/* Visible object */
 					if (obj->known && !ignore_item_ok(obj)) {
-					disturb(player, 0);
+					disturb(player);
 					player->upkeep->running_withpathfind = false;
 					return;
 				}
 
 				/* Get step after */
-				y = y + ddy[pf_result[pf_result_index - 1] - '0'];
-				x = x + ddx[pf_result[pf_result_index - 1] - '0'];
+				grid = loc_sum(grid, ddgrid[path_step_dir[path_step_idx - 1]]);
 
 				/* Known wall, so run the direction we were going */
-				if (square_isknown(cave, y, x) &&
-					!square_ispassable(cave, y, x)) {
+				if (square_isknown(cave, grid) &&
+					!square_ispassable(cave, grid)) {
 					player->upkeep->running_withpathfind = false;
-					run_init(pf_result[pf_result_index] - '0');
+					run_init(path_step_dir[path_step_idx]);
 				}
 			}
 
 			/* Now actually run the step if we're still going */
-			run_cur_dir = pf_result[pf_result_index--] - '0';
+			run_cur_dir = path_step_dir[path_step_idx--];
 		}
 	}
 
 	/* Take time */
-	player->upkeep->energy_use = z_info->move_energy;
+	player->upkeep->energy_use = z_info->move_energy / player->state.num_moves;
 
 	/* Move the player; running straight into a trap == trying to disarm */
 	move_player(run_cur_dir, dir && disarm ? true : false);
