@@ -22,18 +22,31 @@
 #include "datafile.h"
 #include "init.h"
 #include "mon-util.h"
+#include "obj-gear.h"
 #include "obj-knowledge.h"
+#include "obj-util.h"
 #include "player-calcs.h"
 #include "player-timed.h"
 #include "player-util.h"
 
+int PY_FOOD_MAX;
+int PY_FOOD_FULL;
+int PY_FOOD_HUNGRY;
+int PY_FOOD_WEAK;
+int PY_FOOD_FAINT;
+int PY_FOOD_STARVE;
+
 /**
- * The "stun" and "cut" statuses need to be handled by special functions of
- * their own, as they are more complex than the ones handled by the generic
- * code.
- */
-static bool set_stun(struct player *p, int v);
-static bool set_cut(struct player *p, int v);
+ * ------------------------------------------------------------------------
+ * Parsing functions for player_timed.txt
+ * ------------------------------------------------------------------------ */
+
+const char *list_player_flag_names[] = {
+	#define PF(a) #a,
+	#include "list-player-flags.h"
+	#undef PF
+	NULL
+};
 
 struct timed_effect_data timed_effects[TMD_MAX] = {
 	#define TMD(a, b, c)	{ #a, b, c },
@@ -52,10 +65,304 @@ int timed_name_to_idx(const char *name)
     return -1;
 }
 
+/**
+ * List of timed effect names
+ */
+static const char *list_timed_effect_names[] = {
+	#define TMD(a, b, c) #a,
+	#include "list-player-timed.h"
+	#undef TMD
+	"MAX",
+	NULL
+};
+
+static enum parser_error parse_player_timed_name(struct parser *p)
+{
+	const char *name = parser_getstr(p, "name");
+	int index;
+
+	if (grab_name("timed effect",
+			name,
+			list_timed_effect_names,
+			N_ELEMENTS(list_timed_effect_names),
+			&index)) {
+		/* XXX not a desctiptive error */
+		return PARSE_ERROR_INVALID_SPELL_NAME;
+	}
+
+	struct timed_effect_data *t = &timed_effects[index];
+
+	t->index = index;
+	parser_setpriv(p, t);
+
+	return PARSE_ERROR_NONE;
+}
+
+static enum parser_error parse_player_timed_desc(struct parser *p)
+{
+	struct timed_effect_data *t = parser_priv(p);
+	assert(t);
+
+	t->desc = string_append(t->desc, parser_getstr(p, "text"));
+	return PARSE_ERROR_NONE;
+}
+
+static enum parser_error parse_player_timed_end_message(struct parser *p)
+{
+	struct timed_effect_data *t = parser_priv(p);
+	assert(t);
+
+	t->on_end = string_append(t->on_end, parser_getstr(p, "text"));
+	return PARSE_ERROR_NONE;
+}
+
+static enum parser_error parse_player_timed_increase_message(struct parser *p)
+{
+	struct timed_effect_data *t = parser_priv(p);
+	assert(t);
+
+	t->on_increase = string_append(t->on_increase, parser_getstr(p, "text"));
+	return PARSE_ERROR_NONE;
+}
+
+static enum parser_error parse_player_timed_decrease_message(struct parser *p)
+{
+	struct timed_effect_data *t = parser_priv(p);
+	assert(t);
+
+	t->on_decrease = string_append(t->on_decrease, parser_getstr(p, "text"));
+	return PARSE_ERROR_NONE;
+}
+
+static enum parser_error parse_player_timed_message_type(struct parser *p)
+{
+	struct timed_effect_data *t = parser_priv(p);
+	assert(t);
+
+	t->msgt = message_lookup_by_name(parser_getsym(p, "type"));
+
+	return t->msgt < 0 ?
+				PARSE_ERROR_INVALID_MESSAGE :
+				PARSE_ERROR_NONE;
+}
+
+static enum parser_error parse_player_timed_fail(struct parser *p)
+{
+	struct timed_effect_data *t = parser_priv(p);
+	assert(t);
+
+	t->fail_code = parser_getuint(p, "code");
+
+	const char *name = parser_getstr(p, "flag");
+	if (t->fail_code == TMD_FAIL_FLAG_OBJECT) {
+		int flag = lookup_flag(list_obj_flag_names, name);
+		if (flag == FLAG_END)
+			return PARSE_ERROR_INVALID_FLAG;
+		else
+			t->fail = flag;
+	} else if (t->fail_code == TMD_FAIL_FLAG_PLAYER) {
+		int flag = lookup_flag(list_player_flag_names, name);
+		if (flag == FLAG_END)
+			return PARSE_ERROR_INVALID_FLAG;
+		else
+			t->fail = flag;
+	} else if ((t->fail_code == TMD_FAIL_FLAG_RESIST) ||
+			   (t->fail_code == TMD_FAIL_FLAG_VULN)) {
+		size_t i = 0;
+		while (list_element_names[i] && !streq(list_element_names[i], name))
+			i++;
+
+		if (i == ELEM_MAX)
+			return PARSE_ERROR_INVALID_FLAG;
+		else
+			t->fail = i;
+	} else {
+		return PARSE_ERROR_INVALID_FLAG;
+	}
+
+	return PARSE_ERROR_NONE;
+}
+
+static enum parser_error parse_player_timed_grade(struct parser *p)
+{
+	struct timed_effect_data *t = parser_priv(p);
+	struct timed_grade *current = t->grade;
+	struct timed_grade *l = mem_zalloc(sizeof(*l));
+    const char *color = parser_getsym(p, "color");
+    int attr = 0;
+	assert(t);
+
+	/* Make a zero grade structure if there isn't one */
+	if (!current) {
+		t->grade = mem_zalloc(sizeof(struct timed_grade));
+		current = t->grade;
+	}
+
+	/* Move to the highest grade so far */
+	while (current->next) {
+		current = current->next;
+	}
+
+	/* Add the new one */
+	current->next = l;
+	l->grade = current->grade + 1;
+
+    if (strlen(color) > 1) {
+		attr = color_text_to_attr(color);
+    } else {
+		attr = color_char_to_attr(color[0]);
+	}
+    if (attr < 0)
+		return PARSE_ERROR_INVALID_COLOR;
+    l->color = attr;
+
+	l->max = parser_getint(p, "max");
+	l->name = string_make(parser_getsym(p, "name"));
+
+	/* Name may be a dummy (eg hunger)*/
+	if (strlen(l->name) == 1) {
+		string_free(l->name);
+		l->name = NULL;
+	}
+
+	l->up_msg = string_make(parser_getsym(p, "up_msg"));
+
+	/* Message may be a dummy */
+	if (strlen(l->up_msg) == 1) {
+		string_free(l->up_msg);
+		l->up_msg = NULL;
+	}
+
+	if (parser_hasval(p, "down_msg")) {
+		l->down_msg = string_make(parser_getsym(p, "down_msg"));
+	}
+
+	/* Set food constants and deal with percentages */
+	if (streq(t->name, "FOOD")) {
+		l->max *= z_info->food_value;
+		if (streq(l->name, "Starving")) {
+			PY_FOOD_STARVE = l->max;
+		} else if (streq(l->name, "Faint")) {
+			PY_FOOD_FAINT = l->max;
+		} else if (streq(l->name, "Weak")) {
+			PY_FOOD_WEAK = l->max;
+		} else if (streq(l->name, "Hungry")) {
+			PY_FOOD_HUNGRY = l->max;
+		} else if (streq(l->name, "Fed")) {
+			PY_FOOD_FULL = l->max;
+		} else if (streq(l->name, "Full")) {
+			PY_FOOD_MAX = l->max;
+		}
+	}
+
+	return PARSE_ERROR_NONE;
+}
+
+static struct parser *init_parse_player_timed(void)
+{
+	struct parser *p = parser_new();
+	parser_setpriv(p, NULL);
+	parser_reg(p, "name str name", parse_player_timed_name);
+	parser_reg(p, "desc str text", parse_player_timed_desc);
+	parser_reg(p, "on-end str text", parse_player_timed_end_message);
+	parser_reg(p, "on-increase str text", parse_player_timed_increase_message);
+	parser_reg(p, "on-decrease str text", parse_player_timed_decrease_message);
+	parser_reg(p, "msgt sym type", parse_player_timed_message_type);
+	parser_reg(p, "fail uint code str flag", parse_player_timed_fail);
+	parser_reg(p, "grade sym color int max sym name sym up_msg ?sym down_msg", parse_player_timed_grade);
+	return p;
+}
+
+static errr run_parse_player_timed(struct parser *p)
+{
+	return parse_file_quit_not_found(p, "player_timed");
+}
+
+static errr finish_parse_player_timed(struct parser *p)
+{
+	parser_destroy(p);
+	return 0;
+}
+
+static void cleanup_player_timed(void)
+{
+	for (size_t i = 0; i < TMD_MAX; i++) {
+		struct timed_effect_data *effect = &timed_effects[i];
+		struct timed_grade *grade = effect->grade;
+
+		while (grade) {
+			struct timed_grade *next = grade->next;
+			string_free(grade->name);
+			if (grade->up_msg) string_free(grade->up_msg);
+			if (grade->down_msg) string_free(grade->down_msg);
+			mem_free(grade);
+			grade = next;
+		}
+		effect->grade = NULL;
+
+		string_free(effect->desc);
+
+		if (effect->on_end)
+			string_free(effect->on_end);
+		if (effect->on_increase)
+			string_free(effect->on_increase);
+		if (effect->on_decrease)
+			string_free(effect->on_decrease);
+
+		effect->desc        = NULL;
+		effect->on_end      = NULL;
+		effect->on_increase = NULL;
+		effect->on_decrease = NULL;
+	}
+}
+
+struct file_parser player_timed_parser = {
+	"player timed effects",
+	init_parse_player_timed,
+	run_parse_player_timed,
+	finish_parse_player_timed,
+	cleanup_player_timed
+};
+
+
+/**
+ * ------------------------------------------------------------------------
+ * Utilities for more complex or anomolous effects
+ * ------------------------------------------------------------------------ */
+/**
+ * Swap stats at random to temporarily scramble the player's stats.
+ */
+static void player_scramble_stats(struct player *p)
+{
+	int max1, cur1, max2, cur2, i, j, swap;
+
+	/* Fisher-Yates shuffling algorithm */
+	for (i = STAT_MAX - 1; i > 0; --i) {
+		j = randint0(i);
+
+		max1 = p->stat_max[i];
+		cur1 = p->stat_cur[i];
+		max2 = p->stat_max[j];
+		cur2 = p->stat_cur[j];
+
+		p->stat_max[i] = max2;
+		p->stat_cur[i] = cur2;
+		p->stat_max[j] = max1;
+		p->stat_cur[j] = cur1;
+
+		/* Record what we did */
+		swap = p->stat_map[i];
+		p->stat_map[i] = p->stat_map[j];
+		p->stat_map[j] = swap;
+	}
+
+	return;
+}
+
 /**
  * Undo scrambled stats when effect runs out.
  */
-void player_fix_scramble(struct player *p)
+static void player_fix_scramble(struct player *p)
 {
 	/* Figure out what stats should be */
 	int new_cur[STAT_MAX];
@@ -74,6 +381,27 @@ void player_fix_scramble(struct player *p)
 	}
 }
 
+/**
+ * Return true if the player timed effect matches the given string
+ */
+bool player_timed_grade_eq(struct player *p, int idx, char *match)
+{
+	if (p->timed[idx]) {
+		struct timed_grade *grade = timed_effects[idx].grade;
+		while (p->timed[idx] > grade->max) {
+			grade = grade->next;
+		}
+		if (grade->name && streq(grade->name, match)) return true;
+	}
+
+	return false;
+}
+
+
+/**
+ * ------------------------------------------------------------------------
+ * Setting, increasing, decreasing and clearing timed effects
+ * ------------------------------------------------------------------------ */
 /**
  * Set a timed effect.
  */
@@ -83,48 +411,72 @@ bool player_set_timed(struct player *p, int idx, int v, bool notify)
 	assert(idx < TMD_MAX);
 
 	struct timed_effect_data *effect = &timed_effects[idx];
+	struct timed_grade *new_grade = effect->grade;
+	struct timed_grade *current_grade = effect->grade;
+	struct object *weapon = equipped_item_by_slot_name(p, "weapon");
 
-	/* Limit values */
-	v = MIN(v, 10000);
+	/* Lower bound */
 	v = MAX(v, 0);
 
 	/* No change */
-	if (p->timed[idx] == v)
+	if (p->timed[idx] == v) {
 		return false;
+	}
 
-	/* Hack -- call other functions */
-	if (idx == TMD_STUN)
-		return set_stun(p, v);
-	else if (idx == TMD_CUT)
-		return set_cut(p, v);
+	/* Find the grade we will be going to, and the current one */
+	while (v > new_grade->max) {
+		new_grade = new_grade->next;
+		if (!new_grade->next) break;
+	}
+	while (p->timed[idx] > current_grade->max) {
+		current_grade = current_grade->next;
+		if (!current_grade->next) break;
+	}
+
+	/* Upper bound */
+	v = MIN(v, new_grade->max);
 
 	/* Don't mention effects which already match the player state. */
-	if (idx == TMD_OPP_ACID && player_is_immune(p, ELEM_ACID))
+	if (idx == TMD_OPP_ACID && player_is_immune(p, ELEM_ACID)) {
 		notify = false;
-	else if (idx == TMD_OPP_ELEC && player_is_immune(p, ELEM_ELEC))
+	} else if (idx == TMD_OPP_ELEC && player_is_immune(p, ELEM_ELEC)) {
 		notify = false;
-	else if (idx == TMD_OPP_FIRE && player_is_immune(p, ELEM_FIRE))
+	} else if (idx == TMD_OPP_FIRE && player_is_immune(p, ELEM_FIRE)) {
 		notify = false;
-	else if (idx == TMD_OPP_COLD && player_is_immune(p, ELEM_COLD))
+	} else if (idx == TMD_OPP_COLD && player_is_immune(p, ELEM_COLD)) {
 		notify = false;
-	else if (idx == TMD_OPP_CONF && player_of_has(p, OF_PROT_CONF))
+	} else if (idx == TMD_OPP_CONF && player_of_has(p, OF_PROT_CONF)) {
 		notify = false;
+	}
 
-	/* Always mention start or finish, otherwise on request */
-	if (v == 0) {
-		msgt(MSG_RECOVER, "%s", effect->on_end);
+	/* Always mention going up a grade, otherwise on request */
+	if (new_grade->grade > current_grade->grade) {
+		print_custom_message(weapon, new_grade->up_msg, effect->msgt);
 		notify = true;
-	} else if (p->timed[idx] == 0) {
-		msgt(effect->msgt, "%s", effect->on_begin);
+	} else if ((new_grade->grade < current_grade->grade) &&
+			   (new_grade->down_msg)) {
+		print_custom_message(weapon, new_grade->down_msg, effect->msgt);
 		notify = true;
 	} else if (notify) {
-		/* Decrementing */
-		if (p->timed[idx] > v && effect->on_decrease)
-			msgt(effect->msgt, "%s", effect->on_decrease);
+		if (v == 0) {
+			/* Finishing */
+			print_custom_message(weapon, effect->on_end, MSG_RECOVER);
+		} else if (p->timed[idx] > v && effect->on_decrease) {
+			/* Decrementing */
+			print_custom_message(weapon, effect->on_decrease, effect->msgt);
+		} else if (v > p->timed[idx] && effect->on_increase) {
+			/* Incrementing */
+			print_custom_message(weapon, effect->on_increase, effect->msgt);
+		}
+	}
 
-		/* Incrementing */
-		else if (v > p->timed[idx] && effect->on_increase)
-			msgt(effect->msgt, "%s", effect->on_increase);
+	/* Handle stat swap */
+	if (idx == TMD_SCRAMBLE) {
+		if (p->timed[idx] == 0) {
+			player_scramble_stats(p);
+		} else if (v == 0) {
+			player_fix_scramble(p);
+		}
 	}
 
 	/* Use the value */
@@ -135,14 +487,9 @@ bool player_set_timed(struct player *p, int idx, int v, bool notify)
 		player_inc_timed(p, TMD_SLOW, 100, true, false);
 	}
 
-	/* Undo stat swap */
-	if (idx == TMD_SCRAMBLE && v == 0) {
-		player_fix_scramble(p);
-	}
-
 	if (notify) {
 		/* Disturb */
-		disturb(p, 0);
+		disturb(p);
 
 		/* Update the visuals, as appropriate. */
 		p->upkeep->update |= effect->flag_update;
@@ -213,9 +560,14 @@ bool player_inc_check(struct player *p, int idx, bool lore)
 		if (p->state.el_info[effect->fail].res_level < 0) {
 			return false;
 		}
+	} else if (effect->fail_code == TMD_FAIL_FLAG_PLAYER) {
+		/* Effect is inhibited by a player flag */
+		if (player_has(p, effect->fail)) {
+			return false;
+		}
 	}
 
-	/* Special case */
+	/* Special cases */
 	if (effect->index == TMD_POISONED && p->timed[TMD_OPP_POIS])
 		return false;
 
@@ -251,13 +603,16 @@ bool player_inc_timed(struct player *p, int idx, int v, bool notify, bool check)
  */
 bool player_dec_timed(struct player *p, int idx, int v, bool notify)
 {
+	int new_value;
 	assert(idx >= 0);
 	assert(idx < TMD_MAX);
+	new_value = p->timed[idx] - v;
 
-	return player_set_timed(p,
-			idx,
-			p->timed[idx] - v,
-			notify);
+	/* Obey `notify` if not finishing; if finishing, always notify */
+	if (new_value > 0) {
+		return player_set_timed(p, idx, new_value, notify);
+	}
+	return player_set_timed(p, idx, new_value, true);
 }
 
 /**
@@ -271,522 +626,3 @@ bool player_clear_timed(struct player *p, int idx, bool notify)
 	return player_set_timed(p, idx, 0, notify);
 }
 
-/**
- * Set "player->timed[TMD_STUN]", notice observable changes
- *
- * Note the special code to only notice "range" changes.
- */
-static bool set_stun(struct player *p, int v)
-{
-	int old_aux, new_aux;
-
-	bool notice = false;
-
-	/* Hack -- Force good values */
-	v = (v > 10000) ? 10000 : (v < 0) ? 0 : v;
-
-	/* Old state */
-	if (p->timed[TMD_STUN] > 100)
-		/* Knocked out */
-		old_aux = 3;
-	else if (p->timed[TMD_STUN] > 50)
-		/* Heavy stun */
-		old_aux = 2;
-	else if (p->timed[TMD_STUN] > 0)
-		/* Stun */
-		old_aux = 1;
-	else
-		/* None */
-		old_aux = 0;
-
-	/* New state */
-	if (v > 100)
-		/* Knocked out */
-		new_aux = 3;
-	else if (v > 50)
-		/* Heavy stun */
-		new_aux = 2;
-	else if (v > 0)
-		/* Stun */
-		new_aux = 1;
-	else
-		/* None */
-		new_aux = 0;
-
-	/* Increase or decrease stun */
-	if (new_aux > old_aux) {
-		/* Describe the state */
-		switch (new_aux) {
-			/* Stun */
-			case 1:	{
-				msgt(MSG_STUN, "You have been stunned.");
-				break;
-			}
-
-			/* Heavy stun */
-			case 2:	{
-				msgt(MSG_STUN, "You have been heavily stunned.");
-				break;
-			}
-
-			/* Knocked out */
-			case 3:	{
-				msgt(MSG_STUN, "You have been knocked out.");
-				break;
-			}
-		}
-
-		/* Notice */
-		notice = true;
-	} else if (new_aux < old_aux) {
-		/* Describe the state */
-		switch (new_aux) {
-			/* None */
-			case 0: {
-				msgt(MSG_RECOVER, "You are no longer stunned.");
-				disturb(player, 0);
-				break;
-			}
-		}
-
-		/* Notice */
-		notice = true;
-	}
-
-	/* Use the value */
-	p->timed[TMD_STUN] = v;
-
-	/* No change */
-	if (!notice) return (false);
-
-	/* Disturb and update */
-	disturb(player, 0);
-	p->upkeep->update |= (PU_BONUS);
-	p->upkeep->redraw |= (PR_STATUS);
-	handle_stuff(player);
-
-	/* Result */
-	return (true);
-}
-
-/**
- * Set "player->timed[TMD_CUT]", notice observable changes
- *
- * Note the special code to only notice "range" changes.
- */
-static bool set_cut(struct player *p, int v)
-{
-	int old_aux, new_aux;
-
-	bool notice = false;
-
-	/* Hack -- Force good values */
-	v = (v > 10000) ? 10000 : (v < 0) ? 0 : v;
-
-	/* Old state */
-	if (p->timed[TMD_CUT] > TMD_CUT_DEEP)
-		/* Mortal wound */
-		old_aux = 7;
-	else if (p->timed[TMD_CUT] > TMD_CUT_SEVERE)
-		/* Deep gash */
-		old_aux = 6;
-	else if (p->timed[TMD_CUT] > TMD_CUT_NASTY)
-		/* Severe cut */
-		old_aux = 5;
-	else if (p->timed[TMD_CUT] > TMD_CUT_BAD)
-		/* Nasty cut */
-		old_aux = 4;
-	else if (p->timed[TMD_CUT] > TMD_CUT_LIGHT)
-		/* Bad cut */
-		old_aux = 3;
-	else if (p->timed[TMD_CUT] > TMD_CUT_GRAZE)
-		/* Light cut */
-		old_aux = 2;
-	else if (p->timed[TMD_CUT] > TMD_CUT_NONE)
-		/* Graze */
-		old_aux = 1;
-	else
-		/* None */
-		old_aux = 0;
-
-	/* New state */
-	if (v > TMD_CUT_DEEP)
-		/* Mortal wound */
-		new_aux = 7;
-	else if (v > TMD_CUT_SEVERE)
-		/* Deep gash */
-		new_aux = 6;
-	else if (v > TMD_CUT_NASTY)
-		/* Severe cut */
-		new_aux = 5;
-	else if (v > TMD_CUT_BAD)
-		/* Nasty cut */
-		new_aux = 4;
-	else if (v > TMD_CUT_LIGHT)
-		/* Bad cut */
-		new_aux = 3;
-	else if (v > TMD_CUT_GRAZE)
-		/* Light cut */
-		new_aux = 2;
-	else if (v > TMD_CUT_NONE)
-		/* Graze */
-		new_aux = 1;
-	else
-		/* None */
-		new_aux = 0;
-
-	/* Increase or decrease cut */
-	if (new_aux > old_aux) {
-		/* Describe the state */
-		switch (new_aux) {
-			/* Graze */
-			case 1:	{
-				msgt(MSG_CUT, "You have been given a graze.");
-				break;
-			}
-
-			/* Light cut */
-			case 2:	{
-				msgt(MSG_CUT, "You have been given a light cut.");
-				break;
-			}
-
-			/* Bad cut */
-			case 3:	{
-				msgt(MSG_CUT, "You have been given a bad cut.");
-				break;
-			}
-
-			/* Nasty cut */
-			case 4:	{
-				msgt(MSG_CUT, "You have been given a nasty cut.");
-				break;
-			}
-
-			/* Severe cut */
-			case 5:	{
-				msgt(MSG_CUT, "You have been given a severe cut.");
-				break;
-			}
-
-			/* Deep gash */
-			case 6:	{
-				msgt(MSG_CUT, "You have been given a deep gash.");
-				break;
-			}
-
-			/* Mortal wound */
-			case 7:	{
-				msgt(MSG_CUT, "You have been given a mortal wound.");
-				break;
-			}
-		}
-
-		/* Notice */
-		notice = true;
-	} else if (new_aux < old_aux) {
-		/* Describe the state */
-		switch (new_aux) {
-			/* None */
-			case 0:	{
-				msgt(MSG_RECOVER, "You are no longer bleeding.");
-				disturb(player, 0);
-				break;
-			}
-		}
-
-		/* Notice */
-		notice = true;
-	}
-
-	/* Use the value */
-	p->timed[TMD_CUT] = v;
-
-	/* No change */
-	if (!notice) return (false);
-
-	/* Disturb and update */
-	disturb(player, 0);
-	p->upkeep->update |= (PU_BONUS);
-	p->upkeep->redraw |= (PR_STATUS);
-	handle_stuff(player);
-
-	/* Result */
-	return (true);
-}
-
-/**
- * Set "player->food", notice observable changes
- *
- * The "player->food" variable can get as large as 20000, allowing the
- * addition of the most "filling" item, Elvish Waybread, which adds
- * 7500 food units, without overflowing the 32767 maximum limit.
- *
- * Perhaps we should disturb the player with various messages,
- * especially messages about hunger status changes.  XXX XXX XXX
- *
- * Digestion of food is handled in "dungeon.c", in which, normally,
- * the player digests about 20 food units per 100 game turns, more
- * when "fast", more when "regenerating", less with "slow digestion".
- */
-bool player_set_food(struct player *p, int v)
-{
-	int old_aux, new_aux;
-
-	bool notice = false;
-
-	/* Hack -- Force good values */
-	v = MIN(v, PY_FOOD_MAX);
-	v = MAX(v, 0);
-
-	/* Current value */
-	if (p->food < PY_FOOD_FAINT)      old_aux = 0;
-	else if (p->food < PY_FOOD_WEAK)  old_aux = 1;
-	else if (p->food < PY_FOOD_ALERT) old_aux = 2;
-	else if (p->food < PY_FOOD_FULL)  old_aux = 3;
-	else                              old_aux = 4;
-
-	/* New value */
-	if (v < PY_FOOD_FAINT)      new_aux = 0;
-	else if (v < PY_FOOD_WEAK)  new_aux = 1;
-	else if (v < PY_FOOD_ALERT) new_aux = 2;
-	else if (v < PY_FOOD_FULL)  new_aux = 3;
-	else                        new_aux = 4;
-
-	/* Food increase or decrease */
-	if (new_aux > old_aux) {
-		switch (new_aux) {
-			case 1:
-				msg("You are still weak.");
-				break;
-			case 2:
-				msg("You are still hungry.");
-				break;
-			case 3:
-				msg("You are no longer hungry.");
-				break;
-			case 4:
-				msg("You are full!");
-				break;
-		}
-
-		/* Change */
-		notice = true;
-	} else if (new_aux < old_aux) {
-		switch (new_aux) {
-			case 0:
-				msgt(MSG_NOTICE, "You are getting faint from hunger!");
-				break;
-			case 1:
-				msgt(MSG_NOTICE, "You are getting weak from hunger!");
-				break;
-			case 2:
-				msgt(MSG_HUNGRY, "You are getting hungry.");
-				break;
-			case 3:
-				msgt(MSG_NOTICE, "You are no longer full.");
-				break;
-		}
-
-		/* Change */
-		notice = true;
-	}
-
-	/* Use the value */
-	p->food = v;
-
-	/* Nothing to notice */
-	if (!notice) return (false);
-
-	/* Disturb and update */
-	disturb(player, 0);
-	p->upkeep->update |= (PU_BONUS);
-	p->upkeep->redraw |= (PR_STATUS);
-	handle_stuff(player);
-
-	/* Result */
-	return (true);
-}
-
-/*
- * Parsing functions for player_timed.txt
- */
-
-/**
- * List of timed effect names
- */
-static const char *list_timed_effect_names[] = {
-	#define TMD(a, b, c) #a,
-	#include "list-player-timed.h"
-	#undef TMD
-	"MAX",
-	NULL
-};
-
-static enum parser_error parse_player_timed_name(struct parser *p)
-{
-	const char *name = parser_getstr(p, "name");
-	int index;
-
-	if (grab_name("timed effect",
-			name,
-			list_timed_effect_names,
-			N_ELEMENTS(list_timed_effect_names),
-			&index)) {
-		/* XXX not a desctiptive error */
-		return PARSE_ERROR_INVALID_SPELL_NAME;
-	}
-
-	struct timed_effect_data *t = &timed_effects[index];
-
-	t->index = index;
-	parser_setpriv(p, t);
-
-	return PARSE_ERROR_NONE;
-}
-
-static enum parser_error parse_player_timed_desc(struct parser *p)
-{
-	struct timed_effect_data *t = parser_priv(p);
-	assert(t);
-
-	t->desc = string_append(t->desc, parser_getstr(p, "text"));
-	return PARSE_ERROR_NONE;
-}
-
-static enum parser_error parse_player_timed_begin_message(struct parser *p)
-{
-	struct timed_effect_data *t = parser_priv(p);
-	assert(t);
-
-	t->on_begin = string_append(t->on_begin, parser_getstr(p, "text"));
-	return PARSE_ERROR_NONE;
-}
-
-static enum parser_error parse_player_timed_end_message(struct parser *p)
-{
-	struct timed_effect_data *t = parser_priv(p);
-	assert(t);
-
-	t->on_end = string_append(t->on_end, parser_getstr(p, "text"));
-	return PARSE_ERROR_NONE;
-}
-
-static enum parser_error parse_player_timed_increase_message(struct parser *p)
-{
-	struct timed_effect_data *t = parser_priv(p);
-	assert(t);
-
-	t->on_increase = string_append(t->on_increase, parser_getstr(p, "text"));
-	return PARSE_ERROR_NONE;
-}
-
-static enum parser_error parse_player_timed_decrease_message(struct parser *p)
-{
-	struct timed_effect_data *t = parser_priv(p);
-	assert(t);
-
-	t->on_decrease = string_append(t->on_decrease, parser_getstr(p, "text"));
-	return PARSE_ERROR_NONE;
-}
-
-static enum parser_error parse_player_timed_message_type(struct parser *p)
-{
-	struct timed_effect_data *t = parser_priv(p);
-	assert(t);
-
-	t->msgt = message_lookup_by_name(parser_getsym(p, "type"));
-
-	return t->msgt < 0 ?
-				PARSE_ERROR_INVALID_MESSAGE :
-				PARSE_ERROR_NONE;
-}
-
-static enum parser_error parse_player_timed_fail(struct parser *p)
-{
-	struct timed_effect_data *t = parser_priv(p);
-	assert(t);
-
-	t->fail_code = parser_getuint(p, "code");
-
-	const char *name = parser_getstr(p, "flag");
-	if (t->fail_code == TMD_FAIL_FLAG_OBJECT) {
-		int flag = lookup_flag(list_obj_flag_names, name);
-		if (flag == FLAG_END)
-			return PARSE_ERROR_INVALID_FLAG;
-		else
-			t->fail = flag;
-	} else if ((t->fail_code == TMD_FAIL_FLAG_RESIST) ||
-			   (t->fail_code == TMD_FAIL_FLAG_VULN)) {
-		size_t i = 0;
-		while (list_element_names[i] && !streq(list_element_names[i], name))
-			i++;
-
-		if (i == ELEM_MAX)
-			return PARSE_ERROR_INVALID_FLAG;
-		else
-			t->fail = i;
-	} else {
-		return PARSE_ERROR_INVALID_FLAG;
-	}
-
-	return PARSE_ERROR_NONE;
-}
-
-static struct parser *init_parse_player_timed(void)
-{
-	struct parser *p = parser_new();
-	parser_setpriv(p, NULL);
-	parser_reg(p, "name str name", parse_player_timed_name);
-	parser_reg(p, "desc str text", parse_player_timed_desc);
-	parser_reg(p, "on-begin str text", parse_player_timed_begin_message);
-	parser_reg(p, "on-end str text", parse_player_timed_end_message);
-	parser_reg(p, "on-increase str text", parse_player_timed_increase_message);
-	parser_reg(p, "on-decrease str text", parse_player_timed_decrease_message);
-	parser_reg(p, "msgt sym type", parse_player_timed_message_type);
-	parser_reg(p, "fail uint code str flag", parse_player_timed_fail);
-	return p;
-}
-
-static errr run_parse_player_timed(struct parser *p)
-{
-	return parse_file_quit_not_found(p, "player_timed");
-}
-
-static errr finish_parse_player_timed(struct parser *p)
-{
-	parser_destroy(p);
-	return 0;
-}
-
-static void cleanup_player_timed(void)
-{
-	for (size_t i = 0; i < TMD_MAX; i++) {
-		struct timed_effect_data *effect = &timed_effects[i];
-
-		string_free(effect->desc);
-
-		if (effect->on_begin)
-			string_free(effect->on_begin);
-		if (effect->on_end)
-			string_free(effect->on_end);
-		if (effect->on_increase)
-			string_free(effect->on_increase);
-		if (effect->on_decrease)
-			string_free(effect->on_decrease);
-
-		effect->desc        = NULL;
-		effect->on_begin    = NULL;
-		effect->on_end      = NULL;
-		effect->on_increase = NULL;
-		effect->on_decrease = NULL;
-	}
-}
-
-struct file_parser player_timed_parser = {
-	"player timed effects",
-	init_parse_player_timed,
-	run_parse_player_timed,
-	finish_parse_player_timed,
-	cleanup_player_timed
-};
