@@ -22,27 +22,6 @@
 
 #include "player.h"
 
-/**
- * Player food values
- */
-#define PY_FOOD_MAX 	17000	/* Food value (Bloated) */
-#define PY_FOOD_FULL	10000	/* Food value (Normal) */
-#define PY_FOOD_ALERT	2000	/* Food value (Hungry) */
-#define PY_FOOD_WEAK	1000	/* Food value (Weak) */
-#define PY_FOOD_FAINT	500		/* Food value (Fainting) */
-#define PY_FOOD_STARVE	100		/* Food value (Starving) */
-
-/**
- * Player cut timer values
- */
-#define TMD_CUT_NONE    0
-#define TMD_CUT_GRAZE   10
-#define TMD_CUT_LIGHT   25
-#define TMD_CUT_BAD     50
-#define TMD_CUT_NASTY   100
-#define TMD_CUT_SEVERE  200
-#define TMD_CUT_DEEP    1000
-
 /**
  * Timed effects
  */
@@ -60,7 +39,18 @@ enum
 enum {
 	TMD_FAIL_FLAG_OBJECT = 1,
 	TMD_FAIL_FLAG_RESIST,
-	TMD_FAIL_FLAG_VULN
+	TMD_FAIL_FLAG_VULN,
+	TMD_FAIL_FLAG_PLAYER
+};
+
+struct timed_grade {
+	int grade;
+	byte color;
+	int max;
+	char *name;
+	char *up_msg;
+	char *down_msg;
+	struct timed_grade *next;
 };
 
 /**
@@ -73,25 +63,35 @@ struct timed_effect_data {
 
 	int index;
 	char *desc;
-	char *on_begin;
 	char *on_end;
 	char *on_increase;
 	char *on_decrease;
 	int msgt;
 	int fail_code;
 	int fail;
+	struct timed_grade *grade;
 };
 
+/**
+ * Player food values
+ */
+extern int PY_FOOD_MAX; 	/* Food value (Bloated) */
+extern int PY_FOOD_FULL;	/* Food value (Normal) */
+extern int PY_FOOD_HUNGRY;	/* Food value (Hungry) */
+extern int PY_FOOD_WEAK;	/* Food value (Weak) */
+extern int PY_FOOD_FAINT;	/* Food value (Fainting) */
+extern int PY_FOOD_STARVE;	/* Food value (Starving) */
+
 extern struct file_parser player_timed_parser;
 extern struct timed_effect_data timed_effects[TMD_MAX];
 
 int timed_name_to_idx(const char *name);
+bool player_timed_grade_eq(struct player *p, int idx, char *match);
 bool player_set_timed(struct player *p, int idx, int v, bool notify);
 bool player_inc_check(struct player *p, int idx, bool lore);
 bool player_inc_timed(struct player *p, int idx, int v, bool notify,
 					  bool check);
 bool player_dec_timed(struct player *p, int idx, int v, bool notify);
 bool player_clear_timed(struct player *p, int idx, bool notify);
-bool player_set_food(struct player *p, int v);
 
 #endif /* !PLAYER_TIMED_H */
