@@ -19,6 +19,7 @@
 #include "angband.h"
 #include "cave.h"
 #include "cmd-core.h"
+#include "cmds.h"
 #include "game-input.h"
 #include "game-world.h"
 #include "generate.h"
@@ -52,19 +53,19 @@ int dungeon_get_next_level(int dlev, int added)
 
 	/* Get target level */
 	target_level = dlev + added * z_info->stair_skip;
-	
+
 	/* Don't allow levels below max */
 	if (target_level > z_info->max_depth - 1)
 		target_level = z_info->max_depth - 1;
 
 	/* Don't allow levels above the town */
 	if (target_level < 0) target_level = 0;
-	
+
 	/* Check intermediate levels for quests */
 	for (i = dlev; i <= target_level; i++) {
 		if (is_quest(i)) return i;
 	}
-	
+
 	return target_level;
 }
 
@@ -157,22 +158,48 @@ void take_hit(struct player *p, int dam, const char *kb_str)
 	/* Paranoia */
 	if (p->is_dead) return;
 
-	/* Disturb */
-	disturb(p, 1);
-
 	/* Mega-Hack -- Apply "invulnerability" */
 	if (p->timed[TMD_INVULN] && (dam < 9000)) return;
 
+	/* Apply damage reduction */
+	dam -= p->state.dam_red;
+	if (p->state.perc_dam_red) {
+		dam -= (dam * p->state.perc_dam_red) / 100 ;
+	}
+	if (dam <= 0) return;
+
+	/* Disturb */
+	disturb(p);
+
 	/* Hurt the player */
 	p->chp -= dam;
 
+	/* Reward COMBAT_REGEN characters with mana for their lost hitpoints
+	 * Unenviable task of separating what should and should not cause rage
+	 * If we eliminate the most exploitable cases it should be fine.
+	 * All traps and lava currently give mana, which could be exploited  */
+	if (player_has(p, PF_COMBAT_REGEN)  && strcmp(kb_str, "poison")
+		&& strcmp(kb_str, "a fatal wound") && strcmp(kb_str, "starvation")) {
+		/* lose X% of hitpoints get X% of spell points */
+		s32b sp_gain = (MAX((s32b)p->msp, 10) << 16) / (s32b)p->mhp * dam;
+		player_adjust_mana_precise(p, sp_gain);
+	}
+
 	/* Display the hitpoints */
 	p->upkeep->redraw |= (PR_HP);
 
 	/* Dead player */
 	if (p->chp < 0) {
-		/* Allow cheating */
-		if ((p->wizard || OPT(p, cheat_live)) && !get_check("Die? ")) {
+		/* From hell's heart I stab at thee */
+		if (p->timed[TMD_BLOODLUST]
+			&& (p->chp + p->timed[TMD_BLOODLUST] + p->lev >= 0)) {
+			if (randint0(10)) {
+				msg("Your lust for blood keeps you alive!");
+			} else {
+				msg("So great was his prowess and skill in warfare, the Elves said: ");
+				msg("'The Mormegil cannot be slain, save by mischance.'");
+			}
+		} else if ((p->wizard || OPT(p, cheat_live)) && !get_check("Die? ")) {
 			event_signal(EVENT_CHEAT_DEATH);
 		} else {
 			/* Hack -- Note death */
@@ -289,23 +316,27 @@ s16b modify_stat_value(int value, int amount)
 }
 
 /**
- * Regenerate hit points
+ * Regenerate one turn's worth of hit points
  */
 void player_regen_hp(struct player *p)
 {
-	s32b new_chp, new_chp_frac;
-	int old_chp, percent = 0;
-
-	/* Save the old hitpoints */
-	old_chp = p->chp;
+	s32b hp_gain;
+	int percent = 0;/* max 32k -> 50% of mhp; more accurately "pertwobytes" */
+	int fed_pct, old_chp = p->chp;
 
 	/* Default regeneration */
-	if (p->food >= PY_FOOD_WEAK)
+	if (p->timed[TMD_FOOD] >= PY_FOOD_WEAK) {
 		percent = PY_REGEN_NORMAL;
-	else if (p->food >= PY_FOOD_FAINT)
+	} else if (p->timed[TMD_FOOD] >= PY_FOOD_FAINT) {
 		percent = PY_REGEN_WEAK;
-	else if (p->food >= PY_FOOD_STARVE)
+	} else if (p->timed[TMD_FOOD] >= PY_FOOD_STARVE) {
 		percent = PY_REGEN_FAINT;
+	}
+
+	/* Food bonus - better fed players regenerate up to 1/3 faster */
+	fed_pct = p->timed[TMD_FOOD] / z_info->food_value;
+	percent *= 100 + fed_pct / 3;
+	percent /= 100;
 
 	/* Various things speed up regeneration */
 	if (player_of_has(p, OF_REGEN))
@@ -314,7 +345,7 @@ void player_regen_hp(struct player *p)
 		percent *= 2;
 
 	/* Some things slow it down */
-	if (player_of_has(p, OF_IMPAIR_HP))
+	if (player_of_has(p, OF_IMPAIR_HP) || player_has(p, PF_COMBAT_REGEN))
 		percent /= 2;
 
 	/* Various things interfere with physical healing */
@@ -324,29 +355,11 @@ void player_regen_hp(struct player *p)
 	if (p->timed[TMD_CUT]) percent = 0;
 
 	/* Extract the new hitpoints */
-	new_chp = ((long)p->mhp) * percent + PY_REGEN_HPBASE;
-	p->chp += (s16b)(new_chp >> 16);   /* div 65536 */
-
-	/* Check for overflow */
-	if ((p->chp < 0) && (old_chp > 0))
-		p->chp = SHRT_MAX;
-	new_chp_frac = (new_chp & 0xFFFF) + p->chp_frac;	/* mod 65536 */
-	if (new_chp_frac >= 0x10000L) {
-		p->chp_frac = (u16b)(new_chp_frac - 0x10000L);
-		p->chp++;
-	} else {
-		p->chp_frac = (u16b)new_chp_frac;
-	}
-
-	/* Fully healed */
-	if (p->chp >= p->mhp) {
-		p->chp = p->mhp;
-		p->chp_frac = 0;
-	}
+	hp_gain = (s32b)(p->mhp * percent) + PY_REGEN_HPBASE;
+	player_adjust_hp_precise(p, hp_gain);
 
 	/* Notice changes */
 	if (old_chp != p->chp) {
-		p->upkeep->redraw |= (PR_HP);
 		equip_learn_flag(p, OF_REGEN);
 		equip_learn_flag(p, OF_IMPAIR_HP);
 	}
@@ -354,12 +367,12 @@ void player_regen_hp(struct player *p)
 
 
 /**
- * Regenerate mana points
+ * Regenerate one turn's worth of mana
  */
 void player_regen_mana(struct player *p)
 {
-	s32b new_mana, new_mana_frac;
-	int old_csp, percent;
+	s32b sp_gain;
+	int percent, old_csp = p->csp;
 
 	/* Save the old spell points */
 	old_csp = p->csp;
@@ -367,36 +380,30 @@ void player_regen_mana(struct player *p)
 	/* Default regeneration */
 	percent = PY_REGEN_NORMAL;
 
-	/* Various things speed up regeneration */
-	if (player_of_has(p, OF_REGEN))
-		percent *= 2;
-	if (player_resting_can_regenerate(p))
-		percent *= 2;
+	/* Various things speed up regeneration, but shouldn't punish healthy BGs */
+	if (!(player_has(p, PF_COMBAT_REGEN) && p->chp  > p->mhp / 2)) {
+		if (player_of_has(p, OF_REGEN))
+			percent *= 2;
+		if (player_resting_can_regenerate(p))
+			percent *= 2;
+	}
 
 	/* Some things slow it down */
-	if (player_of_has(p, OF_IMPAIR_MANA))
+	if (player_has(p, PF_COMBAT_REGEN)) {
+		percent /= -2;
+	} else if (player_of_has(p, OF_IMPAIR_MANA)) {
 		percent /= 2;
+	}
 
 	/* Regenerate mana */
-	new_mana = ((long)p->msp) * percent + PY_REGEN_MNBASE;
-	p->csp += (s16b)(new_mana >> 16);	/* div 65536 */
+	sp_gain = (s32b)(p->msp * percent);
+	if (percent >= 0)
+		sp_gain += PY_REGEN_MNBASE;
+	sp_gain = player_adjust_mana_precise(p, sp_gain);
 
-	/* check for overflow */
-	if ((p->csp < 0) && (old_csp > 0)) {
-		p->csp = SHRT_MAX;
-	}
-	new_mana_frac = (new_mana & 0xFFFF) + p->csp_frac;	/* mod 65536 */
-	if (new_mana_frac >= 0x10000L) {
-		p->csp_frac = (u16b)(new_mana_frac - 0x10000L);
-		p->csp++;
-	} else {
-		p->csp_frac = (u16b)new_mana_frac;
-	}
-
-	/* Must set frac to zero even if equal */
-	if (p->csp >= p->msp) {
-		p->csp = p->msp;
-		p->csp_frac = 0;
+	/* SP degen heals BGs at double efficiency vs casting */
+	if (sp_gain < 0  && player_has(p, PF_COMBAT_REGEN)) {
+		convert_mana_to_hp(p, -sp_gain << 2);
 	}
 
 	/* Notice changes */
@@ -407,6 +414,121 @@ void player_regen_mana(struct player *p)
 	}
 }
 
+void player_adjust_hp_precise(struct player *p, s32b hp_gain)
+{
+	s32b new_chp;
+	int num, old_chp = p->chp;
+
+	/* Load it all into 4 byte format*/
+	new_chp = (s32b)((p->chp << 16) + p->chp_frac) + hp_gain;
+
+	/* Check for overflow */
+	/*     {new_chp = LONG_MIN;} DAVIDTODO*/
+	if ((new_chp < 0) && (old_chp > 0) && (hp_gain > 0)) {
+		new_chp = INT32_MAX;
+	} else if ((new_chp > 0) && (old_chp < 0) && (hp_gain < 0)) {
+		new_chp = INT32_MIN;
+	}
+
+	/* Break it back down*/
+	p->chp = (s16b)(new_chp >> 16);   /* div 65536 */
+	p->chp_frac = (u16b)(new_chp & 0xFFFF); /* mod 65536 */
+	/*DAVIDTODO neg new_chp ok? I think so because eg a slightly negative
+	 * new_chp will give -1 for chp and very high chp_frac.*/
+
+	/* Fully healed */
+	if (p->chp >= p->mhp) {
+		p->chp = p->mhp;
+		p->chp_frac = 0;
+	}
+
+	num = p->chp - old_chp;
+	if (num == 0)
+		return;
+
+	p->upkeep->redraw |= (PR_HP);
+}
+
+
+/**
+ * Accept a 4 byte signed int, divide it by 65k, and add
+ * to current spell points. p->csp and csp_frac are 2 bytes each.
+ */
+s32b player_adjust_mana_precise(struct player *p, s32b sp_gain)
+{
+	s32b old_csp_long, new_csp_long;
+	int old_csp_short = p->csp;
+
+	if (sp_gain == 0) return 0;
+
+	/* Load it all into 4 byte format*/
+	old_csp_long = (s32b)((p->csp << 16) + p->csp_frac);
+	new_csp_long = old_csp_long + sp_gain;
+
+	/* Check for overflow */
+
+	/* new_csp = LONG_MAX LONG_MIN;} DAVIDTODO produces warning*/
+	if ((new_csp_long < 0) && (old_csp_long > 0) && (sp_gain > 0)) {
+		new_csp_long = INT32_MAX;
+		sp_gain = 0;
+	} else if ((new_csp_long > 0) && (old_csp_long < 0) && (sp_gain < 0)) {
+		new_csp_long = INT32_MIN;
+		sp_gain = 0;
+	}
+
+	/* Break it back down*/
+	p->csp = (s16b)(new_csp_long >> 16);   /* div 65536 */
+	p->csp_frac = (u16b)(new_csp_long & 0xFFFF);    /* mod 65536 */
+
+	/* Max/min SP */
+	if (p->csp >= p->msp) {
+		p->csp = p->msp;
+		p->csp_frac = 0;
+		sp_gain = 0;
+	} else if (p->csp < 0) {
+		p->csp = 0;
+		p->csp_frac = 0;
+		sp_gain = 0;
+	}
+
+	/* Notice changes */
+	if (old_csp_short != p->csp) {
+		p->upkeep->redraw |= (PR_MANA);
+	}
+
+	if (sp_gain == 0) {
+		/* Recalculate */
+		new_csp_long = (s32b)((p->csp << 16) + p->csp_frac);
+		sp_gain = new_csp_long - old_csp_long;
+	}
+
+	return sp_gain;
+}
+
+void convert_mana_to_hp(struct player *p, s32b sp_long) {
+	s32b hp_gain, sp_ratio;
+
+	if (sp_long <= 0 || p->msp == 0 || p->mhp == p->chp) return;
+
+	/* Total HP from max */
+	hp_gain = (s32b)((p->mhp - p->chp) << 16);
+	hp_gain -= (s32b)p->chp_frac;
+
+	/* Spend X% of SP get X/2% of lost HP. E.g., at 50% HP get X/4% */
+	/* Gain stays low at msp<10 because MP gains are generous at msp<10 */
+	/* sp_ratio is max sp to spent sp, doubled to suit target rate. */
+	sp_ratio = (MAX(10, (s32b)p->msp) << 16) * 2 / sp_long;
+
+	/* Limit max healing to 25% of damage; ergo spending > 50% msp
+	 * is inefficient */
+	if (sp_ratio < 4) {sp_ratio = 4;}
+	hp_gain /= sp_ratio;
+
+	/* DAVIDTODO Flavorful comments on large gains would be fun and informative */
+
+	player_adjust_hp_precise(p, hp_gain);
+}
+
 /**
  * Update the player's light fuel
  */
@@ -443,7 +565,7 @@ void player_update_light(struct player *p)
 				if (obj->timeout == 0) obj->timeout++;
 			} else if (obj->timeout == 0) {
 				/* The light is now out */
-				disturb(p, 0);
+				disturb(p);
 				msg("Your light has gone out!");
 
 				/* If it's a torch, now is the time to delete it */
@@ -457,7 +579,7 @@ void player_update_light(struct player *p)
 				}
 			} else if ((obj->timeout < 50) && (!(obj->timeout % 20))) {
 				/* The light is getting dim */
-				disturb(p, 0);
+				disturb(p);
 				msg("Your light is growing faint.");
 			}
 		}
@@ -467,14 +589,151 @@ void player_update_light(struct player *p)
 	p->upkeep->update |= (PU_TORCH);
 }
 
+/**
+ * Find the player's best digging tool.  If forbid_stack is true, ignores
+ * stacks of more than one item.
+ */
+struct object *player_best_digger(struct player *p, bool forbid_stack)
+{
+	struct object *obj, *best = NULL;
+	int best_score = 0;
+
+	for (obj = p->gear; obj; obj = obj->next) {
+		int score = 0;
+		if (!tval_is_melee_weapon(obj)) continue;
+		if (obj->number < 1 || (forbid_stack && obj->number > 1)) continue;
+		if (tval_is_digger(obj)) {
+			if (of_has(obj->flags, OF_DIG_1))
+				score = 1;
+			else if (of_has(obj->flags, OF_DIG_2))
+				score = 2;
+			else if (of_has(obj->flags, OF_DIG_3))
+				score = 3;
+		}
+		score += obj->modifiers[OBJ_MOD_TUNNEL]
+			* p->obj_k->modifiers[OBJ_MOD_TUNNEL];
+		if (score > best_score) {
+			best = obj;
+			best_score = score;
+		}
+	}
+
+	return best;
+}
+
+/**
+ * Melee a random adjacent monster
+ */
+bool player_attack_random_monster(struct player *p)
+{
+	int i, dir = randint0(8);
+
+	/* Confused players get a free pass */
+	if (p->timed[TMD_CONFUSED]) return false;
+
+	/* Look for a monster, attack */
+	for (i = 0; i < 8; i++, dir++) {
+		struct loc grid = loc_sum(player->grid, ddgrid_ddd[dir % 8]);
+		if (square_monster(cave, grid)) {
+			p->upkeep->energy_use = z_info->move_energy;
+			msg("You angrily lash out at a nearby foe!");
+			move_player(ddd[dir % 8], false);
+			return true;
+		}
+	}
+	return false;
+}
+
+/**
+ * Have random bad stuff happen to the player from over-exertion
+ *
+ * This function uses the PY_EXERT_* flags
+ */
+void player_over_exert(struct player *p, int flag, int chance, int amount)
+{
+	if (chance <= 0) return;
+
+	/* CON damage */
+	if (flag & PY_EXERT_CON) {
+		if (randint0(100) < chance) {
+			/* Hack - only permanent with high chance (no-mana casting) */
+			bool perm = (randint0(100) < chance / 2) && (chance >= 50);
+			msg("You have damaged your health!");
+			player_stat_dec(p, STAT_CON, perm);
+		}
+	}
+
+	/* Fainting */
+	if (flag & PY_EXERT_FAINT) {
+		if (randint0(100) < chance) {
+			msg("You faint from the effort!");
+
+			/* Bypass free action */
+			(void)player_inc_timed(p, TMD_PARALYZED, randint1(amount),
+								   true, false);
+		}
+	}
+
+	/* Scrambled stats */
+	if (flag & PY_EXERT_SCRAMBLE) {
+		if (randint0(100) < chance) {
+			(void)player_inc_timed(p, TMD_SCRAMBLE, randint1(amount),
+								   true, true);
+		}
+	}
+
+	/* Cut damage */
+	if (flag & PY_EXERT_CUT) {
+		if (randint0(100) < chance) {
+			msg("Wounds appear on your body!");
+			(void)player_inc_timed(p, TMD_CUT, randint1(amount),
+								   true, false);
+		}
+	}
+
+	/* Confusion */
+	if (flag & PY_EXERT_CONF) {
+		if (randint0(100) < chance) {
+			(void)player_inc_timed(p, TMD_CONFUSED, randint1(amount),
+								   true, true);
+		}
+	}
+
+	/* Hallucination */
+	if (flag & PY_EXERT_HALLU) {
+		if (randint0(100) < chance) {
+			(void)player_inc_timed(p, TMD_IMAGE, randint1(amount),
+								   true, true);
+		}
+	}
+
+	/* Slowing */
+	if (flag & PY_EXERT_SLOW) {
+		if (randint0(100) < chance) {
+			msg("You feel suddenly lethargic.");
+			(void)player_inc_timed(p, TMD_SLOW, randint1(amount),
+								   true, false);
+		}
+	}
+
+	/* HP */
+	if (flag & PY_EXERT_HP) {
+		if (randint0(100) < chance) {
+			msg("You cry out in sudden pain!");
+			take_hit(p, randint1(amount), "over-exertion");
+		}
+	}
+}
+
+
 /**
  * See how much damage the player will take from damaging terrain
  */
-int player_check_terrain_damage(struct player *p, int y, int x)
+int player_check_terrain_damage(struct player *p, struct loc grid)
 {
 	int dam_taken = 0;
 
-	if (square_isfiery(cave, y, x)) {
+	if (square_isfiery(cave, grid)) {
 		int base_dam = 100 + randint1(100);
 		int res = p->state.el_info[ELEM_FIRE].res_level;
 
@@ -493,22 +752,102 @@ int player_check_terrain_damage(struct player *p, int y, int x)
 /**
  * Terrain damages the player
  */
-void player_take_terrain_damage(struct player *p, int y, int x)
+void player_take_terrain_damage(struct player *p, struct loc grid)
 {
-	int dam_taken = player_check_terrain_damage(p, y, x);
+	int dam_taken = player_check_terrain_damage(p, grid);
 
 	if (!dam_taken) {
 		return;
 	}
 
 	/* Damage the player and inventory */
-	take_hit(player, dam_taken, square_feat(cave, y, x)->die_msg);
-	if (square_isfiery(cave, y, x)) {
-		msg(square_feat(cave, y, x)->hurt_msg);
+	take_hit(player, dam_taken, square_feat(cave, grid)->die_msg);
+	if (square_isfiery(cave, grid)) {
+		msg(square_feat(cave, grid)->hurt_msg);
 		inven_damage(player, PROJ_FIRE, dam_taken);
 	}
 }
 
+/**
+ * Find a player shape from the name
+ */
+struct player_shape *lookup_player_shape(const char *name)
+{
+	struct player_shape *shape = shapes;
+	while (shape) {
+		if (streq(shape->name, name)) {
+			return shape;
+		}
+		shape = shape->next;
+	}
+	msg("Could not find %s shape!", name);
+	return NULL;
+}
+
+/**
+ * Find a player shape index from the shape name
+ */
+int shape_name_to_idx(const char *name)
+{
+	struct player_shape *shape = lookup_player_shape(name);
+	if (shape) {
+		return shape->sidx;
+	} else {
+		return -1;
+	}
+}
+
+/**
+ * Find a player shape from the index
+ */
+struct player_shape *player_shape_by_idx(int index)
+{
+	struct player_shape *shape = shapes;
+	while (shape) {
+		if (shape->sidx == index) {
+			return shape;
+		}
+		shape = shape->next;
+	}
+	msg("Could not find shape %d!", index);
+	return NULL;
+}
+
+/**
+ * Revert to normal shape
+ */
+void player_resume_normal_shape(struct player *p)
+{
+	p->shape = lookup_player_shape("normal");
+	msg("You resume your usual shape.");
+
+	/* Kill vampire attack */
+	(void) player_clear_timed(p, TMD_ATT_VAMP, true);
+
+	/* Update */
+	player->upkeep->update |= (PU_BONUS);
+	player->upkeep->redraw |= (PR_TITLE | PR_MISC);
+	handle_stuff(player);
+}
+
+/**
+ * Check if the player is shapechanged
+ */
+bool player_is_shapechanged(struct player *p)
+{
+	return streq(p->shape->name, "normal") ? false : true;
+}
+
+/**
+ * Check if the player is immune from traps
+ */
+bool player_is_trapsafe(struct player *p)
+{
+	if (p->timed[TMD_TRAPSAFE]) return true;
+	if (player_of_has(p, OF_TRAP_IMMUNE)) return true;
+	return false;
+}
+
 /**
  * Return true if the player can cast a spell.
  *
@@ -518,7 +857,7 @@ void player_take_terrain_damage(struct player *p, int y, int x)
  */
 bool player_can_cast(struct player *p, bool show_msg)
 {
-	if (!p->class->magic.spell_realm) {
+	if (!p->class->magic.total_spells) {
 		if (show_msg) {
 			msg("You cannot pray or produce magics.");
 		}
@@ -556,8 +895,31 @@ bool player_can_study(struct player *p, bool show_msg)
 
 	if (!p->upkeep->new_spells) {
 		if (show_msg) {
-			const char *name = p->class->magic.spell_realm->spell_noun;
-			msg("You cannot learn any new %ss!", name);
+			int count;
+			struct magic_realm *r = class_magic_realms(p->class, &count), *r1;
+			char buf[120];
+
+			my_strcpy(buf, r->spell_noun, sizeof(buf));
+			my_strcat(buf, "s", sizeof(buf));
+			r1 = r->next;
+			mem_free(r);
+			r = r1;
+			if (count > 1) {
+				while (r) {
+					count--;
+					if (count) {
+						my_strcat(buf, ", ", sizeof(buf));
+					} else {
+						my_strcat(buf, " or ", sizeof(buf));
+					}
+					my_strcat(buf, r->spell_noun, sizeof(buf));
+					my_strcat(buf, "s", sizeof(buf));
+					r1 = r->next;
+					mem_free(r);
+					r = r1;
+				}
+			}
+			msg("You cannot learn any new %s!", buf);
 		}
 		return false;
 	}
@@ -710,7 +1072,7 @@ bool player_book_has_unlearned_spells(struct player *p)
 	item_num = scan_items(item_list, item_max, USE_INVEN | USE_FLOOR,
 						  obj_can_study);
 	for (i = 0; i < item_num; i++) {
-		const struct class_book *book = object_to_book(item_list[i]);
+		const struct class_book *book = player_object_to_book(p, item_list[i]);
 		if (!book) continue;
 
 		/* Extract spells */
@@ -735,20 +1097,24 @@ bool player_confuse_dir(struct player *p, int *dp, bool too)
 {
 	int dir = *dp;
 
-	if (p->timed[TMD_CONFUSED])
-		if ((dir == 5) || (randint0(100) < 75))
+	if (p->timed[TMD_CONFUSED]) {
+		if ((dir == 5) || (randint0(100) < 75)) {
 			/* Random direction */
 			dir = ddd[randint0(8)];
+		}
+
+	/* Running attempts always fail */
+	if (too) {
+		msg("You are too confused.");
+		return true;
+	}
 
 	if (*dp != dir) {
-		if (too)
-			msg("You are too confused.");
-		else
-			msg("You are confused.");
-
+		msg("You are confused.");
 		*dp = dir;
 		return true;
 	}
+	}
 
 	return false;
 }
@@ -871,26 +1237,27 @@ void player_resting_step_turn(struct player *p)
 void player_resting_complete_special(struct player *p)
 {
 	/* Complete resting */
-	if (player_resting_is_special(p->upkeep->resting)) {
-		if (p->upkeep->resting == REST_ALL_POINTS) {
-			if ((p->chp == p->mhp) && (p->csp == p->msp))
-				/* Stop resting */
-				disturb(p, 0);
-		} else if (p->upkeep->resting == REST_COMPLETE) {
-			if ((p->chp == p->mhp) && (p->csp == p->msp) &&
-				!p->timed[TMD_BLIND] && !p->timed[TMD_CONFUSED] &&
-				!p->timed[TMD_POISONED] && !p->timed[TMD_AFRAID] &&
-				!p->timed[TMD_TERROR] && !p->timed[TMD_STUN] &&
-				!p->timed[TMD_CUT] && !p->timed[TMD_SLOW] &&
-				!p->timed[TMD_PARALYZED] && !p->timed[TMD_IMAGE] &&
-				!p->word_recall && !p->deep_descent)
-				/* Stop resting */
-				disturb(p, 0);
-		} else if (p->upkeep->resting == REST_SOME_POINTS) {
-			if ((p->chp == p->mhp) || (p->csp == p->msp))
-				/* Stop resting */
-				disturb(p, 0);
-		}
+	if (!player_resting_is_special(p->upkeep->resting)) return;
+
+	if (p->upkeep->resting == REST_ALL_POINTS) {
+		if ((p->chp == p->mhp) && (p->csp == p->msp))
+			/* Stop resting */
+			disturb(p);
+	} else if (p->upkeep->resting == REST_COMPLETE) {
+		if ((p->chp == p->mhp) &&
+			(p->csp == p->msp || player_has(p, PF_COMBAT_REGEN)) &&
+			!p->timed[TMD_BLIND] && !p->timed[TMD_CONFUSED] &&
+			!p->timed[TMD_POISONED] && !p->timed[TMD_AFRAID] &&
+			!p->timed[TMD_TERROR] && !p->timed[TMD_STUN] &&
+			!p->timed[TMD_CUT] && !p->timed[TMD_SLOW] &&
+			!p->timed[TMD_PARALYZED] && !p->timed[TMD_IMAGE] &&
+			!p->word_recall && !p->deep_descent)
+			/* Stop resting */
+			disturb(p);
+	} else if (p->upkeep->resting == REST_SOME_POINTS) {
+		if ((p->chp == p->mhp) || (p->csp == p->msp))
+			/* Stop resting */
+			disturb(p);
 	}
 }
 
@@ -942,36 +1309,24 @@ bool player_is_immune(struct player *p, int element)
 	return (p->state.el_info[element].res_level == 3);
 }
 
-/*
- * Extract a "direction" which will move one step from the player location
- * towards the given "target" location (or "5" if no motion necessary).
- */
-int coords_to_dir(struct player *p, int y, int x)
-{
-	return (motion_dir(p->py, p->px, y, x));
-}
-
 /**
  * Places the player at the given coordinates in the cave.
  */
-void player_place(struct chunk *c, struct player *p, int y, int x)
+void player_place(struct chunk *c, struct player *p, struct loc grid)
 {
-	assert(!c->squares[y][x].mon);
+	assert(!square_monster(c, grid));
 
 	/* Save player location */
-	p->py = y;
-	p->px = x;
+	p->grid = grid;
 
 	/* Mark cave grid */
-	c->squares[y][x].mon = -1;
+	square_set_mon(c, grid, -1);
 
 	/* Clear stair creation */
 	p->upkeep->create_down_stair = false;
 	p->upkeep->create_up_stair = false;
 }
 
-
-
 /*
  * Something has happened to disturb the player.
  *
@@ -980,11 +1335,11 @@ void player_place(struct chunk *c, struct player *p, int y, int x)
  * The second arg is currently unused, but could induce output flush.
  *
  * All disturbance cancels repeated commands, resting, and running.
- * 
+ *
  * XXX-AS: Make callers either pass in a command
  * or call cmd_cancel_repeat inside the function calling this
  */
-void disturb(struct player *p, int stop_search)
+void disturb(struct player *p)
 {
 	/* Cancel repeated commands */
 	cmd_cancel_repeat();
@@ -999,6 +1354,9 @@ void disturb(struct player *p, int stop_search)
 	if (p->upkeep->running) {
 		p->upkeep->running = 0;
 
+		/* Cancel queued commands */
+		cmdq_flush();
+
 		/* Check for new panel if appropriate */
 		event_signal(EVENT_PLAYERMOVED);
 		p->upkeep->update |= PU_TORCH;
@@ -1016,7 +1374,7 @@ void disturb(struct player *p, int stop_search)
  */
 void search(struct player *p)
 {
-	int y, x;
+	struct loc grid;
 
 	/* Various conditions mean no searching */
 	if (p->timed[TMD_BLIND] || no_light() ||
@@ -1024,26 +1382,26 @@ void search(struct player *p)
 		return;
 
 	/* Search the nearby grids, which are always in bounds */
-	for (y = (p->py - 1); y <= (p->py + 1); y++) {
-		for (x = (p->px - 1); x <= (p->px + 1); x++) {
+	for (grid.y = (p->grid.y - 1); grid.y <= (p->grid.y + 1); grid.y++) {
+		for (grid.x = (p->grid.x - 1); grid.x <= (p->grid.x + 1); grid.x++) {
 			struct object *obj;
 
 			/* Secret doors */
-			if (square_issecretdoor(cave, y, x)) {
+			if (square_issecretdoor(cave, grid)) {
 				msg("You have found a secret door.");
-				place_closed_door(cave, y, x);
-				disturb(p, 0);
+				place_closed_door(cave, grid);
+				disturb(p);
 			}
 
 			/* Traps on chests */
-			for (obj = square_object(cave, y, x); obj; obj = obj->next) {
+			for (obj = square_object(cave, grid); obj; obj = obj->next) {
 				if (!obj->known || !is_trapped_chest(obj))
 					continue;
 
 				if (obj->known->pval != obj->pval) {
 					msg("You have discovered a trap on the chest!");
 					obj->known->pval = obj->pval;
-					disturb(p, 0);
+					disturb(p);
 				}
 			}
 		}
