@@ -29,6 +29,19 @@
 #define PY_REGEN_HPBASE		1442	/* Min amount hp regen*2^16 */
 #define PY_REGEN_MNBASE		524		/* Min amount mana regen*2^16 */
 
+/* Player over-exertion */
+enum {
+	PY_EXERT_NONE = 0x00,
+	PY_EXERT_CON = 0x01,
+	PY_EXERT_FAINT = 0x02,
+	PY_EXERT_SCRAMBLE = 0x04,
+	PY_EXERT_CUT = 0x08,
+	PY_EXERT_CONF = 0x10,
+	PY_EXERT_HALLU = 0x20,
+	PY_EXERT_SLOW = 0x40,
+	PY_EXERT_HP = 0x80
+};
+
 /**
  * Special values for the number of turns to rest, these need to be
  * negative numbers, as postive numbers are taken to be a turncount,
@@ -55,9 +68,21 @@ void death_knowledge(struct player *p);
 s16b modify_stat_value(int value, int amount);
 void player_regen_hp(struct player *p);
 void player_regen_mana(struct player *p);
+void player_adjust_hp_precise(struct player *p, s32b hp_gain);
+s32b player_adjust_mana_precise(struct player *p, s32b sp_gain);
+void convert_mana_to_hp(struct player *p, s32b sp);
 void player_update_light(struct player *p);
-int player_check_terrain_damage(struct player *p, int y, int x);
-void player_take_terrain_damage(struct player *p, int y, int x);
+void player_over_exert(struct player *p, int flag, int chance, int amount);
+struct object *player_best_digger(struct player *p, bool forbid_stack);
+bool player_attack_random_monster(struct player *p);
+int player_check_terrain_damage(struct player *p, struct loc grid);
+void player_take_terrain_damage(struct player *p, struct loc grid);
+struct player_shape *lookup_player_shape(const char *name);
+int shape_name_to_idx(const char *name);
+struct player_shape *player_shape_by_idx(int index);
+void player_resume_normal_shape(struct player *p);
+bool player_is_shapechanged(struct player *p);
+bool player_is_trapsafe(struct player *p);
 bool player_can_cast(struct player *p, bool show_msg);
 bool player_can_study(struct player *p, bool show_msg);
 bool player_can_read(struct player *p, bool show_msg);
@@ -83,9 +108,8 @@ void player_set_resting_repeat_count(struct player *p, s16b count);
 bool player_of_has(struct player *p, int flag);
 bool player_resists(struct player *p, int element);
 bool player_is_immune(struct player *p, int element);
-int coords_to_dir(struct player *p, int y, int x);
-void player_place(struct chunk *c, struct player *p, int y, int x);
-void disturb(struct player *p, int stop_search);
+void player_place(struct chunk *c, struct player *p, struct loc grid);
+void disturb(struct player *p);
 void search(struct player *p);
 
 #endif /* !PLAYER_UTIL_H */
