@@ -36,8 +36,7 @@
 typedef struct project_feature_handler_context_s {
 	const struct source origin;
 	const int r;
-	const int y;
-	const int x;
+	const struct loc grid;
 	const int dam;
 	const int type;
 	bool obvious;
@@ -47,14 +46,13 @@ typedef void (*project_feature_handler_f)(project_feature_handler_context_t *);
 /* Light up the grid */
 static void project_feature_handler_LIGHT_WEAK(project_feature_handler_context_t *context)
 {
-	const int x = context->x;
-	const int y = context->y;
+	const struct loc grid = context->grid;
 
 	/* Turn on the light */
-	sqinfo_on(cave->squares[y][x].info, SQUARE_GLOW);
+	sqinfo_on(square(cave, grid).info, SQUARE_GLOW);
 
 	/* Grid is in line of sight */
-	if (square_isview(cave, y, x)) {
+	if (square_isview(cave, grid)) {
 		if (!player->timed[TMD_BLIND]) {
 			/* Observe */
 			context->obvious = true;
@@ -68,16 +66,15 @@ static void project_feature_handler_LIGHT_WEAK(project_feature_handler_context_t
 /* Darken the grid */
 static void project_feature_handler_DARK_WEAK(project_feature_handler_context_t *context)
 {
-	const int x = context->x;
-	const int y = context->y;
+	const struct loc grid = context->grid;
 
-	if ((player->depth != 0 || !is_daytime()) && !square_isbright(cave, y, x)) {
+	if ((player->depth != 0 || !is_daytime()) && !square_isbright(cave, grid)) {
 		/* Turn off the light */
-		sqinfo_off(cave->squares[y][x].info, SQUARE_GLOW);
+		sqinfo_off(square(cave, grid).info, SQUARE_GLOW);
 	}
 
 	/* Grid is in line of sight */
-	if (square_isview(cave, y, x)) {
+	if (square_isview(cave, grid)) {
 		/* Observe */
 		context->obvious = true;
 
@@ -89,91 +86,90 @@ static void project_feature_handler_DARK_WEAK(project_feature_handler_context_t
 /* Destroy walls (and doors) */
 static void project_feature_handler_KILL_WALL(project_feature_handler_context_t *context)
 {
-	const int x = context->x;
-	const int y = context->y;
+	const struct loc grid = context->grid;
 
 	/* Non-walls (etc) */
-	if (square_ispassable(cave, y, x) && !square_seemslikewall(cave, y, x))
+	if (square_ispassable(cave, grid) && !square_seemslikewall(cave, grid))
 		return;
 
 	/* Permanent walls */
-	if (square_isperm(cave, y, x)) return;
+	if (square_isperm(cave, grid)) return;
 
 	/* Different treatment for different walls */
-	if (square_isrubble(cave, y, x)) {
+	if (square_isrubble(cave, grid)) {
 		/* Message */
-		if (square_isseen(cave, y, x)) {
+		if (square_isseen(cave, grid)) {
 			msg("The rubble turns into mud!");
 			context->obvious = true;
 
 			/* Forget the wall */
-			square_forget(cave, y, x);
+			square_forget(cave, grid);
 		}
 
 		/* Destroy the rubble */
-		square_destroy_rubble(cave, y, x);
+		square_destroy_rubble(cave, grid);
 
 		/* Hack -- place an object */
 		if (randint0(100) < 10){
-			if (square_isseen(cave, y, x)) {
+			if (square_isseen(cave, grid)) {
 				msg("There was something buried in the rubble!");
 				context->obvious = true;
 			}
-			place_object(cave, y, x, player->depth, false, false,
+			place_object(cave, grid, player->depth, false, false,
 						 ORIGIN_RUBBLE, 0);
 		}
-	} else if (square_isdoor(cave, y, x)) {
+	} else if (square_isdoor(cave, grid)) {
 		/* Hack -- special message */
-		if (square_isseen(cave, y, x)) {
+		if (square_isseen(cave, grid)) {
 			msg("The door turns into mud!");
 			context->obvious = true;
 
 			/* Forget the wall */
-			square_forget(cave, y, x);
+			square_forget(cave, grid);
 		}
 
 		/* Destroy the feature */
-		square_destroy_door(cave, y, x);
-	} else if (square_hasgoldvein(cave, y, x)) {
+		square_destroy_door(cave, grid);
+	} else if (square_hasgoldvein(cave, grid)) {
 		/* Message */
-		if (square_isseen(cave, y, x)) {
+		if (square_isseen(cave, grid)) {
 			msg("The vein turns into mud!");
 			msg("You have found something!");
 			context->obvious = true;
 
 			/* Forget the wall */
-			square_forget(cave, y, x);
+			square_forget(cave, grid);
 		}
 
 		/* Destroy the wall */
-		square_destroy_wall(cave, y, x);
+		square_destroy_wall(cave, grid);
 
 		/* Place some gold */
-		place_gold(cave, y, x, player->depth, ORIGIN_FLOOR);
-	} else if (square_ismagma(cave, y, x) || square_isquartz(cave, y, x)) {
+		place_gold(cave, grid, player->depth, ORIGIN_FLOOR);
+	} else if (square_ismagma(cave, grid) || square_isquartz(cave, grid)) {
 		/* Message */
-		if (square_isseen(cave, y, x)) {
+		if (square_isseen(cave, grid)) {
 			msg("The vein turns into mud!");
 			context->obvious = true;
 
 			/* Forget the wall */
-			square_forget(cave, y, x);
+			square_forget(cave, grid);
 		}
 
 		/* Destroy the wall */
-		square_destroy_wall(cave, y, x);
-	} else if (square_iswall(cave, y, x)) {
+		square_destroy_wall(cave, grid);
+	} else if (square_iswall(cave, grid)) {
 		/* Message */
-		if (square_isseen(cave, y, x)) {
+		if (square_isseen(cave, grid)) {
 			msg("The wall turns into mud!");
 			context->obvious = true;
 
 			/* Forget the wall */
-			square_forget(cave, y, x);
+			square_forget(cave, grid);
 		}
 
 		/* Destroy the wall */
-		square_destroy_wall(cave, y, x);
+		square_destroy_wall(cave, grid);
 	}
 
 	/* Update the visuals */
@@ -183,13 +179,12 @@ static void project_feature_handler_KILL_WALL(project_feature_handler_context_t
 /* Destroy Doors */
 static void project_feature_handler_KILL_DOOR(project_feature_handler_context_t *context)
 {
-	const int x = context->x;
-	const int y = context->y;
+	const struct loc grid = context->grid;
 
 	/* Destroy all doors */
-	if (square_isdoor(cave, y, x)) {
+	if (square_isdoor(cave, grid)) {
 		/* Check line of sight */
-		if (square_isview(cave, y, x)) {
+		if (square_isview(cave, grid)) {
 			/* Message */
 			msg("There is a bright flash of light!");
 			context->obvious = true;
@@ -198,45 +193,47 @@ static void project_feature_handler_KILL_DOOR(project_feature_handler_context_t
 			player->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
 
 			/* Forget the door */
-			square_forget(cave, y, x);
+			square_forget(cave, grid);
 		}
 
 		/* Destroy the feature */
-		square_destroy_door(cave, y, x);
+		square_destroy_door(cave, grid);
 	}
 }
 
 /* Disable traps, unlock doors */
 static void project_feature_handler_KILL_TRAP(project_feature_handler_context_t *context)
 {
-	const int x = context->x;
-	const int y = context->y;
+	const struct loc grid = context->grid;
 
 	/* Reveal secret doors */
-	if (square_issecretdoor(cave, y, x)) {
-		place_closed_door(cave, y, x);
+	if (square_issecretdoor(cave, grid)) {
+		place_closed_door(cave, grid);
 
 		/* Check line of sight */
-		if (square_isseen(cave, y, x))
+		if (square_isseen(cave, grid))
 			context->obvious = true;
 	}
 
 	/* Disable traps, unlock doors */
-	if (square_istrap(cave, y, x)) {
+	if (square_isdisarmabletrap(cave, grid)) {
 		/* Check line of sight */
-		if (square_isview(cave, y, x)) {
+		if (square_isview(cave, grid)) {
 			msg("The trap seizes up.");
 			context->obvious = true;
 		}
 
 		/* Disable the trap */
-		square_disable_trap(cave, y, x);
-	} else if (square_islockeddoor(cave, y, x)) {
+		square_disable_trap(cave, grid);
+
+		/* Visibility change */
+		player->upkeep->update |= (PU_UPDATE_VIEW);
+	} else if (square_islockeddoor(cave, grid)) {
 		/* Unlock the door */
-		square_unlock_door(cave, y, x);
+		square_unlock_door(cave, grid);
 
 		/* Check line of sound */
-		if (square_isview(cave, y, x)) {
+		if (square_isview(cave, grid)) {
 			msg("Click!");
 			context->obvious = true;
 		}
@@ -246,24 +243,23 @@ static void project_feature_handler_KILL_TRAP(project_feature_handler_context_t
 /* Make doors */
 static void project_feature_handler_MAKE_DOOR(project_feature_handler_context_t *context)
 {
-	const int x = context->x;
-	const int y = context->y;
+	const struct loc grid = context->grid;
 
 	/* Require a grid without monsters */
-	if (square_monster(cave, y, x) || square_isplayer(cave, y, x)) return;
+	if (square_monster(cave, grid) || square_isplayer(cave, grid)) return;
 
 	/* Require a floor grid */
-	if (!square_isfloor(cave, y, x)) return;
+	if (!square_isfloor(cave, grid)) return;
 
 	/* Push objects off the grid */
-	if (square_object(cave, y, x))
-		push_object(y,x);
+	if (square_object(cave, grid))
+		push_object(grid);
 
 	/* Create closed door */
-	square_add_door(cave, y, x, true);
+	square_add_door(cave, grid, true);
 
 	/* Observe */
-	if (square_isknown(cave, y, x))
+	if (square_isknown(cave, grid))
 		context->obvious = true;
 
 	/* Update the visuals */
@@ -273,17 +269,16 @@ static void project_feature_handler_MAKE_DOOR(project_feature_handler_context_t
 /* Make traps */
 static void project_feature_handler_MAKE_TRAP(project_feature_handler_context_t *context)
 {
-	const int x = context->x;
-	const int y = context->y;
+	const struct loc grid = context->grid;
 
-	/* Require an "empty", non-warded floor grid */
-	if (!square_isempty(cave, y, x)) return;
-	if (square_iswarded(cave, y, x)) return;
+	/* Require an "empty" floor grid with no existing traps or glyphs */
+	if (!square_isempty(cave, grid)) return;
+	if (square_istrap(cave, grid)) return;
 
 	/* Create a trap, try to notice it */
 	if (one_in_(4)) {
-		square_add_trap(cave, y, x);
-		(void) square_reveal_trap(cave, y, x, false, false);
+		square_add_trap(cave, grid);
+		(void) square_reveal_trap(cave, grid, false, false);
 	}
 	context->obvious = true;
 }
@@ -291,8 +286,7 @@ static void project_feature_handler_MAKE_TRAP(project_feature_handler_context_t
 static void project_feature_handler_ACID(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -301,8 +295,7 @@ static void project_feature_handler_ACID(project_feature_handler_context_t *cont
 static void project_feature_handler_ELEC(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -311,45 +304,48 @@ static void project_feature_handler_ELEC(project_feature_handler_context_t *cont
 static void project_feature_handler_FIRE(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
 
+	/* Removes webs */
+	if (square_iswebbed(cave, context->grid)) {
+		square_destroy_trap(cave, context->grid);
+	}
+
 	/* Can create lava if extremely powerful. */
 	if ((context->dam > randint1(1800) + 600) &&
-		square_isfloor(cave, context->y, context->x)) {
+		square_isfloor(cave, context->grid)) {
 		/* Forget the floor, make lava. */
-		square_unmark(cave, context->y, context->x);
-		square_set_feat(cave, context->y, context->x, FEAT_LAVA);
+		square_unmark(cave, context->grid);
+		square_set_feat(cave, context->grid, FEAT_LAVA);
 
 		/* Objects that have survived should move */
-		push_object(context->y, context->x);
+		push_object(context->grid);
 	}
 }
 
 static void project_feature_handler_COLD(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
 
 	/* Sufficiently intense cold can solidify lava. */
 	if ((context->dam > randint1(900) + 300) &&
-		square_isfiery(cave, context->y, context->x)) {
-		bool occupied = square_isoccupied(cave, context->y, context->x);
+		square_isfiery(cave, context->grid)) {
+		bool occupied = square_isoccupied(cave, context->grid);
 
-		square_unmark(cave, context->y, context->x);
+		square_unmark(cave, context->grid);
 		if (one_in_(2)) {
-			square_set_feat(cave, context->y, context->x, FEAT_FLOOR);
+			square_set_feat(cave, context->grid, FEAT_FLOOR);
 		} else if (one_in_(2) && !occupied) {
-			square_set_feat(cave, context->y, context->x, FEAT_RUBBLE);
+			square_set_feat(cave, context->grid, FEAT_RUBBLE);
 		} else {
-			square_set_feat(cave, context->y, context->x, FEAT_PASS_RUBBLE);
+			square_set_feat(cave, context->grid, FEAT_PASS_RUBBLE);
 		}
 	}
 }
@@ -357,8 +353,7 @@ static void project_feature_handler_COLD(project_feature_handler_context_t *cont
 static void project_feature_handler_POIS(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -379,8 +374,7 @@ static void project_feature_handler_DARK(project_feature_handler_context_t *cont
 static void project_feature_handler_SOUND(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -389,8 +383,7 @@ static void project_feature_handler_SOUND(project_feature_handler_context_t *con
 static void project_feature_handler_SHARD(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -399,8 +392,7 @@ static void project_feature_handler_SHARD(project_feature_handler_context_t *con
 static void project_feature_handler_NEXUS(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -409,8 +401,7 @@ static void project_feature_handler_NEXUS(project_feature_handler_context_t *con
 static void project_feature_handler_NETHER(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -419,8 +410,7 @@ static void project_feature_handler_NETHER(project_feature_handler_context_t *co
 static void project_feature_handler_CHAOS(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -429,8 +419,7 @@ static void project_feature_handler_CHAOS(project_feature_handler_context_t *con
 static void project_feature_handler_DISEN(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -439,8 +428,7 @@ static void project_feature_handler_DISEN(project_feature_handler_context_t *con
 static void project_feature_handler_WATER(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -449,24 +437,23 @@ static void project_feature_handler_WATER(project_feature_handler_context_t *con
 static void project_feature_handler_ICE(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
 
 	/* Sufficiently intense cold can solidify lava. */
 	if ((context->dam > randint1(900) + 300) &&
-		square_isfiery(cave, context->y, context->x)) {
-		bool occupied = square_isoccupied(cave, context->y, context->x);
+		square_isfiery(cave, context->grid)) {
+		bool occupied = square_isoccupied(cave, context->grid);
 
-		square_unmark(cave, context->y, context->x);
+		square_unmark(cave, context->grid);
 		if (one_in_(2)) {
-			square_set_feat(cave, context->y, context->x, FEAT_FLOOR);
+			square_set_feat(cave, context->grid, FEAT_FLOOR);
 		} else if (one_in_(2) && !occupied) {
-			square_set_feat(cave, context->y, context->x, FEAT_RUBBLE);
+			square_set_feat(cave, context->grid, FEAT_RUBBLE);
 		} else {
-			square_set_feat(cave, context->y, context->x, FEAT_PASS_RUBBLE);
+			square_set_feat(cave, context->grid, FEAT_PASS_RUBBLE);
 		}
 	}
 }
@@ -474,8 +461,7 @@ static void project_feature_handler_ICE(project_feature_handler_context_t *conte
 static void project_feature_handler_GRAVITY(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -484,8 +470,7 @@ static void project_feature_handler_GRAVITY(project_feature_handler_context_t *c
 static void project_feature_handler_INERTIA(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -494,8 +479,7 @@ static void project_feature_handler_INERTIA(project_feature_handler_context_t *c
 static void project_feature_handler_FORCE(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -504,8 +488,7 @@ static void project_feature_handler_FORCE(project_feature_handler_context_t *con
 static void project_feature_handler_TIME(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -514,29 +497,27 @@ static void project_feature_handler_TIME(project_feature_handler_context_t *cont
 static void project_feature_handler_PLASMA(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
 
 	/* Can create lava if extremely powerful. */
 	if ((context->dam > randint1(1800) + 600) &&
-		square_isfloor(cave, context->y, context->x)) {
+		square_isfloor(cave, context->grid)) {
 		/* Forget the floor, make lava. */
-		square_unmark(cave, context->y, context->x);
-		square_set_feat(cave, context->y, context->x, FEAT_LAVA);
+		square_unmark(cave, context->grid);
+		square_set_feat(cave, context->grid, FEAT_LAVA);
 
 		/* Objects that have survived should move */
-		push_object(context->y, context->x);
+		push_object(context->grid);
 	}
 }
 
 static void project_feature_handler_METEOR(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -545,8 +526,7 @@ static void project_feature_handler_METEOR(project_feature_handler_context_t *co
 static void project_feature_handler_MISSILE(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -555,8 +535,7 @@ static void project_feature_handler_MISSILE(project_feature_handler_context_t *c
 static void project_feature_handler_MANA(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -565,8 +544,7 @@ static void project_feature_handler_MANA(project_feature_handler_context_t *cont
 static void project_feature_handler_HOLY_ORB(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -575,8 +553,7 @@ static void project_feature_handler_HOLY_ORB(project_feature_handler_context_t *
 static void project_feature_handler_ARROW(project_feature_handler_context_t *context)
 {
 	/* Grid is in line of sight and player is not blind */
-	if (square_isview(cave, context->y, context->x) &&
-		!player->timed[TMD_BLIND]) {
+	if (square_isview(cave, context->grid) && !player->timed[TMD_BLIND]) {
 		/* Observe */
 		context->obvious = true;
 	}
@@ -590,6 +567,10 @@ static void project_feature_handler_AWAY_EVIL(project_feature_handler_context_t
 {
 }
 
+static void project_feature_handler_AWAY_SPIRIT(project_feature_handler_context_t *context)
+{
+}
+
 static void project_feature_handler_AWAY_ALL(project_feature_handler_context_t *context)
 {
 }
@@ -602,6 +583,10 @@ static void project_feature_handler_TURN_EVIL(project_feature_handler_context_t
 {
 }
 
+static void project_feature_handler_TURN_LIVING(project_feature_handler_context_t *context)
+{
+}
+
 static void project_feature_handler_TURN_ALL(project_feature_handler_context_t *context)
 {
 }
@@ -618,6 +603,18 @@ static void project_feature_handler_DISP_ALL(project_feature_handler_context_t *
 {
 }
 
+static void project_feature_handler_SLEEP_UNDEAD(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_SLEEP_EVIL(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_SLEEP_ALL(project_feature_handler_context_t *context)
+{
+}
+
 static void project_feature_handler_MON_CLONE(project_feature_handler_context_t *context)
 {
 }
@@ -642,10 +639,6 @@ static void project_feature_handler_MON_CONF(project_feature_handler_context_t *
 {
 }
 
-static void project_feature_handler_MON_SLEEP(project_feature_handler_context_t *context)
-{
-}
-
 static void project_feature_handler_MON_HOLD(project_feature_handler_context_t *context)
 {
 }
@@ -658,6 +651,10 @@ static void project_feature_handler_MON_DRAIN(project_feature_handler_context_t
 {
 }
 
+static void project_feature_handler_MON_CRUSH(project_feature_handler_context_t *context)
+{
+}
+
 static const project_feature_handler_f feature_handlers[] = {
 	#define ELEM(a) project_feature_handler_##a,
 	#include "list-elements.h"
@@ -687,15 +684,14 @@ static const project_feature_handler_f feature_handlers[] = {
  *
  * Hack -- effects on grids which are memorized but not in view are also seen.
  */
-bool project_f(struct source origin, int r, int y, int x, int dam, int typ)
+bool project_f(struct source origin, int r, struct loc grid, int dam, int typ)
 {
 	bool obvious = false;
 
 	project_feature_handler_context_t context = {
 		origin,
 		r,
-		y,
-		x,
+		grid,
 		dam,
 		typ,
 		obvious,
