@@ -83,9 +83,9 @@ const char *proj_idx_to_name(int type)
 /**
  * Determine the path taken by a projection.
  *
- * The projection will always start from the grid (y1,x1), and will travel
- * towards the grid (y2,x2), touching one grid per unit of distance along
- * the major axis, and stopping when it enters the destination grid or a
+ * The projection will always start from the grid1, and will travel
+ * towards grid2, touching one grid per unit of distance along
+ * the major axis, and stopping when it enters the finish grid or a
  * wall grid, or has travelled the maximum legal distance of "range".
  *
  * Note that "distance" in this function (as in the "update_view()" code)
@@ -97,7 +97,7 @@ const char *proj_idx_to_name(int type)
  * due to the way in which distance is calculated, this function normally
  * uses fewer than "range" grids for the projection path, so the result
  * of this function should never be compared directly to "range".  Note
- * that the initial grid (y1,x1) is never saved into the grid array, not
+ * that the initial grid grid1 is never saved into the grid array, not
  * even if the initial grid is also the final grid.  XXX XXX XXX
  *
  * The "flg" flags can be used to modify the behavior of this function.
@@ -105,7 +105,7 @@ const char *proj_idx_to_name(int type)
  * In particular, the "PROJECT_STOP" and "PROJECT_THRU" flags have the same
  * semantics as they do for the "project" function, namely, that the path
  * will stop as soon as it hits a monster, or that the path will continue
- * through the destination grid, respectively.
+ * through the finish grid, respectively.
  *
  * The "PROJECT_JUMP" flag, which for the "project()" function means to
  * start at a special grid (which makes no sense in this function), means
@@ -115,12 +115,13 @@ const char *proj_idx_to_name(int type)
  * perhaps make use of the "vinfo" array (above).  XXX XXX XXX
  *
  * This function returns the number of grids (if any) in the path.  This
- * function will return zero if and only if (y1,x1) and (y2,x2) are equal.
+ * function will return zero if and only if grid1 and grid2 are equal.
  *
  * This algorithm is similar to, but slightly different from, the one used
  * by "update_view_los()", and very different from the one used by "los()".
  */
-int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int flg)
+int project_path(struct loc *gp, int range, struct loc grid1, struct loc grid2,
+				 int flg)
 {
 	int y, x;
 
@@ -142,26 +143,28 @@ int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int
 	/* Slope */
 	int m;
 
+	/* Possible decoy */
+	struct loc decoy = cave_find_decoy(cave);
 
 	/* No path necessary (or allowed) */
-	if ((x1 == x2) && (y1 == y2)) return (0);
+	if (loc_eq(grid1, grid2)) return (0);
 
 
 	/* Analyze "dy" */
-	if (y2 < y1) {
-		ay = (y1 - y2);
+	if (grid2.y < grid1.y) {
+		ay = (grid1.y - grid2.y);
 		sy = -1;
 	} else {
-		ay = (y2 - y1);
+		ay = (grid2.y - grid1.y);
 		sy = 1;
 	}
 
 	/* Analyze "dx" */
-	if (x2 < x1) {
-		ax = (x1 - x2);
+	if (grid2.x < grid1.x) {
+		ax = (grid1.x - grid2.x);
 		sx = -1;
 	} else {
-		ax = (x2 - x1);
+		ax = (grid2.x - grid1.x);
 		sx = 1;
 	}
 
@@ -182,8 +185,8 @@ int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int
 		m = frac << 1;
 
 		/* Start */
-		y = y1 + sy;
-		x = x1;
+		y = grid1.y + sy;
+		x = grid1.x;
 
 		/* Create the projection path */
 		while (1) {
@@ -193,19 +196,27 @@ int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int
 			/* Hack -- Check maximum range */
 			if ((n + (k >> 1)) >= range) break;
 
-			/* Sometimes stop at destination grid */
+			/* Sometimes stop at finish grid */
 			if (!(flg & (PROJECT_THRU)))
-				if ((x == x2) && (y == y2)) break;
+				if (loc_eq(loc(x, y), grid2)) break;
 
-			/* Stop at non-initial wall grids, except where that would
-			 * leak info during targetting */
-			if (!(flg & (PROJECT_INFO))) {
-				if ((n > 0) && !square_isprojectable(cave, y, x)) break;
-			} else if ((n > 0) && square_isbelievedwall(cave,y,x)) break;
+			/* Don't stop if making paths through rock for generation */
+			if (!(flg & (PROJECT_ROCK))) {
+				/* Stop at non-initial wall grids, except where that would
+				 * leak info during targetting */
+				if (!(flg & (PROJECT_INFO))) {
+					if ((n > 0) && !square_isprojectable(cave, loc(x, y)))
+						break;
+				} else if ((n > 0) && square_isbelievedwall(cave, loc(x, y))) {
+					break;
+				}
+			}
 
-			/* Sometimes stop at non-initial monsters/players */
-			if (flg & (PROJECT_STOP))
-				if ((n > 0) && (cave->squares[y][x].mon != 0)) break;
+			/* Sometimes stop at non-initial monsters/players, decoys */
+			if (flg & (PROJECT_STOP)) {
+				if ((n > 0) && (square(cave, loc(x, y)).mon != 0)) break;
+				if (loc_eq(loc(x, y), decoy)) break;
+			}
 
 			/* Slant */
 			if (m) {
@@ -239,8 +250,8 @@ int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int
 		m = frac << 1;
 
 		/* Start */
-		y = y1;
-		x = x1 + sx;
+		y = grid1.y;
+		x = grid1.x + sx;
 
 		/* Create the projection path */
 		while (1) {
@@ -250,19 +261,27 @@ int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int
 			/* Hack -- Check maximum range */
 			if ((n + (k >> 1)) >= range) break;
 
-			/* Sometimes stop at destination grid */
+			/* Sometimes stop at finish grid */
 			if (!(flg & (PROJECT_THRU)))
-				if ((x == x2) && (y == y2)) break;
+				if (loc_eq(loc(x, y), grid2)) break;
 
-			/* Stop at non-initial wall grids, except where that would
-			 * leak info during targetting */
-			if (!(flg & (PROJECT_INFO))) {			
-				if ((n > 0) && !square_isprojectable(cave, y, x)) break;
-			} else if ((n > 0) && square_isbelievedwall(cave,y,x)) break;
+			/* Don't stop if making paths through rock for generation */
+			if (!(flg & (PROJECT_ROCK))) {
+				/* Stop at non-initial wall grids, except where that would
+				 * leak info during targetting */
+				if (!(flg & (PROJECT_INFO))) {
+					if ((n > 0) && !square_isprojectable(cave, loc(x, y)))
+						break;
+				} else if ((n > 0) && square_isbelievedwall(cave, loc(x, y))) {
+					break;
+				}
+			}
 
-			/* Sometimes stop at non-initial monsters/players */
-			if (flg & (PROJECT_STOP))
-				if ((n > 0) && (cave->squares[y][x].mon != 0)) break;
+			/* Sometimes stop at non-initial monsters/players, decoys */
+			if (flg & (PROJECT_STOP)) {
+				if ((n > 0) && (square(cave, loc(x, y)).mon != 0)) break;
+				if (loc_eq(loc(x, y), decoy)) break;
+			}
 
 			/* Slant */
 			if (m) {
@@ -290,8 +309,8 @@ int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int
 	/* Diagonal */
 	else {
 		/* Start */
-		y = y1 + sy;
-		x = x1 + sx;
+		y = grid1.y + sy;
+		x = grid1.x + sx;
 
 		/* Create the projection path */
 		while (1) {
@@ -301,19 +320,27 @@ int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int
 			/* Hack -- Check maximum range */
 			if ((n + (n >> 1)) >= range) break;
 
-			/* Sometimes stop at destination grid */
+			/* Sometimes stop at finish grid */
 			if (!(flg & (PROJECT_THRU)))
-				if ((x == x2) && (y == y2)) break;
+				if (loc_eq(loc(x, y), grid2)) break;
 
-			/* Stop at non-initial wall grids, except where that would
-			 * leak info during targetting */
-			if (!(flg & (PROJECT_INFO))) {
-				if ((n > 0) && !square_isprojectable(cave, y, x)) break;
-			} else if ((n > 0) && square_isbelievedwall(cave,y,x)) break;
+			/* Don't stop if making paths through rock for generation */
+			if (!(flg & (PROJECT_ROCK))) {
+				/* Stop at non-initial wall grids, except where that would
+				 * leak info during targetting */
+				if (!(flg & (PROJECT_INFO))) {
+					if ((n > 0) && !square_isprojectable(cave, loc(x, y)))
+						break;
+				} else if ((n > 0) && square_isbelievedwall(cave, loc(x, y))) {
+					break;
+				}
+			}
 
-			/* Sometimes stop at non-initial monsters/players */
-			if (flg & (PROJECT_STOP))
-				if ((n > 0) && (cave->squares[y][x].mon != 0)) break;
+			/* Sometimes stop at non-initial monsters/players, decoys */
+			if (flg & (PROJECT_STOP)) {
+				if ((n > 0) && (square(cave, loc(x, y)).mon != 0)) break;
+				if (loc_eq(loc(x, y), decoy)) break;
+			}
 
 			/* Advance */
 			y += sy;
@@ -327,7 +354,7 @@ int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int
 
 
 /**
- * Determine if a bolt spell cast from (y1,x1) to (y2,x2) will arrive
+ * Determine if a bolt spell cast from grid1 to grid2 will arrive
  * at the final destination, assuming that no monster gets in the way,
  * using the project_path() function to check the projection path.
  *
@@ -336,28 +363,28 @@ int project_path(struct loc *gp, int range, int y1, int x1, int y2, int x2, int
  * This function is used to determine if the player can (easily) target
  * a given grid, and if a monster can target the player.
  */
-bool projectable(struct chunk *c, int y1, int x1, int y2, int x2, int flg)
+bool projectable(struct chunk *c, struct loc grid1, struct loc grid2, int flg)
 {
-	int y, x;
-
-	int grid_n = 0;
 	struct loc grid_g[512];
+	int grid_n = 0;
+	int max_range = z_info->max_range;
+
+	/* Check for shortened projection range */
+	if ((flg & PROJECT_SHORT) && player->timed[TMD_COVERTRACKS]) {
+		max_range /= 4;
+	}
 
 	/* Check the projection path */
-	grid_n = project_path(grid_g, z_info->max_range, y1, x1, y2, x2, flg);
+	grid_n = project_path(grid_g, max_range, grid1, grid2, flg);
 
 	/* No grid is ever projectable from itself */
-	if (!grid_n) return (false);
-
-	/* Final grid */
-	y = grid_g[grid_n - 1].y;
-	x = grid_g[grid_n - 1].x;
+	if (!grid_n) return false;
 
 	/* May not end in a wall grid */
-	if (!square_ispassable(c, y, x)) return (false);
+	if (!square_ispassable(c, grid_g[grid_n - 1])) return false;
 
 	/* May not end in an unrequested grid */
-	if ((y != y2) || (x != x2)) return (false);
+	if (!loc_eq(grid_g[grid_n - 1], grid2)) return false;
 
 	/* Assume okay */
 	return (true);
@@ -381,17 +408,18 @@ struct loc origin_get_loc(struct source origin)
 	switch (origin.what) {
 		case SRC_MONSTER: {
 			struct monster *who = cave_monster(cave, origin.which.monster);
-			return loc(who->fx, who->fy);
+			return who ? who->grid : loc(-1, -1);
 		}
 
 		case SRC_TRAP: {
 			struct trap *trap = origin.which.trap;
-			return loc(trap->fx, trap->fy);
+			return trap->grid;
 		}
 
 		case SRC_PLAYER:
-		case SRC_OBJECT:	/* At the moment only worn cursed objects use this */
-			return loc(player->px, player->py);
+		case SRC_OBJECT:	/* Currently only worn cursed objects use this */
+		case SRC_CHEST_TRAP:
+			return player->grid;
 
 		case SRC_NONE:
 			return loc(-1, -1);
@@ -546,7 +574,7 @@ struct loc origin_get_loc(struct source origin)
  * in the blast radius, in case the illumination of the grid was changed,
  * and "update_view()" and "update_monsters()" need to be called.
  */
-bool project(struct source origin, int rad, int y, int x,
+bool project(struct source origin, int rad, struct loc finish,
 			 int dam, int typ, int flg,
 			 int degrees_of_arc, byte diameter_of_source,
 			 const struct object *obj)
@@ -556,8 +584,7 @@ bool project(struct source origin, int rad, int y, int x,
 	u32b dam_temp;
 
 	struct loc centre;
-	struct loc source;
-	struct loc destination;
+	struct loc start;
 
 	int n1y = 0;
 	int n1x = 0;
@@ -597,24 +624,21 @@ bool project(struct source origin, int rad, int y, int x,
 
 	/* No projection path - jump to target */
 	if (flg & PROJECT_JUMP) {
-		source = loc(x, y);
+		start = finish;
 
 		/* Clear the flag */
 		flg &= ~(PROJECT_JUMP);
 	} else {
-		source = origin_get_loc(origin);
+		start = origin_get_loc(origin);
 
-		/* Default to destination grid */
-		if (source.y == -1 && source.x == -1) {
-			source = loc(x, y);
+		/* Default to finish grid */
+		if (start.y == -1 && start.x == -1) {
+			start = finish;
 		}
 	}
 
-	/* Default destination */
-	destination = loc(x, y);
-
 	/* Default center of explosion (if any) */
-	centre = loc(source.x, source.y);
+	centre = start;
 
 	/*
 	 * An arc spell with no width and a non-zero radius is actually a
@@ -630,25 +654,24 @@ bool project(struct source origin, int rad, int y, int x,
 	}
 
 	/*
-	 * If a single grid is both source and destination (for example
+	 * If a single grid is both start and finish (for example
 	 * if PROJECT_JUMP is set), store it; otherwise, travel along the
 	 * projection path.
 	 */
-	if ((source.x == destination.x) && (source.y == destination.y)) {
-		blast_grid[num_grids].y = y;
-		blast_grid[num_grids].x = x;
+	if (loc_eq(start, finish)) {
+		blast_grid[num_grids] =  finish;
+		centre = finish;
 		distance_to_grid[num_grids] = 0;
-		sqinfo_on(cave->squares[y][x].info, SQUARE_PROJECT);
+		sqinfo_on(square(cave, finish).info, SQUARE_PROJECT);
 		num_grids++;
 	} else {
-		/* Calculate the projection path */
-		num_path_grids = project_path(path_grid, z_info->max_range, source.y,
-									  source.x, destination.y, destination.x,
-									  flg);
-
 		/* Start from caster */
-		y = source.y;
-		x = source.x;
+		int y = start.y;
+		int x = start.x;
+
+		/* Calculate the projection path */
+		num_path_grids = project_path(path_grid, z_info->max_range, start,
+									  finish, flg);
 
 		/* Some beams have limited length. */
 		if (flg & (PROJECT_BEAM)) {
@@ -669,7 +692,8 @@ bool project(struct source origin, int rad, int y, int x,
 				int nx = path_grid[i].x;
 
 				/* Hack -- Balls explode before reaching walls. */
-				if (!square_ispassable(cave, ny, nx) && (rad > 0))
+				if (!square_ispassable(cave, path_grid[i]) && (rad > 0) &&
+					!(flg & (PROJECT_BEAM)))
 					break;
 
 				/* Advance */
@@ -682,19 +706,19 @@ bool project(struct source origin, int rad, int y, int x,
 					blast_grid[num_grids].y = y;
 					blast_grid[num_grids].x = x;
 					distance_to_grid[num_grids] = 0;
-					sqinfo_on(cave->squares[y][x].info, SQUARE_PROJECT);
+					sqinfo_on(square(cave, loc(x, y)).info, SQUARE_PROJECT);
 					num_grids++;
 				} else if (i == num_path_grids - 1) {
 					blast_grid[num_grids].y = y;
 					blast_grid[num_grids].x = x;
 					distance_to_grid[num_grids] = 0;
-					sqinfo_on(cave->squares[y][x].info, SQUARE_PROJECT);
+					sqinfo_on(square(cave, loc(x, y)).info, SQUARE_PROJECT);
 					num_grids++;
 				}
 
 				/* Only do visuals if requested and within range limit. */
 				if (!blind && !(flg & (PROJECT_HIDE))) {
-					bool seen = square_isview(cave, y, x);
+					bool seen = square_isview(cave, loc(x, y));
 					bool beam = flg & (PROJECT_BEAM);
 
 					/* Tell the UI to display the bolt */
@@ -703,21 +727,22 @@ bool project(struct source origin, int rad, int y, int x,
 				}
 			}
 		}
-	}
 
-	/* Save the "blast epicenter" */
-	centre.y = y;
-	centre.x = x;
+		/* Save the "blast epicenter" */
+		centre.y = y;
+		centre.x = x;
+	}
 
 	/* Now check for explosions.  Beams have already stored all the grids they
 	 * will affect; all non-beam projections with positive radius explode in
 	 * some way */
 	if ((rad > 0) && (!(flg & (PROJECT_BEAM)))) {
+		int y, x;
+
 		/* Pre-calculate some things for arcs. */
 		if ((flg & (PROJECT_ARC)) && (num_path_grids != 0)) {
 			/* Explosion centers on the caster. */
-			centre.y = source.y;
-			centre.x = source.x;
+			centre = start;
 
 			/* The radius of arcs cannot be more than 20 */
 			if (rad > 20)
@@ -736,19 +761,19 @@ bool project(struct source origin, int rad, int y, int x,
 
 		/* If the explosion centre hasn't been saved already, save it now. */
 		if (num_grids == 0) {
-			blast_grid[num_grids].y = centre.y;
-			blast_grid[num_grids].x = centre.x;
+			blast_grid[num_grids] = centre;
 			distance_to_grid[num_grids] = 0;
-			sqinfo_on(cave->squares[centre.y][centre.x].info, SQUARE_PROJECT);
+			sqinfo_on(square(cave, centre).info, SQUARE_PROJECT);
 			num_grids++;
 		}
 
 		/* Scan every grid that might possibly be in the blast radius. */
 		for (y = centre.y - rad; y <= centre.y + rad; y++) {
 			for (x = centre.x - rad; x <= centre.x + rad; x++) {
+				struct loc grid = loc(x, y);
 
 				/* Center grid has already been stored. */
-				if ((y == centre.y) && (x == centre.x))
+				if (loc_eq(grid, centre))
 					continue;
 
 				/* Precaution: Stay within area limit. */
@@ -756,7 +781,7 @@ bool project(struct source origin, int rad, int y, int x,
 					break;
 
 				/* Ignore "illegal" locations */
-				if (!square_in_bounds(cave, y, x))
+				if (!square_in_bounds(cave, grid))
 					continue;
 
 				/* Most explosions are immediately stopped by walls. If
@@ -765,73 +790,65 @@ bool project(struct source origin, int rad, int y, int x,
 				 * Angband 3.5.0 there are no such explosions - NRM.
 				 * All explosions can affect one layer of terrain which is
 				 * passable but not projectable */
-				if ((flg & (PROJECT_THRU)) || square_ispassable(cave, y, x)) {
+				if ((flg & (PROJECT_THRU)) || square_ispassable(cave, grid)) {
 					/* If this is a wall grid, ... */
-					if (!square_isprojectable(cave, y, x)) {
+					if (!square_isprojectable(cave, grid)) {
+						bool can_see_one = false;
 						/* Check neighbors */
-						for (i = 0, k = 0; i < 8; i++) {
-							int yy = y + ddy_ddd[i];
-							int xx = x + ddx_ddd[i];
-
-							if (los(cave, centre.y, centre.x, yy, xx)) {
-								k++;
+						for (i = 0; i < 8; i++) {
+							struct loc adj_grid = loc_sum(grid, ddgrid_ddd[i]);
+							if (los(cave, centre, adj_grid)) {
+								can_see_one = true;
 								break;
 							}
 						}
 
 						/* Require at least one adjacent grid in LOS. */
-						if (!k)
+						if (!can_see_one)
 							continue;
 					}
-				} else if (!square_isprojectable(cave, y, x))
+				} else if (!square_isprojectable(cave, grid))
 					continue;
 
 				/* Must be within maximum distance. */
-				dist_from_centre  = (distance(centre.y, centre.x, y, x));
+				dist_from_centre  = (distance(centre, grid));
 				if (dist_from_centre > rad)
 					continue;
 
 
-				/* Do we need to consider a  restricted angle? */
+				/* Do we need to consider a restricted angle? */
 				if (flg & (PROJECT_ARC)) {
 					/* Use angle comparison to delineate an arc. */
 					int n2y, n2x, tmp, rotate, diff;
 
 					/* Reorient current grid for table access. */
-					n2y = y - source.y + 20;
-					n2x = x - source.x + 20;
+					n2y = y - start.y + 20;
+					n2x = x - start.x + 20;
 
-					/* 
-					 * Find the angular difference (/2) between 
-					 * the lines to the end of the arc's center-
-					 * line and to the current grid.
+					/* Find the angular difference (/2) between the lines to
+					 * the end of the arc's center-line and to the current grid.
 					 */
 					rotate = 90 - get_angle_to_grid[n1y][n1x];
 					tmp = ABS(get_angle_to_grid[n2y][n2x] + rotate) % 180;
 					diff = ABS(90 - tmp);
 
-					/* 
-					 * If difference is not greater then that 
-					 * allowed, and the grid is in LOS, accept it.
-					 */
-					if (diff < (degrees_of_arc + 6) / 4) {
-						if (los(cave, centre.y, centre.x, y, x)) {
-							blast_grid[num_grids].y = y;
-							blast_grid[num_grids].x = x;
-							distance_to_grid[num_grids] = dist_from_centre;
-							sqinfo_on(cave->squares[y][x].info, SQUARE_PROJECT);
-							num_grids++;
+					/* If difference is greater then that allowed, skip it */
+					if (diff >= (degrees_of_arc + 6) / 4) {
+						/* ...unless it's on the target path */
+						for (i = 0; i < num_path_grids; i++) {
+							if (loc_eq(grid, path_grid[i])) break;
 						}
+						if (i == num_path_grids) continue;
 					}
-				} else {
-					/* Accept all grids in LOS */
-					if (los(cave, centre.y, centre.x, y, x)) {
-						blast_grid[num_grids].y = y;
-						blast_grid[num_grids].x = x;
-						distance_to_grid[num_grids] = dist_from_centre;
-						sqinfo_on(cave->squares[y][x].info, SQUARE_PROJECT);
-						num_grids++;
-					}
+				}
+
+				/* Accept remaining grids if in LOS */
+				if (los(cave, centre, grid)) {
+					blast_grid[num_grids].y = y;
+					blast_grid[num_grids].x = x;
+					distance_to_grid[num_grids] = dist_from_centre;
+					sqinfo_on(square(cave, grid).info, SQUARE_PROJECT);
+					num_grids++;
 				}
 			}
 		}
@@ -849,7 +866,7 @@ bool project(struct source origin, int rad, int y, int x,
 			/* If a particular diameter for the source of the explosion's
 			 * energy is given, it is full strength to that diameter and
 			 * then reduces */
-			dam_temp = (diameter_of_source * dam) / ((i + 1) * 10);
+			dam_temp = (diameter_of_source * dam) / (i + 1);
 			if (dam_temp > (u32b) dam) {
 				dam_temp = dam;
 			}
@@ -860,23 +877,19 @@ bool project(struct source origin, int rad, int y, int x,
 	}
 
 
-	/* Sort the blast grids by distance, starting at the origin. */
+	/* Sort the blast grids by distance from the centre. */
 	for (i = 0, k = 0; i <= rad; i++) {
-		int tmp_y, tmp_x, tmp_d;
-
 		/* Collect all the grids of a given distance together. */
 		for (j = k; j < num_grids; j++) {
 			if (distance_to_grid[j] == i) {
-				tmp_y = blast_grid[k].y;
-				tmp_x = blast_grid[k].x;
-				tmp_d = distance_to_grid[k];
+				struct loc tmp;
+				int tmp_d = distance_to_grid[k];
+				tmp = blast_grid[k];
 
-				blast_grid[k].y = blast_grid[j].y;
-				blast_grid[k].x = blast_grid[j].x;
+				blast_grid[k] = blast_grid[j];
 				distance_to_grid[k] = distance_to_grid[j];
 
-				blast_grid[j].y = tmp_y;
-				blast_grid[j].x = tmp_x;
+				blast_grid[j] = tmp;
 				distance_to_grid[j] = tmp_d;
 
 				/* Write to next slot */
@@ -888,7 +901,7 @@ bool project(struct source origin, int rad, int y, int x,
 	/* Establish which grids are visible - no blast visuals with PROJECT_HIDE */
 	for (i = 0; i < num_grids; i++) {
 		if (panel_contains(blast_grid[i].y, blast_grid[i].x) &&
-			square_isview(cave, blast_grid[i].y, blast_grid[i].x) &&
+			square_isview(cave, blast_grid[i]) &&
 			!blind && !(flg & (PROJECT_HIDE))) {
 			player_sees_grid[i] = true;
 		} else {
@@ -900,16 +913,10 @@ bool project(struct source origin, int rad, int y, int x,
 	event_signal_blast(EVENT_EXPLOSION, typ, num_grids, distance_to_grid,
 					   drawing, player_sees_grid, blast_grid, centre);
 
-	/* Check objects */
+	/* Affect objects on every relevant grid */
 	if (flg & (PROJECT_ITEM)) {
-		/* Scan for objects */
 		for (i = 0; i < num_grids; i++) {
-			/* Get the grid location */
-			y = blast_grid[i].y;
-			x = blast_grid[i].x;
-
-			/* Affect the object in the grid */
-			if (project_o(origin, distance_to_grid[i], y, x,
+			if (project_o(origin, distance_to_grid[i], blast_grid[i],
 						  dam_at_dist[distance_to_grid[i]], typ, obj)) {
 				notice = true;
 			}
@@ -921,28 +928,23 @@ bool project(struct source origin, int rad, int y, int x,
 		bool was_obvious = false;
 		bool did_hit = false;
 		int num_hit = 0;
-		int last_hit_x = 0;
-		int last_hit_y = 0;
+		struct loc last_hit_grid = loc(0, 0);
 
 		/* Scan for monsters */
 		for (i = 0; i < num_grids; i++) {
 			struct monster *mon = NULL;
 
-			/* Get the grid location */
-			y = blast_grid[i].y;
-			x = blast_grid[i].x;
-			
 			/* Check this monster hasn't been processed already */
-			if (!square_isproject(cave, y, x))
+			if (!square_isproject(cave, blast_grid[i]))
 				continue;
 
 			/* Check there is actually a monster here */
-			mon = square_monster(cave, y, x);
+			mon = square_monster(cave, blast_grid[i]);
 			if (mon == NULL)
 				continue;
 
 			/* Affect the monster in the grid */
-			project_m(origin, distance_to_grid[i], y, x,
+			project_m(origin, distance_to_grid[i], blast_grid[i],
 			          dam_at_dist[distance_to_grid[i]], typ, flg,
 			          &did_hit, &was_obvious);
 			if (was_obvious) {
@@ -952,8 +954,7 @@ bool project(struct source origin, int rad, int y, int x,
 				num_hit++;
 
 				/* Monster location may have been updated by project_m() */
-				last_hit_x = mon->fx;
-				last_hit_y = mon->fy;
+				last_hit_grid = mon->grid;
 			}
 		}
 
@@ -962,12 +963,12 @@ bool project(struct source origin, int rad, int y, int x,
 				num_hit == 1 &&
 				!(flg & PROJECT_JUMP)) {
 			/* Location */
-			x = last_hit_x;
-			y = last_hit_y;
+			int x = last_hit_grid.x;
+			int y = last_hit_grid.y;
 
 			/* Track if possible */
-			if (cave->squares[y][x].mon > 0) {
-				struct monster *mon = square_monster(cave, y, x);
+			if (square(cave, loc(x, y)).mon > 0) {
+				struct monster *mon = square_monster(cave, loc(x, y));
 
 				/* Recall and track */
 				if (monster_is_visible(mon)) {
@@ -978,35 +979,36 @@ bool project(struct source origin, int rad, int y, int x,
 		}
 	}
 
-	/* Check player */
+	/* Look for the player, affect them when found */
 	if (flg & (PROJECT_PLAY)) {
-		/* Scan for player */
-		for (i = 0; i < num_grids; i++) {
-			/* Get the grid location */
-			y = blast_grid[i].y;
-			x = blast_grid[i].x;
+		/* Set power */
+		int power = 0;
+		if (origin.what == SRC_MONSTER) {
+			struct monster *mon = cave_monster(cave, origin.which.monster);
+			power = mon->race->spell_power;
 
-			/* Affect the player, or keep scanning */
-			if (project_p(origin, distance_to_grid[i], y, x,
-						  dam_at_dist[distance_to_grid[i]], typ)) {
+			/* Breaths from powerful monsters get power effects as well */
+			if (monster_is_powerful(mon))
+				power = MAX(power, 80);
+		}
+		for (i = 0; i < num_grids; i++) {
+			if (project_p(origin, distance_to_grid[i], blast_grid[i],
+						  dam_at_dist[distance_to_grid[i]], typ, power,
+						  flg & PROJECT_SELF)) {
 				notice = true;
-				if (player->is_dead)
+				if (player->is_dead) {
+					free(dam_at_dist);
 					return notice;
+				}
 				break;
 			}
 		}
 	}
 
-	/* Check features */
+	/* Affect features in every relevant grid */
 	if (flg & (PROJECT_GRID)) {
-		/* Scan for features */
 		for (i = 0; i < num_grids; i++) {
-			/* Get the grid location */
-			y = blast_grid[i].y;
-			x = blast_grid[i].x;
-
-			/* Affect the feature in that grid */
-			if (project_f(origin, distance_to_grid[i], y, x,
+			if (project_f(origin, distance_to_grid[i], blast_grid[i],
 						  dam_at_dist[distance_to_grid[i]], typ)) {
 				notice = true;
 			}
@@ -1015,12 +1017,8 @@ bool project(struct source origin, int rad, int y, int x,
 
 	/* Clear all the processing marks. */
 	for (i = 0; i < num_grids; i++) {
-		/* Get the grid location */
-		y = blast_grid[i].y;
-		x = blast_grid[i].x;
-
 		/* Clear the mark */
-		sqinfo_off(cave->squares[y][x].info, SQUARE_PROJECT);
+		sqinfo_off(square(cave, blast_grid[i]).info, SQUARE_PROJECT);
 	}
 
 	/* Update stuff if needed */
