@@ -96,6 +96,7 @@ int rd_dungeon(void);
 int rd_chunks(void);
 int rd_objects(void);
 int rd_monsters(void);
+int rd_monster_groups(void);
 int rd_history(void);
 int rd_traps(void);
 int rd_null(void);
@@ -121,6 +122,7 @@ void wr_dungeon(void);
 void wr_chunks(void);
 void wr_objects(void);
 void wr_monsters(void);
+void wr_monster_groups(void);
 void wr_ghost(void);
 void wr_history(void);
 void wr_traps(void);
