@@ -22,8 +22,8 @@
 
 #include "cave.h"
 #include "cmd-core.h"
+#include "datafile.h"
 #include "object.h"
-#include "parser.h"
 
 /**
  * List of store indices
@@ -64,6 +64,7 @@ struct store {
 	byte stock_num;				/* Stock -- Number of entries */
 	s16b stock_size;			/* Stock -- Total Size of Array */
 	struct object *stock;		/* Stock -- Actual stock items */
+	struct object *stock_k;		/* Stock -- Stock as known by the character */
 
 	/* Always stock these items */
 	size_t always_size;
@@ -85,7 +86,7 @@ struct store {
 
 extern struct store *stores;
 
-struct store *store_at(struct chunk *c, int y, int x);
+struct store *store_at(struct chunk *c, struct loc grid);
 void store_init(void);
 void free_stores(void);
 void store_stock_list(struct store *store, struct object **list, int n);
