@@ -42,7 +42,7 @@ int test_prefs(void *state) {
 	/* This is a bit of a hack to ensure we have a player struct set up */
 	/* Otherwise race/class dependent graphics will crash */
 	cmdq_push(CMD_BIRTH_RESET);
-	cmdq_execute(CMD_BIRTH);
+	cmdq_execute(CTX_BIRTH);
 
 	event_add_handler(EVENT_MESSAGE, getmsg, &error);
 
