@@ -27,6 +27,10 @@
 #include "player-util.h"
 #include "trap.h"
 
+/**
+ * ------------------------------------------------------------------------
+ * General trap routines
+ * ------------------------------------------------------------------------ */
 struct trap_kind *trap_info;
 
 /**
@@ -59,12 +63,12 @@ struct trap_kind *lookup_trap(const char *desc)
 /**
  * Is there a specific kind of trap in this square?
  */
-bool square_trap_specific(struct chunk *c, int y, int x, int t_idx)
+bool square_trap_specific(struct chunk *c, struct loc grid, int t_idx)
 {
-    struct trap *trap = square_trap(c, y, x);
+    struct trap *trap = square_trap(c, grid);
 	
     /* First, check the trap marker */
-    if (!square_istrap(c, y, x))
+    if (!square_istrap(c, grid))
 		return false;
 	
     /* Scan the square trap list */
@@ -82,12 +86,12 @@ bool square_trap_specific(struct chunk *c, int y, int x, int t_idx)
 /**
  * Is there a trap with a given flag in this square?
  */
-bool square_trap_flag(struct chunk *c, int y, int x, int flag)
+bool square_trap_flag(struct chunk *c, struct loc grid, int flag)
 {
-    struct trap *trap = square_trap(c, y, x);
+    struct trap *trap = square_trap(c, grid);
 
     /* First, check the trap marker */
-    if (!square_istrap(c, y, x))
+    if (!square_istrap(c, grid))
 		return false;
 	
     /* Scan the square trap list */
@@ -110,9 +114,9 @@ bool square_trap_flag(struct chunk *c, int y, int x, int flag)
  *
  * Clear the SQUARE_TRAP flag if none exist.
  */
-static bool square_verify_trap(struct chunk *c, int y, int x, int vis)
+static bool square_verify_trap(struct chunk *c, struct loc grid, int vis)
 {
-    struct trap *trap = square_trap(c, y, x);
+    struct trap *trap = square_trap(c, grid);
     bool trap_exists = false;
 
     /* Scan the square trap list */
@@ -135,34 +139,124 @@ static bool square_verify_trap(struct chunk *c, int y, int x, int vis)
     /* No traps in this location. */
     if (!trap_exists) {
 		/* No traps */
-		sqinfo_off(c->squares[y][x].info, SQUARE_TRAP);
+		sqinfo_off(square(c, grid).info, SQUARE_TRAP);
 
 		/* Take note */
-		square_note_spot(c, y, x);
+		square_note_spot(c, grid);
     }
 
     /* Report failure */
     return false;
 }
 
+/**
+ * Free memory for all traps on a grid
+ */
+void square_free_trap(struct chunk *c, struct loc grid)
+{
+	struct trap *next, *trap = square_trap(c, grid);
+
+	while (trap) {
+		next = trap->next;
+		mem_free(trap);
+		trap = next;
+	}
+}
+
+/**
+ * Remove all traps from a grid.
+ *
+ * Return true if traps were removed.
+ */
+bool square_remove_all_traps(struct chunk *c, struct loc grid)
+{
+	struct trap *trap = square(c, grid).trap;
+	bool were_there_traps = trap == NULL ? false : true;
+
+	assert(square_in_bounds(c, grid));
+	while (trap) {
+		struct trap *next_trap = trap->next;
+		mem_free(trap);
+		trap = next_trap;
+	}
+
+	square_set_trap(c, grid, NULL);
+
+	/* Refresh grids that the character can see */
+	if (square_isseen(c, grid)) {
+		square_light_spot(c, grid);
+	}
+
+	(void)square_verify_trap(c, grid, 0);
+
+	return were_there_traps;
+}
+
+/**
+ * Remove all traps with the given index.
+ *
+ * Return true if traps were removed.
+ */
+bool square_remove_trap(struct chunk *c, struct loc grid, int t_idx_remove)
+{
+	bool removed = false;
+
+	/* Look at the traps in this grid */
+	struct trap *prev_trap = NULL;
+	struct trap *trap = square(c, grid).trap;
+
+	assert(square_in_bounds(c, grid));
+	while (trap) {
+		struct trap *next_trap = trap->next;
+
+		if (t_idx_remove == trap->t_idx) {
+			mem_free(trap);
+			removed = true;
+
+			if (prev_trap) {
+				prev_trap->next = next_trap;
+			} else {
+				square_set_trap(c, grid, next_trap);
+			}
+
+			break;
+		}
+
+		prev_trap = trap;
+		trap = next_trap;
+	}
+
+	/* Refresh grids that the character can see */
+	if (square_isseen(c, grid))
+		square_light_spot(c, grid);
+
+	(void)square_verify_trap(c, grid, 0);
+
+	return removed;
+}
+
+/**
+ * ------------------------------------------------------------------------
+ * Player traps
+ * ------------------------------------------------------------------------ */
 /**
  * Determine if a cave grid is allowed to have player traps in it.
  */
-bool square_player_trap_allowed(struct chunk *c, int y, int x)
+bool square_player_trap_allowed(struct chunk *c, struct loc grid)
 {
-    /*
-     * We currently forbid multiple traps in a grid under normal conditions.
+
+    /* We currently forbid multiple traps in a grid under normal conditions.
      * If this changes, various bits of code elsewhere will have to change too.
      */
-    if (square_istrap(c, y, x))
+    if (square_istrap(c, grid))
 		return false;
 
     /* We currently forbid traps in a grid with objects. */
-    if (square_object(c, y, x))
+    if (square_object(c, grid))
 		return false;
 
     /* Check it's a trappable square */
-    return (square_istrappable(c, y, x));
+    return (square_istrappable(c, grid));
 }
 
 /**
@@ -249,16 +343,16 @@ static int pick_trap(struct chunk *c, int feat, int trap_level)
  * This should be the only function that places traps in the dungeon
  * except the savefile loading code.
  */
-void place_trap(struct chunk *c, int y, int x, int t_idx, int trap_level)
+void place_trap(struct chunk *c, struct loc grid, int t_idx, int trap_level)
 {
 	struct trap *new_trap;
 
     /* We've been called with an illegal index; choose a random trap */
     if ((t_idx <= 0) || (t_idx >= z_info->trap_max)) {
 		/* Require the correct terrain */
-		if (!square_player_trap_allowed(c, y, x)) return;
+		if (!square_player_trap_allowed(c, grid)) return;
 
-		t_idx = pick_trap(c, c->squares[y][x].feat, trap_level);
+		t_idx = pick_trap(c, square(c, grid).feat, trap_level);
     }
 
     /* Failure */
@@ -266,48 +360,35 @@ void place_trap(struct chunk *c, int y, int x, int t_idx, int trap_level)
 
 	/* Allocate a new trap for this grid (at the front of the list) */
 	new_trap = mem_zalloc(sizeof(*new_trap));
-	new_trap->next = square_trap(c, y, x);
-	c->squares[y][x].trap = new_trap;
+	new_trap->next = square_trap(c, grid);
+	square_set_trap(c, grid, new_trap);
 
 	/* Set the details */
 	new_trap->t_idx = t_idx;
 	new_trap->kind = &trap_info[t_idx];
-	new_trap->fy = y;
-	new_trap->fx = x;
+	new_trap->grid = grid;
 	new_trap->power = randcalc(new_trap->kind->power, trap_level, RANDOMISE);
 	trf_copy(new_trap->flags, trap_info[t_idx].flags);
 
 	/* Toggle on the trap marker */
-	sqinfo_on(c->squares[y][x].info, SQUARE_TRAP);
+	sqinfo_on(square(c, grid).info, SQUARE_TRAP);
 
 	/* Redraw the grid */
-	square_light_spot(c, y, x);
-}
-
-/**
- * Free memory for all traps on a grid
- */
-void square_free_trap(struct chunk *c, int y, int x)
-{
-	struct trap *next, *trap = square_trap(c, y, x);
-
-	while (trap) {
-		next = trap->next;
-		mem_free(trap);
-		trap = next;
-	}
+	square_note_spot(c, grid);
+	square_light_spot(c, grid);
 }
 
 /**
  * Reveal some of the player traps in a square
  */
-bool square_reveal_trap(struct chunk *c, int y, int x, bool always, bool domsg)
+bool square_reveal_trap(struct chunk *c, struct loc grid, bool always,
+						bool domsg)
 {
     int found_trap = 0;
-	struct trap *trap = square_trap(c, y, x);
+	struct trap *trap = square_trap(c, grid);
     
     /* Check there is a player trap */
-    if (!square_isplayertrap(c, y, x))
+    if (!square_isplayertrap(c, grid))
 		return false;
 
 	/* Scan the grid */
@@ -326,9 +407,9 @@ bool square_reveal_trap(struct chunk *c, int y, int x, bool always, bool domsg)
 
 		/* Trap is invisible */
 		if (!trf_has(trap->flags, TRF_VISIBLE)) {
-			/* See the trap */
+			/* See the trap (actually, see all the traps) */
 			trf_on(trap->flags, TRF_VISIBLE);
-			square_memorize(c, y, x);
+			square_memorize_traps(c, grid);
 
 			/* We found a trap */
 			found_trap++;
@@ -347,10 +428,10 @@ bool square_reveal_trap(struct chunk *c, int y, int x, bool always, bool domsg)
 		}
 
 		/* Memorize */
-		square_memorize(c, y, x);
+		square_memorize(c, grid);
 
 		/* Redraw */
-		square_light_spot(c, y, x);
+		square_light_spot(c, grid);
     }
 
     /* Return true if we found any traps */
@@ -358,33 +439,34 @@ bool square_reveal_trap(struct chunk *c, int y, int x, bool always, bool domsg)
 }
 
 /**
- * Count the number of player traps in this location.
- *
- * Called with vis = 0 to accept any trap, = 1 to accept only visible
- * traps, and = -1 to accept only invisible traps.
+ * Memorize all the visible traps on a square
  */
-int num_traps(struct chunk *c, int y, int x, int vis)
+void square_memorize_traps(struct chunk *c, struct loc grid)
 {
-    int num = 0;
-	struct trap *trap;
+	struct trap *trap = square(c, grid).trap;
+	struct trap *current = NULL;
+	if (c != cave) return;
 
-	/* Look at the traps in this grid */
-	for (trap = square_trap(c, y, x); trap; trap = trap->next) {
-		/* Require that trap be capable of affecting the character */
-		if (!trf_has(trap->kind->flags, TRF_TRAP)) continue;
-	    
-		/* Require correct visibility */
-		if (vis >= 1) {
-			if (trf_has(trap->flags, TRF_VISIBLE)) num++;
-		} else if (vis <= -1) {
-			if (!trf_has(trap->flags, TRF_VISIBLE)) num++;
-		} else {
-			num++;
+	/* Clear current knowledge */
+	square_remove_all_traps(player->cave, grid);
+
+	/* Copy all visible traps to the known cave */
+	while (trap) {
+		if (square_isvisibletrap(c, grid)) {
+			struct trap *next;
+			if (current) {
+				next = mem_zalloc(sizeof(*next));
+				current->next = next;
+				current = next;
+			} else {
+				current = mem_zalloc(sizeof(*current));
+				player->cave->squares[grid.y][grid.x].trap = current;
+			}
+			memcpy(current, trap, sizeof(*trap));
+			current->next = NULL;
 		}
+		trap = trap->next;
 	}
-
-    /* Return the number of traps */
-    return (num);
 }
 
 /**
@@ -401,25 +483,17 @@ bool trap_check_hit(int power)
 /**
  * Hit a trap. 
  */
-extern void hit_trap(int y, int x)
+extern void hit_trap(struct loc grid, int delayed)
 {
 	bool ident = false;
 	struct trap *trap;
 	struct effect *effect;
 
-    /* Count the hidden traps here */
-    int num = num_traps(cave, y, x, -1);
-
 	/* The player is safe from all traps */
-	if (player->timed[TMD_TRAPSAFE]) return;
-
-    /* Oops.  We've walked right into trouble. */
-    if      (num == 1) msg("You stumble upon a trap!");
-    else if (num >  1) msg("You stumble upon some traps!");
-
+	if (player_is_trapsafe(player)) return;
 
 	/* Look at the traps in this grid */
-	for (trap = square_trap(cave, y, x); trap; trap = trap->next) {
+	for (trap = square_trap(cave, grid); trap; trap = trap->next) {
 		int flag;
 		bool saved = false;
 
@@ -427,8 +501,18 @@ extern void hit_trap(int y, int x)
 		if (!trf_has(trap->kind->flags, TRF_TRAP)) continue;
 		if (trap->timeout) continue;
 
+		if (delayed != trf_has(trap->kind->flags, TRF_DELAY) &&
+		    delayed != -1)
+			continue;
+
 		/* Disturb the player */
-		disturb(player, 0);
+		disturb(player);
+
+		/* Trap immune player learns the rune */
+		if (player_of_has(player, OF_TRAP_IMMUNE)) {
+			equip_learn_flag(player, OF_TRAP_IMMUNE);
+			break;
+		}
 
 		/* Give a message */
 		if (trap->kind->msg)
@@ -460,14 +544,21 @@ extern void hit_trap(int y, int x)
 			if (trap->kind->msg_bad)
 				msg(trap->kind->msg_bad);
 			effect = trap->kind->effect;
-			effect_do(effect, source_trap(trap), NULL, &ident, false, 0, 0, 0);
+			effect_do(effect, source_trap(trap), NULL, &ident, false, 0, 0, 0, NULL);
+
+			/* Trap may have gone */
+			if (!square_trap(cave, grid)) break;
 
 			/* Do any extra effects */
 			if (trap->kind->effect_xtra && one_in_(2)) {
 				if (trap->kind->msg_xtra)
 					msg(trap->kind->msg_xtra);
 				effect = trap->kind->effect_xtra;
-				effect_do(effect, source_trap(trap), NULL, &ident, false, 0, 0, 0);
+				effect_do(effect, source_trap(trap), NULL, &ident, false,
+						  0, 0, 0, NULL);
+
+				/* Trap may have gone */
+				if (!square_trap(cave, grid)) break;
 			}
 		}
 
@@ -478,121 +569,40 @@ extern void hit_trap(int y, int x)
 
 		/* Some traps drop you onto them */
 		if (trf_has(trap->kind->flags, TRF_PIT))
-			monster_swap(player->py, player->px, trap->fy, trap->fx);
+			monster_swap(player->grid, trap->grid);
 
 		/* Some traps disappear after activating, all have a chance to */
 		if (trf_has(trap->kind->flags, TRF_ONETIME) || one_in_(3)) {
-			square_destroy_trap(cave, y, x);
-			square_forget(cave, y, x);
+			square_destroy_trap(cave, grid);
+			square_forget(cave, grid);
 		}
 
 		/* Trap may have gone */
-		if (!square_trap(cave, y, x)) break;
+		if (!square_trap(cave, grid)) break;
 
 		/* Trap becomes visible (always XXX) */
 		trf_on(trap->flags, TRF_VISIBLE);
-		square_memorize(cave, y, x);
+		square_memorize(cave, grid);
 	}
 
     /* Verify traps (remove marker if appropriate) */
-    (void)square_verify_trap(cave, y, x, 0);
+    (void)square_verify_trap(cave, grid, 0);
 }
 
 /**
- * Remove all traps from a grid.
- *
- * Return true if traps were removed.
+ * Disable a trap for the specified number of turns
  */
-bool square_remove_all_traps(struct chunk *c, int y, int x)
-{
-	assert(square_in_bounds(c, y, x));
-
-	struct trap *trap = c->squares[y][x].trap;
-	bool were_there_traps = trap == NULL ? false : true;
-
-	while (trap) {
-		struct trap *next_trap = trap->next;
-		mem_free(trap);
-		trap = next_trap;
-	}
-
-	c->squares[y][x].trap = NULL;
-
-	/* Refresh grids that the character can see */
-	if (square_isseen(c, y, x)) {
-		square_light_spot(c, y, x);
-	}
-
-	(void)square_verify_trap(c, y, x, 0);
-
-	return were_there_traps;
-}
-
-/**
- * Remove traps.
- *
- * If called with t_idx < 0, will remove all traps in the location given.
- * Otherwise, will remove all traps with the given kind.
- *
- * Return true if traps were removed.
- */
-bool square_remove_trap(struct chunk *c, int y, int x, int t_idx_remove)
-{
-	assert(square_in_bounds(c, y, x));
-
-	bool removed = false;
-
-	/* Look at the traps in this grid */
-	struct trap *prev_trap = NULL;
-	struct trap *trap = c->squares[y][x].trap;
-	while (trap) {
-		struct trap *next_trap = trap->next;
-
-		if (t_idx_remove == trap->t_idx) {
-			mem_free(trap);
-			removed = true;
-
-			if (prev_trap) {
-				prev_trap->next = next_trap;
-			} else {
-				c->squares[y][x].trap = next_trap;
-			}
-
-			break;
-		}
-
-		prev_trap = trap;
-		trap = next_trap;
-	}
-
-	/* Refresh grids that the character can see */
-	if (square_isseen(c, y, x))
-		square_light_spot(c, y, x);
-
-	(void)square_verify_trap(c, y, x, 0);
-
-	return removed;
-}
-
-/**
- * Remove traps.
- *
- * If called with t_idx < 0, will remove all traps in the location given.
- * Otherwise, will remove all traps with the given kind.
- *
- * Return true if no traps now exist in this grid.
- */
-bool square_set_trap_timeout(struct chunk *c, int y, int x, bool domsg,
+bool square_set_trap_timeout(struct chunk *c, struct loc grid, bool domsg,
 							 int t_idx, int time)
 {
     bool trap_exists;
 	struct trap *current_trap = NULL;
 
 	/* Bounds check */
-	assert(square_in_bounds(c, y, x));
+	assert(square_in_bounds(c, grid));
 
 	/* Look at the traps in this grid */
-	current_trap = c->squares[y][x].trap;
+	current_trap = square(c, grid).trap;
 	while (current_trap) {
 		/* Get the next trap (may be NULL) */
 		struct trap *next_trap = current_trap->next;
@@ -615,11 +625,11 @@ bool square_set_trap_timeout(struct chunk *c, int y, int x, bool domsg,
     }
 
     /* Refresh grids that the character can see */
-    if (square_isseen(c, y, x))
-		square_light_spot(c, y, x);
+    if (square_isseen(c, grid))
+		square_light_spot(c, grid);
 
     /* Verify traps (remove marker if appropriate) */
-    trap_exists = square_verify_trap(c, y, x, 0);
+    trap_exists = square_verify_trap(c, grid, 0);
 
     /* Report whether any traps exist in this grid */
     return (!trap_exists);
@@ -629,9 +639,9 @@ bool square_set_trap_timeout(struct chunk *c, int y, int x, bool domsg,
  * Give the remaining time for a trap to be disabled; note it chooses the first
  * appropriate trap on the grid
  */
-int square_trap_timeout(struct chunk *c, int y, int x, int t_idx)
+int square_trap_timeout(struct chunk *c, struct loc grid, int t_idx)
 {
-	struct trap *current_trap = c->squares[y][x].trap;
+	struct trap *current_trap = square(c, grid).trap;
 	while (current_trap) {
 		/* Get the next trap (may be NULL) */
 		struct trap *next_trap = current_trap->next;
@@ -654,24 +664,28 @@ int square_trap_timeout(struct chunk *c, int y, int x, int t_idx)
 	return 0;
 }
 
+/**
+ * ------------------------------------------------------------------------
+ * Door locks
+ * ------------------------------------------------------------------------ */
 /**
  * Lock a closed door to a given power
  */
-void square_set_door_lock(struct chunk *c, int y, int x, int power)
+void square_set_door_lock(struct chunk *c, struct loc grid, int power)
 {
 	struct trap_kind *lock = lookup_trap("door lock");
 	struct trap *trap;
 
 	/* Verify it's a closed door */
-	if (!square_iscloseddoor(c, y, x))
+	if (!square_iscloseddoor(c, grid))
 		return;
 
 	/* If there's no lock there, add one */
-	if (!square_trap_specific(c, y, x, lock->tidx))
-		place_trap(c, y, x, lock->tidx, 0);
+	if (!square_trap_specific(c, grid, lock->tidx))
+		place_trap(c, grid, lock->tidx, 0);
 
 	/* Set the power (of all locks - there should be only one) */
-	trap = square_trap(c, y, x);
+	trap = square_trap(c, grid);
 	while (trap) {
 		if (trap->kind == lock)
 			trap->power = power;
@@ -682,21 +696,21 @@ void square_set_door_lock(struct chunk *c, int y, int x, int power)
 /**
  * Return the power of the lock on a door
  */
-int square_door_power(struct chunk *c, int y, int x)
+int square_door_power(struct chunk *c, struct loc grid)
 {
 	struct trap_kind *lock = lookup_trap("door lock");
 	struct trap *trap;
 
 	/* Verify it's a closed door */
-	if (!square_iscloseddoor(c, y, x))
+	if (!square_iscloseddoor(c, grid))
 		return 0;
 
 	/* Is there a lock there? */
-	if (!square_trap_specific(c, y, x, lock->tidx))
+	if (!square_trap_specific(c, grid, lock->tidx))
 		return 0;
 
 	/* Get the power and return it */
-	trap = square_trap(c, y, x);
+	trap = square_trap(c, grid);
 	while (trap) {
 		if (trap->kind == lock)
 			return trap->power;
