@@ -23,6 +23,7 @@
 #include "game-input.h"
 #include "obj-tval.h"
 #include "player.h"
+#include "player-spell.h"
 #include "ui-birth.h"
 #include "ui-display.h"
 #include "ui-game.h"
@@ -230,54 +231,13 @@ static void skill_help(const int r_skills[], const int c_skills[], int mhp, int
 		text_out_e("\n");
 }
 
-static const char *get_flag_desc(bitflag flag)
-{
-	switch (flag)
-	{
-		case OF_SUST_STR: return "Sustains strength";
-		case OF_SUST_DEX: return "Sustains dexterity";
-		case OF_SUST_CON: return "Sustains constitution";
-		case OF_PROT_BLIND: return "Resists blindness";
-		case OF_HOLD_LIFE: return "Sustains experience";
-		case OF_FREE_ACT: return "Resists paralysis";
-		case OF_REGEN: return "Regenerates quickly";
-		case OF_SEE_INVIS: return "Sees invisible creatures";
-
-		default: return "Undocumented flag";
-	}
-}
-
-static const char *get_resist_desc(int element)
-{
-	switch (element)
-	{
-		case ELEM_POIS: return "Resists poison";
-		case ELEM_LIGHT: return "Resists light damage";
-		case ELEM_DARK: return "Resists darkness damage";
-
-		default: return "Undocumented element";
-	}
-}
-
-static const char *get_pflag_desc(bitflag flag)
-{
-	switch (flag)
-	{
-		#define PF(a,b,c) case PF_##a: return c;
-		#include "list-player-flags.h"
-		#undef PF
-	default:
-		abort(); /* compilation consistency problem */
-	}
-}
-
 static void race_help(int i, void *db, const region *l)
 {
 	int j;
-	size_t k;
 	struct player_race *r = player_id2race(i);
 	int len = (STAT_MAX + 1) / 2;
 
+	struct player_ability *ability;
 	int n_flags = 0;
 	int flag_space = 3;
 
@@ -309,28 +269,23 @@ static void race_help(int i, void *db, const region *l)
 	skill_help(r->r_skills, NULL, r->r_mhp, r->r_exp, r->infra);
 	text_out_e("\n");
 
-	for (k = 1; k < OF_MAX; k++) {
+	for (ability = player_abilities; ability; ability = ability->next) {
 		if (n_flags >= flag_space) break;
-		if (!of_has(r->flags, k)) continue;
-		text_out_e("\n%s", get_flag_desc(k));
+		if (streq(ability->type, "object") &&
+			!of_has(r->flags, ability->index)) {
+			continue;
+		} else if (streq(ability->type, "player") &&
+				   !pf_has(r->pflags, ability->index)) {
+			continue;
+		} else if (streq(ability->type, "element") &&
+				   (r->el_info[ability->index].res_level != ability->value)) {
+			continue;
+		}
+		text_out_e("\n%s", ability->name);
 		n_flags++;
 	}
 
-	for (k = 0; k < ELEM_MAX; k++) {
-		if (n_flags >= flag_space) break;
-		if (r->el_info[k].res_level != 1) continue;
-		text_out_e("\n%s", get_resist_desc(k));
-		n_flags++;
-	}
-
-	for (k = 0; k < PF_MAX; k++) {
-		if (n_flags >= flag_space) break;
-		if (!pf_has(r->pflags, k)) continue;
-		text_out_e("\n%s", get_pflag_desc(k));
-		n_flags++;
-	}
-
-	while(n_flags < flag_space) {
+	while (n_flags < flag_space) {
 		text_out_e("\n");
 		n_flags++;
 	}
@@ -342,11 +297,11 @@ static void race_help(int i, void *db, const region *l)
 static void class_help(int i, void *db, const region *l)
 {
 	int j;
-	size_t k;
 	struct player_class *c = player_id2class(i);
 	const struct player_race *r = player->race;
 	int len = (STAT_MAX + 1) / 2;
 
+	struct player_ability *ability;
 	int n_flags = 0;
 	int flag_space = 5;
 
@@ -379,20 +334,49 @@ static void class_help(int i, void *db, const region *l)
 	skill_help(r->r_skills, c->c_skills, r->r_mhp + c->c_mhp,
 			   r->r_exp + c->c_exp, -1);
 
-	if (c->magic.spell_realm)
-		text_out_e("\nLearns %s magic", c->magic.spell_realm->name);
+	if (c->magic.total_spells) {
+		int count;
+		struct magic_realm *realm = class_magic_realms(c, &count), *realm_next;
+		char buf[120];
 
-	for (k = 0; k < PF_MAX; k++) {
-		const char *s;
+		my_strcpy(buf, realm->name, sizeof(buf));
+		realm_next = realm->next;
+		mem_free(realm);
+		realm = realm_next;
+		if (count > 1) {
+			while (realm) {
+				count--;
+				if (count) {
+					my_strcat(buf, ", ", sizeof(buf));
+				} else {
+					my_strcat(buf, " and ", sizeof(buf));
+				}
+				my_strcat(buf, realm->name, sizeof(buf));
+				realm_next = realm->next;
+				mem_free(realm);
+				realm = realm_next;
+			}
+		}
+		text_out_e("\nLearns %s magic", buf);
+	}
+
+	for (ability = player_abilities; ability; ability = ability->next) {
 		if (n_flags >= flag_space) break;
-		if (!pf_has(c->pflags, k)) continue;
-		s = get_pflag_desc(k);
-		if (!s) continue;
-		text_out_e("\n%s", s);
+		if (streq(ability->type, "object") &&
+			!of_has(c->flags, ability->index)) {
+			continue;
+		} else if (streq(ability->type, "player") &&
+				   !pf_has(c->pflags, ability->index)) {
+			continue;
+		} else if (streq(ability->type, "element")) {
+			continue;
+		}
+
+		text_out_e("\n%s", ability->name);
 		n_flags++;
 	}
 
-	while(n_flags < flag_space) {
+	while (n_flags < flag_space) {
 		text_out_e("\n");
 		n_flags++;
 	}
@@ -862,6 +846,9 @@ void get_screen_loc(size_t cursor, int *x, int *y, size_t n_lines, size_t *line_
 {
 	size_t lengths_so_far = 0;
 	size_t i;
+
+	if (!line_starts || !line_lengths) return;
+
 	for (i = 0; i < n_lines; i++) {
 		if (cursor >= line_starts[i]) {
 			if (cursor <= (line_starts[i] + line_lengths[i])) {
@@ -973,9 +960,11 @@ int edit_text(char *buffer, int buflen) {
 				if (atnull) {
 					/* Make sure we have enough room for a new character */
 					if ((cursor + 1) >= buflen) break;
+					if ((len + 1) >= buflen) break;
 				} else {
 					/* Make sure we have enough room to add a new character */
 					if ((cursor + 1) >= buflen) break;
+					if ((len + 1) >= buflen) break;
 
 					/* Move the rest of the buffer along to make room */
 					memmove(&buffer[cursor + 1], &buffer[cursor], len - cursor);
@@ -992,6 +981,8 @@ int edit_text(char *buffer, int buflen) {
 			}
 		}
 
+		mem_free(line_starts);
+		mem_free(line_lengths);
 		textblock_free(tb);
 	}
 
@@ -1027,7 +1018,7 @@ static enum birth_stage get_history_command(void)
 		switch (edit_text(history, sizeof(history))) {
 			case -1:
 				next = BIRTH_BACK;
-
+				break;
 			case 0:
 				cmdq_push(CMD_HISTORY_CHOICE);
 				cmd_set_arg_string(cmdq_peek(), "history", history);
@@ -1104,7 +1095,7 @@ int textui_do_birth(void)
 	bool done = false;
 
 	cmdq_push(CMD_BIRTH_INIT);
-	cmdq_execute(CMD_BIRTH);
+	cmdq_execute(CTX_BIRTH);
 
 	while (!done) {
 
@@ -1243,7 +1234,7 @@ int textui_do_birth(void)
 		current_stage = next;
 
 		/* Execute whatever commands have been sent */
-		cmdq_execute(CMD_BIRTH);
+		cmdq_execute(CTX_BIRTH);
 	}
 
 	return 0;
