@@ -977,16 +977,13 @@ static bool get_file_text(const char *suggested_name, char *path, size_t len)
 	/* Get filename */
 	my_strcpy(buf, suggested_name, sizeof buf);
 	
-	if(!arg_force_name) {
+	if (!arg_force_name) {
 			
 			if (!get_string("File name: ", buf, sizeof buf)) return false;
 
 			/* Make sure it's actually a filename */
 			if (buf[0] == '\0' || buf[0] == ' ') return false;
-	}
-
-	else {
-	
+	} else {
 		int len;
 		time_t ltime;
 		struct tm *today;
@@ -998,8 +995,9 @@ static bool get_file_text(const char *suggested_name, char *path, size_t len)
 		prt("File name: ", 0,0);
 
 		/* Overwrite the ".txt" that was added */
-		len = strlen(buf)-4;
-		strftime(buf+len, sizeof(buf)-len, "-%Y-%m-%d-%H-%M.txt", today);
+		assert(strlen(buf) >= 4);
+		len = strlen(buf) - 4;
+		strftime(buf + len, sizeof(buf) - len, "-%Y-%m-%d-%H-%M.txt", today);
 
 		/* Prompt the user to confirm or cancel the file dump */
 		if (!get_check(format("Confirm writing to %s? ", buf))) return false;
@@ -1159,7 +1157,7 @@ bool textui_get_rep_dir(int *dp, bool allow_5)
 			if (ke.mouse.button == 1) {
 				int y = KEY_GRID_Y(ke);
 				int x = KEY_GRID_X(ke);
-				struct loc from = loc(player->px, player->py);
+				struct loc from = player->grid;
 				struct loc to = loc(x, y);
 
 				dir = pathfind_direction_to(from, to);
@@ -1273,7 +1271,7 @@ bool textui_get_aim_dir(int *dp)
 					dir = 5;
 			} else if (ke.key.code == '\'') {
 				/* Set to closest target */
-				if (target_set_closest(TARGET_KILL))
+				if (target_set_closest(TARGET_KILL, NULL))
 					dir = 5;
 			} else if (ke.key.code == 't' || ke.key.code == '5' ||
 					   ke.key.code == '0' || ke.key.code == '.') {
@@ -1427,7 +1425,7 @@ ui_event textui_get_command(int *count)
 		/* Toggle on cursor if requested */
 		if (OPT(player, highlight_player)) {
 			Term_set_cursor(true);
-			move_cursor_relative(player->py, player->px);
+			move_cursor_relative(player->grid.y, player->grid.x);
 		}
 
 		/* Get a command */
@@ -1442,6 +1440,8 @@ ui_event textui_get_command(int *count)
 			bool keymap_ok = true;
 			switch (ke.key.code) {
 				case '0': {
+					if(ke.key.mods & KC_MOD_KEYPAD) break;
+
 					int c = textui_get_count();
 
 					if (c == -1 || !get_com_ex("Command: ", &ke))
