@@ -37,10 +37,12 @@
 #include "object.h"
 #include "player-calcs.h"
 #include "player-history.h"
+#include "player-util.h"
 #include "store.h"
 #include "target.h"
 #include "trap.h"
 #include "ui-context.h"
+#include "ui-equip-cmp.h"
 #include "ui-history.h"
 #include "ui-menu.h"
 #include "ui-mon-list.h"
@@ -519,20 +521,18 @@ static bool glyph_command(ui_event ke, bool *glyph_picker_ptr,
 			  int height, int width, byte *cur_attr_ptr,
 			  wchar_t *cur_char_ptr, int col, int row)
 {
-        static byte attr_old = 0;
+	static byte attr_old = 0;
 	static char char_old = 0;
 	
 	/* Get mouse movement */
 	if (*glyph_picker_ptr && (ke.type == EVT_MOUSE)) {
-	        byte a = *cur_attr_ptr;
-
 		int mx = logical_width(ke.mouse.x - col);
 		
-		if (ke.mouse.y != row + height/2) return false;
+		if (ke.mouse.y != row + height / 2) return false;
 		
 		if ((mx >= 0) && (mx < MAX_COLORS) && (ke.mouse.button == 1)) {
-		        /* Set the visual */
-		        *cur_attr_ptr = a = mx - 14;
+			/* Set the visual */
+			*cur_attr_ptr = mx - 14;
 
 			/* Accept change */
 			remove_tiles(col, row, glyph_picker_ptr, width, height);
@@ -732,7 +732,7 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 	int g_cur = 0, grp_old = -1; /* group list positions */
 	int o_cur = 0;					/* object list positions */
 	int g_o_count = 0;				 /* object count for group */
-	int oid = -1;  				/* object identifiers */
+	int oid;  				/* object identifiers */
 
 	region title_area = { 0, 0, 0, 4 };
 	region group_region = { 0, 6, MISSING, -2 };
@@ -1101,7 +1101,7 @@ static struct
 	{ L"h",        "Hominids (Elves, Dwarves)" },
 	{ L"M",        "Hydras" },
 	{ L"i",        "Icky Things" },
-	{ L"lFI",      "Insects" },
+	{ L"FI",       "Insects" },
 	{ L"j",        "Jellies" },
 	{ L"K",        "Killer Beetles" },
 	{ L"k",        "Kobolds" },
@@ -1119,6 +1119,7 @@ static struct
 	{ L"S",        "Scorpions/Spiders" },
 	{ L"s",        "Skeletons/Drujs" },
 	{ L"J",        "Snakes" },
+	{ L"l",        "Trees/Ents" },
 	{ L"T",        "Trolls" },
 	{ L"V",        "Vampires" },
 	{ L"W",        "Wights/Wraiths" },
@@ -1167,14 +1168,16 @@ static void display_monster(int col, int row, bool cursor, int oid)
 	big_pad(66, row, a, c);
 
 	/* Display kills */
-	if (rf_has(race->flags, RF_UNIQUE))
+	if (!race->rarity) {
+		put_str(format("%s", "shape"), row, 70);
+	} else if (rf_has(race->flags, RF_UNIQUE)) {
 		put_str(format("%s", (race->max_num == 0)?  " dead" : "alive"),
 				row, 70);
-	else
+	} else {
 		put_str(format("%5d", lore->pkills), row, 70);
+	}
 }
 
-
 static int m_cmp_race(const void *a, const void *b)
 {
 	const int a_val = *(const int *)a;
@@ -1339,10 +1342,8 @@ static void do_cmd_knowledge_monsters(const char *name, int row)
 
 		for (j = 0; j < N_ELEMENTS(monster_group) - 1; j++) {
 			const wchar_t *pat = monster_group[j].chars;
-			if (j == 0 && !rf_has(race->flags, RF_UNIQUE))
-				continue;
-			else if (j > 0 && !wcschr(pat, race->d_char))
-				continue;
+			if (j == 0 && !rf_has(race->flags, RF_UNIQUE)) continue;
+			if (j > 0 && !wcschr(pat, race->d_char)) continue;
 
 			monsters[m_count] = m_count;
 			default_join[m_count].oid = i;
@@ -1377,6 +1378,9 @@ static const grouper object_text_order[] =
  	{TV_MUSHROOM,		"Mushroom"		},
 	{TV_PRAYER_BOOK,	"Priest Book"	},
 	{TV_MAGIC_BOOK,		"Magic Book"	},
+	{TV_NATURE_BOOK,	"Nature Book"	},
+	{TV_SHADOW_BOOK,	"Shadow Book"	},
+	{TV_OTHER_BOOK,		"Mystery Book"	},
 	{TV_LIGHT,			"Light"			},
 	{TV_FLASK,			"Flask"			},
 	{TV_SWORD,			"Sword"			},
@@ -1438,16 +1442,19 @@ static struct object *find_artifact(struct artifact *artifact)
 	struct object *obj;
 
 	/* Ground objects */
-	for (y = 1; y < cave->height; y++)
-		for (x = 1; x < cave->width; x++)
-			for (obj = square_object(cave, y, x); obj; obj = obj->next)
-				if (obj->artifact == artifact)
-					return obj;
+	for (y = 1; y < cave->height; y++) {
+		for (x = 1; x < cave->width; x++) {
+			struct loc grid = loc(x, y);
+			for (obj = square_object(cave, grid); obj; obj = obj->next) {
+				if (obj->artifact == artifact) return obj;
+			}
+		}
+	}
 
 	/* Player objects */
-	for (obj = player->gear; obj; obj = obj->next)
-		if (obj->artifact == artifact)
-			return obj;
+	for (obj = player->gear; obj; obj = obj->next) {
+		if (obj->artifact == artifact) return obj;
+	}
 
 	/* Monster objects */
 	for (i = cave_monster_max(cave) - 1; i >= 1; i--) {
@@ -1455,8 +1462,7 @@ static struct object *find_artifact(struct artifact *artifact)
 		obj = mon ? mon->held_obj : NULL;
 
 		while (obj) {
-			if (obj->artifact == artifact)
-				return obj;
+			if (obj->artifact == artifact) return obj;
 			obj = obj->next;
 		}
 	}
@@ -1464,11 +1470,39 @@ static struct object *find_artifact(struct artifact *artifact)
 	/* Store objects */
 	for (i = 0; i < MAX_STORES; i++) {
 		struct store *s = &stores[i];
-		for (obj = s->stock; obj; obj = obj->next)
-			if (obj->artifact == artifact)
-				return obj;
+		for (obj = s->stock; obj; obj = obj->next) {
+			if (obj->artifact == artifact) return obj;
+		}
 	}
 
+	/* Stored chunk objects */
+	for (i = 0; i < chunk_list_max; i++) {
+		struct chunk *c = chunk_list[i];
+		int j;
+		if (strstr(c->name, "known")) continue;
+
+		/* Ground objects */
+		for (y = 1; y < c->height; y++) {
+			for (x = 1; x < c->width; x++) {
+				struct loc grid = loc(x, y);
+				for (obj = square_object(c, grid); obj; obj = obj->next) {
+					if (obj->artifact == artifact) return obj;
+				}
+			}
+		}
+
+		/* Monster objects */
+		for (j = cave_monster_max(c) - 1; j >= 1; j--) {
+			struct monster *mon = cave_monster(c, j);
+			obj = mon ? mon->held_obj : NULL;
+
+			while (obj) {
+				if (obj->artifact == artifact) return obj;
+				obj = obj->next;
+			}
+		}
+	}	
+
 	return NULL;
 }
 
@@ -1860,6 +1894,9 @@ static int o_cmp_tval(const void *a, const void *b)
 		case TV_LIGHT:
 		case TV_MAGIC_BOOK:
 		case TV_PRAYER_BOOK:
+		case TV_NATURE_BOOK:
+		case TV_SHADOW_BOOK:
+		case TV_OTHER_BOOK:
 		case TV_DRAG_ARMOR:
 			/* leave sorted by sval */
 			break;
@@ -1886,6 +1923,7 @@ static int obj2gid(int oid)
 static wchar_t *o_xchar(int oid)
 {
 	struct object_kind *kind = objkind_byid(oid);
+	if (!kind) return 0;
 
 	if (!kind->flavor || kind->aware)
 		return &kind_x_char[kind->kidx];
@@ -1896,6 +1934,7 @@ static wchar_t *o_xchar(int oid)
 static byte *o_xattr(int oid)
 {
 	struct object_kind *kind = objkind_byid(oid);
+	if (!kind) return NULL;
 
 	if (!kind->flavor || kind->aware)
 		return &kind_x_attr[kind->kidx];
@@ -1913,6 +1952,8 @@ static const char *o_xtra_prompt(int oid)
 	const char *no_insc = ", 's' to toggle ignore, 'r'ecall, '{'";
 	const char *with_insc = ", 's' to toggle ignore, 'r'ecall, '{', '}'";
 
+	if (!kind) return NULL;
+
 	/* Appropriate prompt */
 	if (kind->aware)
 		return kind->note_aware ? with_insc : no_insc;
@@ -1926,6 +1967,7 @@ static const char *o_xtra_prompt(int oid)
 static void o_xtra_act(struct keypress ch, int oid)
 {
 	struct object_kind *k = objkind_byid(oid);
+	if (!k) return;
 
 	/* Toggle ignore */
 	if (ignore_tval(k->tval) && (ch.code == 's' || ch.code == 'S')) {
@@ -2203,13 +2245,13 @@ static void display_feature(int col, int row, bool cursor, int oid )
 		/* Display symbols */
 		col = 65;
 		col += big_pad(col, row, feat_x_attr[LIGHTING_DARK][feat->fidx],
-				feat_x_char[LIGHTING_DARK][feat->fidx]);
+					   feat_x_char[LIGHTING_DARK][feat->fidx]);
 		col += big_pad(col, row, feat_x_attr[LIGHTING_LIT][feat->fidx],
-				feat_x_char[LIGHTING_LIT][feat->fidx]);
+					   feat_x_char[LIGHTING_LIT][feat->fidx]);
 		col += big_pad(col, row, feat_x_attr[LIGHTING_TORCH][feat->fidx],
-				feat_x_char[LIGHTING_TORCH][feat->fidx]);
-		col += big_pad(col, row, feat_x_attr[LIGHTING_LOS][feat->fidx],
-				feat_x_char[LIGHTING_LOS][feat->fidx]);
+					   feat_x_char[LIGHTING_TORCH][feat->fidx]);
+		(void) big_pad(col, row, feat_x_attr[LIGHTING_LOS][feat->fidx],
+					   feat_x_char[LIGHTING_LOS][feat->fidx]);
 	}
 }
 
@@ -2371,7 +2413,7 @@ static void display_trap(int col, int row, bool cursor, int oid )
 				trap_x_char[LIGHTING_LIT][trap->tidx]);
 		col += big_pad(col, row, trap_x_attr[LIGHTING_TORCH][trap->tidx],
 				trap_x_char[LIGHTING_TORCH][trap->tidx]);
-		col += big_pad(col, row, trap_x_attr[LIGHTING_LOS][trap->tidx],
+		(void) big_pad(col, row, trap_x_attr[LIGHTING_LOS][trap->tidx],
 				trap_x_char[LIGHTING_LOS][trap->tidx]);
 	}
 }
@@ -2380,7 +2422,7 @@ static int trap_order(int trap)
 {
 	const struct trap_kind *t = &trap_info[trap];
 
-	if (trf_has(t->flags, TRF_RUNE))
+	if (trf_has(t->flags, TRF_GLYPH))
 		return 0;
 	else if (trf_has(t->flags, TRF_LOCK))
 		return 1;
@@ -2536,6 +2578,11 @@ static void do_cmd_knowledge_history(const char *name, int row)
 	history_display();
 }
 
+static void do_cmd_knowledge_equip_cmp(const char* name, int row)
+{
+	equip_cmp_display();
+}
+
 
 /**
  * Definition of the "player knowledge" menu.
@@ -2552,13 +2599,14 @@ static menu_action knowledge_actions[] =
 { 0, 0, "Display contents of general store", do_cmd_knowledge_store     },
 { 0, 0, "Display contents of armourer",      do_cmd_knowledge_store     },
 { 0, 0, "Display contents of weaponsmith",   do_cmd_knowledge_store     },
-{ 0, 0, "Display contents of temple",   	   do_cmd_knowledge_store     },
+{ 0, 0, "Display contents of bookseller",    do_cmd_knowledge_store     },
 { 0, 0, "Display contents of alchemist",     do_cmd_knowledge_store     },
 { 0, 0, "Display contents of magic shop",    do_cmd_knowledge_store     },
 { 0, 0, "Display contents of black market",  do_cmd_knowledge_store     },
 { 0, 0, "Display contents of home",   	   do_cmd_knowledge_store     },
 { 0, 0, "Display hall of fame",       	   do_cmd_knowledge_scores    },
 { 0, 0, "Display character history",  	   do_cmd_knowledge_history   },
+{ 0, 0, "Display equipable comparison",    do_cmd_knowledge_equip_cmp },
 };
 
 static struct menu knowledge_menu;
@@ -2594,6 +2642,7 @@ void textui_knowledge_init(void)
 			obj_group_order[i] = -1;
 
 		for (i = 0; 0 != object_text_order[i].tval; i++) {
+			if (kb_info[object_text_order[i].tval].num_svals == 0) continue;
 			if (object_text_order[i].name) gid = i;
 			obj_group_order[object_text_order[i].tval] = gid;
 		}
@@ -2608,7 +2657,7 @@ void textui_knowledge_init(void)
 void textui_browse_knowledge(void)
 {
 	int i, rune_max = max_runes();
-	region knowledge_region = { 0, 0, -1, 19 };
+	region knowledge_region = { 0, 0, -1, 2 + (int)N_ELEMENTS(knowledge_actions) };
 
 	/* Runes */
 	knowledge_actions[1].flags = MN_ACT_GRAYED;
@@ -2730,7 +2779,7 @@ void do_cmd_messages(void)
 			Term_putstr(0, hgt - 3 - j, -1, attr, msg);
 
 			/* Highlight "shower" */
-			if (shower[0]) {
+			if (strlen(shower)) {
 				str = msg;
 
 				/* Display matches */
@@ -2751,7 +2800,7 @@ void do_cmd_messages(void)
 				   i, i + j - 1, n, q), 0, 0);
 
 		/* Display prompt (not very informative) */
-		if (shower[0])
+		if (strlen(shower))
 			prt("[Movement keys to navigate, '-' for next, '=' to find]",
 				hgt - 1, 0);
 		else
@@ -2829,7 +2878,7 @@ void do_cmd_messages(void)
 		}
 
 		/* Find the next item */
-		if (ke.key.code == '-' && shower[0]) {
+		if (ke.key.code == '-' && strlen(shower)) {
 			s16b z;
 
 			/* Scan messages */
@@ -2886,7 +2935,9 @@ void do_cmd_inven(void)
 				/* Track the object */
 				track_object(player->upkeep, obj);
 
-				while ((ret = context_menu_object(obj)) == 2);
+				if (!player_is_shapechanged(player)) {
+					while ((ret = context_menu_object(obj)) == 2);
+				}
 			}
 		} else {
 			/* Load screen */
@@ -2929,7 +2980,9 @@ void do_cmd_equip(void)
 				/* Track the object */
 				track_object(player->upkeep, obj);
 
-				while ((ret = context_menu_object(obj)) == 2);
+				if (!player_is_shapechanged(player)) {
+					while ((ret = context_menu_object(obj)) == 2);
+				}
 
 				/* Stay in "equipment" mode */
 				player->upkeep->command_wrk = (USE_EQUIP);
@@ -2975,7 +3028,9 @@ void do_cmd_quiver(void)
 				/* Track the object */
 				track_object(player->upkeep, obj);
 
-				while ((ret = context_menu_object(obj)) == 2);
+				if (!player_is_shapechanged(player)) {
+					while  ((ret = context_menu_object(obj)) == 2);
+				}
 
 				/* Stay in "quiver" mode */
 				player->upkeep->command_wrk = (USE_QUIVER);
@@ -3211,7 +3266,7 @@ static void lookup_symbol(char sym, char *buf, size_t max)
  */
 void do_cmd_query_symbol(void)
 {
-	int i, n;
+	int idx, num;
 	char buf[128];
 
 	char sym;
@@ -3250,9 +3305,9 @@ void do_cmd_query_symbol(void)
 	who = mem_zalloc(z_info->r_max * sizeof(u16b));
 
 	/* Collect matching monsters */
-	for (n = 0, i = 1; i < z_info->r_max - 1; i++) {
-		struct monster_race *race = &r_info[i];
-		struct monster_lore *lore = &l_list[i];
+	for (num = 0, idx = 1; idx < z_info->r_max - 1; idx++) {
+		struct monster_race *race = &r_info[idx];
+		struct monster_lore *lore = &l_list[idx];
 
 		/* Nothing to recall */
 		if (!lore->all_known && !lore->sights)
@@ -3265,14 +3320,13 @@ void do_cmd_query_symbol(void)
 		if (uniq && !rf_has(race->flags, RF_UNIQUE)) continue;
 
 		/* Collect "appropriate" monsters */
-		if (all || char_matches_key(race->d_char, sym)) who[n++] = i;
+		if (all || char_matches_key(race->d_char, sym)) who[num++] = idx;
 	}
 
-	/* Nothing to recall */
-	if (!n) {
-		/* XXX XXX Free the "who" array */
+	/* No monsters to recall */
+	if (!num) {
+		/* Free the "who" array */
 		mem_free(who);
-
 		return;
 	}
 
@@ -3288,35 +3342,32 @@ void do_cmd_query_symbol(void)
 	/* Interpret the response */
 	if (query.code == 'k') {
 		/* Sort by kills (and level) */
-		sort(who, n, sizeof(*who), cmp_pkill);
+		sort(who, num, sizeof(*who), cmp_pkill);
 	} else if (query.code == 'y' || query.code == 'p') {
 		/* Sort by level; accept 'p' as legacy */
-		sort(who, n, sizeof(*who), cmp_level);
+		sort(who, num, sizeof(*who), cmp_level);
 	} else {
 		/* Any unsupported response is "nope, no history please" */
-	
-		/* XXX XXX Free the "who" array */
 		mem_free(who);
-
 		return;
 	}
 
-	/* Start at the end */
-	i = n - 1;
+	/* Start at the end, as the array is sorted lowest to highest */
+	idx = num - 1;
 
 	/* Scan the monster memory */
 	while (1) {
 		textblock *tb;
 
 		/* Extract a race */
-		int r_idx = who[i];
+		int r_idx = who[idx];
 		struct monster_race *race = &r_info[r_idx];
 		struct monster_lore *lore = &l_list[r_idx];
 
-		/* Hack -- Auto-recall */
+		/* Auto-recall */
 		monster_race_track(player->upkeep, race);
 
-		/* Hack -- Handle stuff */
+		/* Do any necessary updates or redraws */
 		handle_stuff(player);
 
 		tb = textblock_new();
@@ -3345,13 +3396,21 @@ void do_cmd_query_symbol(void)
 		/* Stop scanning */
 		if (query.code == ESCAPE) break;
 
-		/* Move to "prev" or "next" monster */
+		/* Move to previous or next monster */
 		if (query.code == '-') {
-			if (++i == n)
-				i = 0;
+			/* Previous is a step forward in the array */
+			idx++;
+			/* Wrap if we're at the end of the array */
+			if (idx == num) {
+				idx = 0;
+			}
 		} else {
-			if (i-- == 0)
-				i = n - 1;
+			/* Next is a step back in the array */
+			idx--;
+			/* Wrap if we're at the start of the array */
+			if (idx < 0) {
+				idx = num - 1;
+			}
 		}
 	}
 
