@@ -1043,6 +1043,13 @@ static void do_dump_autoinsc(const char *title, int row) {
 	dump_pref_file(dump_autoinscriptions, "Dump autoinscriptions", 20);
 }
 
+/**
+ * Write character screen customizations to a file.
+ */
+static void do_dump_charscreen_opt(const char *title, int row) {
+	dump_pref_file(dump_ui_entry_renderers, "Dump char screen options", 20);
+}
+
 /**
  * Load a pref file.
  */
@@ -1451,6 +1458,9 @@ static tval_desc sval_dependent[] =
 	{ TV_MUSHROOM,		"Mushrooms" },
 	{ TV_MAGIC_BOOK,	"Magic books" },
 	{ TV_PRAYER_BOOK,	"Prayer books" },
+	{ TV_NATURE_BOOK,	"Nature books" },
+	{ TV_SHADOW_BOOK,	"Shadow books" },
+	{ TV_OTHER_BOOK,	"Mystery books" },
 	{ TV_LIGHT,			"Lights" },
 	{ TV_FLASK,			"Flasks of oil" },
 	{ TV_GOLD,			"Money" },
@@ -1465,8 +1475,8 @@ bool ignore_tval(int tval)
 	size_t i;
 
 	/* Only ignore if the tval's allowed */
-	for (i = 0; i < N_ELEMENTS(sval_dependent); i++)
-	{
+	for (i = 0; i < N_ELEMENTS(sval_dependent); i++) {
+		if (kb_info[tval].num_svals == 0) continue;
 		if (tval == sval_dependent[i].tval)
 			return true;
 	}
@@ -1601,6 +1611,9 @@ static bool sval_menu(int tval, const char *desc)
 		case TV_LIGHT:
 		case TV_MAGIC_BOOK:
 		case TV_PRAYER_BOOK:
+		case TV_NATURE_BOOK:
+		case TV_SHADOW_BOOK:
+		case TV_OTHER_BOOK:
 		case TV_DRAG_ARMOR:
 		case TV_GOLD:
 			/* leave sorted by sval */
@@ -1628,6 +1641,7 @@ static bool sval_menu(int tval, const char *desc)
 	menu_select(menu, 0, false);
 
 	/* Free memory */
+	mem_free(menu);
 	mem_free(choices);
 
 	/* Load screen */
@@ -1809,6 +1823,7 @@ static menu_action option_actions[] =
 	{ 0, 0, NULL, NULL },
 	{ 0, 's', "Save subwindow setup to pref file", do_dump_options },
 	{ 0, 't', "Save autoinscriptions to pref file", do_dump_autoinsc },
+	{ 0, 'u', "Save char screen options to pref file", do_dump_charscreen_opt },
 	{ 0, 0, NULL, NULL },
 	{ 0, 'l', "Load a user pref file", options_load_pref_file },
 	{ 0, 'k', "Edit keymaps (advanced)", do_cmd_keymaps },
