@@ -27,6 +27,7 @@
 #include "mon-make.h"
 #include "mon-util.h"
 #include "monster.h"
+#include "obj-curse.h"
 #include "obj-desc.h"
 #include "obj-gear.h"
 #include "obj-knowledge.h"
@@ -108,6 +109,7 @@ static void wiz_proj_demo(void)
 	clear_from(0);
 	menu_select(m, 0, false);
 	screen_load();
+	mem_free(m);
 }
 
 
@@ -134,9 +136,6 @@ static s16b get_idx_from_name(char *s)
  */
 static void do_cmd_wiz_hack_nick(void)
 {
-	int py = player->py;
-	int px = player->px;
-
 	int i, y, x;
 
 	char kp;
@@ -147,16 +146,17 @@ static void do_cmd_wiz_hack_nick(void)
 		for (y = Term->offset_y; y < Term->offset_y + SCREEN_HGT; y++)
 			for (x = Term->offset_x; x < Term->offset_x + SCREEN_WID; x++) {
 				byte a = COLOUR_RED;
+				struct loc grid = loc(x, y);
 
-				if (!square_in_bounds_fully(cave, y, x)) continue;
+				if (!square_in_bounds_fully(cave, grid)) continue;
 
 				/* Display proper noise */
 				if (cave->noise.grids[y][x] != i) continue;
 
 				/* Display player/floors/walls */
-				if ((y == py) && (x == px))
+				if (loc_eq(grid, player->grid))
 					print_rel(L'@', a, y, x);
-				else if (square_ispassable(cave, y, x))
+				else if (square_ispassable(cave, grid))
 					print_rel(L'*', a, y, x);
 				else
 					print_rel(L'#', a, y, x);
@@ -176,16 +176,17 @@ static void do_cmd_wiz_hack_nick(void)
 		for (y = Term->offset_y; y < Term->offset_y + SCREEN_HGT; y++)
 			for (x = Term->offset_x; x < Term->offset_x + SCREEN_WID; x++) {
 				byte a = COLOUR_YELLOW;
+				struct loc grid = loc(x, y);
 
-				if (!square_in_bounds_fully(cave, y, x)) continue;
+				if (!square_in_bounds_fully(cave, grid)) continue;
 
 				/* Display proper smell */
 				if (cave->scent.grids[y][x] != i) continue;
 
 				/* Display player/floors/walls */
-				if ((y == py) && (x == px))
+				if (loc_eq(grid, player->grid))
 					print_rel(L'@', a, y, x);
-				else if (square_ispassable(cave, y, x))
+				else if (square_ispassable(cave, grid))
 					print_rel(L'*', a, y, x);
 				else
 					print_rel(L'#', a, y, x);
@@ -271,22 +272,24 @@ static void do_cmd_keylog(void) {
  */
 static void do_cmd_wiz_bamf(void)
 {
-	int x = 0, y = 0;
+	struct loc grid = loc(0, 0);
 
 	/* Use the targeting function. */
 	if (!target_set_interactive(TARGET_LOOK, -1, -1))
 		return;
 
 	/* grab the target coords. */
-	target_get(&x, &y);
+	target_get(&grid);
 
 	/* Test for passable terrain. */
-	if (!square_ispassable(cave, y, x))
+	if (!square_ispassable(cave, grid)) {
 		msg("The square you are aiming for is impassable.");
 
-	/* Teleport to the target */
-	else
-		effect_simple(EF_TELEPORT_TO, source_player(), "0", y, x, 0, NULL);
+	} else {
+		/* Teleport to the target */
+		effect_simple(EF_TELEPORT_TO, source_player(), "0", 0, 0, 0, grid.y,
+					  grid.x, NULL);
+	}
 }
 
 
@@ -600,7 +603,7 @@ static void wiz_create_item_drop_object(struct object *obj)
 	obj->origin_depth = player->depth;
 
 	/* Drop the object from heaven */
-	drop_near(cave, &obj, 0, player->py, player->px, true);
+	drop_near(cave, &obj, 0, player->grid, true, true);
 }
 
 /**
@@ -751,7 +754,7 @@ static bool wiz_create_item_action(struct menu *m, const ui_event *e, int oid)
 	char buf[80];
 	char title[80];
 
-	int choice[60];
+	int choice[70];
 	int num;
 
 	int i;
@@ -811,6 +814,7 @@ static bool wiz_create_item_action(struct menu *m, const ui_event *e, int oid)
 	ret = menu_select(menu, 0, false);
 
 	screen_load();
+	mem_free(menu);
 
 	return (ret.type == EVT_ESCAPE);
 }
@@ -869,6 +873,7 @@ static void wiz_create_item(bool art)
 	menu_select(menu, 0, false);
 
 	screen_load();
+	mem_free(menu);
 	
 	/* Redraw map */
 	player->upkeep->redraw |= (PR_MAP | PR_ITEMLIST);
@@ -981,7 +986,9 @@ static void wiz_reroll_item(struct object *obj)
 	/* Get new copy, hack off slays and brands */
 	new = mem_zalloc(sizeof(*new));
 	object_copy(new, obj);
+	mem_free(new->slays);
 	new->slays = NULL;
+	mem_free(new->brands);
 	new->brands = NULL;
 
 	/* Main loop. Ask for magification and artifactification */
@@ -1052,6 +1059,7 @@ static void wiz_reroll_item(struct object *obj)
 	}
 
 	/* Free the copy */
+	object_wipe(new);
 	mem_free(new);
 }
 
@@ -1263,7 +1271,50 @@ static void wiz_quantity_item(struct object *obj, bool carried)
  */
 static void wiz_tweak_curse(struct object *obj)
 {
-	// DO SOMETHING - NRM
+	const char *p;
+	char tmp_val[80];
+	int val, pval;
+
+	/* Get curse name */
+	p = "Enter curse name or index: ";
+	strnfmt(tmp_val, sizeof(tmp_val), "0");
+	if (! get_string(p, tmp_val, sizeof(tmp_val))) return;
+
+	/* Accept index or name */
+	val = get_idx_from_name(tmp_val);
+	if (! val) {
+		val = lookup_curse(tmp_val);
+	}
+	if (val <= 0 || val >= z_info->curse_max) {
+		return;
+	}
+
+	/* Get power */
+	p = "Enter curse power (0 removes): ";
+	strnfmt(tmp_val, sizeof(tmp_val), "0");
+	if (! get_string(p, tmp_val, 30)) return;
+	pval = get_idx_from_name(tmp_val);
+	if (pval < 0) {
+		return;
+	}
+
+	/* Apply */
+	if (pval) {
+		append_object_curse(obj, val, pval);
+	} else if (obj->curses) {
+		obj->curses[val].power = 0;
+
+		/* Duplicates logic from non-public check_object_curses(). */
+		int i;
+
+		for (i = 0; i < z_info->curse_max; i++) {
+			if (obj->curses[i].power) {
+				return;
+			}
+		}
+		mem_free(obj->curses);
+		obj->curses = NULL;
+	}
 }
 
 
@@ -1372,7 +1423,7 @@ void wiz_cheat_death(void)
 	(void)player_clear_timed(player, TMD_CUT, true);
 
 	/* Prevent starvation */
-	player_set_food(player, PY_FOOD_MAX - 1);
+	player_set_timed(player, TMD_FOOD, PY_FOOD_MAX - 1, false);
 
 	/* Cancel recall */
 	if (player->word_recall)
@@ -1419,14 +1470,14 @@ static void do_cmd_wiz_cure_all(void)
 	}
 
 	/* Restore stats */
-	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_STR, 0, 0, NULL);
-	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_INT, 0, 0, NULL);
-	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_WIS, 0, 0, NULL);
-	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_DEX, 0, 0, NULL);
-	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_CON, 0, 0, NULL);
+	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_STR, 0, 0, 0, 0, NULL);
+	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_INT, 0, 0, 0, 0, NULL);
+	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_WIS, 0, 0, 0, 0, NULL);
+	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_DEX, 0, 0, 0, 0, NULL);
+	effect_simple(EF_RESTORE_STAT, source_player(), "0", STAT_CON, 0, 0, 0, 0, NULL);
 
 	/* Restore the level */
-	effect_simple(EF_RESTORE_EXP, source_player(), "0", 1, 0, 0, NULL);
+	effect_simple(EF_RESTORE_EXP, source_none(), "0", 0, 0, 0, 0, 0, NULL);
 
 	/* Heal the player */
 	player->chp = player->mhp;
@@ -1449,7 +1500,7 @@ static void do_cmd_wiz_cure_all(void)
 	(void)player_clear_timed(player, TMD_AMNESIA, true);
 
 	/* No longer hungry */
-	player_set_food(player, PY_FOOD_MAX - 1);
+	player_set_timed(player, TMD_FOOD, PY_FOOD_FULL - 1, false);
 
 	/* Redraw everything */
 	do_cmd_redraw();
@@ -1581,7 +1632,7 @@ static void do_cmd_wiz_summon(int num)
 	int i;
 
 	for (i = 0; i < num; i++)
-		effect_simple(EF_SUMMON, source_player(), "1", 0, 0, 0, NULL);
+		effect_simple(EF_SUMMON, source_player(), "1", 0, 0, 0, 0, 0, NULL);
 }
 
 
@@ -1590,27 +1641,28 @@ static void do_cmd_wiz_summon(int num)
  */
 static void do_cmd_wiz_named(struct monster_race *r, bool slp)
 {
-	int py = player->py;
-	int px = player->px;
-
-	int i, x, y;
+	int i;
+	struct monster_group_info info = { 0, 0 };
 
 	/* Paranoia */
 	assert(r);
 
 	/* Try 10 times */
 	for (i = 0; i < 10; i++) {
+		struct loc grid;
 		int d = 1;
 
 		/* Pick a location */
-		scatter(cave, &y, &x, py, px, d, true);
+		scatter(cave, &grid, player->grid, d, true);
 
 		/* Require empty grids */
-		if (!square_isempty(cave, y, x)) continue;
+		if (!square_isempty(cave, grid)) continue;
 
 		/* Place it (allow groups) */
-		if (place_new_monster(cave, y, x, r, slp, true, ORIGIN_DROP_WIZARD))
+		if (place_new_monster(cave, grid, r, slp, true, info,
+							  ORIGIN_DROP_WIZARD)) {
 			break;
+		}
 	}
 }
 
@@ -1648,13 +1700,8 @@ static void do_cmd_wiz_zap(int d)
  */
 static void do_cmd_wiz_query(void)
 {
-	int py = player->py;
-	int px = player->px;
-
 	int y, x;
-
 	char cmd;
-
 	int flag = 0;
 
 
@@ -1684,22 +1731,23 @@ static void do_cmd_wiz_query(void)
 	for (y = Term->offset_y; y < Term->offset_y + SCREEN_HGT; y++) {
 		for (x = Term->offset_x; x < Term->offset_x + SCREEN_WID; x++) {
 			byte a = COLOUR_RED;
+			struct loc grid = loc(x, y);
 
-			if (!square_in_bounds_fully(cave, y, x)) continue;
+			if (!square_in_bounds_fully(cave, grid)) continue;
 
 			/* Given flag, show only those grids */
-			if (flag && !sqinfo_has(cave->squares[y][x].info, flag)) continue;
+			if (flag && !sqinfo_has(square(cave, grid).info, flag)) continue;
 
 			/* Given no flag, show known grids */
-			if (!flag && (!square_isknown(cave, y, x))) continue;
+			if (!flag && (!square_isknown(cave, grid))) continue;
 
 			/* Color */
-			if (square_ispassable(cave, y, x)) a = COLOUR_YELLOW;
+			if (square_ispassable(cave, grid)) a = COLOUR_YELLOW;
 
 			/* Display player/floors/walls */
-			if ((y == py) && (x == px))
+			if (loc_eq(grid, player->grid))
 				print_rel(L'@', a, y, x);
-			else if (square_ispassable(cave, y, x))
+			else if (square_ispassable(cave, grid))
 				print_rel(L'*', a, y, x);
 			else
 				print_rel(L'#', a, y, x);
@@ -1722,9 +1770,6 @@ static void do_cmd_wiz_query(void)
  */
 static void do_cmd_wiz_features(void)
 {
-	int py = player->py;
-	int px = player->px;
-
 	int y, x;
 
 	char cmd;
@@ -1768,15 +1813,15 @@ static void do_cmd_wiz_features(void)
 		/* Stairs */
 		case 't': feat = featt; length = 2; break;
 		/* Closed doors */
-		case 'c': feat = featc; length = 8; break;
+		case 'c': feat = featc; length = 1; break;
 		/* Doors */
-		case 'd': feat = featd; length = 11; break;
+		case 'd': feat = featd; length = 4; break;
 		/* Secret doors */
 		case 'h': feat = feath; length = 1; break;
 		/* Magma */
-		case 'm': feat = featm; length = 3; break;
+		case 'm': feat = featm; length = 2; break;
 		/* Quartz */
-		case 'q': feat = featq; length = 3; break;
+		case 'q': feat = featq; length = 2; break;
 		/* Granite */
 		case 'g': feat = featg; length = 1; break;
 		/* Permanent wall */
@@ -1785,30 +1830,33 @@ static void do_cmd_wiz_features(void)
 		case 'r': feat = featr; length = 1; break;
 		/* Passable rubble */
 		case 'a': feat = feata; length = 1; break;
+		/* Invalid entry */
+		default: return;
 	}
 
 	/* Scan map */
 	for (y = Term->offset_y; y < Term->offset_y + SCREEN_HGT; y++) {
 		for (x = Term->offset_x; x < Term->offset_x + SCREEN_WID; x++) {
+			struct loc grid = loc(x, y);
 			byte a = COLOUR_RED;
 			bool show = false;
 			int i;
 
-			if (!square_in_bounds_fully(cave, y, x)) continue;
+			if (!square_in_bounds_fully(cave, loc(x, y))) continue;
 
 			/* Given feature, show only those grids */
 			for (i = 0; i < length; i++)
-				if (cave->squares[y][x].feat == feat[i]) show = true;
+				if (square(cave, grid).feat == feat[i]) show = true;
 
 			/* Color */
-			if (square_ispassable(cave, y, x)) a = COLOUR_YELLOW;
+			if (square_ispassable(cave, grid)) a = COLOUR_YELLOW;
 
 			if (!show) continue;
 
 			/* Display player/floors/walls */
-			if ((y == py) && (x == px))
+			if (loc_eq(grid, player->grid))
 				print_rel(L'@', a, y, x);
-			else if (square_ispassable(cave, y, x))
+			else if (square_ispassable(cave, grid))
 				print_rel(L'*', a, y, x);
 			else
 				print_rel(L'#', a, y, x);
@@ -1831,8 +1879,6 @@ static void do_cmd_wiz_features(void)
  */
 static void wiz_test_kind(int tval)
 {
-	int py = player->py;
-	int px = player->px;
 	int sval;
 
 	struct object *obj, *known_obj;
@@ -1862,7 +1908,7 @@ static void wiz_test_kind(int tval)
 		obj->known = known_obj;
 
 		/* Drop the object from heaven */
-		drop_near(cave, &obj, 0, py, px, true);
+		drop_near(cave, &obj, 0, player->grid, true, true);
 	}
 
 	msg("Done.");
@@ -1930,6 +1976,7 @@ void do_cmd_wiz_effect(void)
 	char dice[80] = "0";
 	int index = -1;
 	int p1 = 0, p2 = 0, p3 = 0;
+	int y = 0, x = 0;
 	bool ident = false;
 
 	/* Avoid the prompt getting in the way */
@@ -1946,6 +1993,12 @@ void do_cmd_wiz_effect(void)
 		/* If not, find the effect with that name */
 		if (index <= EF_NONE || index >= EF_MAX)
 			index = effect_lookup(name);
+
+		/* Failed */
+		if (index <= EF_NONE || index >= EF_MAX) {
+			msg("No effect found.");
+			return;
+		}
 	}
 
 	/* Prompt */
@@ -1956,25 +2009,24 @@ void do_cmd_wiz_effect(void)
 		my_strcpy(dice, "0", sizeof(dice));
 
 	/* Get the parameters */
-	prt("Enter name or number for first parameter: ", 0, 0);
+	prt("Enter name or number for effect subtype: ", 0, 0);
 
 	/* Get the name */
 	if (askfor_aux(name, sizeof(name), NULL)) {
 		/* See if an effect parameter was entered */
-		p1 = effect_param(index, name);
+		p1 = effect_subtype(index, name);
 		if (p1 == -1) p1 = 0;
 	}
 
-	p2 = get_quantity("Enter second parameter: ", 100);
-	p3 = get_quantity("Enter third parameter: ", 100);
+	p2 = get_quantity("Enter second parameter (radius): ", 100);
+	p3 = get_quantity("Enter third parameter (other):", 100);
+	y = get_quantity("Enter y parameter:", 100);
+	x = get_quantity("Enter x parameter:", 100);
 
 	/* Reload the screen */
 	screen_load();
 
-	if (index > EF_NONE && index < EF_MAX)
-		effect_simple(index, source_player(), dice, p1, p2, p3, &ident);
-	else
-		msg("No effect found.");
+	effect_simple(index, source_player(), dice, p1, p2, p3, y, x, &ident);
 
 	if (ident)
 		msg("Identified!");
@@ -1986,9 +2038,6 @@ void do_cmd_wiz_effect(void)
  */
 void get_debug_command(void)
 {
-	int py = player->py;
-	int px = player->px;
-
 	char cmd;
 
 	/* Get a "debug command" */
@@ -2055,13 +2104,13 @@ void get_debug_command(void)
 		/* Detect everything */
 		case 'd':
 		{
-			effect_simple(EF_DETECT_TRAPS, source_player(), "22d40", 0, 0, 0, NULL);
-			effect_simple(EF_DETECT_DOORS, source_player(), "22d40", 0, 0, 0, NULL);
-			effect_simple(EF_DETECT_STAIRS, source_player(), "22d40", 0, 0, 0, NULL);
-			effect_simple(EF_DETECT_GOLD, source_player(), "22d40", 0, 0, 0, NULL);
-			effect_simple(EF_DETECT_OBJECTS, source_player(), "22d40", 0, 0, 0, NULL);
-			effect_simple(EF_DETECT_VISIBLE_MONSTERS, source_player(), "22d40", 0, 0, 0, NULL);
-			effect_simple(EF_DETECT_INVISIBLE_MONSTERS, source_player(), "22d40", 0, 0, 0, NULL);
+			effect_simple(EF_DETECT_TRAPS, source_player(), "0", 0, 0, 0, 22, 40, NULL);
+			effect_simple(EF_DETECT_DOORS, source_player(), "0", 0, 0, 0, 22, 40, NULL);
+			effect_simple(EF_DETECT_STAIRS, source_player(), "0", 0, 0, 0, 22, 40, NULL);
+			effect_simple(EF_DETECT_GOLD, source_player(), "0", 0, 0, 0, 22, 40, NULL);
+			effect_simple(EF_DETECT_OBJECTS, source_player(), "0", 0, 0, 0, 22, 40, NULL);
+			effect_simple(EF_DETECT_VISIBLE_MONSTERS, source_player(), "0", 0, 0, 0, 22, 40, NULL);
+			effect_simple(EF_DETECT_INVISIBLE_MONSTERS, source_player(), "0", 0, 0, 0, 22, 40, NULL);
 			break;
 		}
 
@@ -2106,7 +2155,7 @@ void get_debug_command(void)
 			n= get_quantity("How many good objects? ", 40);
 			screen_load();
 			if (n < 1) n = 1;
-			acquirement(py, px, player->depth, n, false);
+			acquirement(player->grid, player->depth, n, false);
 			break;
 		}
 
@@ -2127,7 +2176,7 @@ void get_debug_command(void)
 		/* Hit all monsters in LOS */
 		case 'H':
 		{
-			effect_simple(EF_PROJECT_LOS, source_player(), "10000", PROJ_DISP_ALL, 0, 0, NULL);
+			effect_simple(EF_PROJECT_LOS, source_player(), "10000", PROJ_DISP_ALL, 0, 0, 0, 0, NULL);
 			break;
 		}
 
@@ -2155,7 +2204,7 @@ void get_debug_command(void)
 		/* Magic Mapping */
 		case 'm':
 		{
-			effect_simple(EF_MAP_AREA, source_player(), "22d40", 0, 0, 0, NULL);
+			effect_simple(EF_MAP_AREA, source_player(), "0", 0, 0, 0, 22, 40, NULL);
 			break;
 		}
 
@@ -2207,7 +2256,7 @@ void get_debug_command(void)
 		case 'p':
 		{
 			const char *near = "10";
-			effect_simple(EF_TELEPORT, source_player(), near, 0, 1, 0, NULL);
+			effect_simple(EF_TELEPORT, source_player(), near, 0, 0, 0, 0, 0, NULL);
 			break;
 		}
 
@@ -2301,14 +2350,14 @@ void get_debug_command(void)
 		case 't':
 		{
 			const char *far = "100";
-			effect_simple(EF_TELEPORT, source_player(), far, 0, 1, 0, NULL);
+			effect_simple(EF_TELEPORT, source_player(), far, 0, 0, 0, 0, 0, NULL);
 			break;
 		}
 
 		/* Create a trap */
 		case 'T':
 		{
-			if (!square_isfloor(cave, player->py, player->px)) {
+			if (!square_isfloor(cave, player->grid)) {
 				msg("You can't place a trap there!");
 				break;
 			} else if (player->depth == 0) {
@@ -2322,7 +2371,7 @@ void get_debug_command(void)
 
 			struct trap_kind *trap = lookup_trap(buf);
 			if (trap) {
-				place_trap(cave, player->py, player->px, trap->tidx, 0);
+				place_trap(cave, player->grid, trap->tidx, 0);
 			} else {
 				msg("Trap not found.");
 			}
@@ -2333,8 +2382,8 @@ void get_debug_command(void)
 		/* Un-hide all monsters */
 		case 'u':
 		{
-			effect_simple(EF_DETECT_VISIBLE_MONSTERS, source_player(), "500d500", 0, 0, 0, NULL);
-			effect_simple(EF_DETECT_INVISIBLE_MONSTERS, source_player(), "500d500", 0, 0, 0, NULL);
+			effect_simple(EF_DETECT_VISIBLE_MONSTERS, source_player(), "0", 0, 0, 0, 500, 500, NULL);
+			effect_simple(EF_DETECT_INVISIBLE_MONSTERS, source_player(), "0", 0, 0, 0, 500, 500, NULL);
 			break;
 		}
 
@@ -2346,7 +2395,7 @@ void get_debug_command(void)
 			n = get_quantity("How many great objects? ", 40);
 			screen_load();
 			if (n < 1) n = 1;
-			acquirement(py, px, player->depth, n, true);
+			acquirement(player->grid, player->depth, n, true);
 			break;
 		}
 
