@@ -585,7 +585,7 @@ char *vformat(const char *fmt, va_list vp)
 		size_t len;
 
 		/* Build the string */
-		VA_COPY(args, vp);
+		va_copy(args, vp);
 		len = vstrnfmt(format_buf, format_len, fmt, args);
 		va_end(args);
 
