@@ -98,7 +98,7 @@ static void textblock_vappend_c(textblock *tb, byte attr, const char *fmt,
 		va_list args;
 		size_t len;
 
-		VA_COPY(args, vp);
+		va_copy(args, vp);
 		len = vstrnfmt(temp_space, temp_len, fmt, args);
 		va_end(args);
 		if (len < temp_len - 1) {
@@ -113,7 +113,7 @@ static void textblock_vappend_c(textblock *tb, byte attr, const char *fmt,
 	/* Get extent of addition in wide chars */
 	new_length = text_mbstowcs(NULL, temp_space, 0);
 	assert(new_length >= 0); /* If this fails, the string was badly formed */
-	textblock_resize_if_needed(tb, new_length);
+	textblock_resize_if_needed(tb, new_length + 1);
 
 	/* Convert to wide chars, into the text block buffer */
 	text_mbstowcs(tb->text + tb->strlen, temp_space, tb->size - tb->strlen);
@@ -162,6 +162,22 @@ void textblock_append_utf8(textblock *tb, const char *utf8_string)
 	tb->strlen += new_length;
 }
 
+/**
+ * Append one textblock to another.
+ *
+ * \param tb is the textblock we are appending to.
+ * \param tba is the textblock to append.
+ */
+void textblock_append_textblock(textblock *tb, const textblock *tba)
+{
+	textblock_resize_if_needed(tb, tba->strlen);
+	(void) memcpy(tb->text + tb->strlen, tba->text,
+		tba->strlen * sizeof(*tb->text));
+	(void) memcpy(tb->attrs + tb->strlen, tba->attrs, tba->strlen);
+	tb->strlen += tba->strlen;
+}
+
+
 /**
  * Add text to a text block, formatted.
  */
